@@ -28,6 +28,7 @@ import org.apache.flink.client.deployment.application.executors.EmbeddedExecutor
 import org.apache.flink.client.deployment.application.executors.EmbeddedExecutorServiceLoader;
 import org.apache.flink.client.program.PackagedProgram;
 import org.apache.flink.configuration.Configuration;
+import org.apache.flink.configuration.DeploymentOptions;
 import org.apache.flink.configuration.PipelineOptionsInternal;
 import org.apache.flink.core.execution.PipelineExecutorServiceLoader;
 import org.apache.flink.runtime.client.DuplicateJobSubmissionException;
@@ -79,9 +80,16 @@ import static org.apache.flink.util.Preconditions.checkNotNull;
 @Internal
 public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
 
+    public static final JobID ZERO_JOB_ID = new JobID(0, 0);
+
+    @VisibleForTesting static final String FAILED_JOB_NAME = "(application driver)";
+
     private static final Logger LOG = LoggerFactory.getLogger(ApplicationDispatcherBootstrap.class);
 
-    public static final JobID ZERO_JOB_ID = new JobID(0, 0);
+    private static boolean isCanceledOrFailed(ApplicationStatus applicationStatus) {
+        return applicationStatus == ApplicationStatus.CANCELED
+                || applicationStatus == ApplicationStatus.FAILED;
+    }
 
     private final PackagedProgram application;
 
@@ -93,7 +101,7 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
 
     private final CompletableFuture<Void> applicationCompletionFuture;
 
-    private final CompletableFuture<Acknowledge> clusterShutdownFuture;
+    private final CompletableFuture<Acknowledge> bootstrapCompletionFuture;
 
     private ScheduledFuture<?> applicationExecutionTask;
 
@@ -112,7 +120,7 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
         this.applicationCompletionFuture =
                 fixJobIdAndRunApplicationAsync(dispatcherGateway, scheduledExecutor);
 
-        this.clusterShutdownFuture = runApplicationAndShutdownClusterAsync(dispatcherGateway);
+        this.bootstrapCompletionFuture = finishBootstrapTasks(dispatcherGateway);
     }
 
     @Override
@@ -137,15 +145,16 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
     }
 
     @VisibleForTesting
-    CompletableFuture<Acknowledge> getClusterShutdownFuture() {
-        return clusterShutdownFuture;
+    CompletableFuture<Acknowledge> getBootstrapCompletionFuture() {
+        return bootstrapCompletionFuture;
     }
 
     /**
-     * Runs the user program entrypoint and shuts down the given dispatcherGateway when the
-     * application completes (either successfully or in case of failure).
+     * Logs final application status and invokes error handler in case of unexpected failures.
+     * Optionally shuts down the given dispatcherGateway when the application completes (either
+     * successfully or in case of failure), depending on the corresponding config option.
      */
-    private CompletableFuture<Acknowledge> runApplicationAndShutdownClusterAsync(
+    private CompletableFuture<Acknowledge> finishBootstrapTasks(
             final DispatcherGateway dispatcherGateway) {
         final CompletableFuture<Acknowledge> shutdownFuture =
                 applicationCompletionFuture
@@ -153,23 +162,18 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
                                 (ignored, t) -> {
                                     if (t == null) {
                                         LOG.info("Application completed SUCCESSFULLY");
-                                        return dispatcherGateway.shutDownCluster(
-                                                ApplicationStatus.SUCCEEDED);
+                                        return finish(
+                                                dispatcherGateway, ApplicationStatus.SUCCEEDED);
                                     }
-                                    final Optional<UnsuccessfulExecutionException> maybeException =
-                                            ExceptionUtils.findThrowable(
-                                                    t, UnsuccessfulExecutionException.class);
-                                    if (maybeException.isPresent()) {
+                                    final Optional<ApplicationStatus> maybeApplicationStatus =
+                                            extractApplicationStatus(t);
+                                    if (maybeApplicationStatus.isPresent()
+                                            && isCanceledOrFailed(maybeApplicationStatus.get())) {
                                         final ApplicationStatus applicationStatus =
-                                                maybeException.get().getStatus();
-                                        if (applicationStatus == ApplicationStatus.CANCELED
-                                                || applicationStatus == ApplicationStatus.FAILED) {
-                                            LOG.info("Application {}: ", applicationStatus, t);
-                                            return dispatcherGateway.shutDownCluster(
-                                                    applicationStatus);
-                                        }
+                                                maybeApplicationStatus.get();
+                                        LOG.info("Application {}: ", applicationStatus, t);
+                                        return finish(dispatcherGateway, applicationStatus);
                                     }
-
                                     if (t instanceof CancellationException) {
                                         LOG.warn(
                                                 "Application has been cancelled because the {} is being stopped.",
@@ -177,7 +181,6 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
                                                         .getSimpleName());
                                         return CompletableFuture.completedFuture(Acknowledge.get());
                                     }
-
                                     LOG.warn("Application failed unexpectedly: ", t);
                                     return FutureUtils.<Acknowledge>completedExceptionally(t);
                                 })
@@ -186,22 +189,38 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
         return shutdownFuture;
     }
 
+    private CompletableFuture<Acknowledge> finish(
+            DispatcherGateway dispatcherGateway, ApplicationStatus applicationStatus) {
+        boolean shouldShutDownOnFinish =
+                configuration.getBoolean(DeploymentOptions.SHUTDOWN_ON_APPLICATION_FINISH);
+        return shouldShutDownOnFinish
+                ? dispatcherGateway.shutDownCluster(applicationStatus)
+                : CompletableFuture.completedFuture(Acknowledge.get());
+    }
+
+    private Optional<ApplicationStatus> extractApplicationStatus(Throwable t) {
+        final Optional<UnsuccessfulExecutionException> maybeException =
+                ExceptionUtils.findThrowable(t, UnsuccessfulExecutionException.class);
+        return maybeException.map(UnsuccessfulExecutionException::getStatus);
+    }
+
     private CompletableFuture<Void> fixJobIdAndRunApplicationAsync(
             final DispatcherGateway dispatcherGateway, final ScheduledExecutor scheduledExecutor) {
-
         final Optional<String> configuredJobId =
                 configuration.getOptional(PipelineOptionsInternal.PIPELINE_FIXED_JOB_ID);
-
+        final boolean submitFailedJobOnApplicationError =
+                configuration.getBoolean(DeploymentOptions.SUBMIT_FAILED_JOB_ON_APPLICATION_ERROR);
         if (!HighAvailabilityMode.isHighAvailabilityModeActivated(configuration)
                 && !configuredJobId.isPresent()) {
-            return runApplicationAsync(dispatcherGateway, scheduledExecutor, false);
+            return runApplicationAsync(
+                    dispatcherGateway, scheduledExecutor, false, submitFailedJobOnApplicationError);
         }
-
         if (!configuredJobId.isPresent()) {
             configuration.set(
                     PipelineOptionsInternal.PIPELINE_FIXED_JOB_ID, ZERO_JOB_ID.toHexString());
         }
-        return runApplicationAsync(dispatcherGateway, scheduledExecutor, true);
+        return runApplicationAsync(
+                dispatcherGateway, scheduledExecutor, true, submitFailedJobOnApplicationError);
     }
 
     /**
@@ -212,7 +231,8 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
     private CompletableFuture<Void> runApplicationAsync(
             final DispatcherGateway dispatcherGateway,
             final ScheduledExecutor scheduledExecutor,
-            final boolean enforceSingleJobExecution) {
+            final boolean enforceSingleJobExecution,
+            final boolean submitFailedJobOnApplicationError) {
         final CompletableFuture<List<JobID>> applicationExecutionFuture = new CompletableFuture<>();
         final Set<JobID> tolerateMissingResult = Collections.synchronizedSet(new HashSet<>());
 
@@ -226,7 +246,8 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
                                         tolerateMissingResult,
                                         dispatcherGateway,
                                         scheduledExecutor,
-                                        enforceSingleJobExecution),
+                                        enforceSingleJobExecution,
+                                        submitFailedJobOnApplicationError),
                         0L,
                         TimeUnit.MILLISECONDS);
 
@@ -250,10 +271,19 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
             final Set<JobID> tolerateMissingResult,
             final DispatcherGateway dispatcherGateway,
             final ScheduledExecutor scheduledExecutor,
-            final boolean enforceSingleJobExecution) {
+            final boolean enforceSingleJobExecution,
+            final boolean submitFailedJobOnApplicationError) {
+        if (submitFailedJobOnApplicationError && !enforceSingleJobExecution) {
+            jobIdsFuture.completeExceptionally(
+                    new ApplicationExecutionException(
+                            String.format(
+                                    "Submission of failed job in case of an application error ('%s') is not supported in non-HA setups.",
+                                    DeploymentOptions.SUBMIT_FAILED_JOB_ON_APPLICATION_ERROR
+                                            .key())));
+            return;
+        }
+        final List<JobID> applicationJobIds = new ArrayList<>(recoveredJobIds);
         try {
-            final List<JobID> applicationJobIds = new ArrayList<>(recoveredJobIds);
-
             final PipelineExecutorServiceLoader executorServiceLoader =
                     new EmbeddedExecutorServiceLoader(
                             applicationJobIds, dispatcherGateway, scheduledExecutor);
@@ -283,6 +313,12 @@ public class ApplicationDispatcherBootstrap implements DispatcherBootstrap {
                 final JobID jobId = maybeDuplicate.get().getJobID();
                 tolerateMissingResult.add(jobId);
                 jobIdsFuture.complete(Collections.singletonList(jobId));
+            } else if (submitFailedJobOnApplicationError && applicationJobIds.isEmpty()) {
+                final JobID failedJobId =
+                        JobID.fromHexString(
+                                configuration.get(PipelineOptionsInternal.PIPELINE_FIXED_JOB_ID));
+                dispatcherGateway.submitFailedJob(failedJobId, FAILED_JOB_NAME, t);
+                jobIdsFuture.complete(Collections.singletonList(failedJobId));
             } else {
                 jobIdsFuture.completeExceptionally(
                         new ApplicationExecutionException("Could not execute application.", t));
