@@ -21,12 +21,14 @@ package org.apache.flink.client.program.rest;
 import org.apache.flink.api.common.JobExecutionResult;
 import org.apache.flink.api.common.JobID;
 import org.apache.flink.api.common.JobStatus;
+import org.apache.flink.api.common.cache.DistributedCache;
 import org.apache.flink.client.cli.DefaultCLI;
 import org.apache.flink.client.deployment.ClusterClientFactory;
 import org.apache.flink.client.deployment.ClusterClientServiceLoader;
 import org.apache.flink.client.deployment.ClusterDescriptor;
 import org.apache.flink.client.deployment.DefaultClusterClientServiceLoader;
 import org.apache.flink.client.deployment.StandaloneClusterId;
+import org.apache.flink.configuration.ConfigConstants;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.JobManagerOptions;
 import org.apache.flink.configuration.RestOptions;
@@ -107,12 +109,15 @@ import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
+import org.junit.rules.TemporaryFolder;
 
 import javax.annotation.Nonnull;
 
+import java.io.File;
 import java.io.IOException;
 import java.net.InetSocketAddress;
 import java.net.URL;
+import java.nio.file.Files;
 import java.util.ArrayDeque;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -153,7 +158,7 @@ import static org.junit.Assert.fail;
 public class RestClusterClientTest extends TestLogger {
 
     private final DispatcherGateway mockRestfulGateway =
-            new TestingDispatcherGateway.Builder().build();
+            TestingDispatcherGateway.newBuilder().build();
 
     private GatewayRetriever<DispatcherGateway> mockGatewayRetriever;
 
@@ -274,7 +279,7 @@ public class RestClusterClientTest extends TestLogger {
 
         @Override
         protected CompletableFuture<JobSubmitResponseBody> handleRequest(
-                @Nonnull HandlerRequest<JobSubmitRequestBody, EmptyMessageParameters> request,
+                @Nonnull HandlerRequest<JobSubmitRequestBody> request,
                 @Nonnull DispatcherGateway gateway)
                 throws RestHandlerException {
             jobSubmitted = true;
@@ -293,7 +298,7 @@ public class RestClusterClientTest extends TestLogger {
 
         @Override
         protected CompletableFuture<EmptyResponseBody> handleRequest(
-                @Nonnull HandlerRequest<EmptyRequestBody, JobCancellationMessageParameters> request,
+                @Nonnull HandlerRequest<EmptyRequestBody> request,
                 @Nonnull DispatcherGateway gateway)
                 throws RestHandlerException {
             jobCanceled = true;
@@ -386,9 +391,7 @@ public class RestClusterClientTest extends TestLogger {
 
             @Override
             protected CompletableFuture<TriggerResponse> handleRequest(
-                    @Nonnull
-                            HandlerRequest<SavepointDisposalRequest, EmptyMessageParameters>
-                                    request,
+                    @Nonnull HandlerRequest<SavepointDisposalRequest> request,
                     @Nonnull DispatcherGateway gateway) {
                 assertThat(request.getRequestBody().getSavepointPath(), is(savepointPath));
                 return CompletableFuture.completedFuture(new TriggerResponse(triggerId));
@@ -412,11 +415,7 @@ public class RestClusterClientTest extends TestLogger {
             @Override
             protected CompletableFuture<AsynchronousOperationResult<AsynchronousOperationInfo>>
                     handleRequest(
-                            @Nonnull
-                                    HandlerRequest<
-                                                    EmptyRequestBody,
-                                                    SavepointDisposalStatusMessageParameters>
-                                            request,
+                            @Nonnull HandlerRequest<EmptyRequestBody> request,
                             @Nonnull DispatcherGateway gateway)
                             throws RestHandlerException {
                 final TriggerId actualTriggerId =
@@ -581,7 +580,7 @@ public class RestClusterClientTest extends TestLogger {
 
         @Override
         protected CompletableFuture<JobExecutionResultResponseBody> handleRequest(
-                @Nonnull HandlerRequest<EmptyRequestBody, JobMessageParameters> request,
+                @Nonnull HandlerRequest<EmptyRequestBody> request,
                 @Nonnull DispatcherGateway gateway) {
             if (jobExecutionResults.hasNext()) {
                 lastJobExecutionResult = jobExecutionResults.next();
@@ -724,6 +723,45 @@ public class RestClusterClientTest extends TestLogger {
         }
     }
 
+    @Test(timeout = 120_000)
+    public void testJobSubmissionWithoutUserArtifact() throws Exception {
+        try (final TestRestServerEndpoint restServerEndpoint =
+                createRestServerEndpoint(new TestJobSubmitHandler())) {
+            try (RestClusterClient<?> restClusterClient =
+                    createRestClusterClient(restServerEndpoint.getServerAddress().getPort())) {
+
+                restClusterClient.submitJob(jobGraph).get();
+            }
+        }
+    }
+
+    @Test(timeout = 120_000)
+    public void testJobSubmissionWithUserArtifact() throws Exception {
+        try (final TestRestServerEndpoint restServerEndpoint =
+                createRestServerEndpoint(new TestJobSubmitHandler())) {
+            try (RestClusterClient<?> restClusterClient =
+                    createRestClusterClient(restServerEndpoint.getServerAddress().getPort())) {
+
+                TemporaryFolder temporaryFolder = new TemporaryFolder();
+                temporaryFolder.create();
+                File file = temporaryFolder.newFile();
+                Files.write(file.toPath(), "hello world".getBytes(ConfigConstants.DEFAULT_CHARSET));
+
+                // Add file path with scheme
+                jobGraph.addUserArtifact(
+                        "file",
+                        new DistributedCache.DistributedCacheEntry(file.toURI().toString(), false));
+
+                // Add file path without scheme
+                jobGraph.addUserArtifact(
+                        "file2",
+                        new DistributedCache.DistributedCacheEntry(file.toURI().getPath(), false));
+
+                restClusterClient.submitJob(jobGraph).get();
+            }
+        }
+    }
+
     @Test
     public void testJobSubmissionFailureCauseForwardedToClient() throws Exception {
         try (final TestRestServerEndpoint restServerEndpoint =
@@ -755,7 +793,7 @@ public class RestClusterClientTest extends TestLogger {
 
         @Override
         protected CompletableFuture<JobSubmitResponseBody> handleRequest(
-                @Nonnull HandlerRequest<JobSubmitRequestBody, EmptyMessageParameters> request,
+                @Nonnull HandlerRequest<JobSubmitRequestBody> request,
                 @Nonnull DispatcherGateway gateway)
                 throws RestHandlerException {
             throw new RestHandlerException("expected", HttpResponseStatus.INTERNAL_SERVER_ERROR);
@@ -804,7 +842,7 @@ public class RestClusterClientTest extends TestLogger {
 
         @Override
         protected CompletableFuture<EmptyResponseBody> handleRequest(
-                @Nonnull HandlerRequest<EmptyRequestBody, EmptyMessageParameters> request,
+                @Nonnull HandlerRequest<EmptyRequestBody> request,
                 @Nonnull DispatcherGateway gateway)
                 throws RestHandlerException {
             final CompletableFuture<EmptyResponseBody> result = responseQueue.poll();
@@ -935,11 +973,7 @@ public class RestClusterClientTest extends TestLogger {
         @Override
         @SuppressWarnings("unchecked")
         protected CompletableFuture<ClientCoordinationResponseBody> handleRequest(
-                @Nonnull
-                        HandlerRequest<
-                                        ClientCoordinationRequestBody,
-                                        ClientCoordinationMessageParameters>
-                                request,
+                @Nonnull HandlerRequest<ClientCoordinationRequestBody> request,
                 @Nonnull DispatcherGateway gateway)
                 throws RestHandlerException {
             try {
@@ -989,7 +1023,7 @@ public class RestClusterClientTest extends TestLogger {
 
         @Override
         protected CompletableFuture<JobAccumulatorsInfo> handleRequest(
-                @Nonnull HandlerRequest<EmptyRequestBody, JobAccumulatorsMessageParameters> request,
+                @Nonnull HandlerRequest<EmptyRequestBody> request,
                 @Nonnull DispatcherGateway gateway)
                 throws RestHandlerException {
             JobAccumulatorsInfo accumulatorsInfo;
@@ -1046,7 +1080,7 @@ public class RestClusterClientTest extends TestLogger {
 
         @Override
         protected CompletableFuture<MultipleJobsDetails> handleRequest(
-                @Nonnull HandlerRequest<EmptyRequestBody, EmptyMessageParameters> request,
+                @Nonnull HandlerRequest<EmptyRequestBody> request,
                 @Nonnull DispatcherGateway gateway)
                 throws RestHandlerException {
             JobDetails running =
@@ -1073,7 +1107,7 @@ public class RestClusterClientTest extends TestLogger {
 
         @Override
         protected CompletableFuture<JobDetailsInfo> handleRequest(
-                @Nonnull HandlerRequest<EmptyRequestBody, JobMessageParameters> request,
+                @Nonnull HandlerRequest<EmptyRequestBody> request,
                 @Nonnull DispatcherGateway gateway)
                 throws RestHandlerException {
             if (!jobDetailsInfo.hasNext()) {
