@@ -48,6 +48,7 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.atomic.AtomicLong;
 
 import static org.mockito.Matchers.any;
 import static org.mockito.Mockito.doAnswer;
@@ -665,6 +666,12 @@ public class ElasticsearchSinkBaseTest {
             }
         }
 
+        @Override
+        public void configureBulkProcessorFlushInterval(
+                BulkProcessor.Builder builder, long flushIntervalMillis) {
+            // no need for this in the test cases here
+        }
+
         @Override
         public void configureBulkProcessorBackoff(
                 BulkProcessor.Builder builder,
@@ -676,6 +683,14 @@ public class ElasticsearchSinkBaseTest {
         public void verifyClientConnection(Client client) {
             // no need for this in the test cases here
         }
+
+        @Override
+        public RequestIndexer createBulkProcessorIndexer(
+                BulkProcessor bulkProcessor,
+                boolean flushOnCheckpoint,
+                AtomicLong numPendingRequestsRef) {
+            return new TestRequestIndexer(bulkProcessor, flushOnCheckpoint, numPendingRequestsRef);
+        }
     }
 
     private static class SimpleSinkFunction<String> implements ElasticsearchSinkFunction<String> {
