@@ -20,8 +20,10 @@ package org.apache.flink.connector.file.sink.writer;
 
 import org.apache.flink.annotation.Internal;
 import org.apache.flink.annotation.VisibleForTesting;
-import org.apache.flink.api.connector.sink.Sink;
-import org.apache.flink.api.connector.sink.SinkWriter;
+import org.apache.flink.api.common.operators.ProcessingTimeService;
+import org.apache.flink.api.connector.sink2.SinkWriter;
+import org.apache.flink.api.connector.sink2.StatefulSink.StatefulSinkWriter;
+import org.apache.flink.api.connector.sink2.TwoPhaseCommittingSink;
 import org.apache.flink.connector.file.sink.FileSink;
 import org.apache.flink.connector.file.sink.FileSinkCommittable;
 import org.apache.flink.core.fs.Path;
@@ -39,6 +41,7 @@ import javax.annotation.Nullable;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
@@ -58,8 +61,10 @@ import static org.apache.flink.util.Preconditions.checkState;
  */
 @Internal
 public class FileWriter<IN>
-        implements SinkWriter<IN, FileSinkCommittable, FileWriterBucketState>,
-                Sink.ProcessingTimeService.ProcessingTimeCallback {
+        implements StatefulSinkWriter<IN, FileWriterBucketState>,
+                TwoPhaseCommittingSink.PrecommittingSinkWriter<IN, FileSinkCommittable>,
+                SinkWriter<IN>,
+                ProcessingTimeService.ProcessingTimeCallback {
 
     private static final Logger LOG = LoggerFactory.getLogger(FileWriter.class);
 
@@ -75,7 +80,7 @@ public class FileWriter<IN>
 
     private final RollingPolicy<IN, String> rollingPolicy;
 
-    private final Sink.ProcessingTimeService processingTimeService;
+    private final ProcessingTimeService processingTimeService;
 
     private final long bucketCheckInterval;
 
@@ -87,7 +92,9 @@ public class FileWriter<IN>
 
     private final OutputFileConfig outputFileConfig;
 
-    private final Counter recordsOutCounter;
+    private final Counter numRecordsSendCounter;
+
+    private boolean endOfInput;
 
     /**
      * A constructor creating a new empty bucket manager.
@@ -107,7 +114,7 @@ public class FileWriter<IN>
             final BucketWriter<IN, String> bucketWriter,
             final RollingPolicy<IN, String> rollingPolicy,
             final OutputFileConfig outputFileConfig,
-            final Sink.ProcessingTimeService processingTimeService,
+            final ProcessingTimeService processingTimeService,
             final long bucketCheckInterval) {
 
         this.basePath = checkNotNull(basePath);
@@ -121,8 +128,7 @@ public class FileWriter<IN>
         this.activeBuckets = new HashMap<>();
         this.bucketerContext = new BucketerContext();
 
-        this.recordsOutCounter =
-                checkNotNull(metricGroup).getIOMetricGroup().getNumRecordsOutCounter();
+        this.numRecordsSendCounter = checkNotNull(metricGroup).getNumRecordsSendCounter();
         this.processingTimeService = checkNotNull(processingTimeService);
         checkArgument(
                 bucketCheckInterval > 0,
@@ -148,7 +154,7 @@ public class FileWriter<IN>
      * @throws IOException if anything goes wrong during retrieving the state or
      *     restoring/committing of any in-progress/pending part files
      */
-    public void initializeState(List<FileWriterBucketState> bucketStates) throws IOException {
+    public void initializeState(Collection<FileWriterBucketState> bucketStates) throws IOException {
         checkNotNull(bucketStates, "The retrieved state was null.");
 
         for (FileWriterBucketState state : bucketStates) {
@@ -179,7 +185,7 @@ public class FileWriter<IN>
     }
 
     @Override
-    public void write(IN element, Context context) throws IOException {
+    public void write(IN element, Context context) throws IOException, InterruptedException {
         // setting the values in the bucketer context
         bucketerContext.update(
                 context.timestamp(),
@@ -189,11 +195,16 @@ public class FileWriter<IN>
         final String bucketId = bucketAssigner.getBucketId(element, bucketerContext);
         final FileWriterBucket<IN> bucket = getOrCreateBucketForBucketId(bucketId);
         bucket.write(element, processingTimeService.getCurrentProcessingTime());
-        recordsOutCounter.inc();
+        numRecordsSendCounter.inc();
     }
 
     @Override
-    public List<FileSinkCommittable> prepareCommit(boolean flush) throws IOException {
+    public void flush(boolean endOfInput) throws IOException, InterruptedException {
+        this.endOfInput = endOfInput;
+    }
+
+    @Override
+    public Collection<FileSinkCommittable> prepareCommit() throws IOException {
         List<FileSinkCommittable> committables = new ArrayList<>();
 
         // Every time before we prepare commit, we first check and remove the inactive
@@ -206,7 +217,7 @@ public class FileWriter<IN>
             if (!entry.getValue().isActive()) {
                 activeBucketIt.remove();
             } else {
-                committables.addAll(entry.getValue().prepareCommit(flush));
+                committables.addAll(entry.getValue().prepareCommit(endOfInput));
             }
         }
 
@@ -263,7 +274,7 @@ public class FileWriter<IN>
     private void registerNextBucketInspectionTimer() {
         final long nextInspectionTime =
                 processingTimeService.getCurrentProcessingTime() + bucketCheckInterval;
-        processingTimeService.registerProcessingTimer(nextInspectionTime, this);
+        processingTimeService.registerTimer(nextInspectionTime, this);
     }
 
     /**
