@@ -40,6 +40,7 @@ import org.apache.flink.table.expressions.ExpressionVisitor;
 import org.apache.flink.table.expressions.FieldReferenceExpression;
 import org.apache.flink.table.expressions.TypeLiteralExpression;
 import org.apache.flink.table.expressions.ValueLiteralExpression;
+import org.apache.flink.table.factories.ManagedTableFactory;
 import org.apache.flink.table.functions.BuiltInFunctionDefinitions;
 import org.apache.flink.table.functions.FunctionDefinition;
 import org.apache.flink.table.functions.hive.conversion.HiveInspectors;
@@ -351,8 +352,9 @@ public class HiveTableUtil {
             ObjectPath tablePath,
             CatalogBaseTable baseTable,
             Table oldHiveTable,
-            HiveConf hiveConf) {
-        Table newHiveTable = instantiateHiveTable(tablePath, baseTable, hiveConf);
+            HiveConf hiveConf,
+            boolean managedTable) {
+        Table newHiveTable = instantiateHiveTable(tablePath, baseTable, hiveConf, managedTable);
         // client.alter_table() requires a valid location
         // thus, if new table doesn't have that, it reuses location of the old table
         if (!newHiveTable.getSd().isSetLocation()) {
@@ -362,7 +364,7 @@ public class HiveTableUtil {
     }
 
     public static Table instantiateHiveTable(
-            ObjectPath tablePath, CatalogBaseTable table, HiveConf hiveConf) {
+            ObjectPath tablePath, CatalogBaseTable table, HiveConf hiveConf, boolean managedTable) {
         final boolean isView = table instanceof CatalogView;
         // let Hive set default parameters for us, e.g. serialization.format
         Table hiveTable =
@@ -371,6 +373,9 @@ public class HiveTableUtil {
         hiveTable.setCreateTime((int) (System.currentTimeMillis() / 1000));
 
         Map<String, String> properties = new HashMap<>(table.getOptions());
+        if (managedTable) {
+            properties.put(CONNECTOR.key(), ManagedTableFactory.DEFAULT_IDENTIFIER);
+        }
         // Table comment
         if (table.getComment() != null) {
             properties.put(HiveCatalogConfig.COMMENT, table.getComment());
