@@ -24,7 +24,6 @@ import org.apache.flink.connector.jdbc.internal.options.JdbcDmlOptions;
 import org.apache.flink.connector.jdbc.internal.options.JdbcLookupOptions;
 import org.apache.flink.connector.jdbc.internal.options.JdbcReadOptions;
 import org.apache.flink.table.api.DataTypes;
-import org.apache.flink.table.api.TableSchema;
 import org.apache.flink.table.catalog.Column;
 import org.apache.flink.table.catalog.ResolvedSchema;
 import org.apache.flink.table.catalog.UniqueConstraint;
@@ -46,8 +45,8 @@ import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 /**
- * Test for {@link JdbcTableSource} and {@link JdbcUpsertTableSink} created by {@link
- * JdbcTableSourceSinkFactory}.
+ * Test for {@link JdbcDynamicTableSource} and {@link JdbcDynamicTableSink} created by {@link
+ * JdbcDynamicTableFactory}.
  */
 public class JdbcDynamicTableFactoryTest {
 
@@ -92,7 +91,7 @@ public class JdbcDynamicTableFactoryTest {
                         options,
                         JdbcReadOptions.builder().build(),
                         lookupOptions,
-                        TableSchema.fromResolvedSchema(SCHEMA));
+                        SCHEMA.toPhysicalRowDataType());
         assertEquals(expectedSource, actualSource);
 
         // validation for sink
@@ -113,10 +112,7 @@ public class JdbcDynamicTableFactoryTest {
                         .build();
         JdbcDynamicTableSink expectedSink =
                 new JdbcDynamicTableSink(
-                        options,
-                        executionOptions,
-                        dmlOptions,
-                        TableSchema.fromResolvedSchema(SCHEMA));
+                        options, executionOptions, dmlOptions, SCHEMA.toPhysicalRowDataType());
         assertEquals(expectedSink, actualSink);
     }
 
@@ -154,10 +150,7 @@ public class JdbcDynamicTableFactoryTest {
                         .build();
         JdbcDynamicTableSource expected =
                 new JdbcDynamicTableSource(
-                        options,
-                        readOptions,
-                        lookupOptions,
-                        TableSchema.fromResolvedSchema(SCHEMA));
+                        options, readOptions, lookupOptions, SCHEMA.toPhysicalRowDataType());
 
         assertEquals(expected, actual);
     }
@@ -187,7 +180,7 @@ public class JdbcDynamicTableFactoryTest {
                         options,
                         JdbcReadOptions.builder().build(),
                         lookupOptions,
-                        TableSchema.fromResolvedSchema(SCHEMA));
+                        SCHEMA.toPhysicalRowDataType());
 
         assertEquals(expected, actual);
     }
@@ -222,10 +215,7 @@ public class JdbcDynamicTableFactoryTest {
 
         JdbcDynamicTableSink expected =
                 new JdbcDynamicTableSink(
-                        options,
-                        executionOptions,
-                        dmlOptions,
-                        TableSchema.fromResolvedSchema(SCHEMA));
+                        options, executionOptions, dmlOptions, SCHEMA.toPhysicalRowDataType());
 
         assertEquals(expected, actual);
     }
@@ -259,10 +249,7 @@ public class JdbcDynamicTableFactoryTest {
 
         JdbcDynamicTableSink expected =
                 new JdbcDynamicTableSink(
-                        options,
-                        executionOptions,
-                        dmlOptions,
-                        TableSchema.fromResolvedSchema(SCHEMA));
+                        options, executionOptions, dmlOptions, SCHEMA.toPhysicalRowDataType());
 
         assertEquals(expected, actual);
     }
@@ -402,6 +389,38 @@ public class JdbcDynamicTableFactoryTest {
         }
     }
 
+    @Test
+    public void testJdbcLookupPropertiesWithExcludeEmptyResult() {
+        Map<String, String> properties = getAllOptions();
+        properties.put("lookup.cache.max-rows", "1000");
+        properties.put("lookup.cache.ttl", "10s");
+        properties.put("lookup.max-retries", "10");
+        properties.put("lookup.cache.caching-missing-key", "true");
+
+        DynamicTableSource actual = createTableSource(SCHEMA, properties);
+
+        JdbcConnectorOptions options =
+                JdbcConnectorOptions.builder()
+                        .setDBUrl("jdbc:derby:memory:mydb")
+                        .setTableName("mytable")
+                        .build();
+        JdbcLookupOptions lookupOptions =
+                JdbcLookupOptions.builder()
+                        .setCacheMaxSize(1000)
+                        .setCacheExpireMs(10_000)
+                        .setMaxRetryTimes(10)
+                        .setCacheMissingKey(true)
+                        .build();
+        JdbcDynamicTableSource expected =
+                new JdbcDynamicTableSource(
+                        options,
+                        JdbcReadOptions.builder().build(),
+                        lookupOptions,
+                        SCHEMA.toPhysicalRowDataType());
+
+        assertEquals(expected, actual);
+    }
+
     private Map<String, String> getAllOptions() {
         Map<String, String> options = new HashMap<>();
         options.put("connector", "jdbc");
