@@ -18,6 +18,8 @@
 
 package org.apache.flink.connector.kafka.source.reader;
 
+import org.apache.flink.annotation.Internal;
+import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.api.connector.source.SourceReaderContext;
 import org.apache.flink.connector.base.source.reader.RecordsWithSplitIds;
 import org.apache.flink.connector.base.source.reader.splitreader.SplitReader;
@@ -54,10 +56,10 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
 import java.util.StringJoiner;
-import java.util.function.Supplier;
 import java.util.stream.Collectors;
 
 /** A {@link SplitReader} implementation that reads records from Kafka partitions. */
+@Internal
 public class KafkaPartitionSplitReader
         implements SplitReader<ConsumerRecord<byte[], byte[]>, KafkaPartitionSplit> {
     private static final Logger LOG = LoggerFactory.getLogger(KafkaPartitionSplitReader.class);
@@ -96,16 +98,9 @@ public class KafkaPartitionSplitReader
         ConsumerRecords<byte[], byte[]> consumerRecords;
         try {
             consumerRecords = consumer.poll(Duration.ofMillis(POLL_TIMEOUT));
-        } catch (WakeupException | IllegalStateException e) {
-            // IllegalStateException will be thrown if the consumer is not assigned any partitions.
-            // This happens if all assigned partitions are invalid or empty (starting offset >=
-            // stopping offset). We just mark empty partitions as finished and return an empty
-            // record container, and this consumer will be closed by SplitFetcherManager.
-            KafkaPartitionSplitRecords recordsBySplits =
-                    new KafkaPartitionSplitRecords(
-                            ConsumerRecords.empty(), kafkaSourceReaderMetrics);
-            markEmptySplitsAsFinished(recordsBySplits);
-            return recordsBySplits;
+        } catch (WakeupException we) {
+            return new KafkaPartitionSplitRecords(
+                    ConsumerRecords.empty(), kafkaSourceReaderMetrics);
         }
         KafkaPartitionSplitRecords recordsBySplits =
                 new KafkaPartitionSplitRecords(consumerRecords, kafkaSourceReaderMetrics);
@@ -136,7 +131,12 @@ public class KafkaPartitionSplitReader
             kafkaSourceReaderMetrics.maybeAddRecordsLagMetric(consumer, tp);
         }
 
-        markEmptySplitsAsFinished(recordsBySplits);
+        // Some splits are discovered as empty when handling split additions. These splits should be
+        // added to finished splits to clean up states in split fetcher and source reader.
+        if (!emptySplits.isEmpty()) {
+            recordsBySplits.finishedSplits.addAll(emptySplits);
+            emptySplits.clear();
+        }
 
         // Unassign the partitions that has finished.
         if (!finishedPartitions.isEmpty()) {
@@ -150,15 +150,6 @@ public class KafkaPartitionSplitReader
         return recordsBySplits;
     }
 
-    private void markEmptySplitsAsFinished(KafkaPartitionSplitRecords recordsBySplits) {
-        // Some splits are discovered as empty when handling split additions. These splits should be
-        // added to finished splits to clean up states in split fetcher and source reader.
-        if (!emptySplits.isEmpty()) {
-            recordsBySplits.finishedSplits.addAll(emptySplits);
-            emptySplits.clear();
-        }
-    }
-
     @Override
     public void handleSplitsChanges(SplitsChange<KafkaPartitionSplit> splitsChange) {
         // Get all the partition assignments and stopping offsets.
@@ -232,6 +223,11 @@ public class KafkaPartitionSplitReader
         consumer.commitAsync(offsetsToCommit, offsetCommitCallback);
     }
 
+    @VisibleForTesting
+    KafkaConsumer<byte[], byte[]> consumer() {
+        return consumer;
+    }
+
     // --------------- private helper method ----------------------
 
     private void parseStartingOffsets(
@@ -306,9 +302,7 @@ public class KafkaPartitionSplitReader
         Map<TopicPartition, Long> endOffset = consumer.endOffsets(partitionsStoppingAtLatest);
         stoppingOffsets.putAll(endOffset);
         if (!partitionsStoppingAtCommitted.isEmpty()) {
-            retryOnWakeup(
-                            () -> consumer.committed(partitionsStoppingAtCommitted),
-                            "getting committed offset as stopping offsets")
+            consumer.committed(partitionsStoppingAtCommitted)
                     .forEach(
                             (tp, offsetAndMetadata) -> {
                                 Preconditions.checkNotNull(
@@ -326,10 +320,7 @@ public class KafkaPartitionSplitReader
         List<TopicPartition> emptyPartitions = new ArrayList<>();
         // If none of the partitions have any records,
         for (TopicPartition tp : consumer.assignment()) {
-            if (retryOnWakeup(
-                            () -> consumer.position(tp),
-                            "getting starting offset to check if split is empty")
-                    >= getStoppingOffset(tp)) {
+            if (consumer.position(tp) >= getStoppingOffset(tp)) {
                 emptyPartitions.add(tp);
             }
         }
@@ -352,10 +343,7 @@ public class KafkaPartitionSplitReader
         if (LOG.isDebugEnabled()) {
             StringJoiner splitsInfo = new StringJoiner(",");
             for (KafkaPartitionSplit split : splitsChange.splits()) {
-                long startingOffset =
-                        retryOnWakeup(
-                                () -> consumer.position(split.getTopicPartition()),
-                                "logging starting position");
+                long startingOffset = consumer.position(split.getTopicPartition());
                 long stoppingOffset = getStoppingOffset(split.getTopicPartition());
                 splitsInfo.add(
                         String.format(
@@ -410,33 +398,6 @@ public class KafkaPartitionSplitReader
         }
     }
 
-    /**
-     * Catch {@link WakeupException} in Kafka consumer call and retry the invocation on exception.
-     *
-     * <p>This helper function handles a race condition as below:
-     *
-     * <ol>
-     *   <li>Fetcher thread finishes a {@link KafkaConsumer#poll(Duration)} call
-     *   <li>Task thread assigns new splits so invokes {@link #wakeUp()}, then the wakeup is
-     *       recorded and held by the consumer
-     *   <li>Later fetcher thread invokes {@link #handleSplitsChanges(SplitsChange)}, and
-     *       interactions with consumer will throw {@link WakeupException} because of the previously
-     *       held wakeup in the consumer
-     * </ol>
-     *
-     * <p>Under this case we need to catch the {@link WakeupException} and retry the operation.
-     */
-    private <V> V retryOnWakeup(Supplier<V> consumerCall, String description) {
-        try {
-            return consumerCall.get();
-        } catch (WakeupException we) {
-            LOG.info(
-                    "Caught WakeupException while executing Kafka consumer call for {}. Will retry the consumer call.",
-                    description);
-            return consumerCall.get();
-        }
-    }
-
     // ---------------- private helper class ------------------------
 
     private static class KafkaPartitionSplitRecords
