@@ -29,15 +29,16 @@ import org.apache.flink.api.common.typeinfo.TypeInformation;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
 import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
-import org.apache.flink.connector.kafka.source.testutils.KafkaMultipleTopicExternalContext;
-import org.apache.flink.connector.kafka.source.testutils.KafkaSingleTopicExternalContext;
-import org.apache.flink.connector.kafka.source.testutils.KafkaSourceTestEnv;
-import org.apache.flink.connectors.test.common.environment.MiniClusterTestEnvironment;
-import org.apache.flink.connectors.test.common.external.DefaultContainerizedExternalSystem;
-import org.apache.flink.connectors.test.common.junit.annotations.ExternalContextFactory;
-import org.apache.flink.connectors.test.common.junit.annotations.ExternalSystem;
-import org.apache.flink.connectors.test.common.junit.annotations.TestEnv;
-import org.apache.flink.connectors.test.common.testsuites.SourceTestSuiteBase;
+import org.apache.flink.connector.kafka.testutils.KafkaSourceExternalContextFactory;
+import org.apache.flink.connector.kafka.testutils.KafkaSourceTestEnv;
+import org.apache.flink.connector.testframe.environment.MiniClusterTestEnvironment;
+import org.apache.flink.connector.testframe.external.DefaultContainerizedExternalSystem;
+import org.apache.flink.connector.testframe.junit.annotations.TestContext;
+import org.apache.flink.connector.testframe.junit.annotations.TestEnv;
+import org.apache.flink.connector.testframe.junit.annotations.TestExternalSystem;
+import org.apache.flink.connector.testframe.junit.annotations.TestSemantics;
+import org.apache.flink.connector.testframe.testsuites.SourceTestSuiteBase;
+import org.apache.flink.streaming.api.CheckpointingMode;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.streaming.api.functions.ProcessFunction;
@@ -47,6 +48,7 @@ import org.apache.flink.streaming.api.operators.StreamMap;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
 import org.apache.flink.util.CloseableIterator;
 import org.apache.flink.util.Collector;
+import org.apache.flink.util.DockerImageVersions;
 
 import org.apache.kafka.clients.consumer.ConsumerRecord;
 import org.apache.kafka.clients.producer.ProducerRecord;
@@ -76,9 +78,10 @@ import java.util.UUID;
 import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.atomic.AtomicInteger;
 
+import static org.apache.flink.connector.kafka.testutils.KafkaSourceExternalContext.SplitMappingMode.PARTITION;
+import static org.apache.flink.connector.kafka.testutils.KafkaSourceExternalContext.SplitMappingMode.TOPIC;
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.jupiter.api.Assertions.assertEquals;
-import static org.junit.jupiter.api.Assertions.assertFalse;
-import static org.junit.jupiter.api.Assertions.assertTrue;
 
 /** Unite test class for {@link KafkaSource}. */
 public class KafkaSourceITCase {
@@ -182,36 +185,33 @@ public class KafkaSourceITCase {
 
             StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
             env.setParallelism(1);
-
-            try (CloseableIterator<Integer> resultIterator =
+            final CloseableIterator<Integer> resultIterator =
                     env.fromSource(
                                     source,
                                     WatermarkStrategy.noWatermarks(),
                                     "testValueOnlyDeserializer")
-                            .executeAndCollect()) {
-                AtomicInteger actualSum = new AtomicInteger();
-                resultIterator.forEachRemaining(actualSum::addAndGet);
+                            .executeAndCollect();
 
-                // Calculate the actual sum of values
-                // Values in a partition should start from partition ID, and end with
-                // (NUM_RECORDS_PER_PARTITION - 1)
-                // e.g. Values in partition 5 should be {5, 6, 7, 8, 9}
-                int expectedSum = 0;
-                for (int partition = 0;
-                        partition < KafkaSourceTestEnv.NUM_PARTITIONS;
-                        partition++) {
-                    for (int value = partition;
-                            value < KafkaSourceTestEnv.NUM_RECORDS_PER_PARTITION;
-                            value++) {
-                        expectedSum += value;
-                    }
+            AtomicInteger actualSum = new AtomicInteger();
+            resultIterator.forEachRemaining(actualSum::addAndGet);
+
+            // Calculate the actual sum of values
+            // Values in a partition should start from partition ID, and end with
+            // (NUM_RECORDS_PER_PARTITION - 1)
+            // e.g. Values in partition 5 should be {5, 6, 7, 8, 9}
+            int expectedSum = 0;
+            for (int partition = 0; partition < KafkaSourceTestEnv.NUM_PARTITIONS; partition++) {
+                for (int value = partition;
+                        value < KafkaSourceTestEnv.NUM_RECORDS_PER_PARTITION;
+                        value++) {
+                    expectedSum += value;
                 }
-
-                // Since we have two topics, the expected sum value should be doubled
-                expectedSum *= 2;
-
-                assertEquals(expectedSum, actualSum.get());
             }
+
+            // Since we have two topics, the expected sum value should be doubled
+            expectedSum *= 2;
+
+            assertEquals(expectedSum, actualSum.get());
         }
 
         @ParameterizedTest(name = "Object reuse in deserializer = {arguments}")
@@ -299,80 +299,23 @@ public class KafkaSourceITCase {
                                         PartitionAndValue value,
                                         ProcessFunction<PartitionAndValue, Long>.Context ctx,
                                         Collector<Long> out) {
-                                    assertTrue(
-                                            ctx.timestamp()
-                                                    >= ctx.timerService().currentWatermark(),
-                                            "Event time should never behind watermark "
-                                                    + "because of per-split watermark multiplexing logic");
+                                    assertThat(ctx.timestamp())
+                                            .as(
+                                                    "Event time should never behind watermark "
+                                                            + "because of per-split watermark multiplexing logic")
+                                            .isGreaterThanOrEqualTo(
+                                                    ctx.timerService().currentWatermark());
                                 }
                             });
             env.execute();
         }
-
-        @Test
-        public void testConsumingEmptyTopic() throws Throwable {
-            String emptyTopic = "emptyTopic-" + UUID.randomUUID();
-            KafkaSourceTestEnv.createTestTopic(emptyTopic, 3, 1);
-            KafkaSource<PartitionAndValue> source =
-                    KafkaSource.<PartitionAndValue>builder()
-                            .setBootstrapServers(KafkaSourceTestEnv.brokerConnectionStrings)
-                            .setTopics(emptyTopic)
-                            .setGroupId("empty-topic-test")
-                            .setDeserializer(new TestingKafkaRecordDeserializationSchema(false))
-                            .setStartingOffsets(OffsetsInitializer.earliest())
-                            .setBounded(OffsetsInitializer.latest())
-                            .build();
-            StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
-            env.setParallelism(1);
-            try (CloseableIterator<PartitionAndValue> iterator =
-                    env.fromSource(
-                                    source,
-                                    WatermarkStrategy.noWatermarks(),
-                                    "testConsumingEmptyTopic")
-                            .executeAndCollect()) {
-                assertFalse(iterator.hasNext());
-            }
-        }
-
-        @Test
-        public void testConsumingTopicWithEmptyPartitions() throws Throwable {
-            String topicWithEmptyPartitions = "topicWithEmptyPartitions-" + UUID.randomUUID();
-            KafkaSourceTestEnv.createTestTopic(
-                    topicWithEmptyPartitions, KafkaSourceTestEnv.NUM_PARTITIONS, 1);
-            List<ProducerRecord<String, Integer>> records =
-                    KafkaSourceTestEnv.getRecordsForTopicWithoutTimestamp(topicWithEmptyPartitions);
-            // Only keep records in partition 5
-            int partitionWithRecords = 5;
-            records.removeIf(record -> record.partition() != partitionWithRecords);
-            KafkaSourceTestEnv.produceToKafka(records);
-            KafkaSourceTestEnv.setupEarliestOffsets(
-                    Collections.singletonList(
-                            new TopicPartition(topicWithEmptyPartitions, partitionWithRecords)));
-
-            KafkaSource<PartitionAndValue> source =
-                    KafkaSource.<PartitionAndValue>builder()
-                            .setBootstrapServers(KafkaSourceTestEnv.brokerConnectionStrings)
-                            .setTopics(topicWithEmptyPartitions)
-                            .setGroupId("topic-with-empty-partition-test")
-                            .setDeserializer(new TestingKafkaRecordDeserializationSchema(false))
-                            .setStartingOffsets(OffsetsInitializer.earliest())
-                            .setBounded(OffsetsInitializer.latest())
-                            .build();
-            StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
-            env.setParallelism(2);
-            executeAndVerify(
-                    env,
-                    env.fromSource(
-                            source,
-                            WatermarkStrategy.noWatermarks(),
-                            "testConsumingTopicWithEmptyPartitions"));
-        }
     }
 
     /** Integration test based on connector testing framework. */
     @Nested
     class IntegrationTests extends SourceTestSuiteBase<String> {
-        private static final String KAFKA_IMAGE_NAME = "confluentinc/cp-kafka:5.5.2";
+        @TestSemantics
+        CheckpointingMode[] semantics = new CheckpointingMode[] {CheckpointingMode.EXACTLY_ONCE};
 
         // Defines test environment on Flink MiniCluster
         @SuppressWarnings("unused")
@@ -380,23 +323,27 @@ public class KafkaSourceITCase {
         MiniClusterTestEnvironment flink = new MiniClusterTestEnvironment();
 
         // Defines external system
-        @ExternalSystem
+        @TestExternalSystem
         DefaultContainerizedExternalSystem<KafkaContainer> kafka =
                 DefaultContainerizedExternalSystem.builder()
-                        .fromContainer(new KafkaContainer(DockerImageName.parse(KAFKA_IMAGE_NAME)))
+                        .fromContainer(
+                                new KafkaContainer(
+                                        DockerImageName.parse(DockerImageVersions.KAFKA)))
                         .build();
 
         // Defines 2 External context Factories, so test cases will be invoked twice using these two
         // kinds of external contexts.
         @SuppressWarnings("unused")
-        @ExternalContextFactory
-        KafkaSingleTopicExternalContext.Factory singleTopic =
-                new KafkaSingleTopicExternalContext.Factory(kafka.getContainer());
+        @TestContext
+        KafkaSourceExternalContextFactory singleTopic =
+                new KafkaSourceExternalContextFactory(
+                        kafka.getContainer(), Collections.emptyList(), PARTITION);
 
         @SuppressWarnings("unused")
-        @ExternalContextFactory
-        KafkaMultipleTopicExternalContext.Factory multipleTopic =
-                new KafkaMultipleTopicExternalContext.Factory(kafka.getContainer());
+        @TestContext
+        KafkaSourceExternalContextFactory multipleTopic =
+                new KafkaSourceExternalContextFactory(
+                        kafka.getContainer(), Collections.emptyList(), TOPIC);
     }
 
     // -----------------
@@ -494,20 +441,15 @@ public class KafkaSourceITCase {
                         resultPerPartition
                                 .computeIfAbsent(partitionAndValue.tp, ignored -> new ArrayList<>())
                                 .add(partitionAndValue.value));
-
-        // Expected elements from partition P should be an integer sequence from P to
-        // NUM_RECORDS_PER_PARTITION.
         resultPerPartition.forEach(
                 (tp, values) -> {
-                    int firstExpectedValue =
-                            Integer.parseInt(tp.substring(tp.lastIndexOf('-') + 1));
+                    int firstExpectedValue = Integer.parseInt(tp.substring(tp.indexOf('-') + 1));
                     for (int i = 0; i < values.size(); i++) {
                         assertEquals(
                                 firstExpectedValue + i,
                                 (int) values.get(i),
                                 String.format(
-                                        "The %d-th value for partition %s should be %d",
-                                        i, tp, firstExpectedValue + i));
+                                        "The %d-th value for partition %s should be %d", i, tp, i));
                     }
                 });
     }
