@@ -19,14 +19,21 @@
 package org.apache.flink.streaming.connectors.kafka.table;
 
 import org.apache.flink.core.execution.JobClient;
+import org.apache.flink.core.testutils.FlinkAssertions;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.functions.sink.SinkFunction;
 import org.apache.flink.streaming.connectors.kafka.partitioner.FlinkKafkaPartitioner;
 import org.apache.flink.table.api.TableResult;
+import org.apache.flink.table.api.config.TableConfigOptions;
 import org.apache.flink.table.data.RowData;
+import org.apache.flink.table.utils.EncodingUtils;
 import org.apache.flink.test.util.SuccessException;
 import org.apache.flink.types.Row;
 
+import org.apache.kafka.clients.consumer.NoOffsetForPartitionException;
+import org.apache.kafka.clients.consumer.OffsetAndMetadata;
+import org.apache.kafka.common.TopicPartition;
+import org.assertj.core.api.Assertions;
 import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
@@ -39,16 +46,21 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Random;
+import java.util.concurrent.ExecutionException;
+import java.util.concurrent.ThreadLocalRandom;
 import java.util.stream.Collectors;
 import java.util.stream.IntStream;
 
+import static org.apache.flink.core.testutils.CommonTestUtils.waitUtil;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaTableTestUtils.collectRows;
 import static org.apache.flink.streaming.connectors.kafka.table.KafkaTableTestUtils.readLines;
 import static org.apache.flink.table.api.config.ExecutionConfigOptions.TABLE_EXEC_SOURCE_IDLE_TIMEOUT;
 import static org.apache.flink.table.utils.TableTestMatchers.deepEqualTo;
+import static org.apache.flink.util.CollectionUtil.entry;
+import static org.apache.flink.util.CollectionUtil.map;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertThat;
 import static org.junit.Assert.fail;
@@ -290,29 +302,20 @@ public class KafkaTableITCase extends KafkaTableTestBase {
                                 + "  %s\n"
                                 + ")",
                         topic, bootstraps, groupId, formatOptions());
-
         tEnv.executeSql(createTable);
 
         String initialValues =
                 "INSERT INTO kafka\n"
                         + "VALUES\n"
-                        + " ('data 1', 1, TIMESTAMP '2020-03-08 13:12:11.123', MAP['k1', X'C0FFEE', 'k2', X'BABE'], TRUE),\n"
+                        + " ('data 1', 1, TIMESTAMP '2020-03-08 13:12:11.123', MAP['k1', X'C0FFEE', 'k2', X'BABE01'], TRUE),\n"
                         + " ('data 2', 2, TIMESTAMP '2020-03-09 13:12:11.123', CAST(NULL AS MAP<STRING, BYTES>), FALSE),\n"
-                        + " ('data 3', 3, TIMESTAMP '2020-03-10 13:12:11.123', MAP['k1', X'10', 'k2', X'20'], TRUE)";
+                        + " ('data 3', 3, TIMESTAMP '2020-03-10 13:12:11.123', MAP['k1', X'102030', 'k2', X'203040'], TRUE)";
         tEnv.executeSql(initialValues).await();
 
         // ---------- Consume stream from Kafka -------------------
 
         final List<Row> result = collectRows(tEnv.sqlQuery("SELECT * FROM kafka"), 3);
 
-        final Map<String, byte[]> headers1 = new HashMap<>();
-        headers1.put("k1", new byte[] {(byte) 0xC0, (byte) 0xFF, (byte) 0xEE});
-        headers1.put("k2", new byte[] {(byte) 0xBA, (byte) 0xBE});
-
-        final Map<String, byte[]> headers3 = new HashMap<>();
-        headers3.put("k1", new byte[] {(byte) 0x10});
-        headers3.put("k2", new byte[] {(byte) 0x20});
-
         final List<Row> expected =
                 Arrays.asList(
                         Row.of(
@@ -321,7 +324,9 @@ public class KafkaTableITCase extends KafkaTableTestBase {
                                 "CreateTime",
                                 LocalDateTime.parse("2020-03-08T13:12:11.123"),
                                 0,
-                                headers1,
+                                map(
+                                        entry("k1", EncodingUtils.decodeHex("C0FFEE")),
+                                        entry("k2", EncodingUtils.decodeHex("BABE01"))),
                                 0,
                                 topic,
                                 true),
@@ -341,7 +346,9 @@ public class KafkaTableITCase extends KafkaTableTestBase {
                                 "CreateTime",
                                 LocalDateTime.parse("2020-03-10T13:12:11.123"),
                                 0,
-                                headers3,
+                                map(
+                                        entry("k1", EncodingUtils.decodeHex("102030")),
+                                        entry("k2", EncodingUtils.decodeHex("203040"))),
                                 0,
                                 topic,
                                 true));
@@ -518,7 +525,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
         // 'value.source.timestamp'` DDL
         // will use the session time zone when convert the changelog time from milliseconds to
         // timestamp
-        tEnv.getConfig().getConfiguration().setString("table.local-time-zone", "UTC");
+        tEnv.getConfig().set(TableConfigOptions.LOCAL_TIME_ZONE, "UTC");
 
         // we always use a different topic name for each parameterized topic,
         // in order to make sure the topic can be created.
@@ -754,9 +761,7 @@ public class KafkaTableITCase extends KafkaTableTestBase {
         // ---------- Produce an event time stream into Kafka -------------------
         String groupId = getStandardProps().getProperty("group.id");
         String bootstraps = getBootstrapServers();
-        tEnv.getConfig()
-                .getConfiguration()
-                .set(TABLE_EXEC_SOURCE_IDLE_TIMEOUT, Duration.ofMillis(100));
+        tEnv.getConfig().set(TABLE_EXEC_SOURCE_IDLE_TIMEOUT, Duration.ofMillis(100));
 
         final String createTable =
                 String.format(
@@ -821,6 +826,164 @@ public class KafkaTableITCase extends KafkaTableTestBase {
         deleteTestTopic(topic);
     }
 
+    @Test
+    public void testStartFromGroupOffsetsLatest() throws Exception {
+        testStartFromGroupOffsets("latest");
+    }
+
+    @Test
+    public void testStartFromGroupOffsetsEarliest() throws Exception {
+        testStartFromGroupOffsets("earliest");
+    }
+
+    @Test
+    public void testStartFromGroupOffsetsNone() {
+        Assertions.assertThatThrownBy(() -> testStartFromGroupOffsetsWithNoneResetStrategy())
+                .satisfies(FlinkAssertions.anyCauseMatches(NoOffsetForPartitionException.class));
+    }
+
+    private List<String> appendNewData(
+            String topic, String tableName, String groupId, int targetNum) throws Exception {
+        waitUtil(
+                () -> {
+                    Map<TopicPartition, OffsetAndMetadata> offsets = getConsumerOffset(groupId);
+                    long sum =
+                            offsets.entrySet().stream()
+                                    .filter(e -> e.getKey().topic().contains(topic))
+                                    .mapToLong(e -> e.getValue().offset())
+                                    .sum();
+                    return sum == targetNum;
+                },
+                Duration.ofMillis(20000),
+                "Can not reach the expected offset before adding new data.");
+        String appendValues =
+                "INSERT INTO "
+                        + tableName
+                        + "\n"
+                        + "VALUES\n"
+                        + " (2, 6),\n"
+                        + " (2, 7),\n"
+                        + " (2, 8)\n";
+        tEnv.executeSql(appendValues).await();
+        return Arrays.asList("+I[2, 6]", "+I[2, 7]", "+I[2, 8]");
+    }
+
+    private TableResult startFromGroupOffset(
+            String tableName, String topic, String groupId, String resetStrategy, String sinkName)
+            throws ExecutionException, InterruptedException {
+        // we always use a different topic name for each parameterized topic,
+        // in order to make sure the topic can be created.
+        createTestTopic(topic, 4, 1);
+
+        // ---------- Produce an event time stream into Kafka -------------------
+        String bootstraps = getBootstrapServers();
+        tEnv.getConfig().set(TABLE_EXEC_SOURCE_IDLE_TIMEOUT, Duration.ofMillis(100));
+
+        final String createTableSql =
+                "CREATE TABLE %s (\n"
+                        + "  `partition_id` INT,\n"
+                        + "  `value` INT\n"
+                        + ") WITH (\n"
+                        + "  'connector' = 'kafka',\n"
+                        + "  'topic' = '%s',\n"
+                        + "  'properties.bootstrap.servers' = '%s',\n"
+                        + "  'properties.group.id' = '%s',\n"
+                        + "  'scan.startup.mode' = 'group-offsets',\n"
+                        + "  'properties.auto.offset.reset' = '%s',\n"
+                        + "  'properties.enable.auto.commit' = 'true',\n"
+                        + "  'properties.auto.commit.interval.ms' = '1000',\n"
+                        + "  'format' = '%s'\n"
+                        + ")";
+        tEnv.executeSql(
+                String.format(
+                        createTableSql,
+                        tableName,
+                        topic,
+                        bootstraps,
+                        groupId,
+                        resetStrategy,
+                        format));
+
+        String initialValues =
+                "INSERT INTO "
+                        + tableName
+                        + "\n"
+                        + "VALUES\n"
+                        + " (0, 0),\n"
+                        + " (0, 1),\n"
+                        + " (0, 2),\n"
+                        + " (1, 3),\n"
+                        + " (1, 4),\n"
+                        + " (1, 5)\n";
+        tEnv.executeSql(initialValues).await();
+
+        // ---------- Consume stream from Kafka -------------------
+
+        env.setParallelism(1);
+        String createSink =
+                "CREATE TABLE "
+                        + sinkName
+                        + "(\n"
+                        + "  `partition_id` INT,\n"
+                        + "  `value` INT\n"
+                        + ") WITH (\n"
+                        + "  'connector' = 'values'\n"
+                        + ")";
+        tEnv.executeSql(createSink);
+
+        return tEnv.executeSql("INSERT INTO " + sinkName + " SELECT * FROM " + tableName);
+    }
+
+    private void testStartFromGroupOffsets(String resetStrategy) throws Exception {
+        // we always use a different topic name for each parameterized topic,
+        // in order to make sure the topic can be created.
+        final String tableName = "Table" + format + resetStrategy;
+        final String topic =
+                "groupOffset_" + format + resetStrategy + ThreadLocalRandom.current().nextLong();
+        String groupId = format + resetStrategy;
+        String sinkName = "mySink" + format + resetStrategy;
+        List<String> expected =
+                Arrays.asList(
+                        "+I[0, 0]", "+I[0, 1]", "+I[0, 2]", "+I[1, 3]", "+I[1, 4]", "+I[1, 5]");
+
+        TableResult tableResult = null;
+        try {
+            tableResult = startFromGroupOffset(tableName, topic, groupId, resetStrategy, sinkName);
+            if ("latest".equals(resetStrategy)) {
+                expected = appendNewData(topic, tableName, groupId, expected.size());
+            }
+            KafkaTableTestUtils.waitingExpectedResults(sinkName, expected, Duration.ofSeconds(15));
+        } finally {
+            // ------------- cleanup -------------------
+            if (tableResult != null) {
+                tableResult.getJobClient().ifPresent(JobClient::cancel);
+            }
+            deleteTestTopic(topic);
+        }
+    }
+
+    private void testStartFromGroupOffsetsWithNoneResetStrategy()
+            throws ExecutionException, InterruptedException {
+        // we always use a different topic name for each parameterized topic,
+        // in order to make sure the topic can be created.
+        final String resetStrategy = "none";
+        final String tableName = resetStrategy + "Table";
+        final String topic = "groupOffset_" + format;
+        String groupId = resetStrategy + (new Random()).nextInt();
+
+        TableResult tableResult = null;
+        try {
+            tableResult = startFromGroupOffset(tableName, topic, groupId, resetStrategy, "MySink");
+            tableResult.await();
+        } finally {
+            // ------------- cleanup -------------------
+            if (tableResult != null) {
+                tableResult.getJobClient().ifPresent(JobClient::cancel);
+            }
+            deleteTestTopic(topic);
+        }
+    }
+
     // --------------------------------------------------------------------------------------------
     // Utilities
     // --------------------------------------------------------------------------------------------
