@@ -26,7 +26,6 @@ import org.apache.flink.configuration.ConfigOptions;
 import org.apache.flink.configuration.description.Description;
 import org.apache.flink.connector.pulsar.common.config.PulsarOptions;
 import org.apache.flink.connector.pulsar.source.config.CursorVerification;
-import org.apache.flink.connector.pulsar.source.enumerator.cursor.StartCursor;
 
 import org.apache.pulsar.client.api.ConsumerCryptoFailureAction;
 import org.apache.pulsar.client.api.SubscriptionInitialPosition;
@@ -48,7 +47,7 @@ import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.SOURC
  * PulsarSourceBuilder#setConfig(ConfigOption, Object)}. The {@link PulsarOptions} is also required
  * for pulsar source.
  *
- * @see PulsarOptions
+ * @see PulsarOptions for shared configure options.
  */
 @PublicEvolving
 @ConfigGroups(
@@ -123,7 +122,7 @@ public final class PulsarSourceOptions {
                                             " We would automatically commit the cursor using the given period (in ms).")
                                     .build());
 
-    public static final ConfigOption<Long> PULSAR_TRANSACTION_TIMEOUT_MILLIS =
+    public static final ConfigOption<Long> PULSAR_READ_TRANSACTION_TIMEOUT =
             ConfigOptions.key(SOURCE_CONFIG_PREFIX + "transactionTimeoutMillis")
                     .longType()
                     .defaultValue(Duration.ofHours(3).toMillis())
@@ -140,6 +139,14 @@ public final class PulsarSourceOptions {
                                             "The value (in ms) should be greater than the checkpoint interval.")
                                     .build());
 
+    /**
+     * @deprecated Use {@link #PULSAR_READ_TRANSACTION_TIMEOUT} instead. This would be removed in
+     *     the next release.
+     */
+    @Deprecated
+    public static final ConfigOption<Long> PULSAR_TRANSACTION_TIMEOUT_MILLIS =
+            PULSAR_READ_TRANSACTION_TIMEOUT;
+
     public static final ConfigOption<Long> PULSAR_MAX_FETCH_TIME =
             ConfigOptions.key(SOURCE_CONFIG_PREFIX + "maxFetchTime")
                     .longType()
@@ -496,12 +503,6 @@ public final class PulsarSourceOptions {
                                             code("PulsarClientException"))
                                     .build());
 
-    /**
-     * @deprecated This option would be reset by {@link StartCursor}, no need to use it anymore.
-     *     Pulsar didn't support this config option before 1.10.1, so we have to remove this config
-     *     option.
-     */
-    @Deprecated
     public static final ConfigOption<SubscriptionInitialPosition>
             PULSAR_SUBSCRIPTION_INITIAL_POSITION =
                     ConfigOptions.key(CONSUMER_CONFIG_PREFIX + "subscriptionInitialPosition")
