@@ -22,6 +22,7 @@ import org.apache.flink.annotation.PublicEvolving;
 import org.apache.flink.api.connector.source.SourceReaderContext;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.connector.base.source.reader.RecordsWithSplitIds;
+import org.apache.flink.connector.pulsar.common.config.PulsarConfiguration;
 import org.apache.flink.connector.pulsar.source.enumerator.cursor.CursorPosition;
 import org.apache.flink.connector.pulsar.source.enumerator.cursor.StartCursor;
 
@@ -29,26 +30,24 @@ import org.apache.pulsar.client.api.ConsumerBuilder;
 import org.apache.pulsar.client.api.SubscriptionMode;
 import org.apache.pulsar.client.api.SubscriptionType;
 
-import java.io.Serializable;
 import java.time.Duration;
 import java.util.Objects;
 
 import static org.apache.flink.connector.base.source.reader.SourceReaderOptions.ELEMENT_QUEUE_CAPACITY;
-import static org.apache.flink.connector.pulsar.common.config.PulsarConfigUtils.getOptionValue;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_AUTO_COMMIT_CURSOR_INTERVAL;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_ENABLE_AUTO_ACKNOWLEDGE_MESSAGE;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_MAX_FETCH_RECORDS;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_MAX_FETCH_TIME;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_PARTITION_DISCOVERY_INTERVAL_MS;
+import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_READ_TRANSACTION_TIMEOUT;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_SUBSCRIPTION_MODE;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_SUBSCRIPTION_NAME;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_SUBSCRIPTION_TYPE;
-import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_TRANSACTION_TIMEOUT_MILLIS;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_VERIFY_INITIAL_OFFSETS;
 
 /** The configure class for pulsar source. */
 @PublicEvolving
-public class SourceConfiguration implements Serializable {
+public class SourceConfiguration extends PulsarConfiguration {
     private static final long serialVersionUID = 8488507275800787580L;
 
     private final int messageQueueCapacity;
@@ -64,20 +63,19 @@ public class SourceConfiguration implements Serializable {
     private final SubscriptionMode subscriptionMode;
 
     public SourceConfiguration(Configuration configuration) {
-        this.messageQueueCapacity = configuration.getInteger(ELEMENT_QUEUE_CAPACITY);
-        this.partitionDiscoveryIntervalMs =
-                configuration.get(PULSAR_PARTITION_DISCOVERY_INTERVAL_MS);
-        this.enableAutoAcknowledgeMessage =
-                configuration.get(PULSAR_ENABLE_AUTO_ACKNOWLEDGE_MESSAGE);
-        this.autoCommitCursorInterval = configuration.get(PULSAR_AUTO_COMMIT_CURSOR_INTERVAL);
-        this.transactionTimeoutMillis = configuration.get(PULSAR_TRANSACTION_TIMEOUT_MILLIS);
-        this.maxFetchTime =
-                getOptionValue(configuration, PULSAR_MAX_FETCH_TIME, Duration::ofMillis);
-        this.maxFetchRecords = configuration.get(PULSAR_MAX_FETCH_RECORDS);
-        this.verifyInitialOffsets = configuration.get(PULSAR_VERIFY_INITIAL_OFFSETS);
-        this.subscriptionName = configuration.get(PULSAR_SUBSCRIPTION_NAME);
-        this.subscriptionType = configuration.get(PULSAR_SUBSCRIPTION_TYPE);
-        this.subscriptionMode = configuration.get(PULSAR_SUBSCRIPTION_MODE);
+        super(configuration);
+
+        this.messageQueueCapacity = getInteger(ELEMENT_QUEUE_CAPACITY);
+        this.partitionDiscoveryIntervalMs = get(PULSAR_PARTITION_DISCOVERY_INTERVAL_MS);
+        this.enableAutoAcknowledgeMessage = get(PULSAR_ENABLE_AUTO_ACKNOWLEDGE_MESSAGE);
+        this.autoCommitCursorInterval = get(PULSAR_AUTO_COMMIT_CURSOR_INTERVAL);
+        this.transactionTimeoutMillis = get(PULSAR_READ_TRANSACTION_TIMEOUT);
+        this.maxFetchTime = get(PULSAR_MAX_FETCH_TIME, Duration::ofMillis);
+        this.maxFetchRecords = get(PULSAR_MAX_FETCH_RECORDS);
+        this.verifyInitialOffsets = get(PULSAR_VERIFY_INITIAL_OFFSETS);
+        this.subscriptionName = get(PULSAR_SUBSCRIPTION_NAME);
+        this.subscriptionType = get(PULSAR_SUBSCRIPTION_TYPE);
+        this.subscriptionMode = get(PULSAR_SUBSCRIPTION_MODE);
     }
 
     /** The capacity of the element queue in the source reader. */
@@ -85,10 +83,6 @@ public class SourceConfiguration implements Serializable {
         return messageQueueCapacity;
     }
 
-    /**
-     * We would override the interval into a negative number when we set the connector with bounded
-     * stop cursor.
-     */
     public boolean isEnablePartitionDiscovery() {
         return getPartitionDiscoveryIntervalMs() > 0;
     }
