@@ -30,28 +30,24 @@ import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtil
 /**
  * A stop cursor that initialize the position to the latest message id. The offsets initialization
  * are taken care of by the {@code PulsarPartitionSplitReaderBase} instead of by the {@code
- * PulsarSourceEnumerator}. We would include the latest message available in Pulsar by default.
+ * PulsarSourceEnumerator}.
  */
 public class LatestMessageStopCursor implements StopCursor {
+    private static final long serialVersionUID = 1702059838323965723L;
 
     private MessageId messageId;
-    private final boolean inclusive;
-
-    public LatestMessageStopCursor(boolean inclusive) {
-        this.inclusive = inclusive;
-    }
-
-    @Override
-    public StopCondition shouldStop(Message<?> message) {
-        MessageId current = message.getMessageId();
-        return StopCondition.compare(messageId, current, inclusive);
-    }
 
     @Override
     public void open(PulsarAdmin admin, TopicPartition partition) {
         if (messageId == null) {
             String topic = partition.getFullTopicName();
-            this.messageId = sneakyAdmin(() -> admin.topics().getLastMessageId(topic));
+            messageId = sneakyAdmin(() -> admin.topics().getLastMessageId(topic));
         }
     }
+
+    @Override
+    public boolean shouldStop(Message<?> message) {
+        MessageId id = message.getMessageId();
+        return id.compareTo(messageId) >= 0;
+    }
 }
