@@ -24,16 +24,18 @@ import org.apache.flink.configuration.Configuration;
 import org.apache.flink.connector.base.source.reader.RecordsWithSplitIds;
 import org.apache.flink.connector.base.source.reader.splitreader.SplitsAddition;
 import org.apache.flink.connector.pulsar.source.config.SourceConfiguration;
+import org.apache.flink.connector.pulsar.source.enumerator.cursor.StartCursor;
 import org.apache.flink.connector.pulsar.source.enumerator.cursor.StopCursor;
 import org.apache.flink.connector.pulsar.source.enumerator.topic.TopicPartition;
 import org.apache.flink.connector.pulsar.source.reader.message.PulsarMessage;
 import org.apache.flink.connector.pulsar.source.split.PulsarPartitionSplit;
 import org.apache.flink.connector.pulsar.testutils.PulsarTestSuiteBase;
 import org.apache.flink.connector.pulsar.testutils.extension.TestOrderlinessExtension;
-import org.apache.flink.connectors.test.common.junit.extensions.TestLoggerExtension;
+import org.apache.flink.util.TestLoggerExtension;
 
-import org.apache.pulsar.client.admin.PulsarAdmin;
+import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.MessageId;
+import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.SubscriptionType;
 import org.apache.pulsar.client.impl.MessageIdImpl;
 import org.junit.jupiter.api.TestTemplate;
@@ -60,6 +62,7 @@ import static java.time.Duration.ofSeconds;
 import static java.util.Collections.singletonList;
 import static org.apache.commons.lang3.RandomStringUtils.randomAlphabetic;
 import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtils.sneakyAdmin;
+import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtils.sneakyThrow;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_ENABLE_AUTO_ACKNOWLEDGE_MESSAGE;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_MAX_FETCH_RECORDS;
 import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSAR_MAX_FETCH_TIME;
@@ -67,6 +70,7 @@ import static org.apache.flink.connector.pulsar.source.PulsarSourceOptions.PULSA
 import static org.apache.flink.connector.pulsar.source.enumerator.topic.TopicNameUtils.topicNameWithPartition;
 import static org.apache.flink.connector.pulsar.source.enumerator.topic.TopicRange.createFullRange;
 import static org.apache.flink.connector.pulsar.source.reader.deserializer.PulsarDeserializationSchema.flinkSchema;
+import static org.apache.flink.connector.pulsar.testutils.PulsarTestCommonUtils.isAssignableFromParameterContext;
 import static org.apache.flink.connector.pulsar.testutils.extension.TestOrderlinessExtension.PULSAR_SOURCE_READER_SUBSCRIPTION_TYPE_STORE_KEY;
 import static org.apache.flink.connector.pulsar.testutils.extension.TestOrderlinessExtension.PULSAR_TEST_RESOURCE_NAMESPACE;
 import static org.apache.flink.connector.pulsar.testutils.runtime.PulsarRuntimeOperator.DEFAULT_PARTITIONS;
@@ -88,17 +92,14 @@ public abstract class PulsarPartitionSplitReaderTestBase extends PulsarTestSuite
     PulsarSplitReaderInvocationContextProvider provider =
             new PulsarSplitReaderInvocationContextProvider();
 
-    protected Configuration readerConfig() {
+    /** Default source config: max message 1, fetch timeout 1s. */
+    private SourceConfiguration sourceConfig() {
         Configuration config = operator().config();
         config.set(PULSAR_MAX_FETCH_RECORDS, 1);
         config.set(PULSAR_MAX_FETCH_TIME, 1000L);
         config.set(PULSAR_SUBSCRIPTION_NAME, randomAlphabetic(10));
         config.set(PULSAR_ENABLE_AUTO_ACKNOWLEDGE_MESSAGE, true);
 
-        return config;
-    }
-
-    protected SourceConfiguration sourceConfig(Configuration config) {
         return new SourceConfiguration(config);
     }
 
@@ -134,31 +135,15 @@ public abstract class PulsarPartitionSplitReaderTestBase extends PulsarTestSuite
                 new PulsarPartitionSplit(partition, StopCursor.never(), null, null);
         SplitsAddition<PulsarPartitionSplit> addition = new SplitsAddition<>(singletonList(split));
 
-        // Create the subscription and set the start position for this reader.
-        // Remember not to use Consumer.seek(startPosition)
-        SourceConfiguration sourceConfiguration = reader.sourceConfiguration;
-        PulsarAdmin pulsarAdmin = reader.pulsarAdmin;
-        String subscriptionName = sourceConfiguration.getSubscriptionName();
-        List<String> subscriptions =
-                sneakyAdmin(() -> pulsarAdmin.topics().getSubscriptions(topicName));
-        if (!subscriptions.contains(subscriptionName)) {
-            // If this subscription is not available. Just create it.
-            sneakyAdmin(
-                    () ->
-                            pulsarAdmin
-                                    .topics()
-                                    .createSubscription(
-                                            topicName, subscriptionName, startPosition));
-        } else {
-            // Reset the subscription if this is existed.
-            sneakyAdmin(
-                    () ->
-                            pulsarAdmin
-                                    .topics()
-                                    .resetCursor(topicName, subscriptionName, startPosition));
+        // create consumer and seek before split changes
+        try (Consumer<byte[]> consumer = reader.createPulsarConsumer(partition)) {
+            // inclusive messageId
+            StartCursor startCursor = StartCursor.fromMessageId(startPosition);
+            startCursor.seekPosition(partition.getTopic(), partition.getPartitionId(), consumer);
+        } catch (PulsarClientException e) {
+            sneakyThrow(e);
         }
 
-        // Accept the split and start consuming.
         reader.handleSplitsChanges(addition);
     }
 
@@ -200,7 +185,7 @@ public abstract class PulsarPartitionSplitReaderTestBase extends PulsarTestSuite
         if (verify) {
             assertThat(messages).as("We should fetch the expected size").hasSize(expectedCount);
             if (boundedness == Boundedness.CONTINUOUS_UNBOUNDED) {
-                assertThat(finishedSplits).as("Split should not be marked as finished").isEmpty();
+                assertThat(finishedSplits).as("Split should not be marked as finished").hasSize(0);
             } else {
                 assertThat(finishedSplits).as("Split should be marked as finished").hasSize(1);
             }
@@ -215,7 +200,7 @@ public abstract class PulsarPartitionSplitReaderTestBase extends PulsarTestSuite
         String topicName = randomAlphabetic(10);
 
         // Add a split
-        handleSplit(splitReader, topicName, 0, MessageId.latest);
+        seekStartPositionAndHandleSplit(splitReader, topicName, 0);
 
         // Poll once with a null message
         PulsarMessage<String> message1 = fetchedMessage(splitReader);
@@ -239,7 +224,7 @@ public abstract class PulsarPartitionSplitReaderTestBase extends PulsarTestSuite
     void consumeMessageCreatedAfterHandleSplitChangesAndFetch(
             PulsarPartitionSplitReaderBase<String> splitReader) {
         String topicName = randomAlphabetic(10);
-        handleSplit(splitReader, topicName, 0, MessageId.latest);
+        seekStartPositionAndHandleSplit(splitReader, topicName, 0);
         operator().sendMessage(topicNameWithPartition(topicName, 0), STRING, randomAlphabetic(10));
         fetchedMessages(splitReader, 1, true);
     }
@@ -345,20 +330,17 @@ public abstract class PulsarPartitionSplitReaderTestBase extends PulsarTestSuite
 
     /** Create a split reader with max message 1, fetch timeout 1s. */
     private PulsarPartitionSplitReaderBase<String> splitReader(SubscriptionType subscriptionType) {
-        Configuration configuration = readerConfig();
         if (subscriptionType == SubscriptionType.Failover) {
             return new PulsarOrderedPartitionSplitReader<>(
                     operator().client(),
                     operator().admin(),
-                    configuration,
-                    sourceConfig(configuration),
+                    sourceConfig(),
                     flinkSchema(new SimpleStringSchema()));
         } else {
             return new PulsarUnorderedPartitionSplitReader<>(
                     operator().client(),
                     operator().admin(),
-                    configuration,
-                    sourceConfig(configuration),
+                    sourceConfig(),
                     flinkSchema(new SimpleStringSchema()),
                     null);
         }
@@ -408,10 +390,8 @@ public abstract class PulsarPartitionSplitReaderTestBase extends PulsarTestSuite
                                 ParameterContext parameterContext,
                                 ExtensionContext extensionContext)
                                 throws ParameterResolutionException {
-                            return parameterContext
-                                    .getParameter()
-                                    .getType()
-                                    .equals(PulsarPartitionSplitReaderBase.class);
+                            return isAssignableFromParameterContext(
+                                    PulsarPartitionSplitReaderBase.class, parameterContext);
                         }
 
                         @Override
