@@ -19,9 +19,11 @@
 package org.apache.flink.connector.pulsar.testutils.runtime;
 
 import org.apache.flink.configuration.Configuration;
+import org.apache.flink.connector.base.DeliveryGuarantee;
+import org.apache.flink.connector.pulsar.common.config.PulsarConfiguration;
 import org.apache.flink.connector.pulsar.source.enumerator.topic.TopicPartition;
 import org.apache.flink.connector.pulsar.source.enumerator.topic.TopicRange;
-import org.apache.flink.connectors.test.common.external.ExternalContext;
+import org.apache.flink.connector.testframe.external.ExternalContext;
 
 import org.apache.flink.shaded.guava30.com.google.common.base.Strings;
 
@@ -29,14 +31,13 @@ import org.apache.pulsar.client.admin.PulsarAdmin;
 import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.admin.PulsarAdminException.NotFoundException;
 import org.apache.pulsar.client.api.Consumer;
-import org.apache.pulsar.client.api.ConsumerBuilder;
 import org.apache.pulsar.client.api.Message;
 import org.apache.pulsar.client.api.MessageId;
 import org.apache.pulsar.client.api.Producer;
-import org.apache.pulsar.client.api.ProducerBuilder;
 import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.Schema;
+import org.apache.pulsar.client.api.SubscriptionInitialPosition;
 import org.apache.pulsar.client.api.TypedMessageBuilder;
 import org.apache.pulsar.client.api.transaction.TransactionCoordinatorClient;
 import org.apache.pulsar.client.api.transaction.TxnID;
@@ -48,8 +49,10 @@ import java.io.Closeable;
 import java.io.IOException;
 import java.time.Duration;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.List;
+import java.util.Map;
 import java.util.Random;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutionException;
@@ -59,20 +62,25 @@ import java.util.stream.Stream;
 import static java.util.Collections.emptyList;
 import static java.util.Collections.singletonList;
 import static java.util.concurrent.TimeUnit.MILLISECONDS;
+import static java.util.function.Function.identity;
 import static java.util.stream.Collectors.toList;
+import static java.util.stream.Collectors.toMap;
 import static org.apache.commons.lang3.RandomStringUtils.randomAlphanumeric;
+import static org.apache.flink.connector.base.DeliveryGuarantee.EXACTLY_ONCE;
 import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_ADMIN_URL;
+import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_ENABLE_TRANSACTION;
 import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_SERVICE_URL;
 import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtils.sneakyAdmin;
 import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtils.sneakyClient;
 import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtils.sneakyThrow;
+import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_SEND_TIMEOUT_MS;
+import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_WRITE_DELIVERY_GUARANTEE;
+import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_WRITE_TRANSACTION_TIMEOUT;
 import static org.apache.flink.connector.pulsar.source.enumerator.topic.TopicNameUtils.topicName;
 import static org.apache.flink.connector.pulsar.source.enumerator.topic.TopicNameUtils.topicNameWithPartition;
 import static org.apache.flink.util.Preconditions.checkArgument;
-import static org.apache.pulsar.client.api.SubscriptionInitialPosition.Earliest;
 import static org.apache.pulsar.client.api.SubscriptionMode.Durable;
 import static org.apache.pulsar.client.api.SubscriptionType.Exclusive;
-import static org.apache.pulsar.common.partition.PartitionedTopicMetadata.NON_PARTITIONED;
 
 /**
  * A pulsar cluster operator used for operating pulsar instance. It's serializable for using in
@@ -166,11 +174,11 @@ public class PulsarRuntimeOperator implements Closeable {
      *
      * @param topic The name of the topic.
      * @param numberOfPartitions The number of partitions. We would create a non-partitioned topic
-     *     if this number is zero.
+     *     if this number if zero.
      */
     public void createTopic(String topic, int numberOfPartitions) {
         checkArgument(numberOfPartitions >= 0);
-        if (numberOfPartitions == 0) {
+        if (numberOfPartitions <= 0) {
             createNonPartitionedTopic(topic);
         } else {
             createPartitionedTopic(topic, numberOfPartitions);
@@ -188,7 +196,7 @@ public class PulsarRuntimeOperator implements Closeable {
                 sneakyAdmin(() -> admin().topics().getPartitionedTopicMetadata(topic));
         checkArgument(
                 metadata.partitions < newPartitionsNum,
-                "The new partition size which should greater than previous size.");
+                "The new partition size which should exceed previous size.");
 
         sneakyAdmin(() -> admin().topics().updatePartitionedTopic(topic, newPartitionsNum));
     }
@@ -212,11 +220,9 @@ public class PulsarRuntimeOperator implements Closeable {
             return;
         }
 
-        // Close all the available consumers and producers.
         removeConsumers(topic);
         removeProducers(topic);
-
-        if (metadata.partitions == NON_PARTITIONED) {
+        if (metadata.partitions <= 0) {
             sneakyAdmin(() -> admin().topics().delete(topicName));
         } else {
             sneakyAdmin(() -> admin().topics().deletePartitionedTopic(topicName));
@@ -239,6 +245,22 @@ public class PulsarRuntimeOperator implements Closeable {
         }
     }
 
+    /**
+     * Query a list of topics. Convert the topic metadata into a list of topic partitions. Return a
+     * mapping for topic and its partitions.
+     */
+    public Map<String, List<TopicPartition>> topicsInfo(String... topics) {
+        return topicsInfo(Arrays.asList(topics));
+    }
+
+    /**
+     * Query a list of topics. Convert the topic metadata into a list of topic partitions. Return a
+     * mapping for topic and its partitions.
+     */
+    public Map<String, List<TopicPartition>> topicsInfo(Collection<String> topics) {
+        return topics.stream().collect(toMap(identity(), this::topicInfo));
+    }
+
     /**
      * Send a single message to Pulsar, return the message id after the ack from Pulsar.
      *
@@ -428,7 +450,7 @@ public class PulsarRuntimeOperator implements Closeable {
         return admin;
     }
 
-    /** The configuration for constructing {@link Configuration}. */
+    /** The configuration for constructing {@link PulsarConfiguration}. */
     public Configuration config() {
         Configuration configuration = new Configuration();
         configuration.set(PULSAR_SERVICE_URL, serviceUrl());
@@ -436,6 +458,19 @@ public class PulsarRuntimeOperator implements Closeable {
         return configuration;
     }
 
+    /** Create the sink configuration with common settings. */
+    public Configuration sinkConfig(DeliveryGuarantee deliveryGuarantee) {
+        Configuration configuration = config();
+        configuration.set(PULSAR_WRITE_DELIVERY_GUARANTEE, deliveryGuarantee);
+        if (deliveryGuarantee == EXACTLY_ONCE) {
+            configuration.set(PULSAR_WRITE_TRANSACTION_TIMEOUT, Duration.ofMinutes(5).toMillis());
+            configuration.set(PULSAR_ENABLE_TRANSACTION, true);
+            configuration.set(PULSAR_SEND_TIMEOUT_MS, 0L);
+        }
+
+        return configuration;
+    }
+
     /** This method is used for test framework. You can't close this operator manually. */
     @Override
     public void close() throws IOException {
@@ -483,13 +518,12 @@ public class PulsarRuntimeOperator implements Closeable {
                 topicProducers.computeIfAbsent(
                         index,
                         i -> {
-                            ProducerBuilder<T> builder =
-                                    client().newProducer(schema)
-                                            .topic(topic)
-                                            .enableBatching(false)
-                                            .enableMultiSchema(true);
-
-                            return sneakyClient(builder::create);
+                            try {
+                                return client().newProducer(schema).topic(topic).create();
+                            } catch (PulsarClientException e) {
+                                sneakyThrow(e);
+                                return null;
+                            }
                         });
     }
 
@@ -506,15 +540,19 @@ public class PulsarRuntimeOperator implements Closeable {
                 topicConsumers.computeIfAbsent(
                         index,
                         i -> {
-                            ConsumerBuilder<T> builder =
-                                    client().newConsumer(schema)
-                                            .topic(topic)
-                                            .subscriptionName(SUBSCRIPTION_NAME)
-                                            .subscriptionMode(Durable)
-                                            .subscriptionType(Exclusive)
-                                            .subscriptionInitialPosition(Earliest);
-
-                            return sneakyClient(builder::subscribe);
+                            try {
+                                return client().newConsumer(schema)
+                                        .topic(topic)
+                                        .subscriptionName(SUBSCRIPTION_NAME)
+                                        .subscriptionMode(Durable)
+                                        .subscriptionType(Exclusive)
+                                        .subscriptionInitialPosition(
+                                                SubscriptionInitialPosition.Earliest)
+                                        .subscribe();
+                            } catch (PulsarClientException e) {
+                                sneakyThrow(e);
+                                return null;
+                            }
                         });
     }
 
@@ -523,7 +561,11 @@ public class PulsarRuntimeOperator implements Closeable {
         ConcurrentHashMap<Integer, Producer<?>> integerProducers = producers.remove(topicName);
         if (integerProducers != null) {
             for (Producer<?> producer : integerProducers.values()) {
-                sneakyClient(producer::close);
+                try {
+                    producer.close();
+                } catch (PulsarClientException e) {
+                    sneakyThrow(e);
+                }
             }
         }
     }
@@ -533,7 +575,11 @@ public class PulsarRuntimeOperator implements Closeable {
         ConcurrentHashMap<Integer, Consumer<?>> integerConsumers = consumers.remove(topicName);
         if (integerConsumers != null) {
             for (Consumer<?> consumer : integerConsumers.values()) {
-                sneakyClient(consumer::close);
+                try {
+                    consumer.close();
+                } catch (PulsarClientException e) {
+                    sneakyThrow(e);
+                }
             }
         }
     }
