@@ -22,6 +22,7 @@ import org.apache.flink.annotation.PublicEvolving;
 import org.apache.flink.annotation.docs.Documentation;
 import org.apache.flink.configuration.description.Description;
 import org.apache.flink.configuration.description.InlineElement;
+import org.apache.flink.configuration.description.TextElement;
 
 import static org.apache.flink.configuration.ClusterOptions.UserSystemExitMode.THROW;
 import static org.apache.flink.configuration.ConfigOptions.key;
@@ -36,6 +37,7 @@ public class ClusterOptions {
     @Documentation.Section(Documentation.Sections.EXPERT_FAULT_TOLERANCE)
     public static final ConfigOption<Long> INITIAL_REGISTRATION_TIMEOUT =
             ConfigOptions.key("cluster.registration.initial-timeout")
+                    .longType()
                     .defaultValue(100L)
                     .withDescription(
                             "Initial registration timeout between cluster components in milliseconds.");
@@ -43,6 +45,7 @@ public class ClusterOptions {
     @Documentation.Section(Documentation.Sections.EXPERT_FAULT_TOLERANCE)
     public static final ConfigOption<Long> MAX_REGISTRATION_TIMEOUT =
             ConfigOptions.key("cluster.registration.max-timeout")
+                    .longType()
                     .defaultValue(30000L)
                     .withDescription(
                             "Maximum registration timeout between cluster components in milliseconds.");
@@ -50,6 +53,7 @@ public class ClusterOptions {
     @Documentation.Section(Documentation.Sections.EXPERT_FAULT_TOLERANCE)
     public static final ConfigOption<Long> ERROR_REGISTRATION_DELAY =
             ConfigOptions.key("cluster.registration.error-delay")
+                    .longType()
                     .defaultValue(10000L)
                     .withDescription(
                             "The pause made after an registration attempt caused an exception (other than timeout) in milliseconds.");
@@ -57,6 +61,7 @@ public class ClusterOptions {
     @Documentation.Section(Documentation.Sections.EXPERT_FAULT_TOLERANCE)
     public static final ConfigOption<Long> REFUSED_REGISTRATION_DELAY =
             ConfigOptions.key("cluster.registration.refused-registration-delay")
+                    .longType()
                     .defaultValue(30000L)
                     .withDescription(
                             "The pause made after the registration attempt was refused in milliseconds.");
@@ -64,6 +69,7 @@ public class ClusterOptions {
     @Documentation.Section(Documentation.Sections.EXPERT_FAULT_TOLERANCE)
     public static final ConfigOption<Long> CLUSTER_SERVICES_SHUTDOWN_TIMEOUT =
             ConfigOptions.key("cluster.services.shutdown-timeout")
+                    .longType()
                     .defaultValue(30000L)
                     .withDescription(
                             "The shutdown timeout for cluster services like executors in milliseconds.");
@@ -81,6 +87,7 @@ public class ClusterOptions {
     @Documentation.Section(Documentation.Sections.EXPERT_SCHEDULING)
     public static final ConfigOption<Boolean> EVENLY_SPREAD_OUT_SLOTS_STRATEGY =
             ConfigOptions.key("cluster.evenly-spread-out-slots")
+                    .booleanType()
                     .defaultValue(false)
                     .withDescription(
                             Description.builder()
@@ -121,6 +128,14 @@ public class ClusterOptions {
                                             code(HALT_ON_FATAL_ERROR.key()), code(THROW.name()))
                                     .build());
 
+    @Documentation.Section(Documentation.Sections.EXPERT_CLUSTER)
+    public static final ConfigOption<Integer> THREAD_DUMP_STACKTRACE_MAX_DEPTH =
+            key("cluster.thread-dump.stacktrace-max-depth")
+                    .intType()
+                    .defaultValue(8)
+                    .withDescription(
+                            "The maximum stacktrace depth of TaskManager and JobManager's thread dump web-frontend displayed.");
+
     @Documentation.Section(Documentation.Sections.EXPERT_SCHEDULING)
     public static final ConfigOption<Boolean> ENABLE_FINE_GRAINED_RESOURCE_MANAGEMENT =
             ConfigOptions.key("cluster.fine-grained-resource-management.enabled")
@@ -149,6 +164,47 @@ public class ClusterOptions {
                                     UncaughtExceptionHandleMode.LOG.name(),
                                     UncaughtExceptionHandleMode.FAIL.name()));
 
+    @Documentation.OverrideDefault("io.tmp.dirs")
+    @Documentation.Section(Documentation.Sections.EXPERT_CLUSTER)
+    public static final ConfigOption<String> PROCESS_WORKING_DIR_BASE =
+            ConfigOptions.key("process.working-dir")
+                    .stringType()
+                    .noDefaultValue()
+                    .withDescription(
+                            Description.builder()
+                                    .text(
+                                            "Local working directory for Flink processes. "
+                                                    + "The working directory can be used to store information that can be used upon process recovery. "
+                                                    + "If not configured, then it will default to a randomly picked temporary directory defined via %s.",
+                                            TextElement.code(CoreOptions.TMP_DIRS.key()))
+                                    .build());
+
+    @Documentation.Section(Documentation.Sections.EXPERT_CLUSTER)
+    public static final ConfigOption<String> JOB_MANAGER_PROCESS_WORKING_DIR_BASE =
+            ConfigOptions.key("process.jobmanager.working-dir")
+                    .stringType()
+                    .noDefaultValue()
+                    .withFallbackKeys(PROCESS_WORKING_DIR_BASE.key())
+                    .withDescription(
+                            Description.builder()
+                                    .text(
+                                            "Working directory for Flink JobManager processes. The working directory can be used to store information that can be used upon process recovery. If not configured, then it will default to %s.",
+                                            TextElement.code(PROCESS_WORKING_DIR_BASE.key()))
+                                    .build());
+
+    @Documentation.Section(Documentation.Sections.EXPERT_CLUSTER)
+    public static final ConfigOption<String> TASK_MANAGER_PROCESS_WORKING_DIR_BASE =
+            ConfigOptions.key("process.taskmanager.working-dir")
+                    .stringType()
+                    .noDefaultValue()
+                    .withFallbackKeys(PROCESS_WORKING_DIR_BASE.key())
+                    .withDescription(
+                            Description.builder()
+                                    .text(
+                                            "Working directory for Flink TaskManager processes. The working directory can be used to store information that can be used upon process recovery. If not configured, then it will default to %s.",
+                                            TextElement.code(PROCESS_WORKING_DIR_BASE.key()))
+                                    .build());
+
     public static JobManagerOptions.SchedulerType getSchedulerType(Configuration configuration) {
         if (isAdaptiveSchedulerEnabled(configuration) || isReactiveModeEnabled(configuration)) {
             return JobManagerOptions.SchedulerType.Adaptive;
