@@ -19,6 +19,7 @@
 package org.apache.flink.tests.util.kafka;
 
 import org.apache.flink.api.common.time.Deadline;
+import org.apache.flink.core.testutils.CommonTestUtils;
 
 import org.apache.kafka.clients.CommonClientConfigs;
 import org.apache.kafka.clients.admin.AdminClient;
@@ -32,6 +33,8 @@ import org.apache.kafka.clients.producer.KafkaProducer;
 import org.apache.kafka.clients.producer.Producer;
 import org.apache.kafka.clients.producer.ProducerConfig;
 import org.apache.kafka.clients.producer.ProducerRecord;
+import org.apache.kafka.common.PartitionInfo;
+import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.serialization.BytesDeserializer;
 import org.apache.kafka.common.serialization.BytesSerializer;
 import org.apache.kafka.common.serialization.Deserializer;
@@ -49,7 +52,6 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
-import java.util.regex.Pattern;
 
 /** A utility class that exposes common methods over a {@link KafkaContainer}. */
 public class KafkaContainerClient {
@@ -89,7 +91,7 @@ public class KafkaContainerClient {
             String groupId,
             String topic,
             Deserializer<T> valueDeserializer)
-            throws IOException {
+            throws Exception {
         Properties props = new Properties();
         props.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, container.getBootstrapServers());
         props.put(ConsumerConfig.GROUP_ID_CONFIG, groupId);
@@ -99,7 +101,8 @@ public class KafkaContainerClient {
         final List<T> messages = Collections.synchronizedList(new ArrayList<>(expectedNumMessages));
         try (Consumer<Bytes, T> consumer =
                 new KafkaConsumer<>(props, new BytesDeserializer(), valueDeserializer)) {
-            consumer.subscribe(Pattern.compile(topic));
+            waitUntilTopicAvailableThenAssign(topic, consumer, Duration.ofSeconds(60));
+            // Keep polling until getting expected number of messages
             final Deadline deadline = Deadline.fromNow(Duration.ofSeconds(120));
             while (deadline.hasTimeLeft() && messages.size() < expectedNumMessages) {
                 LOG.info(
@@ -117,4 +120,20 @@ public class KafkaContainerClient {
             return messages;
         }
     }
+
+    private void waitUntilTopicAvailableThenAssign(
+            String topic, Consumer<?, ?> consumer, Duration timeout) throws Exception {
+        // Wait until reading topic is available
+        CommonTestUtils.waitUtil(
+                () -> consumer.listTopics(Duration.ofSeconds(1)).containsKey(topic),
+                timeout,
+                String.format("Cannot get information for topic \"%s\" within timeout", topic));
+        // Assign all partitions of the reading topic
+        List<PartitionInfo> partitions = consumer.listTopics().get(topic);
+        List<TopicPartition> tps = new ArrayList<>();
+        partitions.forEach(partition -> tps.add(new TopicPartition(topic, partition.partition())));
+        consumer.assign(tps);
+        // Seek offsets to beginning in order to consume all records in topic
+        consumer.seekToBeginning(tps);
+    }
 }
