@@ -20,9 +20,8 @@ package org.apache.flink.tests.util.kafka;
 
 import org.apache.flink.api.common.time.Deadline;
 import org.apache.flink.tests.util.TestUtils;
-import org.apache.flink.tests.util.categories.TravisGroup1;
-import org.apache.flink.tests.util.flink.FlinkContainer;
 import org.apache.flink.tests.util.flink.SQLJobSubmission;
+import org.apache.flink.tests.util.flink.container.FlinkContainers;
 import org.apache.flink.tests.util.kafka.containers.SchemaRegistryContainer;
 import org.apache.flink.util.DockerImageVersions;
 
@@ -35,11 +34,10 @@ import org.apache.avro.Schema;
 import org.apache.avro.SchemaBuilder;
 import org.apache.avro.generic.GenericRecordBuilder;
 import org.apache.kafka.common.serialization.StringDeserializer;
+import org.junit.After;
 import org.junit.Before;
 import org.junit.ClassRule;
-import org.junit.Rule;
 import org.junit.Test;
-import org.junit.experimental.categories.Category;
 import org.junit.rules.Timeout;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -60,7 +58,6 @@ import static org.hamcrest.CoreMatchers.equalTo;
 import static org.junit.Assert.assertThat;
 
 /** End-to-end test for SQL client using Avro Confluent Registry format. */
-@Category(value = {TravisGroup1.class})
 public class SQLClientSchemaRegistryITCase {
     private static final Logger LOG = LoggerFactory.getLogger(SQLClientSchemaRegistryITCase.class);
     private static final Slf4jLogConsumer LOG_CONSUMER = new Slf4jLogConsumer(LOG);
@@ -72,36 +69,41 @@ public class SQLClientSchemaRegistryITCase {
     private static final Path sqlToolBoxJar = TestUtils.getResource(".*SqlToolbox.jar");
     private final Path sqlConnectorKafkaJar = TestUtils.getResource(".*kafka.jar");
 
-    @ClassRule private static final Network network = Network.newNetwork();
+    @ClassRule public static final Network NETWORK = Network.newNetwork();
 
     @ClassRule public static final Timeout TIMEOUT = new Timeout(10, TimeUnit.MINUTES);
 
     @ClassRule
-    private static final KafkaContainer kafka =
+    public static final KafkaContainer KAFKA =
             new KafkaContainer(DockerImageName.parse(DockerImageVersions.KAFKA))
-                    .withNetwork(network)
+                    .withNetwork(NETWORK)
                     .withNetworkAliases(INTER_CONTAINER_KAFKA_ALIAS)
                     .withLogConsumer(LOG_CONSUMER);
 
     @ClassRule
-    private static final SchemaRegistryContainer registry =
-            new SchemaRegistryContainer("5.5.2")
+    public static final SchemaRegistryContainer REGISTRY =
+            new SchemaRegistryContainer("6.2.2")
                     .withKafka(INTER_CONTAINER_KAFKA_ALIAS + ":9092")
-                    .withNetwork(network)
+                    .withNetwork(NETWORK)
                     .withNetworkAliases(INTER_CONTAINER_REGISTRY_ALIAS)
-                    .dependsOn(kafka);
+                    .dependsOn(KAFKA);
 
-    @Rule
-    public final FlinkContainer flink =
-            FlinkContainer.builder().build().withNetwork(network).dependsOn(kafka);
+    public final FlinkContainers flink =
+            FlinkContainers.builder().setNetwork(NETWORK).setLogger(LOG).dependsOn(KAFKA).build();
 
     private KafkaContainerClient kafkaClient;
     private CachedSchemaRegistryClient registryClient;
 
     @Before
-    public void setUp() {
-        kafkaClient = new KafkaContainerClient(kafka);
-        registryClient = new CachedSchemaRegistryClient(registry.getSchemaRegistryUrl(), 10);
+    public void setUp() throws Exception {
+        flink.start();
+        kafkaClient = new KafkaContainerClient(KAFKA);
+        registryClient = new CachedSchemaRegistryClient(REGISTRY.getSchemaRegistryUrl(), 10);
+    }
+
+    @After
+    public void tearDown() {
+        flink.stop();
     }
 
     @Test
