@@ -21,8 +21,6 @@ package org.apache.flink.tests.util.hbase;
 import org.apache.flink.api.common.time.Deadline;
 import org.apache.flink.tests.util.TestUtils;
 import org.apache.flink.tests.util.cache.DownloadCache;
-import org.apache.flink.tests.util.categories.PreCommit;
-import org.apache.flink.tests.util.categories.TravisGroup1;
 import org.apache.flink.tests.util.flink.ClusterController;
 import org.apache.flink.tests.util.flink.FlinkResource;
 import org.apache.flink.tests.util.flink.FlinkResourceSetup;
@@ -68,7 +66,7 @@ import static org.junit.Assert.assertThat;
 
 /** End-to-end test for the HBase connectors. */
 @RunWith(Parameterized.class)
-@Category(value = {TravisGroup1.class, PreCommit.class, FailsOnJava11.class})
+@Category(value = {FailsOnJava11.class})
 @Ignore("FLINK-21519")
 public class SQLClientHBaseITCase extends TestLogger {
 
