@@ -52,7 +52,11 @@ function setup_kafka_dist {
 function setup_confluent_dist {
   # download confluent
   mkdir -p $TEST_DATA_DIR
-  CONFLUENT_URL="http://packages.confluent.io/archive/$CONFLUENT_MAJOR_VERSION/confluent-oss-$CONFLUENT_VERSION-2.11.tar.gz"
+  if [[ $CONFLUENT_MAJOR_VERSION =~ ^[6] ]]; then
+    CONFLUENT_URL="http://packages.confluent.io/archive/$CONFLUENT_MAJOR_VERSION/confluent-community-$CONFLUENT_VERSION.tar.gz"
+  else
+    CONFLUENT_URL="http://packages.confluent.io/archive/$CONFLUENT_MAJOR_VERSION/confluent-community-$CONFLUENT_VERSION-2.12.tar.gz"
+  fi
   echo "Downloading confluent from $CONFLUENT_URL"
   cache_path=$(get_artifact $CONFLUENT_URL)
   ln "$cache_path" "${TEST_DATA_DIR}/confluent.tgz"
@@ -95,7 +99,7 @@ function start_kafka_cluster {
 
   start_time=$(date +%s)
   #
-  # Wait for the broker info to appear in ZK. We assume propery registration once an entry
+  # Wait for the broker info to appear in ZK. We assume property registration once an entry
   # similar to this is in ZK: {"listener_security_protocol_map":{"PLAINTEXT":"PLAINTEXT"},"endpoints":["PLAINTEXT://my-host:9092"],"jmx_port":-1,"host":"honorary-pig","timestamp":"1583157804932","port":9092,"version":4}
   #
   while ! [[ $($KAFKA_DIR/bin/zookeeper-shell.sh localhost:2181 get /brokers/ids/0 2>&1) =~ .*listener_security_protocol_map.* ]]; do
