@@ -19,6 +19,7 @@ package org.apache.flink.formats.avro;
 
 import org.apache.flink.annotation.PublicEvolving;
 import org.apache.flink.api.common.serialization.AbstractDeserializationSchema;
+import org.apache.flink.api.common.serialization.DeserializationSchema;
 import org.apache.flink.api.common.typeinfo.BasicArrayTypeInfo;
 import org.apache.flink.api.common.typeinfo.TypeInformation;
 import org.apache.flink.api.common.typeinfo.Types;
@@ -76,8 +77,13 @@ import java.util.TimeZone;
  *
  * <p>Note: Changes in this class need to be kept in sync with the corresponding runtime class
  * {@link AvroRowSerializationSchema} and schema converter {@link AvroSchemaConverter}.
+ *
+ * @deprecated The format was developed for the Table API users and will not be maintained for
+ *     DataStream API users anymore. Either use Table API or switch to Data Stream, defining your
+ *     own {@link DeserializationSchema}.
  */
 @PublicEvolving
+@Deprecated
 public class AvroRowDeserializationSchema extends AbstractDeserializationSchema<Row> {
     /** Used for time conversions into SQL types. */
     private static final TimeZone LOCAL_TZ = TimeZone.getDefault();
@@ -245,15 +251,16 @@ public class AvroRowDeserializationSchema extends AbstractDeserializationSchema<
                 final int size = types.size();
                 final Schema actualSchema;
                 if (size == 2 && types.get(0).getType() == Schema.Type.NULL) {
-                    return convertAvroType(types.get(1), info, object);
+                    actualSchema = types.get(1);
                 } else if (size == 2 && types.get(1).getType() == Schema.Type.NULL) {
-                    return convertAvroType(types.get(0), info, object);
+                    actualSchema = types.get(0);
                 } else if (size == 1) {
-                    return convertAvroType(types.get(0), info, object);
+                    actualSchema = types.get(0);
                 } else {
                     // generic type
                     return object;
                 }
+                return convertAvroType(actualSchema, info, object);
             case FIXED:
                 final byte[] fixedBytes = ((GenericFixed) object).bytes();
                 if (info == Types.BIG_DEC) {
