@@ -39,6 +39,11 @@ public class CsvFilesystemBatchITCase {
     /** General IT cases for CsvRowDataFilesystem in batch mode. */
     public static class GeneralCsvFilesystemBatchITCase extends BatchFileSystemITCaseBase {
 
+        @Override
+        public boolean supportsReadingMetadata() {
+            return false;
+        }
+
         @Override
         public String[] formatProperties() {
             List<String> ret = new ArrayList<>();
@@ -55,6 +60,11 @@ public class CsvFilesystemBatchITCase {
      */
     public static class EnrichedCsvFilesystemBatchITCase extends BatchFileSystemITCaseBase {
 
+        @Override
+        public boolean supportsReadingMetadata() {
+            return false;
+        }
+
         @Override
         public String[] formatProperties() {
             List<String> ret = new ArrayList<>();
@@ -70,7 +80,22 @@ public class CsvFilesystemBatchITCase {
             new File(path).mkdirs();
             File file = new File(path, "test_file");
             file.createNewFile();
-            FileUtils.writeFileUtf8(file, "x5,5,1,1\n" + "x5,5,2,2,2\n" + "x5,5,1,1");
+            FileUtils.writeFileUtf8(
+                    file, "x5,5,1,1\n" + "x5,5,2,2,2\n" + "x5,5,3,3,3,3\n" + "x5,5,1,1");
+
+            check(
+                    "select * from nonPartitionedTable",
+                    Arrays.asList(Row.of("x5", 5, 1, 1), Row.of("x5", 5, 1, 1)));
+        }
+
+        @Test
+        public void testParseErrorLast() throws Exception {
+            String path = new URI(resultPath()).getPath();
+            new File(path).mkdirs();
+            File file = new File(path, "test_file");
+            file.createNewFile();
+            FileUtils.writeFileUtf8(
+                    file, "x5,5,1,1\n" + "x5,5,2,2,2\n" + "x5,5,1,1\n" + "x5,5,3,3,3,3\n");
 
             check(
                     "select * from nonPartitionedTable",
