@@ -26,6 +26,11 @@ import java.util.List;
 /** ITCase to test csv format for {@link CsvFileSystemFormatFactory} in stream mode. */
 public class CsvFilesystemStreamITCase extends StreamFileSystemITCaseBase {
 
+    @Override
+    public boolean supportsReadingMetadata() {
+        return false;
+    }
+
     @Override
     public String[] formatProperties() {
         List<String> ret = new ArrayList<>();
