@@ -60,7 +60,7 @@ public class KubernetesConfigOptions {
     public static final ConfigOption<ServiceExposedType> REST_SERVICE_EXPOSED_TYPE =
             key("kubernetes.rest-service.exposed.type")
                     .enumType(ServiceExposedType.class)
-                    .defaultValue(ServiceExposedType.LoadBalancer)
+                    .defaultValue(ServiceExposedType.ClusterIP)
                     .withDescription(
                             "The exposed type of the rest service. "
                                     + "The exposed rest service could be used to access the Flink’s Web UI and REST endpoint.");
@@ -146,6 +146,22 @@ public class KubernetesConfigOptions {
                     .defaultValue(1.0)
                     .withDescription("The number of cpu used by job manager");
 
+    public static final ConfigOption<Double> JOB_MANAGER_CPU_LIMIT_FACTOR =
+            key("kubernetes.jobmanager.cpu.limit-factor")
+                    .doubleType()
+                    .defaultValue(1.0)
+                    .withDescription(
+                            "The limit factor of cpu used by job manager. "
+                                    + "The resources limit cpu will be set to cpu * limit-factor.");
+
+    public static final ConfigOption<Double> JOB_MANAGER_MEMORY_LIMIT_FACTOR =
+            key("kubernetes.jobmanager.memory.limit-factor")
+                    .doubleType()
+                    .defaultValue(1.0)
+                    .withDescription(
+                            "The limit factor of memory used by job manager. "
+                                    + "The resources limit memory will be set to memory * limit-factor.");
+
     public static final ConfigOption<Double> TASK_MANAGER_CPU =
             key("kubernetes.taskmanager.cpu")
                     .doubleType()
@@ -154,6 +170,22 @@ public class KubernetesConfigOptions {
                             "The number of cpu used by task manager. By default, the cpu is set "
                                     + "to the number of slots per TaskManager");
 
+    public static final ConfigOption<Double> TASK_MANAGER_CPU_LIMIT_FACTOR =
+            key("kubernetes.taskmanager.cpu.limit-factor")
+                    .doubleType()
+                    .defaultValue(1.0)
+                    .withDescription(
+                            "The limit factor of cpu used by task manager. "
+                                    + "The resources limit cpu will be set to cpu * limit-factor.");
+
+    public static final ConfigOption<Double> TASK_MANAGER_MEMORY_LIMIT_FACTOR =
+            key("kubernetes.taskmanager.memory.limit-factor")
+                    .doubleType()
+                    .defaultValue(1.0)
+                    .withDescription(
+                            "The limit factor of memory used by task manager. "
+                                    + "The resources limit memory will be set to memory * limit-factor.");
+
     public static final ConfigOption<ImagePullPolicy> CONTAINER_IMAGE_PULL_POLICY =
             key("kubernetes.container.image.pull-policy")
                     .enumType(ImagePullPolicy.class)
@@ -458,6 +490,14 @@ public class KubernetesConfigOptions {
                                     + "It will help to achieve faster recovery. "
                                     + "Notice that high availability should be enabled when starting standby JobManagers.");
 
+    public static final ConfigOption<Boolean> KUBERNETES_HOSTNETWORK_ENABLED =
+            key("kubernetes.hostnetwork.enabled")
+                    .booleanType()
+                    .defaultValue(false)
+                    .withDescription(
+                            "Whether to enable HostNetwork mode. "
+                                    + "The HostNetwork allows the pod could use the node network namespace instead of the individual pod network namespace. Please note that the JobManager service account should have the permission to update Kubernetes service.");
+
     private static String getDefaultFlinkImage() {
         // The default container image that ties to the exact needed versions of both Flink and
         // Scala.
