@@ -23,6 +23,7 @@ import org.apache.flink.client.program.PackagedProgramUtils;
 import org.apache.flink.configuration.ConfigOption;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.PipelineOptions;
+import org.apache.flink.kubernetes.configuration.KubernetesConfigOptions;
 import org.apache.flink.kubernetes.highavailability.KubernetesCheckpointStoreUtil;
 import org.apache.flink.kubernetes.highavailability.KubernetesJobGraphStoreUtil;
 import org.apache.flink.kubernetes.highavailability.KubernetesStateHandleStore;
@@ -36,18 +37,24 @@ import org.apache.flink.runtime.checkpoint.DefaultCompletedCheckpointStore;
 import org.apache.flink.runtime.checkpoint.DefaultCompletedCheckpointStoreUtils;
 import org.apache.flink.runtime.highavailability.HighAvailabilityServicesUtils;
 import org.apache.flink.runtime.jobgraph.JobGraph;
+import org.apache.flink.runtime.jobgraph.RestoreMode;
 import org.apache.flink.runtime.jobmanager.DefaultJobGraphStore;
 import org.apache.flink.runtime.jobmanager.JobGraphStore;
 import org.apache.flink.runtime.jobmanager.NoOpJobGraphStoreWatcher;
 import org.apache.flink.runtime.leaderelection.LeaderInformation;
 import org.apache.flink.runtime.persistence.RetrievableStateStorageHelper;
 import org.apache.flink.runtime.persistence.filesystem.FileSystemStateStorageHelper;
+import org.apache.flink.runtime.state.SharedStateRegistryFactory;
+import org.apache.flink.util.ExceptionUtils;
+import org.apache.flink.util.FlinkException;
 import org.apache.flink.util.FlinkRuntimeException;
+import org.apache.flink.util.Preconditions;
 import org.apache.flink.util.StringUtils;
 import org.apache.flink.util.function.FunctionUtils;
 
 import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.dataformat.yaml.YAMLMapper;
 
+import io.fabric8.kubernetes.api.model.ConfigMapBuilder;
 import io.fabric8.kubernetes.api.model.Container;
 import io.fabric8.kubernetes.api.model.ContainerBuilder;
 import io.fabric8.kubernetes.api.model.KubernetesResource;
@@ -63,17 +70,21 @@ import java.io.File;
 import java.net.URI;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.UUID;
+import java.util.concurrent.CompletionException;
 import java.util.concurrent.Executor;
 import java.util.stream.Collectors;
 
 import static org.apache.flink.kubernetes.utils.Constants.CHECKPOINT_ID_KEY_PREFIX;
 import static org.apache.flink.kubernetes.utils.Constants.COMPLETED_CHECKPOINT_FILE_SUFFIX;
 import static org.apache.flink.kubernetes.utils.Constants.JOB_GRAPH_STORE_KEY_PREFIX;
+import static org.apache.flink.kubernetes.utils.Constants.LABEL_CONFIGMAP_TYPE_HIGH_AVAILABILITY;
 import static org.apache.flink.kubernetes.utils.Constants.LEADER_ADDRESS_KEY;
 import static org.apache.flink.kubernetes.utils.Constants.LEADER_SESSION_ID_KEY;
 import static org.apache.flink.kubernetes.utils.Constants.SUBMITTED_JOBGRAPH_FILE_PREFIX;
@@ -86,6 +97,9 @@ public class KubernetesUtils {
 
     private static final YAMLMapper yamlMapper = new YAMLMapper();
 
+    private static final String LEADER_PREFIX = "org.apache.flink.k8s.leader.";
+    private static final char LEADER_INFORMATION_SEPARATOR = ',';
+
     /**
      * Check whether the port config option is a fixed port. If not, the fallback port will be set
      * to configuration.
@@ -283,6 +297,7 @@ public class KubernetesUtils {
      * @param lockIdentity lock identity to check the leadership
      * @param maxNumberOfCheckpointsToRetain max number of checkpoints to retain on state store
      *     handle
+     * @param restoreMode the mode in which the job is restoring
      * @return a {@link DefaultCompletedCheckpointStore} with {@link KubernetesStateHandleStore}.
      * @throws Exception when create the storage helper failed
      */
@@ -291,8 +306,11 @@ public class KubernetesUtils {
             FlinkKubeClient kubeClient,
             Executor executor,
             String configMapName,
-            String lockIdentity,
-            int maxNumberOfCheckpointsToRetain)
+            @Nullable String lockIdentity,
+            int maxNumberOfCheckpointsToRetain,
+            SharedStateRegistryFactory sharedStateRegistryFactory,
+            Executor ioExecutor,
+            RestoreMode restoreMode)
             throws Exception {
 
         final RetrievableStateStorageHelper<CompletedCheckpoint> stateStorage =
@@ -307,12 +325,16 @@ public class KubernetesUtils {
                         stateStorage,
                         k -> k.startsWith(CHECKPOINT_ID_KEY_PREFIX),
                         lockIdentity);
+        Collection<CompletedCheckpoint> checkpoints =
+                DefaultCompletedCheckpointStoreUtils.retrieveCompletedCheckpoints(
+                        stateHandleStore, KubernetesCheckpointStoreUtil.INSTANCE);
+
         return new DefaultCompletedCheckpointStore<>(
                 maxNumberOfCheckpointsToRetain,
                 stateHandleStore,
                 KubernetesCheckpointStoreUtil.INSTANCE,
-                DefaultCompletedCheckpointStoreUtils.retrieveCompletedCheckpoints(
-                        stateHandleStore, KubernetesCheckpointStoreUtil.INSTANCE),
+                checkpoints,
+                sharedStateRegistryFactory.create(ioExecutor, checkpoints, restoreMode),
                 executor);
     }
 
@@ -321,7 +343,9 @@ public class KubernetesUtils {
      *
      * @param resourceRequirements resource requirements in pod template
      * @param mem Memory in mb.
+     * @param memoryLimitFactor limit factor for the memory, used to set the limit resources.
      * @param cpu cpu.
+     * @param cpuLimitFactor limit factor for the cpu, used to set the limit resources.
      * @param externalResources external resources
      * @param externalResourceConfigKeys config keys of external resources
      * @return KubernetesResource requirements.
@@ -329,18 +353,23 @@ public class KubernetesUtils {
     public static ResourceRequirements getResourceRequirements(
             ResourceRequirements resourceRequirements,
             int mem,
+            double memoryLimitFactor,
             double cpu,
+            double cpuLimitFactor,
             Map<String, ExternalResource> externalResources,
             Map<String, String> externalResourceConfigKeys) {
         final Quantity cpuQuantity = new Quantity(String.valueOf(cpu));
+        final Quantity cpuLimitQuantity = new Quantity(String.valueOf(cpu * cpuLimitFactor));
         final Quantity memQuantity = new Quantity(mem + Constants.RESOURCE_UNIT_MB);
+        final Quantity memQuantityLimit =
+                new Quantity(((int) (mem * memoryLimitFactor)) + Constants.RESOURCE_UNIT_MB);
 
         ResourceRequirementsBuilder resourceRequirementsBuilder =
                 new ResourceRequirementsBuilder(resourceRequirements)
                         .addToRequests(Constants.RESOURCE_NAME_MEMORY, memQuantity)
                         .addToRequests(Constants.RESOURCE_NAME_CPU, cpuQuantity)
-                        .addToLimits(Constants.RESOURCE_NAME_MEMORY, memQuantity)
-                        .addToLimits(Constants.RESOURCE_NAME_CPU, cpuQuantity);
+                        .addToLimits(Constants.RESOURCE_NAME_MEMORY, memQuantityLimit)
+                        .addToLimits(Constants.RESOURCE_NAME_CPU, cpuLimitQuantity);
 
         // Add the external resources to resource requirement.
         for (Map.Entry<String, ExternalResource> externalResource : externalResources.entrySet()) {
@@ -486,11 +515,106 @@ public class KubernetesUtils {
         }
     }
 
+    /** Checks if hostNetwork is enabled. */
+    public static boolean isHostNetwork(Configuration configuration) {
+        return configuration.getBoolean(KubernetesConfigOptions.KUBERNETES_HOSTNETWORK_ENABLED);
+    }
+
+    /**
+     * Creates a config map with the given name if it does not exist.
+     *
+     * @param flinkKubeClient to use for creating the config map
+     * @param configMapName name of the config map
+     * @param clusterId clusterId to which the map belongs
+     * @throws FlinkException if the config map could not be created
+     */
+    public static void createConfigMapIfItDoesNotExist(
+            FlinkKubeClient flinkKubeClient, String configMapName, String clusterId)
+            throws FlinkException {
+
+        int attempt = 0;
+        CompletionException lastException = null;
+
+        final int maxAttempts = 10;
+        final KubernetesConfigMap configMap =
+                new KubernetesConfigMap(
+                        new ConfigMapBuilder()
+                                .withNewMetadata()
+                                .withName(configMapName)
+                                .withLabels(
+                                        getConfigMapLabels(
+                                                clusterId, LABEL_CONFIGMAP_TYPE_HIGH_AVAILABILITY))
+                                .endMetadata()
+                                .build());
+
+        while (!flinkKubeClient.getConfigMap(configMapName).isPresent() && attempt < maxAttempts) {
+            try {
+                flinkKubeClient.createConfigMap(configMap).join();
+            } catch (CompletionException e) {
+                // retrying
+                lastException = ExceptionUtils.firstOrSuppressed(e, lastException);
+            }
+
+            attempt++;
+        }
+
+        if (attempt >= maxAttempts && lastException != null) {
+            throw new FlinkException(
+                    String.format("Could not create the config map %s.", configMapName),
+                    lastException);
+        }
+    }
+
     /** Cluster components. */
     public enum ClusterComponent {
         JOB_MANAGER,
         TASK_MANAGER
     }
 
+    public static String encodeLeaderInformation(LeaderInformation leaderInformation) {
+        Preconditions.checkArgument(leaderInformation.getLeaderSessionID() != null);
+        Preconditions.checkArgument(leaderInformation.getLeaderAddress() != null);
+
+        return leaderInformation.getLeaderSessionID().toString()
+                + LEADER_INFORMATION_SEPARATOR
+                + leaderInformation.getLeaderAddress();
+    }
+
+    public static Optional<LeaderInformation> parseLeaderInformationSafely(String value) {
+        try {
+            return Optional.of(parseLeaderInformation(value));
+        } catch (Throwable throwable) {
+            LOG.debug("Could not parse value {} into LeaderInformation.", value, throwable);
+            return Optional.empty();
+        }
+    }
+
+    private static LeaderInformation parseLeaderInformation(String value) {
+        final int splitIndex = value.indexOf(LEADER_INFORMATION_SEPARATOR);
+
+        Preconditions.checkState(
+                splitIndex >= 0,
+                String.format(
+                        "Expecting '<session_id>%c<leader_address>'",
+                        LEADER_INFORMATION_SEPARATOR));
+
+        final UUID leaderSessionId = UUID.fromString(value.substring(0, splitIndex));
+        final String leaderAddress = value.substring(splitIndex + 1);
+
+        return LeaderInformation.known(leaderSessionId, leaderAddress);
+    }
+
+    public static String createSingleLeaderKey(String componentId) {
+        return LEADER_PREFIX + componentId;
+    }
+
+    public static boolean isSingleLeaderKey(String key) {
+        return key.startsWith(LEADER_PREFIX);
+    }
+
+    public static String extractLeaderName(String key) {
+        return key.substring(LEADER_PREFIX.length());
+    }
+
     private KubernetesUtils() {}
 }
