@@ -20,6 +20,7 @@ package org.apache.flink.kubernetes;
 
 import org.apache.flink.kubernetes.configuration.KubernetesConfigOptions;
 import org.apache.flink.kubernetes.kubeclient.decorators.ExternalServiceDecorator;
+import org.apache.flink.kubernetes.kubeclient.services.HeadlessClusterIPService;
 import org.apache.flink.kubernetes.utils.Constants;
 import org.apache.flink.util.Preconditions;
 
@@ -144,7 +145,8 @@ public class KubernetesClientTestBase extends KubernetesTestBase {
         return buildExternalService(
                 KubernetesConfigOptions.ServiceExposedType.LoadBalancer,
                 servicePort,
-                serviceStatus);
+                serviceStatus,
+                false);
     }
 
     protected Service buildExternalServiceWithNodePort() {
@@ -162,7 +164,10 @@ public class KubernetesClientTestBase extends KubernetesTestBase {
                         .build();
 
         return buildExternalService(
-                KubernetesConfigOptions.ServiceExposedType.NodePort, servicePort, serviceStatus);
+                KubernetesConfigOptions.ServiceExposedType.NodePort,
+                servicePort,
+                serviceStatus,
+                false);
     }
 
     protected Service buildExternalServiceWithClusterIP() {
@@ -174,13 +179,26 @@ public class KubernetesClientTestBase extends KubernetesTestBase {
                         .build();
 
         return buildExternalService(
-                KubernetesConfigOptions.ServiceExposedType.ClusterIP, servicePort, null);
+                KubernetesConfigOptions.ServiceExposedType.ClusterIP, servicePort, null, false);
+    }
+
+    protected Service buildExternalServiceWithHeadlessClusterIP() {
+        final ServicePort servicePort =
+                new ServicePortBuilder()
+                        .withName(Constants.REST_PORT_NAME)
+                        .withPort(REST_PORT)
+                        .withNewTargetPort(REST_PORT)
+                        .build();
+
+        return buildExternalService(
+                KubernetesConfigOptions.ServiceExposedType.ClusterIP, servicePort, null, true);
     }
 
     private Service buildExternalService(
             KubernetesConfigOptions.ServiceExposedType serviceExposedType,
             ServicePort servicePort,
-            @Nullable ServiceStatus serviceStatus) {
+            @Nullable ServiceStatus serviceStatus,
+            boolean isHeadlessSvc) {
         final ServiceBuilder serviceBuilder =
                 new ServiceBuilder()
                         .editOrNewMetadata()
@@ -196,6 +214,12 @@ public class KubernetesClientTestBase extends KubernetesTestBase {
             serviceBuilder.withStatus(serviceStatus);
         }
 
+        if (isHeadlessSvc) {
+            serviceBuilder
+                    .editOrNewSpec()
+                    .withClusterIP(HeadlessClusterIPService.HEADLESS_CLUSTER_IP)
+                    .endSpec();
+        }
         return serviceBuilder.build();
     }
 }
