@@ -18,6 +18,7 @@
 
 package org.apache.flink.cep.operator;
 
+import org.apache.flink.FlinkVersion;
 import org.apache.flink.api.common.typeinfo.BasicTypeInfo;
 import org.apache.flink.api.java.functions.KeySelector;
 import org.apache.flink.cep.Event;
@@ -26,6 +27,7 @@ import org.apache.flink.cep.nfa.NFA;
 import org.apache.flink.cep.nfa.compiler.NFACompiler;
 import org.apache.flink.cep.pattern.Pattern;
 import org.apache.flink.cep.pattern.conditions.SimpleCondition;
+import org.apache.flink.cep.utils.CepOperatorTestUtilities;
 import org.apache.flink.runtime.checkpoint.OperatorSubtaskState;
 import org.apache.flink.streaming.api.watermark.Watermark;
 import org.apache.flink.streaming.api.windowing.time.Time;
@@ -33,7 +35,6 @@ import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
 import org.apache.flink.streaming.util.KeyedOneInputStreamOperatorTestHarness;
 import org.apache.flink.streaming.util.OneInputStreamOperatorTestHarness;
 import org.apache.flink.streaming.util.OperatorSnapshotUtil;
-import org.apache.flink.testutils.migration.MigrationVersion;
 
 import org.junit.Ignore;
 import org.junit.Test;
@@ -46,7 +47,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.concurrent.ConcurrentLinkedQueue;
 
-import static org.apache.flink.cep.operator.CepOperatorTestUtilities.getKeyedCepOpearator;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
 
@@ -62,29 +62,29 @@ public class CEPMigrationTest {
 
     /**
      * TODO change this to the corresponding savepoint version to be written (e.g. {@link
-     * MigrationVersion#v1_3} for 1.3) TODO and remove all @Ignore annotations on write*Snapshot()
+     * FlinkVersion#v1_3} for 1.3) TODO and remove all @Ignore annotations on write*Snapshot()
      * methods to generate savepoints TODO Note: You should generate the savepoint based on the
      * release branch instead of the master.
      */
-    private final MigrationVersion flinkGenerateSavepointVersion = null;
+    private final FlinkVersion flinkGenerateSavepointVersion = null;
 
-    private final MigrationVersion migrateVersion;
+    private final FlinkVersion migrateVersion;
 
     @Parameterized.Parameters(name = "Migration Savepoint: {0}")
-    public static Collection<MigrationVersion> parameters() {
+    public static Collection<FlinkVersion> parameters() {
         return Arrays.asList(
-                MigrationVersion.v1_6,
-                MigrationVersion.v1_7,
-                MigrationVersion.v1_8,
-                MigrationVersion.v1_9,
-                MigrationVersion.v1_10,
-                MigrationVersion.v1_11,
-                MigrationVersion.v1_12,
-                MigrationVersion.v1_13,
-                MigrationVersion.v1_14);
+                FlinkVersion.v1_6,
+                FlinkVersion.v1_7,
+                FlinkVersion.v1_8,
+                FlinkVersion.v1_9,
+                FlinkVersion.v1_10,
+                FlinkVersion.v1_11,
+                FlinkVersion.v1_12,
+                FlinkVersion.v1_13,
+                FlinkVersion.v1_14);
     }
 
-    public CEPMigrationTest(MigrationVersion migrateVersion) {
+    public CEPMigrationTest(FlinkVersion migrateVersion) {
         this.migrateVersion = migrateVersion;
     }
 
@@ -109,7 +109,7 @@ public class CEPMigrationTest {
 
         OneInputStreamOperatorTestHarness<Event, Map<String, List<Event>>> harness =
                 new KeyedOneInputStreamOperatorTestHarness<>(
-                        getKeyedCepOpearator(false, new NFAFactory()),
+                        CepOperatorTestUtilities.getKeyedCepOperator(false, new NFAFactory()),
                         keySelector,
                         BasicTypeInfo.INT_TYPE_INFO);
 
@@ -158,7 +158,7 @@ public class CEPMigrationTest {
 
         OneInputStreamOperatorTestHarness<Event, Map<String, List<Event>>> harness =
                 new KeyedOneInputStreamOperatorTestHarness<>(
-                        getKeyedCepOpearator(false, new NFAFactory()),
+                        CepOperatorTestUtilities.getKeyedCepOperator(false, new NFAFactory()),
                         keySelector,
                         BasicTypeInfo.INT_TYPE_INFO);
 
@@ -222,7 +222,7 @@ public class CEPMigrationTest {
 
             harness =
                     new KeyedOneInputStreamOperatorTestHarness<>(
-                            getKeyedCepOpearator(false, new NFAFactory()),
+                            CepOperatorTestUtilities.getKeyedCepOperator(false, new NFAFactory()),
                             keySelector,
                             BasicTypeInfo.INT_TYPE_INFO);
 
@@ -276,7 +276,7 @@ public class CEPMigrationTest {
 
         OneInputStreamOperatorTestHarness<Event, Map<String, List<Event>>> harness =
                 new KeyedOneInputStreamOperatorTestHarness<>(
-                        getKeyedCepOpearator(false, new NFAFactory()),
+                        CepOperatorTestUtilities.getKeyedCepOperator(false, new NFAFactory()),
                         keySelector,
                         BasicTypeInfo.INT_TYPE_INFO);
 
@@ -323,7 +323,7 @@ public class CEPMigrationTest {
 
         OneInputStreamOperatorTestHarness<Event, Map<String, List<Event>>> harness =
                 new KeyedOneInputStreamOperatorTestHarness<>(
-                        getKeyedCepOpearator(false, new NFAFactory()),
+                        CepOperatorTestUtilities.getKeyedCepOperator(false, new NFAFactory()),
                         keySelector,
                         BasicTypeInfo.INT_TYPE_INFO);
 
@@ -403,7 +403,7 @@ public class CEPMigrationTest {
 
             harness =
                     new KeyedOneInputStreamOperatorTestHarness<>(
-                            getKeyedCepOpearator(false, new NFAFactory()),
+                            CepOperatorTestUtilities.getKeyedCepOperator(false, new NFAFactory()),
                             keySelector,
                             BasicTypeInfo.INT_TYPE_INFO);
 
@@ -456,7 +456,8 @@ public class CEPMigrationTest {
 
         OneInputStreamOperatorTestHarness<Event, Map<String, List<Event>>> harness =
                 new KeyedOneInputStreamOperatorTestHarness<>(
-                        getKeyedCepOpearator(false, new SinglePatternNFAFactory()),
+                        CepOperatorTestUtilities.getKeyedCepOperator(
+                                false, new SinglePatternNFAFactory()),
                         keySelector,
                         BasicTypeInfo.INT_TYPE_INFO);
 
@@ -494,7 +495,8 @@ public class CEPMigrationTest {
 
         OneInputStreamOperatorTestHarness<Event, Map<String, List<Event>>> harness =
                 new KeyedOneInputStreamOperatorTestHarness<>(
-                        getKeyedCepOpearator(false, new SinglePatternNFAFactory()),
+                        CepOperatorTestUtilities.getKeyedCepOperator(
+                                false, new SinglePatternNFAFactory()),
                         keySelector,
                         BasicTypeInfo.INT_TYPE_INFO);
 
@@ -552,7 +554,8 @@ public class CEPMigrationTest {
 
         OneInputStreamOperatorTestHarness<Event, Map<String, List<Event>>> harness =
                 new KeyedOneInputStreamOperatorTestHarness<>(
-                        getKeyedCepOpearator(false, new NFAComplexConditionsFactory()),
+                        CepOperatorTestUtilities.getKeyedCepOperator(
+                                false, new NFAComplexConditionsFactory()),
                         keySelector,
                         BasicTypeInfo.INT_TYPE_INFO);
 
@@ -591,7 +594,8 @@ public class CEPMigrationTest {
 
         OneInputStreamOperatorTestHarness<Event, Map<String, List<Event>>> harness =
                 new KeyedOneInputStreamOperatorTestHarness<>(
-                        getKeyedCepOpearator(false, new NFAComplexConditionsFactory()),
+                        CepOperatorTestUtilities.getKeyedCepOperator(
+                                false, new NFAComplexConditionsFactory()),
                         keySelector,
                         BasicTypeInfo.INT_TYPE_INFO);
 
