@@ -21,19 +21,17 @@ package org.apache.flink.state.api;
 import org.apache.flink.api.common.state.ValueState;
 import org.apache.flink.api.common.state.ValueStateDescriptor;
 import org.apache.flink.api.common.typeinfo.Types;
-import org.apache.flink.api.java.DataSet;
-import org.apache.flink.api.java.ExecutionEnvironment;
 import org.apache.flink.api.java.tuple.Tuple2;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.MemorySize;
 import org.apache.flink.contrib.streaming.state.EmbeddedRocksDBStateBackend;
-import org.apache.flink.contrib.streaming.state.RocksDBStateBackend;
-import org.apache.flink.core.fs.Path;
 import org.apache.flink.runtime.state.StateBackend;
-import org.apache.flink.runtime.state.filesystem.FsStateBackend;
 import org.apache.flink.runtime.state.hashmap.HashMapStateBackend;
 import org.apache.flink.state.api.functions.KeyedStateBootstrapFunction;
 import org.apache.flink.state.api.functions.KeyedStateReaderFunction;
+import org.apache.flink.state.api.utils.JobResultRetriever;
+import org.apache.flink.streaming.api.datastream.DataStream;
+import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.test.util.AbstractTestBase;
 import org.apache.flink.util.AbstractID;
 import org.apache.flink.util.Collector;
@@ -74,11 +72,7 @@ public class SavepointDeepCopyTest extends AbstractTestBase {
 
     @Parameterized.Parameters(name = "State Backend: {0}")
     public static Collection<StateBackend> data() {
-        return Arrays.asList(
-                new FsStateBackend(new Path("file:///tmp").toUri()),
-                new RocksDBStateBackend(new FsStateBackend(new Path("file:///tmp").toUri())),
-                new HashMapStateBackend(),
-                new EmbeddedRocksDBStateBackend());
+        return Arrays.asList(new HashMapStateBackend(), new EmbeddedRocksDBStateBackend());
     }
 
     /** To bootstrapper a savepoint for testing. */
@@ -137,15 +131,11 @@ public class SavepointDeepCopyTest extends AbstractTestBase {
      */
     @Test
     public void testSavepointDeepCopy() throws Exception {
+        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
 
-        // set up the execution environment
-        ExecutionEnvironment env = ExecutionEnvironment.getExecutionEnvironment();
+        DataStream<String> words = env.fromElements(TEXT.split(" "));
 
-        // construct DataSet
-        DataSet<String> words = env.fromElements(TEXT.split(" "));
-
-        // create BootstrapTransformation
-        BootstrapTransformation<String> transformation =
+        StateBootstrapTransformation<String> transformation =
                 OperatorTransformation.bootstrapWith(words)
                         .keyBy(e -> e)
                         .transform(new WordMapBootstrapper());
@@ -153,9 +143,7 @@ public class SavepointDeepCopyTest extends AbstractTestBase {
         File savepointUrl1 = createAndRegisterTempFile(new AbstractID().toHexString());
         String savepointPath1 = savepointUrl1.getPath();
 
-        // create a savepoint with BootstrapTransformations (one per operator)
-        // write the created savepoint to a given path
-        Savepoint.create(backend, 128)
+        SavepointWriter.newSavepoint(backend, 128)
                 .withConfiguration(FS_SMALL_FILE_THRESHOLD, FILE_STATE_SIZE_THRESHOLD)
                 .withOperator("Operator1", transformation)
                 .write(savepointPath1);
@@ -175,8 +163,8 @@ public class SavepointDeepCopyTest extends AbstractTestBase {
         File savepointUrl2 = createAndRegisterTempFile(new AbstractID().toHexString());
         String savepointPath2 = savepointUrl2.getPath();
 
-        ExistingSavepoint savepoint2 =
-                Savepoint.load(env, savepointPath1, backend)
+        SavepointWriter savepoint2 =
+                SavepointWriter.fromExistingSavepoint(savepointPath1, backend)
                         .withConfiguration(FS_SMALL_FILE_THRESHOLD, FILE_STATE_SIZE_THRESHOLD);
 
         savepoint2.withOperator("Operator2", transformation).write(savepointPath2);
@@ -196,13 +184,16 @@ public class SavepointDeepCopyTest extends AbstractTestBase {
                 stateFiles1,
                 everyItem(isIn(stateFiles2)));
 
-        // Try to load savepoint2 and read the state of "Operator1" (which has not been
+        // Try to fromExistingSavepoint savepoint2 and read the state of "Operator1" (which has not
+        // been
         // touched/changed when savepoint2
         // was created) and make sure the number of keys remain same
         long actuallyKeyNum =
-                Savepoint.load(env, savepointPath2, backend)
-                        .readKeyedState("Operator1", new ReadFunction())
-                        .count();
+                JobResultRetriever.collect(
+                                SavepointReader.read(env, savepointPath2, backend)
+                                        .readKeyedState("Operator1", new ReadFunction()))
+                        .size();
+
         long expectedKeyNum = Arrays.stream(TEXT.split(" ")).distinct().count();
         Assert.assertEquals(
                 "Unexpected number of keys in the state of Operator1",
