@@ -19,6 +19,7 @@
 package org.apache.flink.metrics.prometheus;
 
 import org.apache.flink.annotation.PublicEvolving;
+import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.metrics.Metric;
 import org.apache.flink.metrics.reporter.InstantiateViaFactory;
 import org.apache.flink.metrics.reporter.MetricReporter;
@@ -29,6 +30,7 @@ import io.prometheus.client.CollectorRegistry;
 import io.prometheus.client.exporter.PushGateway;
 
 import java.io.IOException;
+import java.net.URL;
 import java.util.Map;
 
 /**
@@ -44,14 +46,15 @@ public class PrometheusPushGatewayReporter extends AbstractPrometheusReporter im
     private final String jobName;
     private final Map<String, String> groupingKey;
     private final boolean deleteOnShutdown;
+    @VisibleForTesting final URL hostUrl;
 
     PrometheusPushGatewayReporter(
-            String host,
-            int port,
+            URL hostUrl,
             String jobName,
             Map<String, String> groupingKey,
             final boolean deleteOnShutdown) {
-        this.pushGateway = new PushGateway(host + ':' + port);
+        this.hostUrl = hostUrl;
+        this.pushGateway = new PushGateway(hostUrl);
         this.jobName = Preconditions.checkNotNull(jobName);
         this.groupingKey = Preconditions.checkNotNull(groupingKey);
         this.deleteOnShutdown = deleteOnShutdown;
