@@ -274,7 +274,7 @@ function install_tox() {
     # tox 3.14.0 depends on both 0.19 and 0.23 of importlib_metadata at the same time and
     # conda will try to install both these two versions and it will cause problems occasionally.
     # Using pip as the package manager could avoid this problem.
-    $CURRENT_DIR/install_command.sh -q virtualenv==16.0.0 tox==3.14.0 2>&1 >/dev/null
+    $CURRENT_DIR/install_command.sh -q virtualenv==20.10.0 tox==3.14.0 2>&1 >/dev/null
     if [ $? -ne 0 ]; then
         echo "pip install tox failed \
         please try to exec the script again.\
@@ -755,10 +755,6 @@ SUPPORTED_INSTALLATION_COMPONENTS=()
 get_all_supported_install_components
 
 INSTALLATION_COMPONENTS=()
-
-# whether remove the installed python environment.
-CLEAN_UP_FLAG=0
-
 # parse_opts
 USAGE="
 usage: $0 [options]
@@ -785,9 +781,8 @@ Examples:
   ./lint-python                 =>  exec all checks.
   ./lint-python -f              =>  reinstall environment with all components and exec all checks.
   ./lint-python -l              =>  list all checks supported.
-  ./lint-python -r              =>  clean up python environment.
 "
-while getopts "hfs:i:e:lr" arg; do
+while getopts "hfs:i:e:l" arg; do
     case "$arg" in
         h)
             printf "%s\\n" "$USAGE"
@@ -812,10 +807,6 @@ while getopts "hfs:i:e:lr" arg; do
             done
             exit 2
             ;;
-        r)
-            printf "clean up python environment:\n"
-            CLEAN_UP_FLAG=1
-            ;;
         ?)
             printf "ERROR: did not recognize option '%s', please try -h\\n" "$1"
             exit 1
@@ -825,15 +816,6 @@ done
 
 # decides whether to skip check stage
 skip_checks=0
-
-if [[ ${CLEAN_UP_FLAG} -eq 1 ]]; then
-    printf "clean up python environment"
-    rm -rf ${CONDA_HOME}
-    rm -rf ${STAGE_FILE}
-    rm -rf ${FLINK_PYTHON_DIR}/.tox
-    skip_checks=1
-fi
-
 if [ ! -z "$INSTALLATION_COMPONENTS" ]; then
     parse_component_args
     skip_checks=1
@@ -852,9 +834,7 @@ else
 fi
 
 # install environment
-if [[ ${CLEAN_UP_FLAG} -eq 0 ]]; then
-    install_environment
-fi
+install_environment
 
 pushd "$FLINK_PYTHON_DIR" &> /dev/null
 # exec all selected checks
