@@ -632,15 +632,13 @@ class DataStreamTests(object):
                 self.map_state = runtime_context.get_map_state(map_state_descriptor)
 
             def process_element(self, value, ctx):
-                import time
-                time.sleep(1)
                 current_value = self.value_state.value()
                 self.value_state.update(value[0])
                 current_list = [_ for _ in self.list_state.get()]
                 self.list_state.add(value[0])
-                map_entries_string = []
-                for k, v in self.map_state.items():
-                    map_entries_string.append(str(k) + ': ' + str(v))
+                map_entries = {k: v for k, v in self.map_state.items()}
+                keys = sorted(map_entries.keys())
+                map_entries_string = [str(k) + ': ' + str(map_entries[k]) for k in keys]
                 map_entries_string = '{' + ', '.join(map_entries_string) + '}'
                 self.map_state.put(value[0], value[1])
                 current_key = ctx.get_current_key()
@@ -814,14 +812,6 @@ class DataStreamTests(object):
         expected = ['(hi,3)']
         self.assert_equals_sorted(expected, results)
 
-    def test_java_list_deserialization(self):
-        row_type_info = Types.ROW_NAMED(['list'], [Types.LIST(Types.INT())])
-        ds = self.env.from_collection([Row(list=[1, 2, 3])], type_info=row_type_info)
-        ds.map(lambda e: str(e), Types.STRING()).add_sink(self.test_sink)
-        self.env.execute('test_java_list_deserialization')
-        expected = ['Row(list=[1, 2, 3])']
-        self.assert_equals(self.test_sink.get_results(), expected)
-
 
 class StreamingModeDataStreamTests(DataStreamTests, PyFlinkStreamingTestCase):
     def test_data_stream_name(self):
