@@ -17,6 +17,7 @@
 ################################################################################
 import collections
 import datetime
+import unittest
 from decimal import Decimal
 
 import pandas as pd
@@ -246,10 +247,10 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
         self.t_env.create_temporary_function("my_sum", SumAggregateFunction())
         # trigger the finish bundle more frequently to ensure testing the communication
         # between RemoteKeyedStateBackend and the StateGrpcService.
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.fn-execution.bundle.size", "2")
         # trigger the cache eviction in a bundle.
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.state.cache-size", "1")
         t = self.t_env.from_elements([(1, 'Hi', 'Hello'),
                                       (3, 'Hi', 'hi'),
@@ -262,7 +263,7 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
                            pd.DataFrame([[3, 12, 12, 12.0]], columns=['a', 'b', 'c', 'd']))
 
     def test_mixed_with_built_in_functions_with_retract(self):
-        self.t_env.get_config().get_configuration().set_string("parallelism.default", "1")
+        self.t_env.get_config().set("parallelism.default", "1")
         self.t_env.create_temporary_system_function(
             "concat",
             ConcatAggregateFunction())
@@ -310,7 +311,7 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
         self.assertEqual(result[len(result) - 1], expected)
 
     def test_mixed_with_built_in_functions_without_retract(self):
-        self.t_env.get_config().get_configuration().set_string("parallelism.default", "1")
+        self.t_env.get_config().set("parallelism.default", "1")
         self.t_env.create_temporary_system_function(
             "concat",
             ConcatAggregateFunction())
@@ -356,10 +357,10 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
 
     def test_list_view(self):
         my_concat = udaf(ListViewConcatAggregateFunction())
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.fn-execution.bundle.size", "2")
         # trigger the cache eviction in a bundle.
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.state.cache-size", "2")
         t = self.t_env.from_elements([(1, 'Hi', 'Hello'),
                                       (3, 'Hi', 'hi'),
@@ -379,14 +380,14 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
     def test_map_view(self):
         my_count = udaf(CountDistinctAggregateFunction())
         self.t_env.get_config().set_idle_state_retention(datetime.timedelta(days=1))
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.fn-execution.bundle.size", "2")
         # trigger the cache eviction in a bundle.
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.state.cache-size", "1")
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.map-state.read-cache-size", "1")
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.map-state.write-cache-size", "1")
         t = self.t_env.from_elements(
             [(1, 'Hi_', 'hi'),
@@ -413,10 +414,10 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
     def test_data_view_clear(self):
         my_count = udaf(CountDistinctAggregateFunction())
         self.t_env.get_config().set_idle_state_retention(datetime.timedelta(days=1))
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.fn-execution.bundle.size", "2")
         # trigger the cache eviction in a bundle.
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.state.cache-size", "1")
         t = self.t_env.from_elements(
             [(2, 'hello', 'hello'),
@@ -430,16 +431,16 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
     def test_map_view_iterate(self):
         test_iterate = udaf(TestIterateAggregateFunction())
         self.t_env.get_config().set_idle_state_retention(datetime.timedelta(days=1))
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.fn-execution.bundle.size", "2")
         # trigger the cache eviction in a bundle.
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.state.cache-size", "2")
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.map-state.read-cache-size", "2")
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.map-state.write-cache-size", "2")
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.map-state.iterate-response-batch-size", "2")
         t = self.t_env.from_elements(
             [(1, 'Hi_', 'hi'),
@@ -507,12 +508,12 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
 
     def test_clean_state(self):
         self.t_env.register_function("my_count", CountAggregateFunction())
-        self.t_env.get_config().get_configuration().set_string("parallelism.default", "1")
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set("parallelism.default", "1")
+        self.t_env.get_config().set(
             "python.fn-execution.bundle.size", "1")
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "python.state.cache-size", "0")
-        self.t_env.get_config().get_configuration().set_string(
+        self.t_env.get_config().set(
             "table.exec.state.ttl", "2ms")
 
         self.t_env.execute_sql("""
@@ -586,7 +587,7 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
                             "+I[1, 2018-03-11 04:00:00.0, 2018-03-11 05:00:00.0, 1, 1]"])
 
     def test_tumbling_group_window_over_count(self):
-        self.t_env.get_config().get_configuration().set_string("parallelism.default", "1")
+        self.t_env.get_config().set("parallelism.default", "1")
         # create source file path
         tmp_dir = self.tempdir
         data = [
@@ -700,7 +701,7 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
                             "+I[1, 2018-03-11 04:00:00.0, 2018-03-11 05:00:00.0, 8]"])
 
     def test_sliding_group_window_over_count(self):
-        self.t_env.get_config().get_configuration().set_string("parallelism.default", "1")
+        self.t_env.get_config().set("parallelism.default", "1")
         # create source file path
         tmp_dir = self.tempdir
         data = [
@@ -808,6 +809,7 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
                             "+I[1, 2018-03-11 03:10:00.0, 2018-03-11 04:10:00.0, 2]",
                             "+I[1, 2018-03-11 04:20:00.0, 2018-03-11 04:50:00.0, 1]"])
 
+    @unittest.skip("Python UDFs are currently unsupported in JSON plan")
     def test_execute_group_aggregate_from_json_plan(self):
         # create source file path
         tmp_dir = self.tempdir
@@ -840,12 +842,13 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
 
         self.t_env.create_temporary_function("my_sum", SumAggregateFunction())
 
-        json_plan = self.t_env._j_tenv.getJsonPlan("INSERT INTO sink_table "
-                                                   "SELECT a, my_sum(b) FROM source_table "
-                                                   "GROUP BY a")
+        json_plan = self.t_env._j_tenv.compilePlanSql("INSERT INTO sink_table "
+                                                      "SELECT a, my_sum(b) FROM source_table "
+                                                      "GROUP BY a")
         from py4j.java_gateway import get_method
-        get_method(self.t_env._j_tenv.executeJsonPlan(json_plan), "await")()
+        get_method(self.t_env._j_tenv.executePlan(json_plan), "await")()
 
+    @unittest.skip("Python UDFs are currently unsupported in JSON plan")
     def test_execute_group_window_aggregate_from_json_plan(self):
         # create source file path
         tmp_dir = self.tempdir
@@ -893,16 +896,18 @@ class StreamTableAggregateTests(PyFlinkStreamTableTestCase):
 
         self.t_env.create_temporary_function("my_count", CountAggregateFunction())
 
-        json_plan = self.t_env._j_tenv.getJsonPlan("INSERT INTO sink_table "
-                                                   "SELECT a, "
-                                                   "SESSION_START(rowtime, INTERVAL '30' MINUTE), "
-                                                   "SESSION_END(rowtime, INTERVAL '30' MINUTE), "
-                                                   "my_count(c) "
-                                                   "FROM source_table "
-                                                   "GROUP BY "
-                                                   "a, b, SESSION(rowtime, INTERVAL '30' MINUTE)")
+        json_plan = self.t_env._j_tenv.compilePlanSql("INSERT INTO sink_table "
+                                                      "SELECT a, "
+                                                      "SESSION_START("
+                                                      "rowtime, INTERVAL '30' MINUTE), "
+                                                      "SESSION_END(rowtime, INTERVAL '30' MINUTE), "
+                                                      "my_count(c) "
+                                                      "FROM source_table "
+                                                      "GROUP BY "
+                                                      "a, b, "
+                                                      "SESSION(rowtime, INTERVAL '30' MINUTE)")
         from py4j.java_gateway import get_method
-        get_method(self.t_env._j_tenv.executeJsonPlan(json_plan), "await")()
+        get_method(self.t_env._j_tenv.executePlan(json_plan), "await")()
 
         import glob
         lines = [line.strip() for file in glob.glob(sink_path + '/*') for line in open(file, 'r')]
