@@ -17,8 +17,10 @@
 ################################################################################
 import datetime
 import os
+import sys
 import unittest
 
+import pytest
 import pytz
 
 from pyflink.table import DataTypes, expressions as expr
@@ -32,7 +34,7 @@ class UserDefinedFunctionTests(object):
 
     def test_scalar_function(self):
         # test metric disabled.
-        self.t_env.get_config().get_configuration().set_string('python.metric.enabled', 'false')
+        self.t_env.get_config().set('python.metric.enabled', 'false')
         # test lambda function
         add_one = udf(lambda i: i + 1, result_type=DataTypes.BIGINT())
 
@@ -52,8 +54,9 @@ class UserDefinedFunctionTests(object):
 
         # check memory limit is set
         @udf(result_type=DataTypes.BIGINT())
-        def check_memory_limit():
-            assert os.environ['_PYTHON_WORKER_MEMORY_LIMIT'] is not None
+        def check_memory_limit(exec_mode):
+            if exec_mode == "process":
+                assert os.environ['_PYTHON_WORKER_MEMORY_LIMIT'] is not None
             return 1
 
         table_sink = source_sink_utils.TestAppendSink(
@@ -62,10 +65,12 @@ class UserDefinedFunctionTests(object):
              DataTypes.BIGINT(), DataTypes.BIGINT(), DataTypes.BIGINT()])
         self.t_env.register_table_sink("Results", table_sink)
 
+        execution_mode = self.t_env.get_config().get("python.execution-mode", "process")
+
         t = self.t_env.from_elements([(1, 2, 3), (2, 5, 6), (3, 1, 9)], ['a', 'b', 'c'])
         t.where(add_one(t.b) <= 3).select(
             add_one(t.a), subtract_one(t.b), add(t.a, t.c), add_one_callable(t.a),
-            add_one_partial(t.a), check_memory_limit(), t.a) \
+            add_one_partial(t.a), check_memory_limit(execution_mode), t.a) \
             .execute_insert("Results").wait()
         actual = source_sink_utils.results()
         self.assert_equals(actual, ["+I[2, 1, 4, 2, 2, 1, 1]", "+I[4, 0, 12, 4, 4, 1, 3]"])
@@ -197,8 +202,7 @@ class UserDefinedFunctionTests(object):
                              "cast ('2014-09-13' as DATE),"
                              "cast ('12:00:00' as TIME),"
                              "cast ('1999-9-10 05:20:10' as TIMESTAMP))"
-                             " from test_table").insert_into("Results")
-        self.t_env.execute("test")
+                             " from test_table").execute_insert("Results").wait()
         actual = source_sink_utils.results()
         self.assert_equals(actual, ["+I[3, 8]", "+I[3, 9]", "+I[3, 10]"])
 
@@ -216,8 +220,14 @@ class UserDefinedFunctionTests(object):
         self.assert_equals(actual, ["+I[2]", "+I[6]", "+I[3]"])
 
     def test_open(self):
-        self.t_env.get_config().get_configuration().set_string('python.metric.enabled', 'true')
-        subtract = udf(Subtract(), result_type=DataTypes.BIGINT())
+        self.t_env.get_config().set('python.metric.enabled', 'true')
+        execution_mode = self.t_env.get_config().get("python.execution-mode", None)
+
+        if execution_mode == "process":
+            subtract = udf(SubtractWithMetrics(), result_type=DataTypes.BIGINT())
+        else:
+            subtract = udf(Subtract(), result_type=DataTypes.BIGINT())
+
         table_sink = source_sink_utils.TestAppendSink(
             ['a', 'b'], [DataTypes.BIGINT(), DataTypes.BIGINT()])
         self.t_env.register_table_sink("Results", table_sink)
@@ -729,6 +739,7 @@ class PyFlinkStreamUserDefinedFunctionTests(UserDefinedFunctionTests,
         actual = source_sink_utils.results()
         self.assert_equals(actual, ["+I[1970-01-01T00:00:00.123Z]"])
 
+    @unittest.skip("Python UDFs are currently unsupported in JSON plan")
     def test_execute_from_json_plan(self):
         # create source file path
         tmp_dir = self.tempdir
@@ -765,12 +776,12 @@ class PyFlinkStreamUserDefinedFunctionTests(UserDefinedFunctionTests,
         add_one = udf(lambda i: i + 1, result_type=DataTypes.BIGINT())
         self.t_env.create_temporary_system_function("add_one", add_one)
 
-        json_plan = self.t_env._j_tenv.getJsonPlan("INSERT INTO sink_table SELECT "
-                                                   "a, "
-                                                   "add_one(b) "
-                                                   "FROM source_table")
+        json_plan = self.t_env._j_tenv.compilePlanSql("INSERT INTO sink_table SELECT "
+                                                      "a, "
+                                                      "add_one(b) "
+                                                      "FROM source_table")
         from py4j.java_gateway import get_method
-        get_method(self.t_env._j_tenv.executeJsonPlan(json_plan), "await")()
+        get_method(self.t_env._j_tenv.executePlan(json_plan), "await")()
 
         import glob
         lines = [line.strip() for file in glob.glob(sink_path + '/*') for line in open(file, 'r')]
@@ -783,6 +794,12 @@ class PyFlinkBatchUserDefinedFunctionTests(UserDefinedFunctionTests,
     pass
 
 
+@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7")
+class PyFlinkEmbeddedThreadTests(UserDefinedFunctionTests, PyFlinkBatchTableTestCase):
+    def setUp(self):
+        super(PyFlinkEmbeddedThreadTests, self).setUp()
+        self.t_env.get_config().set("python.execution-mode", "thread")
+
 # test specify the input_types
 @udf(input_types=[DataTypes.BIGINT(), DataTypes.BIGINT()], result_type=DataTypes.BIGINT())
 def add(i, j):
@@ -795,7 +812,7 @@ class SubtractOne(ScalarFunction):
         return i - 1
 
 
-class Subtract(ScalarFunction, unittest.TestCase):
+class SubtractWithMetrics(ScalarFunction, unittest.TestCase):
 
     def open(self, function_context):
         self.subtracted_value = 1
@@ -810,6 +827,18 @@ class Subtract(ScalarFunction, unittest.TestCase):
         return i - self.subtracted_value
 
 
+class Subtract(ScalarFunction, unittest.TestCase):
+
+    def open(self, function_context):
+        self.subtracted_value = 1
+        self.counter_sum = 0
+
+    def eval(self, i):
+        # counter
+        self.counter_sum += i
+        return i - self.subtracted_value
+
+
 class CallablePlus(object):
 
     def __call__(self, col):
