@@ -50,8 +50,7 @@ public final class TimerRegistration {
             InternalTimerService internalTimerService,
             KeyContext keyContext,
             TypeSerializer namespaceSerializer,
-            TypeSerializer<Row> timerDataSerializer)
-            throws Exception {
+            TypeSerializer<Row> timerDataSerializer) {
         this.keyedStateBackend = keyedStateBackend;
         this.internalTimerService = internalTimerService;
         this.keyContext = keyContext;
