@@ -31,7 +31,7 @@ import org.apache.flink.table.data.GenericRowData;
 import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.data.StringData;
 import org.apache.flink.table.functions.python.PythonAggregateFunctionInfo;
-import org.apache.flink.table.planner.typeutils.DataViewUtils;
+import org.apache.flink.table.runtime.dataview.DataViewSpec;
 import org.apache.flink.table.runtime.operators.python.scalar.PythonScalarFunctionOperatorTestBase;
 import org.apache.flink.table.runtime.utils.PassThroughStreamTableAggregatePythonFunctionRunner;
 import org.apache.flink.table.runtime.utils.PythonTestUtils;
@@ -40,7 +40,6 @@ import org.apache.flink.table.types.logical.RowType;
 import org.junit.Test;
 
 import java.io.IOException;
-import java.util.HashMap;
 import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.function.Function;
 
@@ -258,7 +257,7 @@ public class PythonStreamGroupTableAggregateOperatorTest
                     inputType,
                     outputType,
                     aggregateFunctions,
-                    new DataViewUtils.DataViewSpec[0][0],
+                    new DataViewSpec[0][0],
                     grouping,
                     indexOfCountStar,
                     generateUpdateBefore,
@@ -270,12 +269,11 @@ public class PythonStreamGroupTableAggregateOperatorTest
         public PythonFunctionRunner createPythonFunctionRunner() {
             return new PassThroughStreamTableAggregatePythonFunctionRunner(
                     getRuntimeContext().getTaskName(),
-                    PythonTestUtils.createTestEnvironmentManager(),
+                    PythonTestUtils.createTestProcessEnvironmentManager(),
                     userDefinedFunctionInputType,
                     outputType,
                     STREAM_GROUP_TABLE_AGGREGATE_URN,
                     getUserDefinedFunctionsProto(),
-                    new HashMap<>(),
                     PythonTestUtils.createMockFlinkMetricContainer(),
                     getKeyedStateBackend(),
                     getKeySerializer(),
