@@ -25,8 +25,13 @@ import org.apache.flink.python.PythonOptions;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
 import org.apache.flink.streaming.util.OneInputStreamOperatorTestHarness;
 import org.apache.flink.table.api.DataTypes;
+import org.apache.flink.table.api.TableConfig;
+import org.apache.flink.table.connector.Projection;
 import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.functions.python.PythonFunctionInfo;
+import org.apache.flink.table.planner.codegen.CodeGeneratorContext;
+import org.apache.flink.table.planner.codegen.ProjectionCodeGenerator;
+import org.apache.flink.table.runtime.generated.GeneratedProjection;
 import org.apache.flink.table.runtime.operators.python.aggregate.arrow.AbstractArrowPythonAggregateFunctionOperator;
 import org.apache.flink.table.runtime.utils.PassThroughPythonAggregateFunctionRunner;
 import org.apache.flink.table.runtime.utils.PythonTestUtils;
@@ -38,7 +43,6 @@ import org.apache.flink.table.types.logical.VarCharType;
 import org.junit.Test;
 
 import java.util.Arrays;
-import java.util.HashMap;
 import java.util.List;
 import java.util.concurrent.ConcurrentLinkedQueue;
 
@@ -216,24 +220,47 @@ public class BatchArrowPythonOverWindowAggregateFunctionOperatorTest
     public AbstractArrowPythonAggregateFunctionOperator getTestOperator(
             Configuration config,
             PythonFunctionInfo[] pandasAggregateFunctions,
-            RowType inputType,
-            RowType outputType,
+            RowType inputRowType,
+            RowType outputRowType,
             int[] groupingSet,
             int[] udafInputOffsets) {
+        RowType udfInputType = (RowType) Projection.of(udafInputOffsets).project(inputRowType);
+        RowType udfOutputType =
+                (RowType)
+                        Projection.range(
+                                        inputRowType.getFieldCount(), outputRowType.getFieldCount())
+                                .project(outputRowType);
+
         return new PassThroughBatchArrowPythonOverWindowAggregateFunctionOperator(
                 config,
                 pandasAggregateFunctions,
-                inputType,
-                outputType,
+                inputRowType,
+                udfInputType,
+                udfOutputType,
                 new long[] {0L, Long.MIN_VALUE},
                 new long[] {0L, 2L},
                 new boolean[] {true, false},
                 new int[] {0},
-                groupingSet,
-                groupingSet,
-                udafInputOffsets,
                 3,
-                true);
+                true,
+                ProjectionCodeGenerator.generateProjection(
+                        CodeGeneratorContext.apply(new TableConfig()),
+                        "UdafInputProjection",
+                        inputRowType,
+                        udfInputType,
+                        udafInputOffsets),
+                ProjectionCodeGenerator.generateProjection(
+                        CodeGeneratorContext.apply(new TableConfig()),
+                        "GroupKey",
+                        inputRowType,
+                        (RowType) Projection.of(groupingSet).project(inputRowType),
+                        groupingSet),
+                ProjectionCodeGenerator.generateProjection(
+                        CodeGeneratorContext.apply(new TableConfig()),
+                        "GroupSet",
+                        inputRowType,
+                        (RowType) Projection.of(groupingSet).project(inputRowType),
+                        groupingSet));
     }
 
     private static class PassThroughBatchArrowPythonOverWindowAggregateFunctionOperator
@@ -243,42 +270,43 @@ public class BatchArrowPythonOverWindowAggregateFunctionOperatorTest
                 Configuration config,
                 PythonFunctionInfo[] pandasAggFunctions,
                 RowType inputType,
-                RowType outputType,
+                RowType udfInputType,
+                RowType udfOutputType,
                 long[] lowerBoundary,
                 long[] upperBoundary,
                 boolean[] isRangeWindow,
                 int[] aggWindowIndex,
-                int[] groupKey,
-                int[] groupingSet,
-                int[] udafInputOffsets,
                 int inputTimeFieldIndex,
-                boolean asc) {
+                boolean asc,
+                GeneratedProjection inputGeneratedProjection,
+                GeneratedProjection groupKeyGeneratedProjection,
+                GeneratedProjection groupSetGeneratedProjection) {
             super(
                     config,
                     pandasAggFunctions,
                     inputType,
-                    outputType,
+                    udfInputType,
+                    udfOutputType,
                     lowerBoundary,
                     upperBoundary,
                     isRangeWindow,
                     aggWindowIndex,
-                    groupKey,
-                    groupingSet,
-                    udafInputOffsets,
                     inputTimeFieldIndex,
-                    asc);
+                    asc,
+                    inputGeneratedProjection,
+                    groupKeyGeneratedProjection,
+                    groupSetGeneratedProjection);
         }
 
         @Override
         public PythonFunctionRunner createPythonFunctionRunner() {
             return new PassThroughPythonAggregateFunctionRunner(
                     getRuntimeContext().getTaskName(),
-                    PythonTestUtils.createTestEnvironmentManager(),
-                    userDefinedFunctionInputType,
-                    userDefinedFunctionOutputType,
+                    PythonTestUtils.createTestProcessEnvironmentManager(),
+                    udfInputType,
+                    udfOutputType,
                     getFunctionUrn(),
                     getUserDefinedFunctionsProto(),
-                    new HashMap<>(),
                     PythonTestUtils.createMockFlinkMetricContainer(),
                     true);
         }
