@@ -60,9 +60,7 @@ import java.time.Duration;
 import java.util.Collection;
 import java.util.List;
 import java.util.Map;
-import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.Future;
 import java.util.stream.Collectors;
 
 import static org.hamcrest.CoreMatchers.containsString;
@@ -445,8 +443,6 @@ public class WebFrontendITCase extends TestLogger {
 
         private volatile boolean isRunning = true;
 
-        private final CompletableFuture<Void> terminationFuture = new CompletableFuture<>();
-
         public BlockingInvokable(Environment environment) {
             super(environment);
         }
@@ -454,19 +450,14 @@ public class WebFrontendITCase extends TestLogger {
         @Override
         public void invoke() throws Exception {
             latch.countDown();
-            try {
-                while (isRunning) {
-                    Thread.sleep(100);
-                }
-            } finally {
-                terminationFuture.complete(null);
+            while (isRunning) {
+                Thread.sleep(100);
             }
         }
 
         @Override
-        public Future<Void> cancel() {
+        public void cancel() {
             this.isRunning = false;
-            return terminationFuture;
         }
 
         public static void reset() {
