@@ -85,8 +85,7 @@ public class JarDeleteHandlerTest extends TestLogger {
     public void testDeleteJarById() throws Exception {
         assertThat(Files.exists(jarDir.resolve(TEST_JAR_NAME)), equalTo(true));
 
-        final HandlerRequest<EmptyRequestBody, JarDeleteMessageParameters> request =
-                createRequest(TEST_JAR_NAME);
+        final HandlerRequest<EmptyRequestBody> request = createRequest(TEST_JAR_NAME);
         jarDeleteHandler.handleRequest(request, restfulGateway).get();
 
         assertThat(Files.exists(jarDir.resolve(TEST_JAR_NAME)), equalTo(false));
@@ -94,8 +93,7 @@ public class JarDeleteHandlerTest extends TestLogger {
 
     @Test
     public void testDeleteUnknownJar() throws Exception {
-        final HandlerRequest<EmptyRequestBody, JarDeleteMessageParameters> request =
-                createRequest("doesnotexist.jar");
+        final HandlerRequest<EmptyRequestBody> request = createRequest("doesnotexist.jar");
         try {
             jarDeleteHandler.handleRequest(request, restfulGateway).get();
         } catch (final ExecutionException e) {
@@ -118,8 +116,7 @@ public class JarDeleteHandlerTest extends TestLogger {
     public void testFailedDelete() throws Exception {
         makeJarDirReadOnly();
 
-        final HandlerRequest<EmptyRequestBody, JarDeleteMessageParameters> request =
-                createRequest(TEST_JAR_NAME);
+        final HandlerRequest<EmptyRequestBody> request = createRequest(TEST_JAR_NAME);
         try {
             jarDeleteHandler.handleRequest(request, restfulGateway).get();
         } catch (final ExecutionException e) {
@@ -134,13 +131,14 @@ public class JarDeleteHandlerTest extends TestLogger {
         }
     }
 
-    private static HandlerRequest<EmptyRequestBody, JarDeleteMessageParameters> createRequest(
-            final String jarFileName) throws HandlerRequestException {
-        return new HandlerRequest<>(
+    private static HandlerRequest<EmptyRequestBody> createRequest(final String jarFileName)
+            throws HandlerRequestException {
+        return HandlerRequest.resolveParametersAndCreate(
                 EmptyRequestBody.getInstance(),
                 new JarDeleteMessageParameters(),
                 Collections.singletonMap(JarIdPathParameter.KEY, jarFileName),
-                Collections.emptyMap());
+                Collections.emptyMap(),
+                Collections.emptyList());
     }
 
     private void makeJarDirReadOnly() {
