@@ -25,10 +25,10 @@ import org.apache.flink.runtime.rest.handler.HandlerRequest;
 import org.apache.flink.runtime.rest.messages.EmptyMessageParameters;
 import org.apache.flink.runtime.rest.messages.EmptyRequestBody;
 import org.apache.flink.runtime.rest.messages.JobPlanInfo;
-import org.apache.flink.runtime.testutils.TestingUtils;
 import org.apache.flink.runtime.webmonitor.RestfulGateway;
 import org.apache.flink.runtime.webmonitor.TestingDispatcherGateway;
 import org.apache.flink.runtime.webmonitor.retriever.GatewayRetriever;
+import org.apache.flink.testutils.TestingUtils;
 
 import java.nio.file.Path;
 import java.util.Collections;
@@ -105,12 +105,11 @@ public class JarHandlers {
 
     public static String uploadJar(
             JarUploadHandler handler, Path jar, RestfulGateway restfulGateway) throws Exception {
-        HandlerRequest<EmptyRequestBody, EmptyMessageParameters> uploadRequest =
-                new HandlerRequest<>(
+
+        HandlerRequest<EmptyRequestBody> uploadRequest =
+                HandlerRequest.create(
                         EmptyRequestBody.getInstance(),
                         EmptyMessageParameters.getInstance(),
-                        Collections.emptyMap(),
-                        Collections.emptyMap(),
                         Collections.singletonList(jar.toFile()));
         final JarUploadResponseBody uploadResponse =
                 handler.handleRequest(uploadRequest, restfulGateway).get();
@@ -119,8 +118,8 @@ public class JarHandlers {
 
     public static JarListInfo listJars(JarListHandler handler, RestfulGateway restfulGateway)
             throws Exception {
-        HandlerRequest<EmptyRequestBody, EmptyMessageParameters> listRequest =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> listRequest =
+                HandlerRequest.create(
                         EmptyRequestBody.getInstance(), EmptyMessageParameters.getInstance());
         return handler.handleRequest(listRequest, restfulGateway).get();
     }
@@ -130,8 +129,8 @@ public class JarHandlers {
             throws Exception {
         JarPlanMessageParameters planParameters =
                 JarPlanGetHeaders.getInstance().getUnresolvedMessageParameters();
-        HandlerRequest<JarPlanRequestBody, JarPlanMessageParameters> planRequest =
-                new HandlerRequest<>(
+        HandlerRequest<JarPlanRequestBody> planRequest =
+                HandlerRequest.resolveParametersAndCreate(
                         new JarPlanRequestBody(),
                         planParameters,
                         Collections.singletonMap(
@@ -146,8 +145,8 @@ public class JarHandlers {
             throws Exception {
         final JarRunMessageParameters runParameters =
                 JarRunHeaders.getInstance().getUnresolvedMessageParameters();
-        HandlerRequest<JarRunRequestBody, JarRunMessageParameters> runRequest =
-                new HandlerRequest<>(
+        HandlerRequest<JarRunRequestBody> runRequest =
+                HandlerRequest.resolveParametersAndCreate(
                         new JarRunRequestBody(),
                         runParameters,
                         Collections.singletonMap(
@@ -162,8 +161,8 @@ public class JarHandlers {
             throws Exception {
         JarDeleteMessageParameters deleteParameters =
                 JarDeleteHeaders.getInstance().getUnresolvedMessageParameters();
-        HandlerRequest<EmptyRequestBody, JarDeleteMessageParameters> deleteRequest =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> deleteRequest =
+                HandlerRequest.resolveParametersAndCreate(
                         EmptyRequestBody.getInstance(),
                         deleteParameters,
                         Collections.singletonMap(
