@@ -16,14 +16,14 @@
  * limitations under the License.
  */
 
-export interface JobBackpressureInterface {
+export interface JobBackpressure {
   status: string;
   'backpressure-level': string;
   'end-timestamp': number;
-  subtasks: JobBackpressureSubtaskInterface[];
+  subtasks: JobBackpressureSubtask[];
 }
 
-export interface JobBackpressureSubtaskInterface {
+export interface JobBackpressureSubtask {
   subtask: number;
   'backpressure-level': string;
   ratio: number;
