@@ -18,21 +18,18 @@
 
 import { HttpClient } from '@angular/common/http';
 import { Injectable } from '@angular/core';
+import { Observable } from 'rxjs';
 import { map } from 'rxjs/operators';
+
 import { BASE_URL, LONG_MIN_VALUE } from 'config';
 
 @Injectable({
   providedIn: 'root'
 })
 export class MetricsService {
-  constructor(private httpClient: HttpClient) {}
+  constructor(private readonly httpClient: HttpClient) {}
 
-  /**
-   * Get available metric list
-   * @param jobId
-   * @param vertexId
-   */
-  getAllAvailableMetrics(jobId: string, vertexId: string) {
+  public getAllAvailableMetrics(jobId: string, vertexId: string): Observable<Array<{ id: string; value: string }>> {
     return this.httpClient
       .get<Array<{ id: string; value: string }>>(`${BASE_URL}/jobs/${jobId}/vertices/${vertexId}/metrics`)
       .pipe(
@@ -52,13 +49,11 @@ export class MetricsService {
       );
   }
 
-  /**
-   * Get metric data
-   * @param jobId
-   * @param vertexId
-   * @param listOfMetricName
-   */
-  getMetrics(jobId: string, vertexId: string, listOfMetricName: string[]) {
+  public getMetrics(
+    jobId: string,
+    vertexId: string,
+    listOfMetricName: string[]
+  ): Observable<{ timestamp: number; values: { [p: string]: number } }> {
     const metricName = listOfMetricName.join(',');
     return this.httpClient
       .get<Array<{ id: string; value: string }>>(
@@ -78,13 +73,13 @@ export class MetricsService {
       );
   }
 
-  /**
-   * Get aggregated metric data from all subtasks of the given vertexId
-   * @param jobId
-   * @param vertexId
-   * @param listOfMetricName
-   */
-  getAggregatedMetrics(jobId: string, vertexId: string, listOfMetricName: string[], aggregate: string = "max") {
+  /** Get aggregated metric data from all subtasks of the given vertexId. */
+  public getAggregatedMetrics(
+    jobId: string,
+    vertexId: string,
+    listOfMetricName: string[],
+    aggregate: string = 'max'
+  ): Observable<{ [p: string]: number }> {
     const metricName = listOfMetricName.join(',');
     return this.httpClient
       .get<Array<{ id: string; min: number; max: number; avg: number; sum: number }>>(
@@ -95,20 +90,20 @@ export class MetricsService {
           const result: { [id: string]: number } = {};
           arr.forEach(item => {
             switch (aggregate) {
-              case "min":
+              case 'min':
                 result[item.id] = +item.min;
                 break;
-              case "max":
+              case 'max':
                 result[item.id] = +item.max;
                 break;
-              case "avg":
+              case 'avg':
                 result[item.id] = +item.avg;
                 break;
-              case "sum":
+              case 'sum':
                 result[item.id] = +item.sum;
                 break;
               default:
-                throw new Error("Unsupported aggregate: " + aggregate);
+                throw new Error(`Unsupported aggregate: ${aggregate}`);
             }
           });
           return result;
@@ -116,20 +111,16 @@ export class MetricsService {
       );
   }
 
-  /**
-   * Gets the watermarks for a given vertex id.
-   * @param jobId
-   * @param vertexId
-   */
-  getWatermarks(jobId: string, vertexId: string) {
+  public getWatermarks(
+    jobId: string,
+    vertexId: string
+  ): Observable<{ lowWatermark: number; watermarks: { [p: string]: number } }> {
     return this.httpClient
-      .get<Array<{ id: string; value: string }>>(
-        `${BASE_URL}/jobs/${jobId}/vertices/${vertexId}/watermarks`
-      )
+      .get<Array<{ id: string; value: string }>>(`${BASE_URL}/jobs/${jobId}/vertices/${vertexId}/watermarks`)
       .pipe(
         map(arr => {
           let minValue = NaN;
-          let lowWatermark = NaN;
+          let lowWatermark: number;
           const watermarks: { [id: string]: number } = {};
           arr.forEach(item => {
             const value = parseInt(item.value, 10);
@@ -148,7 +139,7 @@ export class MetricsService {
             lowWatermark,
             watermarks
           };
-      })
-    );
+        })
+      );
   }
 }
