@@ -17,7 +17,8 @@
  */
 
 import { ChangeDetectionStrategy, ChangeDetectorRef, Component, Input } from '@angular/core';
-import { NodesItemCorrectInterface } from 'interfaces';
+
+import { NodesItemCorrect } from 'interfaces';
 
 @Component({
   selector: '[flink-node]',
@@ -46,37 +47,40 @@ export class NodeComponent {
 
   decodeHTML(value: string): string | null {
     const parser = new DOMParser();
-    const dom = parser.parseFromString('<!doctype html><body>' + value, 'text/html');
+    const dom = parser.parseFromString(`<!doctype html><body>${value}`, 'text/html');
     return dom.body.textContent;
   }
 
   @Input()
-  set node(value: NodesItemCorrectInterface) {
-    const description = this.decodeHTML(value.description);
+  set node(value: NodesItemCorrect) {
+    let description = this.decodeHTML(value.description);
+    if (value.detail) {
+      description = this.decodeHTML(value.detail.name);
+    }
     this.operator = this.decodeHTML(value.operator);
     this.operatorStrategy = this.decodeHTML(value.operator_strategy);
     this.parallelism = value.parallelism;
     this.lowWatermark = value.lowWatermark;
     if (this.isValid(value.backPressuredPercentage)) {
-        this.backPressuredPercentage = value.backPressuredPercentage
+      this.backPressuredPercentage = value.backPressuredPercentage;
     }
     if (this.isValid(value.busyPercentage)) {
-        this.busyPercentage = value.busyPercentage;
+      this.busyPercentage = value.busyPercentage;
     }
     this.height = value.height || 0;
     this.id = value.id;
     if (description && description.length > 300) {
-      this.description = description.slice(0, 300) + '...';
+      this.description = `${description.slice(0, 300)}...`;
     } else {
       this.description = description;
     }
   }
 
-  isValid = (value?: number) => {
-    return value || value === 0 || value === NaN;
-  }
+  isValid = (value?: number): boolean => {
+    return !!value || value === 0 || value === NaN;
+  };
 
-  toRGBA = (d: string) => {
+  toRGBA = (d: string): number[] => {
     const l = d.length;
     const rgba = [];
     const hex = parseInt(d.slice(1), 16);
@@ -87,7 +91,7 @@ export class NodeComponent {
     return rgba;
   };
 
-  blend = (from: string, to: string, p = 0.5) => {
+  blend = (from: string, to: string, p = 0.5): string => {
     from = from.trim();
     to = to.trim();
     const b = p < 0;
@@ -95,56 +99,70 @@ export class NodeComponent {
     const f = this.toRGBA(from);
     const t = this.toRGBA(to);
     if (to[0] === 'r') {
-      return 'rgb' + (to[3] === 'a' ? 'a(' : '(') +
-        Math.round(((t[0] - f[0]) * p) + f[0]) + ',' +
-        Math.round(((t[1] - f[1]) * p) + f[1]) + ',' +
-        Math.round(((t[2] - f[2]) * p) + f[2]) + (
-          f[3] < 0 && t[3] < 0 ? '' : ',' + (
-            f[3] > -1 && t[3] > -1
-              ? Math.round((((t[3] - f[3]) * p) + f[3]) * 10000) / 10000
-              : t[3] < 0 ? f[3] : t[3]
-          )
-        ) + ')';
+      return `rgb${to[3] === 'a' ? 'a(' : '('}${Math.round((t[0] - f[0]) * p + f[0])},${Math.round(
+        (t[1] - f[1]) * p + f[1]
+      )},${Math.round((t[2] - f[2]) * p + f[2])}${
+        f[3] < 0 && t[3] < 0
+          ? ''
+          : `,${
+              f[3] > -1 && t[3] > -1 ? Math.round(((t[3] - f[3]) * p + f[3]) * 10000) / 10000 : t[3] < 0 ? f[3] : t[3]
+            }`
+      })`;
     }
 
-    return '#' + (0x100000000 + ((
-        f[3] > -1 && t[3] > -1
-          ? Math.round((((t[3] - f[3]) * p) + f[3]) * 255)
-          : t[3] > -1 ? Math.round(t[3] * 255) : f[3] > -1 ? Math.round(f[3] * 255) : 255
-      ) * 0x1000000) +
-      (Math.round(((t[0] - f[0]) * p) + f[0]) * 0x10000) +
-      (Math.round(((t[1] - f[1]) * p) + f[1]) * 0x100) +
-      Math.round(((t[2] - f[2]) * p) + f[2])
-    ).toString(16).slice(f[3] > -1 || t[3] > -1 ? 1 : 3);
+    return `#${(
+      0x100000000 +
+      (f[3] > -1 && t[3] > -1
+        ? Math.round(((t[3] - f[3]) * p + f[3]) * 255)
+        : t[3] > -1
+        ? Math.round(t[3] * 255)
+        : f[3] > -1
+        ? Math.round(f[3] * 255)
+        : 255) *
+        0x1000000 +
+      Math.round((t[0] - f[0]) * p + f[0]) * 0x10000 +
+      Math.round((t[1] - f[1]) * p + f[1]) * 0x100 +
+      Math.round((t[2] - f[2]) * p + f[2])
+    )
+      .toString(16)
+      .slice(f[3] > -1 || t[3] > -1 ? 1 : 3)}`;
   };
 
   constructor(protected cd: ChangeDetectorRef) {}
 
   /**
    * Update and check node component
+   *
    * @param node
    */
-  update(node: NodesItemCorrectInterface): void {
+  update(node: NodesItemCorrect): void {
     this.node = node;
     this.backgroundColor = this.backgroundDefaultColor;
     this.borderColor = this.borderDefaultColor;
     if (node.busyPercentage) {
-      this.backgroundColor = this.blend(this.backgroundColor, this.backgroundBusyColor, node.busyPercentage / 100.0 );
+      this.backgroundColor = this.blend(this.backgroundColor, this.backgroundBusyColor, node.busyPercentage / 100.0);
       this.borderColor = this.blend(this.borderColor, this.borderBusyColor, node.busyPercentage / 100.0);
     }
     if (node.backPressuredPercentage) {
-      this.backgroundColor = this.blend(this.backgroundColor, this.backgroundBackPressuredColor, node.backPressuredPercentage / 100.0);
-      this.borderColor = this.blend(this.borderColor, this.borderBackPressuredColor, node.backPressuredPercentage / 100.0);
+      this.backgroundColor = this.blend(
+        this.backgroundColor,
+        this.backgroundBackPressuredColor,
+        node.backPressuredPercentage / 100.0
+      );
+      this.borderColor = this.blend(
+        this.borderColor,
+        this.borderBackPressuredColor,
+        node.backPressuredPercentage / 100.0
+      );
     }
     this.cd.markForCheck();
   }
 
   prettyPrint(value: number | undefined): string {
     if (value === undefined || isNaN(value)) {
-      return "N/A"
-    }
-    else {
-      return value + "%";
+      return 'N/A';
+    } else {
+      return `${value}%`;
     }
   }
 }
