@@ -26,12 +26,14 @@ import org.apache.flink.configuration.Configuration;
 import org.apache.flink.core.fs.Path;
 import org.apache.flink.runtime.blob.PermanentBlobKey;
 import org.apache.flink.runtime.jobgraph.tasks.JobCheckpointingSettings;
-import org.apache.flink.runtime.jobmanager.scheduler.CoLocationGroupDesc;
+import org.apache.flink.runtime.jobmanager.scheduler.CoLocationGroup;
 import org.apache.flink.runtime.jobmanager.scheduler.SlotSharingGroup;
 import org.apache.flink.util.InstantiationUtil;
 import org.apache.flink.util.IterableUtils;
 import org.apache.flink.util.SerializedValue;
 
+import javax.annotation.Nullable;
+
 import java.io.IOException;
 import java.io.Serializable;
 import java.net.URISyntaxException;
@@ -80,8 +82,7 @@ public class JobGraph implements Serializable {
     /** Name of this job. */
     private final String jobName;
 
-    /** The mode in which the job is scheduled. */
-    private ScheduleMode scheduleMode = ScheduleMode.LAZY_FROM_SOURCES;
+    private JobType jobType = JobType.BATCH;
 
     /**
      * Whether approximate local recovery is enabled. This flag will be removed together with legacy
@@ -135,7 +136,7 @@ public class JobGraph implements Serializable {
      * @param jobId The id of the job. A random ID is generated, if {@code null} is passed.
      * @param jobName The name of the job.
      */
-    public JobGraph(JobID jobId, String jobName) {
+    public JobGraph(@Nullable JobID jobId, String jobName) {
         this.jobID = jobId == null ? new JobID() : jobId;
         this.jobName = jobName == null ? "(unnamed job)" : jobName;
 
@@ -147,29 +148,6 @@ public class JobGraph implements Serializable {
         }
     }
 
-    /**
-     * Constructs a new job graph with no name, a random job ID, the given {@link ExecutionConfig},
-     * and the given job vertices. The ExecutionConfig will be serialized and can't be modified
-     * afterwards.
-     *
-     * @param vertices The vertices to add to the graph.
-     */
-    public JobGraph(JobVertex... vertices) {
-        this(null, vertices);
-    }
-
-    /**
-     * Constructs a new job graph with the given name, the given {@link ExecutionConfig}, a random
-     * job ID, and the given job vertices. The ExecutionConfig will be serialized and can't be
-     * modified afterwards.
-     *
-     * @param jobName The name of the job.
-     * @param vertices The vertices to add to the graph.
-     */
-    public JobGraph(String jobName, JobVertex... vertices) {
-        this(null, jobName, vertices);
-    }
-
     /**
      * Constructs a new job graph with the given name, the given {@link ExecutionConfig}, the given
      * jobId or a random one if null supplied, and the given job vertices. The ExecutionConfig will
@@ -179,7 +157,7 @@ public class JobGraph implements Serializable {
      * @param jobName The name of the job.
      * @param vertices The vertices to add to the graph.
      */
-    public JobGraph(JobID jobId, String jobName, JobVertex... vertices) {
+    public JobGraph(@Nullable JobID jobId, String jobName, JobVertex... vertices) {
         this(jobId, jobName);
 
         for (JobVertex vertex : vertices) {
@@ -231,12 +209,12 @@ public class JobGraph implements Serializable {
         return serializedExecutionConfig;
     }
 
-    public void setScheduleMode(ScheduleMode scheduleMode) {
-        this.scheduleMode = scheduleMode;
+    public void setJobType(JobType type) {
+        this.jobType = type;
     }
 
-    public ScheduleMode getScheduleMode() {
-        return scheduleMode;
+    public JobType getJobType() {
+        return jobType;
     }
 
     public void enableApproximateLocalRecovery(boolean enabled) {
@@ -275,7 +253,14 @@ public class JobGraph implements Serializable {
      */
     public void setExecutionConfig(ExecutionConfig executionConfig) throws IOException {
         checkNotNull(executionConfig, "ExecutionConfig must not be null.");
-        this.serializedExecutionConfig = new SerializedValue<>(executionConfig);
+        setSerializedExecutionConfig(new SerializedValue<>(executionConfig));
+    }
+
+    void setSerializedExecutionConfig(SerializedValue<ExecutionConfig> serializedExecutionConfig) {
+        this.serializedExecutionConfig =
+                checkNotNull(
+                        serializedExecutionConfig,
+                        "The serialized ExecutionConfig must not be null.");
     }
 
     /**
@@ -331,15 +316,16 @@ public class JobGraph implements Serializable {
         return Collections.unmodifiableSet(slotSharingGroups);
     }
 
-    public Set<CoLocationGroupDesc> getCoLocationGroupDescriptors() {
-        // invoke distinct() on CoLocationGroup first to avoid creating
-        // multiple CoLocationGroupDec from one CoLocationGroup
-        final Set<CoLocationGroupDesc> coLocationGroups =
+    /**
+     * Returns all {@link CoLocationGroup} instances associated with this {@code JobGraph}.
+     *
+     * @return The associated {@code CoLocationGroup} instances.
+     */
+    public Set<CoLocationGroup> getCoLocationGroups() {
+        final Set<CoLocationGroup> coLocationGroups =
                 IterableUtils.toStream(getVertices())
                         .map(JobVertex::getCoLocationGroup)
                         .filter(Objects::nonNull)
-                        .distinct()
-                        .map(CoLocationGroupDesc::from)
                         .collect(Collectors.toSet());
         return Collections.unmodifiableSet(coLocationGroups);
     }
@@ -466,9 +452,10 @@ public class JobGraph implements Serializable {
     private void addNodesThatHaveNoNewPredecessors(
             JobVertex start, List<JobVertex> target, Set<JobVertex> remaining) {
 
-        // forward traverse over all produced data sets and all their consumers
+        // forward traverse over all produced data sets
         for (IntermediateDataSet dataSet : start.getProducedDataSets()) {
-            for (JobEdge edge : dataSet.getConsumers()) {
+            JobEdge edge = dataSet.getConsumer();
+            if (edge != null) {
 
                 // a vertex can be added, if it has no predecessors that are still in the
                 // 'remaining' set
