@@ -18,6 +18,7 @@
 
 package org.apache.flink.runtime.minicluster;
 
+import org.apache.flink.annotation.Internal;
 import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.api.common.JobExecutionResult;
 import org.apache.flink.api.common.JobID;
@@ -30,34 +31,41 @@ import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.ConfigurationUtils;
 import org.apache.flink.configuration.HighAvailabilityOptions;
 import org.apache.flink.configuration.IllegalConfigurationException;
-import org.apache.flink.runtime.akka.AkkaUtils;
+import org.apache.flink.core.execution.SavepointFormatType;
 import org.apache.flink.runtime.blob.BlobCacheService;
 import org.apache.flink.runtime.blob.BlobClient;
 import org.apache.flink.runtime.blob.BlobServer;
+import org.apache.flink.runtime.blob.BlobUtils;
 import org.apache.flink.runtime.client.ClientUtils;
 import org.apache.flink.runtime.client.JobExecutionException;
 import org.apache.flink.runtime.client.JobStatusMessage;
-import org.apache.flink.runtime.clusterframework.ApplicationStatus;
 import org.apache.flink.runtime.clusterframework.types.ResourceID;
-import org.apache.flink.runtime.concurrent.ExponentialBackoffRetryStrategy;
-import org.apache.flink.runtime.concurrent.FutureUtils;
 import org.apache.flink.runtime.dispatcher.DispatcherGateway;
 import org.apache.flink.runtime.dispatcher.DispatcherId;
 import org.apache.flink.runtime.dispatcher.MemoryExecutionGraphInfoStore;
+import org.apache.flink.runtime.dispatcher.TriggerSavepointMode;
 import org.apache.flink.runtime.entrypoint.ClusterEntrypointUtils;
 import org.apache.flink.runtime.entrypoint.ClusterInformation;
+import org.apache.flink.runtime.entrypoint.WorkingDirectory;
 import org.apache.flink.runtime.entrypoint.component.DefaultDispatcherResourceManagerComponentFactory;
 import org.apache.flink.runtime.entrypoint.component.DispatcherResourceManagerComponent;
 import org.apache.flink.runtime.entrypoint.component.DispatcherResourceManagerComponentFactory;
 import org.apache.flink.runtime.executiongraph.AccessExecutionGraph;
+import org.apache.flink.runtime.executiongraph.ArchivedExecutionGraph;
 import org.apache.flink.runtime.externalresource.ExternalResourceInfoProvider;
 import org.apache.flink.runtime.heartbeat.HeartbeatServices;
 import org.apache.flink.runtime.highavailability.HighAvailabilityServices;
+import org.apache.flink.runtime.highavailability.HighAvailabilityServicesFactory;
 import org.apache.flink.runtime.highavailability.HighAvailabilityServicesUtils;
+import org.apache.flink.runtime.highavailability.nonha.embedded.EmbeddedHaServices;
 import org.apache.flink.runtime.highavailability.nonha.embedded.EmbeddedHaServicesWithLeadershipControl;
 import org.apache.flink.runtime.highavailability.nonha.embedded.HaLeadershipControl;
 import org.apache.flink.runtime.jobgraph.JobGraph;
 import org.apache.flink.runtime.jobgraph.OperatorID;
+import org.apache.flink.runtime.jobgraph.RestoreMode;
+import org.apache.flink.runtime.jobgraph.SavepointConfigOptions;
+import org.apache.flink.runtime.jobgraph.SavepointRestoreSettings;
+import org.apache.flink.runtime.jobmanager.HighAvailabilityMode;
 import org.apache.flink.runtime.jobmaster.JobResult;
 import org.apache.flink.runtime.leaderretrieval.LeaderRetrievalService;
 import org.apache.flink.runtime.messages.Acknowledge;
@@ -72,14 +80,15 @@ import org.apache.flink.runtime.operators.coordination.CoordinationRequest;
 import org.apache.flink.runtime.operators.coordination.CoordinationResponse;
 import org.apache.flink.runtime.resourcemanager.ResourceManagerGateway;
 import org.apache.flink.runtime.resourcemanager.ResourceManagerId;
+import org.apache.flink.runtime.resourcemanager.ResourceOverview;
 import org.apache.flink.runtime.resourcemanager.StandaloneResourceManagerFactory;
 import org.apache.flink.runtime.rpc.FatalErrorHandler;
 import org.apache.flink.runtime.rpc.RpcService;
+import org.apache.flink.runtime.rpc.RpcSystem;
 import org.apache.flink.runtime.rpc.RpcUtils;
-import org.apache.flink.runtime.rpc.akka.AkkaRpcServiceUtils;
+import org.apache.flink.runtime.scheduler.ExecutionGraphInfo;
 import org.apache.flink.runtime.taskexecutor.TaskExecutor;
 import org.apache.flink.runtime.taskexecutor.TaskManagerRunner;
-import org.apache.flink.runtime.util.ExecutorThreadFactory;
 import org.apache.flink.runtime.webmonitor.retriever.LeaderRetriever;
 import org.apache.flink.runtime.webmonitor.retriever.MetricQueryServiceRetriever;
 import org.apache.flink.runtime.webmonitor.retriever.impl.RpcGatewayRetriever;
@@ -88,13 +97,20 @@ import org.apache.flink.util.AutoCloseableAsync;
 import org.apache.flink.util.ExceptionUtils;
 import org.apache.flink.util.ExecutorUtils;
 import org.apache.flink.util.FlinkException;
+import org.apache.flink.util.InstantiationUtil;
+import org.apache.flink.util.Reference;
 import org.apache.flink.util.SerializedValue;
+import org.apache.flink.util.concurrent.ExecutorThreadFactory;
+import org.apache.flink.util.concurrent.ExponentialBackoffRetryStrategy;
+import org.apache.flink.util.concurrent.FutureUtils;
+import org.apache.flink.util.function.BiFunctionWithException;
 import org.apache.flink.util.function.FunctionUtils;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import javax.annotation.Nonnull;
+import javax.annotation.Nullable;
 import javax.annotation.concurrent.GuardedBy;
 
 import java.io.IOException;
@@ -116,6 +132,7 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.TimeUnit;
 import java.util.function.Function;
+import java.util.function.Supplier;
 import java.util.stream.Collectors;
 
 import static org.apache.flink.util.Preconditions.checkNotNull;
@@ -140,6 +157,7 @@ public class MiniCluster implements AutoCloseableAsync {
     private final TerminatingFatalErrorHandlerFactory
             taskManagerTerminatingFatalErrorHandlerFactory =
                     new TerminatingFatalErrorHandlerFactory();
+    private final Supplier<Reference<RpcSystem>> rpcSystemSupplier;
 
     private CompletableFuture<Void> terminationFuture;
 
@@ -158,6 +176,9 @@ public class MiniCluster implements AutoCloseableAsync {
     @GuardedBy("lock")
     private final Collection<RpcService> rpcServices;
 
+    @GuardedBy("lock")
+    private HighAvailabilityServicesFactory haServicesFactory;
+
     @GuardedBy("lock")
     private HighAvailabilityServices haServices;
 
@@ -195,9 +216,15 @@ public class MiniCluster implements AutoCloseableAsync {
     @GuardedBy("lock")
     private RpcServiceFactory taskManagerRpcServiceFactory;
 
+    @GuardedBy("lock")
+    private WorkingDirectory workingDirectory;
+
     /** Flag marking the mini cluster as started/running. */
     private volatile boolean running;
 
+    @GuardedBy("lock")
+    private Reference<RpcSystem> rpcSystem;
+
     // ------------------------------------------------------------------------
 
     /**
@@ -206,6 +233,15 @@ public class MiniCluster implements AutoCloseableAsync {
      * @param miniClusterConfiguration The configuration for the mini cluster
      */
     public MiniCluster(MiniClusterConfiguration miniClusterConfiguration) {
+        this(
+                miniClusterConfiguration,
+                () -> Reference.owned(RpcSystem.load(miniClusterConfiguration.getConfiguration())));
+    }
+
+    public MiniCluster(
+            MiniClusterConfiguration miniClusterConfiguration,
+            Supplier<Reference<RpcSystem>> rpcSystemSupplier) {
+
         this.miniClusterConfiguration =
                 checkNotNull(miniClusterConfiguration, "config may not be null");
         this.rpcServices =
@@ -223,6 +259,8 @@ public class MiniCluster implements AutoCloseableAsync {
         running = false;
 
         this.taskManagers = new ArrayList<>(miniClusterConfiguration.getNumTaskManagers());
+
+        this.rpcSystemSupplier = rpcSystemSupplier;
     }
 
     public CompletableFuture<URI> getRestAddress() {
@@ -274,10 +312,22 @@ public class MiniCluster implements AutoCloseableAsync {
                     miniClusterConfiguration.getRpcServiceSharing() == RpcServiceSharing.SHARED;
 
             try {
+                workingDirectory =
+                        WorkingDirectory.create(
+                                ClusterEntrypointUtils.generateWorkingDirectoryFile(
+                                        configuration,
+                                        Optional.empty(),
+                                        "minicluster_" + ResourceID.generate()));
+
                 initializeIOFormatClasses(configuration);
 
+                rpcSystem = rpcSystemSupplier.get();
+
                 LOG.info("Starting Metrics Registry");
-                metricRegistry = createMetricRegistry(configuration);
+                metricRegistry =
+                        createMetricRegistry(
+                                configuration,
+                                rpcSystem.deref().getMaximumMessageSizeInBytes(configuration));
 
                 // bring up all the RPC services
                 LOG.info("Starting RPC Service(s)");
@@ -287,13 +337,14 @@ public class MiniCluster implements AutoCloseableAsync {
 
                 if (useSingleRpcService) {
                     // we always need the 'commonRpcService' for auxiliary calls
-                    commonRpcService = createLocalRpcService(configuration);
+                    commonRpcService = createLocalRpcService(configuration, rpcSystem.deref());
                     final CommonRpcServiceFactory commonRpcServiceFactory =
                             new CommonRpcServiceFactory(commonRpcService);
                     taskManagerRpcServiceFactory = commonRpcServiceFactory;
                     dispatcherResourceManagerComponentRpcServiceFactory = commonRpcServiceFactory;
                     metricQueryServiceRpcService =
-                            MetricUtils.startLocalMetricsRpcService(configuration);
+                            MetricUtils.startLocalMetricsRpcService(
+                                    configuration, rpcSystem.deref());
                 } else {
 
                     // start a new service per component, possibly with custom bind addresses
@@ -315,21 +366,27 @@ public class MiniCluster implements AutoCloseableAsync {
                                     configuration,
                                     jobManagerExternalAddress,
                                     jobManagerExternalPortRange,
-                                    jobManagerBindAddress);
+                                    jobManagerBindAddress,
+                                    rpcSystem.deref());
                     taskManagerRpcServiceFactory =
                             new DedicatedRpcServiceFactory(
                                     configuration,
                                     taskManagerExternalAddress,
                                     taskManagerExternalPortRange,
-                                    taskManagerBindAddress);
+                                    taskManagerBindAddress,
+                                    rpcSystem.deref());
 
                     // we always need the 'commonRpcService' for auxiliary calls
                     // bind to the JobManager address with port 0
                     commonRpcService =
-                            createRemoteRpcService(configuration, jobManagerBindAddress, 0);
+                            createRemoteRpcService(
+                                    configuration, jobManagerBindAddress, 0, rpcSystem.deref());
                     metricQueryServiceRpcService =
                             MetricUtils.startRemoteMetricsRpcService(
-                                    configuration, commonRpcService.getAddress());
+                                    configuration,
+                                    commonRpcService.getAddress(),
+                                    null,
+                                    rpcSystem.deref());
                 }
 
                 metricRegistry.startQueryService(metricQueryServiceRpcService, null);
@@ -345,16 +402,24 @@ public class MiniCluster implements AutoCloseableAsync {
                         Executors.newFixedThreadPool(
                                 ClusterEntrypointUtils.getPoolSize(configuration),
                                 new ExecutorThreadFactory("mini-cluster-io"));
+
+                haServicesFactory = createHighAvailabilityServicesFactory(configuration);
+
                 haServices = createHighAvailabilityServices(configuration, ioExecutor);
 
-                blobServer = new BlobServer(configuration, haServices.createBlobStore());
+                blobServer =
+                        BlobUtils.createBlobServer(
+                                configuration,
+                                Reference.borrowed(workingDirectory.getBlobStorageDirectory()),
+                                haServices.createBlobStore());
                 blobServer.start();
 
                 heartbeatServices = HeartbeatServices.fromConfiguration(configuration);
 
                 blobCacheService =
-                        new BlobCacheService(
+                        BlobUtils.createBlobCacheService(
                                 configuration,
+                                Reference.borrowed(workingDirectory.getBlobStorageDirectory()),
                                 haServices.createBlobStore(),
                                 new InetSocketAddress(
                                         InetAddress.getLocalHost(), blobServer.getPort()));
@@ -424,27 +489,17 @@ public class MiniCluster implements AutoCloseableAsync {
                 createDispatcherResourceManagerComponents(
                         configuration,
                         dispatcherResourceManagerComponentRpcServiceFactory,
-                        haServices,
                         blobServer,
                         heartbeatServices,
                         metricRegistry,
                         metricQueryServiceRetriever,
                         new ShutDownFatalErrorHandler()));
 
-        final Collection<CompletableFuture<ApplicationStatus>> shutDownFutures =
-                new ArrayList<>(dispatcherResourceManagerComponents.size());
-
-        for (DispatcherResourceManagerComponent dispatcherResourceManagerComponent :
-                dispatcherResourceManagerComponents) {
-            final CompletableFuture<ApplicationStatus> shutDownFuture =
-                    dispatcherResourceManagerComponent.getShutDownFuture();
-            FutureUtils.assertNoException(
-                    shutDownFuture.thenRun(dispatcherResourceManagerComponent::closeAsync));
-            shutDownFutures.add(shutDownFuture);
-        }
-
-        FutureUtils.assertNoException(
-                FutureUtils.completeAll(shutDownFutures).thenRun(this::closeAsync));
+        FutureUtils.completeAll(
+                        dispatcherResourceManagerComponents.stream()
+                                .map(DispatcherResourceManagerComponent::getShutDownFuture)
+                                .collect(Collectors.toList()))
+                .whenComplete((ignored, exception) -> closeAsync());
     }
 
     @VisibleForTesting
@@ -452,7 +507,6 @@ public class MiniCluster implements AutoCloseableAsync {
             createDispatcherResourceManagerComponents(
                     Configuration configuration,
                     RpcServiceFactory rpcServiceFactory,
-                    HighAvailabilityServices haServices,
                     BlobServer blobServer,
                     HeartbeatServices heartbeatServices,
                     MetricRegistry metricRegistry,
@@ -461,9 +515,11 @@ public class MiniCluster implements AutoCloseableAsync {
                     throws Exception {
         DispatcherResourceManagerComponentFactory dispatcherResourceManagerComponentFactory =
                 createDispatcherResourceManagerComponentFactory();
-        return Collections.singleton(
+
+        final DispatcherResourceManagerComponent dispatcherResourceManagerComponent =
                 dispatcherResourceManagerComponentFactory.create(
                         configuration,
+                        ResourceID.generate(),
                         ioExecutor,
                         rpcServiceFactory.createRpcService(),
                         haServices,
@@ -472,29 +528,59 @@ public class MiniCluster implements AutoCloseableAsync {
                         metricRegistry,
                         new MemoryExecutionGraphInfoStore(),
                         metricQueryServiceRetriever,
-                        fatalErrorHandler));
+                        fatalErrorHandler);
+        FutureUtils.assertNoException(
+                dispatcherResourceManagerComponent
+                        .getShutDownFuture()
+                        .thenCompose(
+                                applicationStatus ->
+                                        dispatcherResourceManagerComponent.stopApplication(
+                                                applicationStatus, null)));
+        return Collections.singleton(dispatcherResourceManagerComponent);
     }
 
-    @Nonnull
-    DispatcherResourceManagerComponentFactory createDispatcherResourceManagerComponentFactory() {
+    protected DispatcherResourceManagerComponentFactory
+            createDispatcherResourceManagerComponentFactory() {
         return DefaultDispatcherResourceManagerComponentFactory.createSessionComponentFactory(
                 StandaloneResourceManagerFactory.getInstance());
     }
 
+    private HighAvailabilityServicesFactory createHighAvailabilityServicesFactory(
+            Configuration configuration) {
+        final HaServices customMiniClusterHaServicesMode = miniClusterConfiguration.getHaServices();
+        if (customMiniClusterHaServicesMode == HaServices.WITH_LEADERSHIP_CONTROL) {
+            // special feature of MiniClusters to allow the control of leadership
+            // EmbeddedLeaderElection requires a single instance for leader election across multiple
+            // JobManager instances on the same JVM (after FLINK-24038 was introduced); therefore,
+            // SingletonHighAvailabilityServicesFactory is utilized here
+            return new SingletonHighAvailabilityServicesFactory(
+                    (config, embeddedLeaderElectionExecutor) ->
+                            new EmbeddedHaServicesWithLeadershipControl(
+                                    embeddedLeaderElectionExecutor));
+        } else if (customMiniClusterHaServicesMode != HaServices.CONFIGURED) {
+            throw new IllegalConfigurationException(
+                    "Unknown HA Services Mode configured in MiniCluster configuration: "
+                            + customMiniClusterHaServicesMode);
+        }
+
+        final HighAvailabilityMode highAvailabilityMode =
+                HighAvailabilityMode.fromConfig(configuration);
+        if (highAvailabilityMode == HighAvailabilityMode.NONE) {
+            // basic EmbeddedLeaderElection requires a single instance for leader election across
+            // multiple JobManager instances on the same JVM (after FLINK-24038 was introduced);
+            // therefore, SingletonHighAvailabilityServicesFactory is utilized here
+            return new SingletonHighAvailabilityServicesFactory(
+                    (config, embeddedLeaderElectionExecutor) ->
+                            new EmbeddedHaServices(embeddedLeaderElectionExecutor));
+        } else {
+            return new RegularHighAvailabilityServicesFactory();
+        }
+    }
+
     @VisibleForTesting
     protected HighAvailabilityServices createHighAvailabilityServices(
             Configuration configuration, Executor executor) throws Exception {
-        LOG.info("Starting high-availability services");
-        final HaServices haServices = miniClusterConfiguration.getHaServices();
-        switch (haServices) {
-            case WITH_LEADERSHIP_CONTROL:
-                return new EmbeddedHaServicesWithLeadershipControl(executor);
-            case CONFIGURED:
-                return HighAvailabilityServicesUtils.createAvailableOrEmbeddedServices(
-                        configuration, executor);
-            default:
-                throw new IllegalConfigurationException("Unkown HA Services " + haServices);
-        }
+        return haServicesFactory.createHAServices(configuration, executor);
     }
 
     /**
@@ -514,6 +600,10 @@ public class MiniCluster implements AutoCloseableAsync {
         }
     }
 
+    protected HighAvailabilityServices getHaServices() {
+        return haServices;
+    }
+
     /**
      * Shuts down the mini cluster, failing all currently executing jobs. The mini cluster can be
      * started again by calling the {@link #start()} method again.
@@ -562,7 +652,11 @@ public class MiniCluster implements AutoCloseableAsync {
                                     remainingServicesTerminationFuture,
                                     () -> terminateExecutors(shutdownTimeoutMillis));
 
-                    executorsTerminationFuture.whenComplete(
+                    final CompletableFuture<Void> deleteDirectoriesFuture =
+                            FutureUtils.runAfterwards(
+                                    executorsTerminationFuture, this::deleteDirectories);
+
+                    deleteDirectoriesFuture.whenComplete(
                             (Void ignored, Throwable throwable) -> {
                                 if (throwable != null) {
                                     terminationFuture.completeExceptionally(
@@ -603,7 +697,7 @@ public class MiniCluster implements AutoCloseableAsync {
     private void startTaskManagers() throws Exception {
         final int numTaskManagers = miniClusterConfiguration.getNumTaskManagers();
 
-        LOG.info("Starting {} TaskManger(s)", numTaskManagers);
+        LOG.info("Starting {} TaskManager(s)", numTaskManagers);
 
         for (int i = 0; i < numTaskManagers; i++) {
             startTaskManager();
@@ -635,6 +729,7 @@ public class MiniCluster implements AutoCloseableAsync {
                             blobCacheService,
                             useLocalCommunication(),
                             ExternalResourceInfoProvider.NO_EXTERNAL_RESOURCES,
+                            workingDirectory.createSubWorkingDirectory("tm_" + taskManagers.size()),
                             taskManagerTerminatingFatalErrorHandlerFactory.create(
                                     taskManagers.size()));
 
@@ -648,6 +743,21 @@ public class MiniCluster implements AutoCloseableAsync {
         return miniClusterConfiguration.getNumTaskManagers() == 1;
     }
 
+    @VisibleForTesting
+    public Configuration getConfiguration() {
+        return miniClusterConfiguration.getConfiguration();
+    }
+
+    // HACK: temporary hack to make the changelog state backend tests work with forced
+    // full snapshots. This option should be removed once changelog state backend supports forced
+    // full snapshots
+    @Internal private boolean overrideRestoreModeForChangelogStateBackend;
+
+    @Internal
+    public void overrideRestoreModeForChangelogStateBackend() {
+        this.overrideRestoreModeForChangelogStateBackend = true;
+    }
+
     @GuardedBy("lock")
     private Collection<? extends CompletableFuture<Void>> terminateTaskManagers() {
         final Collection<CompletableFuture<Void>> terminationFutures =
@@ -681,6 +791,14 @@ public class MiniCluster implements AutoCloseableAsync {
     //  Accessing jobs
     // ------------------------------------------------------------------------
 
+    public CompletableFuture<ArchivedExecutionGraph> getArchivedExecutionGraph(JobID jobId) {
+        return runDispatcherCommand(
+                dispatcherGateway ->
+                        dispatcherGateway
+                                .requestExecutionGraphInfo(jobId, rpcTimeout)
+                                .thenApply(ExecutionGraphInfo::getArchivedExecutionGraph));
+    }
+
     public CompletableFuture<Collection<JobStatusMessage>> listJobs() {
         return runDispatcherCommand(
                 dispatcherGateway ->
@@ -712,19 +830,42 @@ public class MiniCluster implements AutoCloseableAsync {
     }
 
     public CompletableFuture<String> triggerSavepoint(
-            JobID jobId, String targetDirectory, boolean cancelJob) {
+            JobID jobId,
+            String targetDirectory,
+            boolean cancelJob,
+            SavepointFormatType formatType) {
         return runDispatcherCommand(
                 dispatcherGateway ->
-                        dispatcherGateway.triggerSavepoint(
-                                jobId, targetDirectory, cancelJob, rpcTimeout));
+                        dispatcherGateway.triggerSavepointAndGetLocation(
+                                jobId,
+                                targetDirectory,
+                                formatType,
+                                cancelJob
+                                        ? TriggerSavepointMode.CANCEL_WITH_SAVEPOINT
+                                        : TriggerSavepointMode.SAVEPOINT,
+                                rpcTimeout));
+    }
+
+    public CompletableFuture<String> triggerCheckpoint(JobID jobID) {
+        return runDispatcherCommand(
+                dispatcherGateway -> dispatcherGateway.triggerCheckpoint(jobID, rpcTimeout));
     }
 
     public CompletableFuture<String> stopWithSavepoint(
-            JobID jobId, String targetDirectory, boolean terminate) {
+            JobID jobId,
+            String targetDirectory,
+            boolean terminate,
+            SavepointFormatType formatType) {
         return runDispatcherCommand(
                 dispatcherGateway ->
-                        dispatcherGateway.stopWithSavepoint(
-                                jobId, targetDirectory, terminate, rpcTimeout));
+                        dispatcherGateway.stopWithSavepointAndGetLocation(
+                                jobId,
+                                targetDirectory,
+                                formatType,
+                                terminate
+                                        ? TriggerSavepointMode.TERMINATE_WITH_SAVEPOINT
+                                        : TriggerSavepointMode.SUSPEND_WITH_SAVEPOINT,
+                                rpcTimeout));
     }
 
     public CompletableFuture<Acknowledge> disposeSavepoint(String savepointPath) {
@@ -747,6 +888,12 @@ public class MiniCluster implements AutoCloseableAsync {
                                 jobId, operatorId, serializedRequest, rpcTimeout));
     }
 
+    public CompletableFuture<ResourceOverview> getResourceOverview() {
+        return runResourceManagerCommand(
+                resourceManagerGateway ->
+                        resourceManagerGateway.requestResourceOverview(rpcTimeout));
+    }
+
     private <T> CompletableFuture<T> runDispatcherCommand(
             Function<DispatcherGateway, CompletableFuture<T>> dispatcherCommand) {
         return getDispatcherGatewayFuture()
@@ -754,6 +901,13 @@ public class MiniCluster implements AutoCloseableAsync {
                 .thenCompose(Function.identity());
     }
 
+    private <T> CompletableFuture<T> runResourceManagerCommand(
+            Function<ResourceManagerGateway, CompletableFuture<T>> resourceManagerCommand) {
+        return getResourceManagerGatewayFuture()
+                .thenApply(resourceManagerCommand)
+                .thenCompose(Function.identity());
+    }
+
     // ------------------------------------------------------------------------
     //  running jobs
     // ------------------------------------------------------------------------
@@ -817,21 +971,41 @@ public class MiniCluster implements AutoCloseableAsync {
     }
 
     public CompletableFuture<JobSubmissionResult> submitJob(JobGraph jobGraph) {
+        // When MiniCluster uses the local RPC, the provided JobGraph is passed directly to the
+        // Dispatcher. This means that any mutations to the JG can affect the Dispatcher behaviour,
+        // so we rather clone it to guard against this.
+        final JobGraph clonedJobGraph = cloneJobGraph(jobGraph);
+        checkRestoreModeForChangelogStateBackend(clonedJobGraph);
         final CompletableFuture<DispatcherGateway> dispatcherGatewayFuture =
                 getDispatcherGatewayFuture();
         final CompletableFuture<InetSocketAddress> blobServerAddressFuture =
                 createBlobServerAddress(dispatcherGatewayFuture);
         final CompletableFuture<Void> jarUploadFuture =
-                uploadAndSetJobFiles(blobServerAddressFuture, jobGraph);
+                uploadAndSetJobFiles(blobServerAddressFuture, clonedJobGraph);
         final CompletableFuture<Acknowledge> acknowledgeCompletableFuture =
                 jarUploadFuture
                         .thenCombine(
                                 dispatcherGatewayFuture,
                                 (Void ack, DispatcherGateway dispatcherGateway) ->
-                                        dispatcherGateway.submitJob(jobGraph, rpcTimeout))
+                                        dispatcherGateway.submitJob(clonedJobGraph, rpcTimeout))
                         .thenCompose(Function.identity());
         return acknowledgeCompletableFuture.thenApply(
-                (Acknowledge ignored) -> new JobSubmissionResult(jobGraph.getJobID()));
+                (Acknowledge ignored) -> new JobSubmissionResult(clonedJobGraph.getJobID()));
+    }
+
+    // HACK: temporary hack to make the randomized changelog state backend tests work with forced
+    // full snapshots. This option should be removed once changelog state backend supports forced
+    // full snapshots
+    private void checkRestoreModeForChangelogStateBackend(JobGraph jobGraph) {
+        final SavepointRestoreSettings savepointRestoreSettings =
+                jobGraph.getSavepointRestoreSettings();
+        if (overrideRestoreModeForChangelogStateBackend
+                && savepointRestoreSettings.getRestoreMode() == RestoreMode.NO_CLAIM) {
+            final Configuration conf = new Configuration();
+            SavepointRestoreSettings.toConfiguration(savepointRestoreSettings, conf);
+            conf.set(SavepointConfigOptions.RESTORE_MODE, RestoreMode.LEGACY);
+            jobGraph.setSavepointRestoreSettings(SavepointRestoreSettings.fromConfiguration(conf));
+        }
     }
 
     public CompletableFuture<JobResult> requestJobResult(JobID jobId) {
@@ -854,6 +1028,13 @@ public class MiniCluster implements AutoCloseableAsync {
         }
     }
 
+    private CompletableFuture<ResourceManagerGateway> getResourceManagerGatewayFuture() {
+        synchronized (lock) {
+            checkState(running, "MiniCluster is not yet running or has already been shut down.");
+            return resourceManagerGatewayRetriever.getFuture();
+        }
+    }
+
     private CompletableFuture<Void> uploadAndSetJobFiles(
             final CompletableFuture<InetSocketAddress> blobServerAddressFuture,
             final JobGraph job) {
@@ -895,10 +1076,12 @@ public class MiniCluster implements AutoCloseableAsync {
      * Factory method to create the metric registry for the mini cluster.
      *
      * @param config The configuration of the mini cluster
+     * @param maximumMessageSizeInBytes the maximum message size
      */
-    protected MetricRegistryImpl createMetricRegistry(Configuration config) {
+    protected MetricRegistryImpl createMetricRegistry(
+            Configuration config, long maximumMessageSizeInBytes) {
         return new MetricRegistryImpl(
-                MetricRegistryConfiguration.fromConfiguration(config),
+                MetricRegistryConfiguration.fromConfiguration(config, maximumMessageSizeInBytes),
                 ReporterSetup.fromConfiguration(config, null));
     }
 
@@ -908,15 +1091,17 @@ public class MiniCluster implements AutoCloseableAsync {
      * @param configuration Flink configuration.
      * @param bindAddress The address to bind the RPC service to.
      * @param bindPort The port range to bind the RPC service to.
+     * @param rpcSystem
      * @return The instantiated RPC service
      */
     protected RpcService createRemoteRpcService(
-            Configuration configuration, String bindAddress, int bindPort) throws Exception {
-        return AkkaRpcServiceUtils.remoteServiceBuilder(
-                        configuration, bindAddress, String.valueOf(bindPort))
+            Configuration configuration, String bindAddress, int bindPort, RpcSystem rpcSystem)
+            throws Exception {
+        return rpcSystem
+                .remoteServiceBuilder(configuration, bindAddress, String.valueOf(bindPort))
                 .withBindAddress(bindAddress)
                 .withBindPort(bindPort)
-                .withCustomConfig(AkkaUtils.testDispatcherConfig())
+                .withExecutorConfiguration(RpcUtils.getTestForkJoinExecutorConfiguration())
                 .createAndStart();
     }
 
@@ -927,18 +1112,20 @@ public class MiniCluster implements AutoCloseableAsync {
      * @param externalAddress The external address to access the RPC service.
      * @param externalPortRange The external port range to access the RPC service.
      * @param bindAddress The address to bind the RPC service to.
+     * @param rpcSystem
      * @return The instantiated RPC service
      */
     protected RpcService createRemoteRpcService(
             Configuration configuration,
             String externalAddress,
             String externalPortRange,
-            String bindAddress)
+            String bindAddress,
+            RpcSystem rpcSystem)
             throws Exception {
-        return AkkaRpcServiceUtils.remoteServiceBuilder(
-                        configuration, externalAddress, externalPortRange)
+        return rpcSystem
+                .remoteServiceBuilder(configuration, externalAddress, externalPortRange)
                 .withBindAddress(bindAddress)
-                .withCustomConfig(AkkaUtils.testDispatcherConfig())
+                .withExecutorConfiguration(RpcUtils.getTestForkJoinExecutorConfiguration())
                 .createAndStart();
     }
 
@@ -946,11 +1133,14 @@ public class MiniCluster implements AutoCloseableAsync {
      * Factory method to instantiate the local RPC service.
      *
      * @param configuration Flink configuration.
+     * @param rpcSystem
      * @return The instantiated RPC service
      */
-    protected RpcService createLocalRpcService(Configuration configuration) throws Exception {
-        return AkkaRpcServiceUtils.localServiceBuilder(configuration)
-                .withCustomConfig(AkkaUtils.testDispatcherConfig())
+    protected RpcService createLocalRpcService(Configuration configuration, RpcSystem rpcSystem)
+            throws Exception {
+        return rpcSystem
+                .localServiceBuilder(configuration)
+                .withExecutorConfiguration(RpcUtils.getTestForkJoinExecutorConfiguration())
                 .createAndStart();
     }
 
@@ -1050,6 +1240,14 @@ public class MiniCluster implements AutoCloseableAsync {
                 haServices = null;
             }
 
+            try {
+                if (rpcSystem.isOwned()) {
+                    rpcSystem.deref().close();
+                }
+            } catch (Exception e) {
+                exception = ExceptionUtils.firstOrSuppressed(e, exception);
+            }
+
             if (exception != null) {
                 throw exception;
             }
@@ -1088,6 +1286,22 @@ public class MiniCluster implements AutoCloseableAsync {
         }
     }
 
+    private void deleteDirectories() throws IOException {
+        synchronized (lock) {
+            if (workingDirectory != null) {
+                workingDirectory.delete();
+            }
+        }
+    }
+
+    private static JobGraph cloneJobGraph(JobGraph jobGraph) {
+        try {
+            return InstantiationUtil.clone(jobGraph);
+        } catch (ClassNotFoundException | IOException e) {
+            throw new IllegalStateException("Unable to clone the provided JobGraph.", e);
+        }
+    }
+
     /** Internal factory for {@link RpcService}. */
     protected interface RpcServiceFactory {
         RpcService createRpcService() throws Exception;
@@ -1115,23 +1329,30 @@ public class MiniCluster implements AutoCloseableAsync {
         private final String externalAddress;
         private final String externalPortRange;
         private final String bindAddress;
+        private final RpcSystem rpcSystem;
 
         DedicatedRpcServiceFactory(
                 Configuration configuration,
                 String externalAddress,
                 String externalPortRange,
-                String bindAddress) {
+                String bindAddress,
+                RpcSystem rpcSystem) {
             this.configuration = configuration;
             this.externalAddress = externalAddress;
             this.externalPortRange = externalPortRange;
             this.bindAddress = bindAddress;
+            this.rpcSystem = rpcSystem;
         }
 
         @Override
         public RpcService createRpcService() throws Exception {
             final RpcService rpcService =
                     MiniCluster.this.createRemoteRpcService(
-                            configuration, externalAddress, externalPortRange, bindAddress);
+                            configuration,
+                            externalAddress,
+                            externalPortRange,
+                            bindAddress,
+                            rpcSystem);
 
             synchronized (lock) {
                 rpcServices.add(rpcService);
@@ -1210,4 +1431,45 @@ public class MiniCluster implements AutoCloseableAsync {
          */
         WITH_LEADERSHIP_CONTROL
     }
+
+    /**
+     * SingletonHighAvailabilityServicesFactory is used for scenarios that are not truly high
+     * available and rely on having a single HighAvailabilityServices object.
+     */
+    private static class SingletonHighAvailabilityServicesFactory
+            implements HighAvailabilityServicesFactory {
+
+        private final BiFunctionWithException<
+                        Configuration, Executor, HighAvailabilityServices, Exception>
+                creationCallback;
+
+        @Nullable private HighAvailabilityServices haServices;
+
+        public SingletonHighAvailabilityServicesFactory(
+                BiFunctionWithException<
+                                Configuration, Executor, HighAvailabilityServices, Exception>
+                        creationCallback) {
+            this.creationCallback = creationCallback;
+        }
+
+        @Override
+        public HighAvailabilityServices createHAServices(
+                Configuration configuration, Executor executor) throws Exception {
+            if (haServices == null) {
+                haServices = creationCallback.apply(configuration, executor);
+            }
+
+            return this.haServices;
+        }
+    }
+
+    private class RegularHighAvailabilityServicesFactory
+            implements HighAvailabilityServicesFactory {
+        @Override
+        public HighAvailabilityServices createHAServices(
+                Configuration configuration, Executor executor) throws Exception {
+            return HighAvailabilityServicesUtils.createAvailableOrEmbeddedServices(
+                    configuration, executor, new ShutDownFatalErrorHandler());
+        }
+    }
 }
