@@ -20,12 +20,15 @@ package org.apache.flink.runtime.state.filesystem;
 
 import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.api.common.JobID;
+import org.apache.flink.core.fs.DuplicatingFileSystem;
 import org.apache.flink.core.fs.FileSystem;
 import org.apache.flink.core.fs.Path;
+import org.apache.flink.runtime.state.CheckpointStateOutputStream;
+import org.apache.flink.runtime.state.CheckpointStateToolset;
 import org.apache.flink.runtime.state.CheckpointStorageLocation;
 import org.apache.flink.runtime.state.CheckpointStorageLocationReference;
 import org.apache.flink.runtime.state.CheckpointStreamFactory;
-import org.apache.flink.runtime.state.CheckpointStreamFactory.CheckpointStateOutputStream;
+import org.apache.flink.runtime.state.NotDuplicatingCheckpointStateToolset;
 import org.apache.flink.runtime.state.filesystem.FsCheckpointStreamFactory.FsCheckpointStateOutputStream;
 
 import javax.annotation.Nullable;
@@ -109,18 +112,15 @@ public class FsCheckpointStorageAccess extends AbstractFsCheckpointStorageAccess
     }
 
     @Override
-    public void initializeBaseLocations() throws IOException {
+    public void initializeBaseLocationsForCheckpoint() throws IOException {
         fileSystem.mkdirs(sharedStateDirectory);
         fileSystem.mkdirs(taskOwnedStateDirectory);
-        baseLocationsInitialized = true;
     }
 
     @Override
     public CheckpointStorageLocation initializeLocationForCheckpoint(long checkpointId)
             throws IOException {
-        checkArgument(checkpointId >= 0, "Illegal negative checkpoint id: %d.", checkpointId);
-        checkArgument(
-                baseLocationsInitialized, "The base checkpoint location has not been initialized.");
+        checkArgument(checkpointId >= 0, "Illegal negative checkpoint id: %s.", checkpointId);
 
         // prepare all the paths needed for the checkpoints
         final Path checkpointDir = createCheckpointDirectory(checkpointsDirectory, checkpointId);
@@ -179,6 +179,16 @@ public class FsCheckpointStorageAccess extends AbstractFsCheckpointStorageAccess
                 taskOwnedStateDirectory, fileSystem, writeBufferSize, fileSizeThreshold);
     }
 
+    @Override
+    public CheckpointStateToolset createTaskOwnedCheckpointStateToolset() {
+        if (fileSystem instanceof DuplicatingFileSystem) {
+            return new FsCheckpointStateToolset(
+                    taskOwnedStateDirectory, (DuplicatingFileSystem) fileSystem);
+        } else {
+            return new NotDuplicatingCheckpointStateToolset();
+        }
+    }
+
     @Override
     protected CheckpointStorageLocation createSavepointLocation(FileSystem fs, Path location) {
         final CheckpointStorageLocationReference reference = encodePathAsReference(location);
