@@ -20,18 +20,16 @@ package org.apache.flink.runtime.checkpoint;
 
 import org.apache.flink.api.common.JobStatus;
 import org.apache.flink.api.java.tuple.Tuple2;
+import org.apache.flink.core.execution.SavepointFormatType;
 import org.apache.flink.runtime.OperatorIDPair;
 import org.apache.flink.runtime.checkpoint.CheckpointCoordinatorTestingUtils.CheckpointCoordinatorBuilder;
-import org.apache.flink.runtime.checkpoint.VertexFinishedStateChecker.VertexFinishedState;
-import org.apache.flink.runtime.concurrent.ManuallyTriggeredScheduledExecutor;
 import org.apache.flink.runtime.executiongraph.Execution;
 import org.apache.flink.runtime.executiongraph.ExecutionGraph;
 import org.apache.flink.runtime.executiongraph.ExecutionJobVertex;
-import org.apache.flink.runtime.io.network.partition.ResultPartitionType;
-import org.apache.flink.runtime.jobgraph.DistributionPattern;
-import org.apache.flink.runtime.jobgraph.JobVertex;
 import org.apache.flink.runtime.jobgraph.JobVertexID;
 import org.apache.flink.runtime.jobgraph.OperatorID;
+import org.apache.flink.runtime.jobgraph.RestoreMode;
+import org.apache.flink.runtime.jobgraph.SavepointRestoreSettings;
 import org.apache.flink.runtime.jobgraph.tasks.CheckpointCoordinatorConfiguration.CheckpointCoordinatorConfigurationBuilder;
 import org.apache.flink.runtime.messages.checkpoint.AcknowledgeCheckpoint;
 import org.apache.flink.runtime.state.ChainedStateHandle;
@@ -41,10 +39,11 @@ import org.apache.flink.runtime.state.KeyedStateHandle;
 import org.apache.flink.runtime.state.OperatorStateHandle;
 import org.apache.flink.runtime.state.SharedStateRegistry;
 import org.apache.flink.runtime.state.testutils.TestCompletedCheckpointStorageLocation;
-import org.apache.flink.runtime.testtasks.NoOpInvokable;
 import org.apache.flink.runtime.testutils.CommonTestUtils;
-import org.apache.flink.util.FlinkRuntimeException;
+import org.apache.flink.types.BooleanValue;
 import org.apache.flink.util.TestLogger;
+import org.apache.flink.util.concurrent.Executors;
+import org.apache.flink.util.concurrent.ManuallyTriggeredScheduledExecutor;
 
 import org.apache.flink.shaded.guava30.com.google.common.collect.Iterables;
 
@@ -52,9 +51,9 @@ import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.ExpectedException;
 import org.junit.rules.TemporaryFolder;
 
+import java.io.File;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -66,9 +65,11 @@ import java.util.Map;
 import java.util.Objects;
 import java.util.Random;
 import java.util.Set;
+import java.util.concurrent.CompletableFuture;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
+import static java.util.Collections.emptyList;
 import static java.util.Collections.singletonList;
 import static org.apache.flink.runtime.checkpoint.CheckpointCoordinatorTestingUtils.compareKeyedState;
 import static org.apache.flink.runtime.checkpoint.CheckpointCoordinatorTestingUtils.comparePartitionableState;
@@ -84,6 +85,7 @@ import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.ArgumentMatchers.eq;
 import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.verify;
 
@@ -167,8 +169,6 @@ public class CheckpointCoordinatorRestoringTest extends TestLogger {
 
     @Rule public TemporaryFolder tmpFolder = new TemporaryFolder();
 
-    @Rule public ExpectedException thrown = ExpectedException.none();
-
     @Before
     public void setUp() throws Exception {
         manuallyTriggeredScheduledExecutor = new ManuallyTriggeredScheduledExecutor();
@@ -235,7 +235,7 @@ public class CheckpointCoordinatorRestoringTest extends TestLogger {
         final ExecutionGraph executionGraph = createExecutionGraph(vertices);
         final EmbeddedCompletedCheckpointStore store =
                 new EmbeddedCompletedCheckpointStore(
-                        completedCheckpoints.size(), completedCheckpoints);
+                        completedCheckpoints.size(), completedCheckpoints, RestoreMode.DEFAULT);
 
         // set up the coordinator and validate the initial state
         final CheckpointCoordinator coordinator =
@@ -257,7 +257,9 @@ public class CheckpointCoordinatorRestoringTest extends TestLogger {
             for (OperatorState taskState : completedCheckpoint.getOperatorStates().values()) {
                 for (OperatorSubtaskState subtaskState : taskState.getStates()) {
                     verify(subtaskState, times(2))
-                            .registerSharedStates(any(SharedStateRegistry.class));
+                            .registerSharedStates(
+                                    any(SharedStateRegistry.class),
+                                    eq(completedCheckpoint.getCheckpointID()));
                 }
             }
         }
@@ -772,13 +774,18 @@ public class CheckpointCoordinatorRestoringTest extends TestLogger {
                         Collections.<MasterState>emptyList(),
                         CheckpointProperties.forCheckpoint(
                                 CheckpointRetentionPolicy.NEVER_RETAIN_AFTER_TERMINATION),
-                        new TestCompletedCheckpointStorageLocation());
+                        new TestCompletedCheckpointStorageLocation(),
+                        null);
 
         // set up the coordinator and validate the initial state
+        SharedStateRegistry sharedStateRegistry =
+                SharedStateRegistry.DEFAULT_FACTORY.create(
+                        Executors.directExecutor(), emptyList(), RestoreMode.DEFAULT);
         CheckpointCoordinator coord =
                 new CheckpointCoordinatorBuilder()
                         .setExecutionGraph(newGraph)
-                        .setCompletedCheckpointStore(storeFor(() -> {}, completedCheckpoint))
+                        .setCompletedCheckpointStore(
+                                storeFor(sharedStateRegistry, () -> {}, completedCheckpoint))
                         .setTimer(manuallyTriggeredScheduledExecutor)
                         .build();
 
@@ -936,12 +943,16 @@ public class CheckpointCoordinatorRestoringTest extends TestLogger {
     }
 
     static CompletedCheckpointStore storeFor(
-            Runnable postCleanupAction, CompletedCheckpoint... checkpoints) throws Exception {
+            SharedStateRegistry sharedStateRegistry,
+            Runnable postCleanupAction,
+            CompletedCheckpoint... checkpoints)
+            throws Exception {
         StandaloneCompletedCheckpointStore store =
                 new StandaloneCompletedCheckpointStore(checkpoints.length);
         CheckpointsCleaner checkpointsCleaner = new CheckpointsCleaner();
         for (final CompletedCheckpoint checkpoint : checkpoints) {
-            store.addCheckpoint(checkpoint, checkpointsCleaner, postCleanupAction);
+            store.addCheckpointAndSubsumeOldestOne(
+                    checkpoint, checkpointsCleaner, postCleanupAction);
         }
         return store;
     }
@@ -1081,8 +1092,9 @@ public class CheckpointCoordinatorRestoringTest extends TestLogger {
                         Collections.emptyList(),
                         CheckpointProperties.forCheckpoint(
                                 CheckpointRetentionPolicy.NEVER_RETAIN_AFTER_TERMINATION),
-                        new TestCompletedCheckpointStorageLocation());
-        completedCheckpointStore.addCheckpoint(
+                        new TestCompletedCheckpointStorageLocation(),
+                        null);
+        completedCheckpointStore.addCheckpointAndSubsumeOldestOne(
                 completedCheckpoint, new CheckpointsCleaner(), () -> {});
 
         CheckpointCoordinator coord =
@@ -1106,298 +1118,98 @@ public class CheckpointCoordinatorRestoringTest extends TestLogger {
     }
 
     @Test
-    public void testRestoringPartiallyFinishedChainsFailsWithoutUidHash() throws Exception {
-        // If useUidHash is set to false, the operator states would still be keyed with the
-        // generated ID, which simulates the case of restoring a checkpoint taken after jobs
-        // started. The checker should still be able to access the stored state correctly, otherwise
-        // it would mark op1 as running and pass the check wrongly.
-        testRestoringPartiallyFinishedChainsFails(false);
-    }
-
-    @Test
-    public void testRestoringPartiallyFinishedChainsFailsWithUidHash() throws Exception {
-        testRestoringPartiallyFinishedChainsFails(true);
-    }
-
-    private void testRestoringPartiallyFinishedChainsFails(boolean useUidHash) throws Exception {
-        final JobVertexID jobVertexID1 = new JobVertexID();
-        final JobVertexID jobVertexID2 = new JobVertexID();
-        // The op1 has uidHash set.
-        OperatorIDPair op1 = OperatorIDPair.of(new OperatorID(), new OperatorID());
-        OperatorIDPair op2 = OperatorIDPair.generatedIDOnly(new OperatorID());
-        OperatorIDPair op3 = OperatorIDPair.generatedIDOnly(new OperatorID());
-
-        final ExecutionGraph graph =
+    public void testJobGraphModificationsAreCheckedForInitialCheckpoint() throws Exception {
+        final JobVertexID jobVertexID = new JobVertexID();
+        ExecutionGraph graph =
                 new CheckpointCoordinatorTestingUtils.CheckpointExecutionGraphBuilder()
-                        .addJobVertex(jobVertexID2, 1, 1, singletonList(op3), true)
-                        .addJobVertex(jobVertexID1, 1, 1, Arrays.asList(op1, op2), true)
+                        .addJobVertex(jobVertexID, 1, 1)
                         .build();
-
-        Map<OperatorID, OperatorState> operatorStates = new HashMap<>();
-        operatorStates.put(
-                useUidHash ? op1.getUserDefinedOperatorID().get() : op1.getGeneratedOperatorID(),
-                new FullyFinishedOperatorState(op1.getGeneratedOperatorID(), 1, 1));
-        operatorStates.put(
-                op2.getGeneratedOperatorID(),
-                new OperatorState(op2.getGeneratedOperatorID(), 1, 1));
-        CompletedCheckpointStore store = new EmbeddedCompletedCheckpointStore();
-        store.addCheckpoint(
+        CompletedCheckpointStore completedCheckpointStore = new EmbeddedCompletedCheckpointStore();
+        CompletedCheckpoint completedCheckpoint =
                 new CompletedCheckpoint(
                         graph.getJobID(),
                         2,
                         System.currentTimeMillis(),
                         System.currentTimeMillis() + 3000,
-                        operatorStates,
+                        Collections.emptyMap(),
                         Collections.emptyList(),
                         CheckpointProperties.forCheckpoint(
                                 CheckpointRetentionPolicy.NEVER_RETAIN_AFTER_TERMINATION),
-                        new TestCompletedCheckpointStorageLocation()),
-                new CheckpointsCleaner(),
-                () -> {});
+                        new TestCompletedCheckpointStorageLocation(),
+                        null);
+        completedCheckpointStore.addCheckpointAndSubsumeOldestOne(
+                completedCheckpoint, new CheckpointsCleaner(), () -> {});
 
-        // set up the coordinator and validate the initial state
-        CheckpointCoordinator coord =
+        BooleanValue checked = new BooleanValue(false);
+        CheckpointCoordinator restoreCoordinator =
                 new CheckpointCoordinatorBuilder()
                         .setExecutionGraph(graph)
-                        .setCompletedCheckpointStore(store)
-                        .setTimer(manuallyTriggeredScheduledExecutor)
+                        .setCompletedCheckpointStore(completedCheckpointStore)
+                        .setVertexFinishedStateCheckerFactory(
+                                (vertices, states) ->
+                                        new VertexFinishedStateChecker(vertices, states) {
+                                            @Override
+                                            public void validateOperatorsFinishedState() {
+                                                checked.set(true);
+                                            }
+                                        })
                         .build();
-
-        Set<ExecutionJobVertex> vertices = new HashSet<>();
-        vertices.add(graph.getJobVertex(jobVertexID1));
-
-        thrown.expect(FlinkRuntimeException.class);
-        thrown.expectMessage(
-                "Can not restore vertex "
-                        + "anon("
-                        + jobVertexID1
-                        + ")"
-                        + " which contain mixed operator finished state: [ALL_RUNNING, FULLY_FINISHED]");
-        coord.restoreInitialCheckpointIfPresent(vertices);
+        restoreCoordinator.restoreInitialCheckpointIfPresent(
+                new HashSet<>(graph.getAllVertices().values()));
+        assertTrue(
+                "The finished states should be checked when job is restored on startup",
+                checked.get());
     }
 
     @Test
-    public void testAddingRunningOperatorBeforeFinishedOneFails() throws Exception {
-        JobVertexID jobVertexID2 = new JobVertexID();
-
-        testAddingOperatorsBeforePartiallyOrFullyFinishedOne(
-                new JobVertexID(),
-                "vert1",
-                VertexFinishedState.ALL_RUNNING,
-                jobVertexID2,
-                "vert2",
-                VertexFinishedState.FULLY_FINISHED,
-                new DistributionPattern[] {DistributionPattern.ALL_TO_ALL},
-                FlinkRuntimeException.class,
-                "Illegal JobGraph modification. Cannot run a program with fully finished vertices"
-                        + " predeceased with the ones not fully finished. Task vertex vert2"
-                        + "("
-                        + jobVertexID2
-                        + ")"
-                        + " has a predecessor not fully finished");
-    }
-
-    @Test
-    public void testAddingPartiallyFinishedOperatorBeforeFinishedOneFails() throws Exception {
-        JobVertexID jobVertexID2 = new JobVertexID();
-
-        testAddingOperatorsBeforePartiallyOrFullyFinishedOne(
-                new JobVertexID(),
-                "vert1",
-                VertexFinishedState.PARTIALLY_FINISHED,
-                jobVertexID2,
-                "vert2",
-                VertexFinishedState.FULLY_FINISHED,
-                new DistributionPattern[] {DistributionPattern.ALL_TO_ALL},
-                FlinkRuntimeException.class,
-                "Illegal JobGraph modification. Cannot run a program with fully finished vertices"
-                        + " predeceased with the ones not fully finished. Task vertex vert2"
-                        + "("
-                        + jobVertexID2
-                        + ")"
-                        + " has a predecessor not fully finished");
-    }
-
-    @Test
-    public void testAddingAllRunningOperatorBeforePartiallyFinishedOneWithAllToAllFails()
-            throws Exception {
-        JobVertexID jobVertexID2 = new JobVertexID();
-
-        testAddingOperatorsBeforePartiallyOrFullyFinishedOne(
-                new JobVertexID(),
-                "vert1",
-                VertexFinishedState.ALL_RUNNING,
-                jobVertexID2,
-                "vert2",
-                VertexFinishedState.PARTIALLY_FINISHED,
-                new DistributionPattern[] {DistributionPattern.ALL_TO_ALL},
-                FlinkRuntimeException.class,
-                "Illegal JobGraph modification. Cannot run a program with partially finished vertices"
-                        + " predeceased with running or partially finished ones and connected via the ALL_TO_ALL edges. "
-                        + "Task vertex vert2"
-                        + "("
-                        + jobVertexID2
-                        + ")"
-                        + " has a all running predecessor");
-    }
-
-    @Test
-    public void testAddingPartiallyFinishedOperatorBeforePartiallyFinishedOneWithAllToAllFails()
-            throws Exception {
-        JobVertexID jobVertexID2 = new JobVertexID();
-
-        testAddingOperatorsBeforePartiallyOrFullyFinishedOne(
-                new JobVertexID(),
-                "vert1",
-                VertexFinishedState.PARTIALLY_FINISHED,
-                jobVertexID2,
-                "vert2",
-                VertexFinishedState.PARTIALLY_FINISHED,
-                new DistributionPattern[] {DistributionPattern.ALL_TO_ALL},
-                FlinkRuntimeException.class,
-                "Illegal JobGraph modification. Cannot run a program with partially finished vertices"
-                        + " predeceased with running or partially finished ones and connected via the ALL_TO_ALL edges. "
-                        + "Task vertex vert2"
-                        + "("
-                        + jobVertexID2
-                        + ")"
-                        + " has a partially finished predecessor");
-    }
-
-    @Test
-    public void
-            testAddingPartiallyFinishedOperatorBeforePartiallyFinishedOneWithPointwiseAndAllToAllFails()
-                    throws Exception {
-        JobVertexID jobVertexID2 = new JobVertexID();
-
-        testAddingOperatorsBeforePartiallyOrFullyFinishedOne(
-                new JobVertexID(),
-                "vert1",
-                VertexFinishedState.PARTIALLY_FINISHED,
-                jobVertexID2,
-                "vert2",
-                VertexFinishedState.PARTIALLY_FINISHED,
-                new DistributionPattern[] {
-                    DistributionPattern.POINTWISE, DistributionPattern.ALL_TO_ALL
-                },
-                FlinkRuntimeException.class,
-                "Illegal JobGraph modification. Cannot run a program with partially finished vertices"
-                        + " predeceased with running or partially finished ones and connected via the ALL_TO_ALL edges. "
-                        + "Task vertex vert2"
-                        + "("
-                        + jobVertexID2
-                        + ")"
-                        + " has a partially finished predecessor");
-    }
-
-    @Test
-    public void testAddingAllRunningOperatorBeforePartiallyFinishedOneFails() throws Exception {
-        JobVertexID jobVertexID2 = new JobVertexID();
-
-        testAddingOperatorsBeforePartiallyOrFullyFinishedOne(
-                new JobVertexID(),
-                "vert1",
-                VertexFinishedState.ALL_RUNNING,
-                jobVertexID2,
-                "vert2",
-                VertexFinishedState.PARTIALLY_FINISHED,
-                new DistributionPattern[] {DistributionPattern.POINTWISE},
-                FlinkRuntimeException.class,
-                "Illegal JobGraph modification. Cannot run a program with partially finished vertices"
-                        + " predeceased with all running ones. "
-                        + "Task vertex vert2"
-                        + "("
-                        + jobVertexID2
-                        + ")"
-                        + " has a all running predecessor");
-    }
-
-    private void testAddingOperatorsBeforePartiallyOrFullyFinishedOne(
-            JobVertexID firstVertexId,
-            String firstVertexName,
-            VertexFinishedState firstOperatorFinishedState,
-            JobVertexID secondVertexId,
-            String secondVertexName,
-            VertexFinishedState secondOperatorFinishedState,
-            DistributionPattern[] distributionPatterns,
-            Class<? extends Throwable> expectedExceptionalClass,
-            String expectedMessage)
-            throws Exception {
-        OperatorIDPair op1 = OperatorIDPair.generatedIDOnly(new OperatorID());
-        OperatorIDPair op2 = OperatorIDPair.generatedIDOnly(new OperatorID());
-        JobVertex vertex1 = new JobVertex(firstVertexName, firstVertexId, singletonList(op1));
-        JobVertex vertex2 = new JobVertex(secondVertexName, secondVertexId, singletonList(op2));
-        vertex1.setInvokableClass(NoOpInvokable.class);
-        vertex2.setInvokableClass(NoOpInvokable.class);
-
-        final ExecutionGraph graph =
+    public void testJobGraphModificationsAreCheckedForSavepoint() throws Exception {
+        final JobVertexID jobVertexID = new JobVertexID();
+        ExecutionGraph graph =
                 new CheckpointCoordinatorTestingUtils.CheckpointExecutionGraphBuilder()
-                        .addJobVertex(vertex1, true)
-                        .addJobVertex(vertex2, false)
-                        .setDistributionPattern(distributionPatterns[0])
+                        .addJobVertex(jobVertexID, 1, 1)
                         .build();
-
-        // Adds the additional edges
-        for (int i = 1; i < distributionPatterns.length; ++i) {
-            vertex2.connectNewDataSetAsInput(
-                    vertex1, distributionPatterns[i], ResultPartitionType.PIPELINED);
-        }
-
-        Map<OperatorID, OperatorState> operatorStates = new HashMap<>();
-        operatorStates.put(
-                op1.getGeneratedOperatorID(),
-                createOperatorState(op1.getGeneratedOperatorID(), firstOperatorFinishedState));
-        operatorStates.put(
-                op2.getGeneratedOperatorID(),
-                createOperatorState(op2.getGeneratedOperatorID(), secondOperatorFinishedState));
-
-        CompletedCheckpointStore store = new EmbeddedCompletedCheckpointStore();
-        store.addCheckpoint(
-                new CompletedCheckpoint(
-                        graph.getJobID(),
-                        2,
-                        System.currentTimeMillis(),
-                        System.currentTimeMillis() + 3000,
-                        operatorStates,
-                        Collections.emptyList(),
-                        CheckpointProperties.forCheckpoint(
-                                CheckpointRetentionPolicy.NEVER_RETAIN_AFTER_TERMINATION),
-                        new TestCompletedCheckpointStorageLocation()),
-                new CheckpointsCleaner(),
-                () -> {});
-
-        // set up the coordinator and validate the initial state
-        CheckpointCoordinator coord =
+        CheckpointCoordinator coordinator =
                 new CheckpointCoordinatorBuilder()
                         .setExecutionGraph(graph)
-                        .setCompletedCheckpointStore(store)
                         .setTimer(manuallyTriggeredScheduledExecutor)
                         .build();
+        File savepointPath = tmpFolder.newFolder();
+        CompletableFuture<CompletedCheckpoint> savepointFuture =
+                coordinator.triggerSavepoint(
+                        "file://" + savepointPath.getAbsolutePath(), SavepointFormatType.CANONICAL);
+        manuallyTriggeredScheduledExecutor.triggerAll();
+        long pendingSavepointId =
+                coordinator.getPendingCheckpoints().keySet().stream().findFirst().get();
+        coordinator.receiveAcknowledgeMessage(
+                new AcknowledgeCheckpoint(
+                        graph.getJobID(),
+                        graph.getJobVertex(jobVertexID)
+                                .getTaskVertices()[0]
+                                .getCurrentExecutionAttempt()
+                                .getAttemptId(),
+                        pendingSavepointId),
+                "localhost");
+        assertTrue(savepointFuture.isDone());
 
-        Set<ExecutionJobVertex> vertices = new HashSet<>();
-        vertices.add(graph.getJobVertex(vertex1.getID()));
-        vertices.add(graph.getJobVertex(vertex2.getID()));
-
-        thrown.expect(expectedExceptionalClass);
-        thrown.expectMessage(expectedMessage);
-
-        coord.restoreInitialCheckpointIfPresent(vertices);
-    }
-
-    private OperatorState createOperatorState(
-            OperatorID operatorId, VertexFinishedState finishedState) {
-        switch (finishedState) {
-            case ALL_RUNNING:
-                return new OperatorState(operatorId, 2, 2);
-            case PARTIALLY_FINISHED:
-                OperatorState operatorState = new OperatorState(operatorId, 2, 2);
-                operatorState.putState(0, FinishedOperatorSubtaskState.INSTANCE);
-                return operatorState;
-            case FULLY_FINISHED:
-                return new FullyFinishedOperatorState(operatorId, 2, 2);
-            default:
-                throw new UnsupportedOperationException(
-                        "Not supported finished state: " + finishedState);
-        }
+        BooleanValue checked = new BooleanValue(false);
+        CheckpointCoordinator restoreCoordinator =
+                new CheckpointCoordinatorBuilder()
+                        .setExecutionGraph(graph)
+                        .setVertexFinishedStateCheckerFactory(
+                                (vertices, states) ->
+                                        new VertexFinishedStateChecker(vertices, states) {
+                                            @Override
+                                            public void validateOperatorsFinishedState() {
+                                                checked.set(true);
+                                            }
+                                        })
+                        .build();
+        restoreCoordinator.restoreSavepoint(
+                SavepointRestoreSettings.forPath(savepointFuture.get().getExternalPointer()),
+                graph.getAllVertices(),
+                getClass().getClassLoader());
+        assertTrue(
+                "The finished states should be checked when job is restored on startup",
+                checked.get());
     }
 }
