@@ -19,11 +19,13 @@
 package org.apache.flink.runtime.checkpoint;
 
 import org.apache.flink.api.common.JobID;
+import org.apache.flink.core.execution.SavepointFormatType;
 import org.apache.flink.runtime.OperatorIDPair;
 import org.apache.flink.runtime.checkpoint.metadata.CheckpointMetadata;
 import org.apache.flink.runtime.executiongraph.ExecutionJobVertex;
 import org.apache.flink.runtime.jobgraph.JobVertexID;
 import org.apache.flink.runtime.jobgraph.OperatorID;
+import org.apache.flink.runtime.jobgraph.RestoreMode;
 import org.apache.flink.runtime.state.CompletedCheckpointStorageLocation;
 import org.apache.flink.runtime.state.OperatorStreamStateHandle;
 import org.apache.flink.runtime.state.StreamStateHandle;
@@ -71,7 +73,14 @@ public class CheckpointMetadataLoadingTest {
                 createTasks(operatorId, parallelism, parallelism);
 
         final CompletedCheckpoint loaded =
-                Checkpoints.loadAndValidateCheckpoint(jobId, tasks, testSavepoint, cl, false);
+                Checkpoints.loadAndValidateCheckpoint(
+                        jobId,
+                        tasks,
+                        testSavepoint,
+                        cl,
+                        false,
+                        CheckpointProperties.forSavepoint(false, SavepointFormatType.CANONICAL),
+                        RestoreMode.NO_CLAIM);
 
         assertEquals(jobId, loaded.getJobId());
         assertEquals(checkpointId, loaded.getCheckpointID());
@@ -89,7 +98,14 @@ public class CheckpointMetadataLoadingTest {
                 createTasks(operatorId, parallelism, parallelism + 1);
 
         try {
-            Checkpoints.loadAndValidateCheckpoint(new JobID(), tasks, testSavepoint, cl, false);
+            Checkpoints.loadAndValidateCheckpoint(
+                    new JobID(),
+                    tasks,
+                    testSavepoint,
+                    cl,
+                    false,
+                    CheckpointProperties.forSavepoint(false, SavepointFormatType.CANONICAL),
+                    RestoreMode.NO_CLAIM);
             fail("Did not throw expected Exception");
         } catch (IllegalStateException expected) {
             assertTrue(expected.getMessage().contains("Max parallelism mismatch"));
@@ -109,7 +125,14 @@ public class CheckpointMetadataLoadingTest {
         final Map<JobVertexID, ExecutionJobVertex> tasks = Collections.emptyMap();
 
         try {
-            Checkpoints.loadAndValidateCheckpoint(new JobID(), tasks, testSavepoint, cl, false);
+            Checkpoints.loadAndValidateCheckpoint(
+                    new JobID(),
+                    tasks,
+                    testSavepoint,
+                    cl,
+                    false,
+                    CheckpointProperties.forSavepoint(false, SavepointFormatType.CANONICAL),
+                    RestoreMode.NO_CLAIM);
             fail("Did not throw expected Exception");
         } catch (IllegalStateException expected) {
             assertTrue(expected.getMessage().contains("allowNonRestoredState"));
@@ -129,7 +152,14 @@ public class CheckpointMetadataLoadingTest {
         final Map<JobVertexID, ExecutionJobVertex> tasks = Collections.emptyMap();
 
         final CompletedCheckpoint loaded =
-                Checkpoints.loadAndValidateCheckpoint(new JobID(), tasks, testSavepoint, cl, true);
+                Checkpoints.loadAndValidateCheckpoint(
+                        new JobID(),
+                        tasks,
+                        testSavepoint,
+                        cl,
+                        true,
+                        CheckpointProperties.forSavepoint(false, SavepointFormatType.CANONICAL),
+                        RestoreMode.NO_CLAIM);
 
         assertTrue(loaded.getOperatorStates().isEmpty());
     }
@@ -152,7 +182,14 @@ public class CheckpointMetadataLoadingTest {
         final Map<JobVertexID, ExecutionJobVertex> tasks = Collections.emptyMap();
 
         try {
-            Checkpoints.loadAndValidateCheckpoint(new JobID(), tasks, testSavepoint, cl, false);
+            Checkpoints.loadAndValidateCheckpoint(
+                    new JobID(),
+                    tasks,
+                    testSavepoint,
+                    cl,
+                    false,
+                    CheckpointProperties.forSavepoint(false, SavepointFormatType.CANONICAL),
+                    RestoreMode.NO_CLAIM);
             fail("Did not throw expected Exception");
         } catch (IllegalStateException expected) {
             assertTrue(expected.getMessage().contains("allowNonRestoredState"));
