@@ -18,13 +18,14 @@
 
 package org.apache.flink.runtime.dispatcher;
 
+import org.apache.flink.api.common.JobID;
 import org.apache.flink.api.common.JobStatus;
 import org.apache.flink.api.common.time.Time;
-import org.apache.flink.configuration.BlobServerOptions;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.runtime.blob.BlobServer;
 import org.apache.flink.runtime.blob.VoidBlobStore;
 import org.apache.flink.runtime.clusterframework.ApplicationStatus;
+import org.apache.flink.runtime.dispatcher.cleanup.TestingCleanupRunnerFactory;
 import org.apache.flink.runtime.entrypoint.ClusterEntrypoint;
 import org.apache.flink.runtime.heartbeat.HeartbeatServices;
 import org.apache.flink.runtime.highavailability.TestingHighAvailabilityServices;
@@ -39,9 +40,11 @@ import org.apache.flink.runtime.rest.handler.legacy.utils.ArchivedExecutionGraph
 import org.apache.flink.runtime.rpc.RpcUtils;
 import org.apache.flink.runtime.rpc.TestingRpcService;
 import org.apache.flink.runtime.scheduler.ExecutionGraphInfo;
+import org.apache.flink.runtime.testutils.TestingJobResultStore;
 import org.apache.flink.runtime.util.TestingFatalErrorHandlerResource;
 import org.apache.flink.util.TestLogger;
 
+import org.assertj.core.api.Assertions;
 import org.junit.AfterClass;
 import org.junit.Before;
 import org.junit.BeforeClass;
@@ -50,7 +53,7 @@ import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
 
-import javax.annotation.Nonnull;
+import javax.annotation.Nullable;
 
 import java.io.IOException;
 import java.util.concurrent.CompletableFuture;
@@ -93,7 +96,8 @@ public class MiniDispatcherTest extends TestLogger {
 
     private TestingHighAvailabilityServices highAvailabilityServices;
 
-    private TestingJobManagerRunnerFactory testingJobManagerRunnerFactory;
+    private TestingJobMasterServiceLeadershipRunnerFactory testingJobManagerRunnerFactory;
+    private TestingCleanupRunnerFactory testingCleanupRunnerFactory;
 
     @BeforeClass
     public static void setupClass() throws IOException {
@@ -109,17 +113,16 @@ public class MiniDispatcherTest extends TestLogger {
         rpcService = new TestingRpcService();
         configuration = new Configuration();
 
-        configuration.setString(
-                BlobServerOptions.STORAGE_DIRECTORY, temporaryFolder.newFolder().getAbsolutePath());
-
-        blobServer = new BlobServer(configuration, new VoidBlobStore());
+        blobServer =
+                new BlobServer(configuration, temporaryFolder.newFolder(), new VoidBlobStore());
     }
 
     @Before
     public void setup() throws Exception {
         highAvailabilityServices = new TestingHighAvailabilityServicesBuilder().build();
 
-        testingJobManagerRunnerFactory = new TestingJobManagerRunnerFactory();
+        testingJobManagerRunnerFactory = new TestingJobMasterServiceLeadershipRunnerFactory();
+        testingCleanupRunnerFactory = new TestingCleanupRunnerFactory();
     }
 
     @AfterClass
@@ -152,6 +155,27 @@ public class MiniDispatcherTest extends TestLogger {
         }
     }
 
+    /** Tests that the {@link MiniDispatcher} recovers the single job with which it was started. */
+    @Test
+    public void testDirtyJobResultCleanup() throws Exception {
+        final JobID jobId = new JobID();
+        final MiniDispatcher miniDispatcher =
+                createMiniDispatcher(
+                        ClusterEntrypoint.ExecutionMode.DETACHED,
+                        null,
+                        TestingJobResultStore.createSuccessfulJobResult(jobId));
+
+        miniDispatcher.start();
+
+        try {
+            final TestingJobManagerRunner testingCleanupRunner =
+                    testingCleanupRunnerFactory.takeCreatedJobManagerRunner();
+            assertThat(testingCleanupRunner.getJobID(), is(jobId));
+        } finally {
+            RpcUtils.terminateRpcEndpoint(miniDispatcher, timeout);
+        }
+    }
+
     /**
      * Tests that in detached mode, the {@link MiniDispatcher} will complete the future that signals
      * job termination.
@@ -200,7 +224,7 @@ public class MiniDispatcherTest extends TestLogger {
                                     .build()));
 
             testingJobManagerRunner.getTerminationFuture().get();
-            assertThat(miniDispatcher.getShutDownFuture().isDone(), is(false));
+            Assertions.assertThat(miniDispatcher.getShutDownFuture()).isNotDone();
         } finally {
             RpcUtils.terminateRpcEndpoint(miniDispatcher, timeout);
         }
@@ -276,9 +300,16 @@ public class MiniDispatcherTest extends TestLogger {
     // Utilities
     // --------------------------------------------------------
 
-    @Nonnull
     private MiniDispatcher createMiniDispatcher(ClusterEntrypoint.ExecutionMode executionMode)
             throws Exception {
+        return createMiniDispatcher(executionMode, jobGraph, null);
+    }
+
+    private MiniDispatcher createMiniDispatcher(
+            ClusterEntrypoint.ExecutionMode executionMode,
+            @Nullable JobGraph recoveredJobGraph,
+            @Nullable JobResult recoveredDirtyJob)
+            throws Exception {
         return new MiniDispatcher(
                 rpcService,
                 DispatcherId.generate(),
@@ -292,11 +323,15 @@ public class MiniDispatcherTest extends TestLogger {
                         testingFatalErrorHandlerResource.getFatalErrorHandler(),
                         VoidHistoryServerArchivist.INSTANCE,
                         null,
+                        new DispatcherOperationCaches(),
                         UnregisteredMetricGroups.createUnregisteredJobManagerMetricGroup(),
                         highAvailabilityServices.getJobGraphStore(),
+                        highAvailabilityServices.getJobResultStore(),
                         testingJobManagerRunnerFactory,
+                        testingCleanupRunnerFactory,
                         ForkJoinPool.commonPool()),
-                jobGraph,
+                recoveredJobGraph,
+                recoveredDirtyJob,
                 (dispatcher, scheduledExecutor, errorHandler) -> new NoOpDispatcherBootstrap(),
                 executionMode);
     }
