@@ -19,16 +19,13 @@
 package org.apache.flink.runtime.io.network.buffer;
 
 import org.apache.flink.core.memory.MemorySegment;
+import org.apache.flink.runtime.execution.CancelTaskException;
 import org.apache.flink.util.TestLogger;
 
-import org.apache.flink.shaded.guava30.com.google.common.collect.Lists;
-
 import org.junit.After;
 import org.junit.AfterClass;
 import org.junit.Before;
-import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.Timeout;
 import org.mockito.Mockito;
 
 import javax.annotation.Nullable;
@@ -41,12 +38,10 @@ import java.util.Deque;
 import java.util.List;
 import java.util.concurrent.Callable;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
-import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import static org.apache.flink.util.Preconditions.checkNotNull;
@@ -56,7 +51,6 @@ import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
 import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.verify;
 
@@ -73,8 +67,6 @@ public class LocalBufferPoolTest extends TestLogger {
 
     private static final ExecutorService executor = Executors.newCachedThreadPool();
 
-    @Rule public Timeout timeout = new Timeout(10, TimeUnit.SECONDS);
-
     @Before
     public void setupLocalBufferPool() throws Exception {
         networkBufferPool = new NetworkBufferPool(numBuffers, memorySegmentSize);
@@ -129,7 +121,7 @@ public class LocalBufferPoolTest extends TestLogger {
             assertEquals(2, bufferPool3.getNumberOfAvailableMemorySegments());
 
             bufferPool3.lazyDestroy();
-            assertThrows(IllegalStateException.class, () -> bufferPool3.reserveSegments(1));
+            assertThrows(CancelTaskException.class, () -> bufferPool3.reserveSegments(1));
         } finally {
             networkBufferPool.destroy();
         }
@@ -221,18 +213,6 @@ public class LocalBufferPoolTest extends TestLogger {
         }
     }
 
-    @Test
-    public void testRequestAfterDestroy() {
-        localBufferPool.lazyDestroy();
-
-        try {
-            localBufferPool.requestBuffer();
-            fail("Call should have failed with an IllegalStateException");
-        } catch (IllegalStateException e) {
-            // we expect exactly that
-        }
-    }
-
     @Test
     public void testSetNumAfterDestroyDoesNotProactivelyFetchSegments() {
         localBufferPool.setNumBuffers(2);
@@ -396,63 +376,6 @@ public class LocalBufferPoolTest extends TestLogger {
         }
     }
 
-    @Test
-    public void testDestroyDuringBlockingRequest() throws Exception {
-        // Config
-        final int numberOfBuffers = 1;
-
-        localBufferPool.setNumBuffers(numberOfBuffers);
-
-        final CountDownLatch sync = new CountDownLatch(1);
-
-        final Callable<List<Buffer>> requester =
-                new Callable<List<Buffer>>() {
-
-                    // Request all buffers in a blocking manner.
-                    @Override
-                    public List<Buffer> call() throws Exception {
-                        final List<Buffer> requested = Lists.newArrayList();
-
-                        // Request all available buffers
-                        for (int i = 0; i < numberOfBuffers; i++) {
-                            final Buffer buffer = checkNotNull(localBufferPool.requestBuffer());
-                            requested.add(buffer);
-                        }
-
-                        // Notify that we've requested all buffers
-                        sync.countDown();
-
-                        // Try to request the next buffer (but pool should be destroyed either right
-                        // before
-                        // the request or more likely during the request).
-                        try {
-                            localBufferPool.requestBufferBuilderBlocking();
-                            fail("Call should have failed with an IllegalStateException");
-                        } catch (IllegalStateException e) {
-                            // we expect exactly that
-                        }
-
-                        return requested;
-                    }
-                };
-
-        Future<List<Buffer>> f = executor.submit(requester);
-
-        sync.await();
-
-        localBufferPool.lazyDestroy();
-
-        // Increase the likelihood that the requested is currently in the request call
-        Thread.sleep(50);
-
-        // This should return immediately if everything works as expected
-        List<Buffer> requestedBuffers = f.get(60, TimeUnit.SECONDS);
-
-        for (Buffer buffer : requestedBuffers) {
-            buffer.recycleBuffer();
-        }
-    }
-
     @Test
     public void testBoundedBuffer() throws Exception {
         localBufferPool.lazyDestroy();
