@@ -18,9 +18,11 @@
 
 package org.apache.flink.runtime.io.network.buffer;
 
+import org.apache.flink.core.fs.CloseableRegistry;
 import org.apache.flink.core.memory.MemorySegment;
 import org.apache.flink.core.testutils.CheckedThread;
 import org.apache.flink.core.testutils.OneShotLatch;
+import org.apache.flink.runtime.execution.CancelTaskException;
 import org.apache.flink.util.TestLogger;
 
 import org.junit.Test;
@@ -31,6 +33,7 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
+import java.util.Optional;
 import java.util.concurrent.ArrayBlockingQueue;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.CompletableFuture;
@@ -44,6 +47,7 @@ import static org.hamcrest.CoreMatchers.instanceOf;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.nullValue;
 import static org.hamcrest.MatcherAssert.assertThat;
+import static org.hamcrest.Matchers.equalTo;
 import static org.hamcrest.Matchers.hasProperty;
 import static org.hamcrest.core.IsCollectionContaining.hasItem;
 import static org.hamcrest.core.IsNot.not;
@@ -213,22 +217,22 @@ public class NetworkBufferPoolTest extends TestLogger {
         // can request no more buffers
         try {
             fixedPool.requestBuffer();
-            fail("Should fail with an IllegalStateException");
-        } catch (IllegalStateException e) {
+            fail("Should fail with an CancelTaskException");
+        } catch (CancelTaskException e) {
             // yippie!
         }
 
         try {
             boundedPool.requestBuffer();
-            fail("Should fail with an IllegalStateException");
-        } catch (IllegalStateException e) {
+            fail("Should fail with an CancelTaskException");
+        } catch (CancelTaskException e) {
             // that's the way we like it, aha, aha
         }
 
         try {
             nonFixedPool.requestBuffer();
-            fail("Should fail with an IllegalStateException");
-        } catch (IllegalStateException e) {
+            fail("Should fail with an CancelTaskException");
+        } catch (CancelTaskException e) {
             // stayin' alive
         }
 
@@ -321,6 +325,71 @@ public class NetworkBufferPoolTest extends TestLogger {
         }
     }
 
+    @Test
+    public void testSegmentsUsage() throws IOException {
+        try (CloseableRegistry closeableRegistry = new CloseableRegistry()) {
+            NetworkBufferPool globalPool = new NetworkBufferPool(50, 128);
+            closeableRegistry.registerCloseable(globalPool::destroy);
+
+            BufferPool bufferPool1 = globalPool.createBufferPool(10, 20);
+
+            assertEquals(20, globalPool.getNumberOfRequestedMemorySegments());
+            assertEquals(40, globalPool.getRequestedSegmentsUsage());
+            assertThat(globalPool.getUsageWarning(), equalTo(Optional.empty()));
+
+            closeableRegistry.registerCloseable(
+                    (globalPool.createBufferPool(5, Integer.MAX_VALUE))::lazyDestroy);
+
+            assertEquals(30, globalPool.getNumberOfRequestedMemorySegments());
+            assertEquals(60, globalPool.getRequestedSegmentsUsage());
+            assertThat(globalPool.getUsageWarning(), equalTo(Optional.empty()));
+
+            closeableRegistry.registerCloseable((globalPool.createBufferPool(10, 30))::lazyDestroy);
+
+            assertEquals(60, globalPool.getNumberOfRequestedMemorySegments());
+            assertEquals(120, globalPool.getRequestedSegmentsUsage());
+            assertThat(
+                    globalPool.getUsageWarning(),
+                    equalTo(
+                            Optional.of(
+                                    "Memory usage [120%] is too high to satisfy all of the requests. "
+                                            + "This can severely impact network throughput. "
+                                            + "Please consider increasing available network memory, "
+                                            + "or decreasing configured size of network buffer pools. ("
+                                            + "totalMemory=6.250kb (6400 bytes), "
+                                            + "requestedMemory=7.500kb (7680 bytes), "
+                                            + "missingMemory=1.250kb (1280 bytes))")));
+            assertThat(globalPool.getUsageWarning(), equalTo(Optional.empty()));
+
+            BufferPool bufferPool2 = globalPool.createBufferPool(10, 20);
+
+            assertEquals(80, globalPool.getNumberOfRequestedMemorySegments());
+            assertEquals(160, globalPool.getRequestedSegmentsUsage());
+            assertThat(
+                    globalPool.getUsageWarning(),
+                    equalTo(
+                            Optional.of(
+                                    "Memory usage [160%] is too high to satisfy all of the requests. "
+                                            + "This can severely impact network throughput. "
+                                            + "Please consider increasing available network memory, "
+                                            + "or decreasing configured size of network buffer pools. ("
+                                            + "totalMemory=6.250kb (6400 bytes), "
+                                            + "requestedMemory=10.000kb (10240 bytes), "
+                                            + "missingMemory=3.750kb (3840 bytes))")));
+
+            bufferPool2.lazyDestroy();
+            bufferPool1.lazyDestroy();
+
+            assertEquals(40, globalPool.getNumberOfRequestedMemorySegments());
+            assertEquals(40 * 128, globalPool.getRequestedMemory());
+            assertEquals(80, globalPool.getRequestedSegmentsUsage());
+            assertThat(
+                    globalPool.getUsageWarning(),
+                    equalTo(Optional.of("Memory usage [80%] went back to normal")));
+            assertThat(globalPool.getUsageWarning(), equalTo(Optional.empty()));
+        }
+    }
+
     /**
      * Tests {@link NetworkBufferPool#requestUnpooledMemorySegments(int)} with the invalid argument
      * to cause exception.
