@@ -18,8 +18,10 @@
 
 package org.apache.flink.runtime.io.network.partition.consumer;
 
+import org.apache.flink.configuration.Configuration;
 import org.apache.flink.core.memory.MemorySegmentProvider;
 import org.apache.flink.runtime.checkpoint.channel.ChannelStateWriter;
+import org.apache.flink.runtime.deployment.SubpartitionIndexRange;
 import org.apache.flink.runtime.io.network.NettyShuffleEnvironment;
 import org.apache.flink.runtime.io.network.buffer.BufferDecompressor;
 import org.apache.flink.runtime.io.network.buffer.BufferPool;
@@ -29,12 +31,17 @@ import org.apache.flink.runtime.io.network.partition.PartitionProducerStateProvi
 import org.apache.flink.runtime.io.network.partition.ResultPartitionType;
 import org.apache.flink.runtime.jobgraph.IntermediateDataSetID;
 import org.apache.flink.runtime.taskmanager.NettyShuffleEnvironmentConfiguration;
+import org.apache.flink.runtime.throughput.BufferDebloatConfiguration;
+import org.apache.flink.runtime.throughput.BufferDebloater;
+import org.apache.flink.runtime.throughput.ThroughputCalculator;
+import org.apache.flink.util.clock.SystemClock;
 import org.apache.flink.util.function.SupplierWithException;
 
 import javax.annotation.Nullable;
 
 import java.io.IOException;
 import java.util.function.BiFunction;
+import java.util.function.Function;
 import java.util.stream.IntStream;
 
 /** Utility class to encapsulate the logic of building a {@link SingleInputGate} instance. */
@@ -49,7 +56,7 @@ public class SingleInputGateBuilder {
 
     private ResultPartitionType partitionType = ResultPartitionType.PIPELINED;
 
-    private int consumedSubpartitionIndex = 0;
+    private SubpartitionIndexRange subpartitionIndexRange = new SubpartitionIndexRange(0, 0);
 
     private int gateIndex = 0;
 
@@ -68,6 +75,10 @@ public class SingleInputGateBuilder {
     private BiFunction<InputChannelBuilder, SingleInputGate, InputChannel> channelFactory = null;
 
     private SupplierWithException<BufferPool, IOException> bufferPoolFactory = NoOpBufferPool::new;
+    private BufferDebloatConfiguration bufferDebloatConfiguration =
+            BufferDebloatConfiguration.fromConfiguration(new Configuration());
+    private Function<BufferDebloatConfiguration, ThroughputCalculator> createThroughputCalculator =
+            config -> new ThroughputCalculator(SystemClock.getInstance());
 
     public SingleInputGateBuilder setPartitionProducerStateProvider(
             PartitionProducerStateProvider partitionProducerStateProvider) {
@@ -81,8 +92,9 @@ public class SingleInputGateBuilder {
         return this;
     }
 
-    public SingleInputGateBuilder setConsumedSubpartitionIndex(int consumedSubpartitionIndex) {
-        this.consumedSubpartitionIndex = consumedSubpartitionIndex;
+    public SingleInputGateBuilder setSubpartitionIndexRange(
+            SubpartitionIndexRange subpartitionIndexRange) {
+        this.subpartitionIndexRange = subpartitionIndexRange;
         return this;
     }
 
@@ -132,6 +144,18 @@ public class SingleInputGateBuilder {
         return this;
     }
 
+    public SingleInputGateBuilder setBufferDebloatConfiguration(
+            BufferDebloatConfiguration configuration) {
+        this.bufferDebloatConfiguration = configuration;
+        return this;
+    }
+
+    public SingleInputGateBuilder setThroughputCalculator(
+            Function<BufferDebloatConfiguration, ThroughputCalculator> createThroughputCalculator) {
+        this.createThroughputCalculator = createThroughputCalculator;
+        return this;
+    }
+
     public SingleInputGate build() {
         SingleInputGate gate =
                 new SingleInputGate(
@@ -139,13 +163,15 @@ public class SingleInputGateBuilder {
                         gateIndex,
                         intermediateDataSetID,
                         partitionType,
-                        consumedSubpartitionIndex,
+                        subpartitionIndexRange,
                         numberOfChannels,
                         partitionProducerStateProvider,
                         bufferPoolFactory,
                         bufferDecompressor,
                         segmentProvider,
-                        bufferSize);
+                        bufferSize,
+                        createThroughputCalculator.apply(bufferDebloatConfiguration),
+                        maybeCreateBufferDebloater(gateIndex));
         if (channelFactory != null) {
             gate.setInputChannels(
                     IntStream.range(0, numberOfChannels)
@@ -160,4 +186,18 @@ public class SingleInputGateBuilder {
         }
         return gate;
     }
+
+    private BufferDebloater maybeCreateBufferDebloater(int gateIndex) {
+        if (bufferDebloatConfiguration.isEnabled()) {
+            return new BufferDebloater(
+                    gateIndex,
+                    bufferDebloatConfiguration.getTargetTotalBufferSize().toMillis(),
+                    bufferDebloatConfiguration.getMaxBufferSize(),
+                    bufferDebloatConfiguration.getMinBufferSize(),
+                    bufferDebloatConfiguration.getBufferDebloatThresholdPercentages(),
+                    bufferDebloatConfiguration.getNumberOfSamples());
+        }
+
+        return null;
+    }
 }
