@@ -39,11 +39,15 @@ import org.apache.flink.util.clock.SystemClock;
 import org.apache.flink.util.concurrent.FutureUtils;
 
 import org.junit.Test;
+import org.junit.runner.RunWith;
+import org.junit.runners.Parameterized;
 
 import javax.annotation.Nonnull;
 
+import java.io.IOException;
 import java.time.Duration;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.CompletableFuture;
@@ -55,6 +59,7 @@ import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.Assert.fail;
 
 /** Tests for the {@link DeclarativeSlotPoolBridge}. */
+@RunWith(Parameterized.class)
 public class DeclarativeSlotPoolBridgeTest extends TestLogger {
 
     private static final Time rpcTimeout = Time.seconds(20);
@@ -62,6 +67,18 @@ public class DeclarativeSlotPoolBridgeTest extends TestLogger {
     private static final JobMasterId jobMasterId = JobMasterId.generate();
     private final ComponentMainThreadExecutor mainThreadExecutor =
             ComponentMainThreadExecutorServiceAdapter.forMainThread();
+    private final RequestSlotMatchingStrategy requestSlotMatchingStrategy;
+
+    @Parameterized.Parameters(name = "RequestSlotMatchingStrategy: {0}")
+    public static Collection<RequestSlotMatchingStrategy> data() throws IOException {
+        return Arrays.asList(
+                SimpleRequestSlotMatchingStrategy.INSTANCE,
+                PreferredAllocationRequestSlotMatchingStrategy.INSTANCE);
+    }
+
+    public DeclarativeSlotPoolBridgeTest(RequestSlotMatchingStrategy requestSlotMatchingStrategy) {
+        this.requestSlotMatchingStrategy = requestSlotMatchingStrategy;
+    }
 
     @Test
     public void testSlotOffer() throws Exception {
@@ -72,7 +89,8 @@ public class DeclarativeSlotPoolBridgeTest extends TestLogger {
         final TestingDeclarativeSlotPoolFactory declarativeSlotPoolFactory =
                 new TestingDeclarativeSlotPoolFactory(TestingDeclarativeSlotPool.builder());
         try (DeclarativeSlotPoolBridge declarativeSlotPoolBridge =
-                createDeclarativeSlotPoolBridge(declarativeSlotPoolFactory)) {
+                createDeclarativeSlotPoolBridge(
+                        declarativeSlotPoolFactory, requestSlotMatchingStrategy)) {
 
             declarativeSlotPoolBridge.start(jobMasterId, "localhost", mainThreadExecutor);
 
@@ -93,7 +111,8 @@ public class DeclarativeSlotPoolBridgeTest extends TestLogger {
         final TestingDeclarativeSlotPoolFactory declarativeSlotPoolFactory =
                 new TestingDeclarativeSlotPoolFactory(TestingDeclarativeSlotPool.builder());
         try (DeclarativeSlotPoolBridge declarativeSlotPoolBridge =
-                createDeclarativeSlotPoolBridge(declarativeSlotPoolFactory)) {
+                createDeclarativeSlotPoolBridge(
+                        declarativeSlotPoolFactory, requestSlotMatchingStrategy)) {
 
             declarativeSlotPoolBridge.start(jobMasterId, "localhost", mainThreadExecutor);
 
@@ -141,7 +160,8 @@ public class DeclarativeSlotPoolBridgeTest extends TestLogger {
         final TestingDeclarativeSlotPoolFactory declarativeSlotPoolFactory =
                 new TestingDeclarativeSlotPoolFactory(builder);
         try (DeclarativeSlotPoolBridge declarativeSlotPoolBridge =
-                createDeclarativeSlotPoolBridge(declarativeSlotPoolFactory)) {
+                createDeclarativeSlotPoolBridge(
+                        declarativeSlotPoolFactory, requestSlotMatchingStrategy)) {
             declarativeSlotPoolBridge.start(jobMasterId, "localhost", mainThreadExecutor);
 
             final SlotRequestId slotRequestId = new SlotRequestId();
@@ -157,7 +177,8 @@ public class DeclarativeSlotPoolBridgeTest extends TestLogger {
     @Test
     public void testNoConcurrentModificationWhenSuspendingAndReleasingSlot() throws Exception {
         try (DeclarativeSlotPoolBridge declarativeSlotPoolBridge =
-                createDeclarativeSlotPoolBridge(new DefaultDeclarativeSlotPoolFactory())) {
+                createDeclarativeSlotPoolBridge(
+                        new DefaultDeclarativeSlotPoolFactory(), requestSlotMatchingStrategy)) {
 
             declarativeSlotPoolBridge.start(jobMasterId, "localhost", mainThreadExecutor);
 
@@ -198,7 +219,8 @@ public class DeclarativeSlotPoolBridgeTest extends TestLogger {
     @Test
     public void testAcceptingOfferedSlotsWithoutResourceManagerConnected() throws Exception {
         try (DeclarativeSlotPoolBridge declarativeSlotPoolBridge =
-                createDeclarativeSlotPoolBridge(new DefaultDeclarativeSlotPoolFactory())) {
+                createDeclarativeSlotPoolBridge(
+                        new DefaultDeclarativeSlotPoolFactory(), requestSlotMatchingStrategy)) {
 
             declarativeSlotPoolBridge.start(jobMasterId, "localhost", mainThreadExecutor);
 
@@ -220,16 +242,53 @@ public class DeclarativeSlotPoolBridgeTest extends TestLogger {
         }
     }
 
+    @Test
+    public void testIfJobIsRestartingAllOfferedSlotsWillBeRegistered() throws Exception {
+        final CompletableFuture<Void> registerSlotsCalledFuture = new CompletableFuture<>();
+        final TestingDeclarativeSlotPoolFactory declarativeSlotPoolFactory =
+                new TestingDeclarativeSlotPoolFactory(
+                        TestingDeclarativeSlotPool.builder()
+                                .setRegisterSlotsFunction(
+                                        (slotOffers,
+                                                taskManagerLocation,
+                                                taskManagerGateway,
+                                                aLong) ->
+                                                registerSlotsCalledFuture.complete(null)));
+
+        try (DeclarativeSlotPoolBridge declarativeSlotPoolBridge =
+                createDeclarativeSlotPoolBridge(
+                        declarativeSlotPoolFactory, requestSlotMatchingStrategy)) {
+            declarativeSlotPoolBridge.start(jobMasterId, "localhost", mainThreadExecutor);
+
+            declarativeSlotPoolBridge.setIsJobRestarting(true);
+
+            final LocalTaskManagerLocation localTaskManagerLocation =
+                    new LocalTaskManagerLocation();
+            declarativeSlotPoolBridge.registerTaskManager(localTaskManagerLocation.getResourceID());
+
+            declarativeSlotPoolBridge.offerSlots(
+                    localTaskManagerLocation,
+                    new SimpleAckingTaskManagerGateway(),
+                    Collections.singleton(
+                            new SlotOffer(new AllocationID(), 0, ResourceProfile.ANY)));
+
+            // make sure that the register slots method is called
+            registerSlotsCalledFuture.join();
+        }
+    }
+
     @Nonnull
     static DeclarativeSlotPoolBridge createDeclarativeSlotPoolBridge(
-            DeclarativeSlotPoolFactory declarativeSlotPoolFactory) {
+            DeclarativeSlotPoolFactory declarativeSlotPoolFactory,
+            RequestSlotMatchingStrategy requestSlotMatchingStrategy) {
         return new DeclarativeSlotPoolBridge(
                 jobId,
                 declarativeSlotPoolFactory,
                 SystemClock.getInstance(),
                 rpcTimeout,
                 Time.seconds(20),
-                Time.seconds(20));
+                Time.seconds(20),
+                requestSlotMatchingStrategy);
     }
 
     static PhysicalSlot createAllocatedSlot(AllocationID allocationID) {
