@@ -21,9 +21,12 @@ package org.apache.flink.runtime.metrics.groups;
 import org.apache.flink.metrics.Counter;
 import org.apache.flink.metrics.SimpleCounter;
 import org.apache.flink.runtime.executiongraph.IOMetrics;
+import org.apache.flink.runtime.jobgraph.IntermediateResultPartitionID;
 
 import org.junit.Test;
 
+import java.util.Map;
+
 import static org.hamcrest.Matchers.greaterThanOrEqualTo;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
@@ -55,11 +58,16 @@ public class TaskIOMetricGroupTest {
         taskIO.getNumBytesOutCounter().inc(250L);
         taskIO.getNumBuffersOutCounter().inc(3L);
         taskIO.getIdleTimeMsPerSecond().markStart();
-        taskIO.getBackPressuredTimePerSecond().markStart();
-        long sleepTime = 2L;
-        Thread.sleep(sleepTime);
+        taskIO.getSoftBackPressuredTimePerSecond().markStart();
+        long softSleepTime = 2L;
+        Thread.sleep(softSleepTime);
         taskIO.getIdleTimeMsPerSecond().markEnd();
-        taskIO.getBackPressuredTimePerSecond().markEnd();
+        taskIO.getSoftBackPressuredTimePerSecond().markEnd();
+
+        long hardSleepTime = 4L;
+        taskIO.getHardBackPressuredTimePerSecond().markStart();
+        Thread.sleep(hardSleepTime);
+        taskIO.getHardBackPressuredTimePerSecond().markEnd();
 
         IOMetrics io = taskIO.createSnapshot();
         assertEquals(32L, io.getNumRecordsIn());
@@ -67,8 +75,36 @@ public class TaskIOMetricGroupTest {
         assertEquals(100L, io.getNumBytesIn());
         assertEquals(250L, io.getNumBytesOut());
         assertEquals(3L, taskIO.getNumBuffersOutCounter().getCount());
-        assertThat(taskIO.getIdleTimeMsPerSecond().getCount(), greaterThanOrEqualTo(sleepTime));
+        assertThat(taskIO.getIdleTimeMsPerSecond().getCount(), greaterThanOrEqualTo(softSleepTime));
         assertThat(
-                taskIO.getBackPressuredTimePerSecond().getCount(), greaterThanOrEqualTo(sleepTime));
+                taskIO.getSoftBackPressuredTimePerSecond().getCount(),
+                greaterThanOrEqualTo(softSleepTime));
+        assertThat(
+                taskIO.getHardBackPressuredTimePerSecond().getCount(),
+                greaterThanOrEqualTo(hardSleepTime));
+    }
+
+    @Test
+    public void testNumBytesProducedOfPartitionsMetrics() {
+        TaskMetricGroup task = UnregisteredMetricGroups.createUnregisteredTaskMetricGroup();
+        TaskIOMetricGroup taskIO = task.getIOMetricGroup();
+
+        Counter c1 = new SimpleCounter();
+        c1.inc(32L);
+        Counter c2 = new SimpleCounter();
+        c2.inc(64L);
+
+        IntermediateResultPartitionID resultPartitionID1 = new IntermediateResultPartitionID();
+        IntermediateResultPartitionID resultPartitionID2 = new IntermediateResultPartitionID();
+
+        taskIO.registerNumBytesProducedCounterForPartition(resultPartitionID1, c1);
+        taskIO.registerNumBytesProducedCounterForPartition(resultPartitionID2, c2);
+
+        Map<IntermediateResultPartitionID, Long> numBytesProducedOfPartitions =
+                taskIO.createSnapshot().getNumBytesProducedOfPartitions();
+
+        assertEquals(2, numBytesProducedOfPartitions.size());
+        assertEquals(32L, numBytesProducedOfPartitions.get(resultPartitionID1).longValue());
+        assertEquals(64L, numBytesProducedOfPartitions.get(resultPartitionID2).longValue());
     }
 }
