@@ -611,7 +611,7 @@ public class MiniClusterITCase extends TestLogger {
         try (final MiniCluster miniCluster = new MiniCluster(cfg)) {
             miniCluster.start();
 
-            final JobVertex failingJobVertex = new OutOfMemoryJobVertex();
+            final JobVertex failingJobVertex = new OutOfMemoryInFinalizationJobVertex();
             failingJobVertex.setInvokableClass(NoOpInvokable.class);
             failingJobVertex.setParallelism(parallelism);
 
@@ -653,14 +653,7 @@ public class MiniClusterITCase extends TestLogger {
         try (final MiniCluster miniCluster = new MiniCluster(cfg)) {
             miniCluster.start();
 
-            final JobVertex failingJobVertex =
-                    new JobVertex("FailingInFinalization") {
-
-                        @Override
-                        public void initializeOnMaster(ClassLoader loader) {
-                            throw new OutOfMemoryError("Java heap space");
-                        }
-                    };
+            final JobVertex failingJobVertex = new OutOfMemoryInInitializationVertex();
             failingJobVertex.setInvokableClass(NoOpInvokable.class);
             failingJobVertex.setParallelism(parallelism);
 
@@ -738,9 +731,9 @@ public class MiniClusterITCase extends TestLogger {
         }
     }
 
-    private static class OutOfMemoryJobVertex extends JobVertex {
+    private static class OutOfMemoryInFinalizationJobVertex extends JobVertex {
 
-        private OutOfMemoryJobVertex() {
+        private OutOfMemoryInFinalizationJobVertex() {
             super("FailingInFinalization");
         }
 
@@ -749,4 +742,16 @@ public class MiniClusterITCase extends TestLogger {
             throw new OutOfMemoryError("Java heap space");
         }
     }
+
+    private static class OutOfMemoryInInitializationVertex extends JobVertex {
+
+        OutOfMemoryInInitializationVertex() {
+            super("FailingInInitialization");
+        }
+
+        @Override
+        public void initializeOnMaster(ClassLoader loader) {
+            throw new OutOfMemoryError("Java heap space");
+        }
+    }
 }
