@@ -28,6 +28,7 @@ public class MockOperatorCoordinatorContext implements OperatorCoordinator.Conte
     private final OperatorID operatorID;
     private final ClassLoader userCodeClassLoader;
     private final int numSubtasks;
+    private final CoordinatorStore coordinatorStore = new CoordinatorStoreImpl();
 
     private boolean jobFailed;
     private Throwable jobFailureReason;
@@ -72,6 +73,11 @@ public class MockOperatorCoordinatorContext implements OperatorCoordinator.Conte
         return userCodeClassLoader;
     }
 
+    @Override
+    public CoordinatorStore getCoordinatorStore() {
+        return coordinatorStore;
+    }
+
     // -------------------------------
 
     public boolean isJobFailed() {
