@@ -21,12 +21,12 @@ package org.apache.flink.runtime.resourcemanager;
 import org.apache.flink.api.common.time.Time;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.runtime.clusterframework.ApplicationStatus;
+import org.apache.flink.runtime.clusterframework.types.ResourceID;
 import org.apache.flink.runtime.entrypoint.ClusterInformation;
 import org.apache.flink.runtime.heartbeat.HeartbeatServices;
 import org.apache.flink.runtime.heartbeat.TestingHeartbeatServices;
 import org.apache.flink.runtime.highavailability.TestingHighAvailabilityServices;
 import org.apache.flink.runtime.leaderelection.TestingLeaderElectionService;
-import org.apache.flink.runtime.metrics.MetricNames;
 import org.apache.flink.runtime.metrics.MetricRegistry;
 import org.apache.flink.runtime.metrics.util.TestingMetricRegistry;
 import org.apache.flink.runtime.rpc.RpcUtils;
@@ -36,17 +36,12 @@ import org.apache.flink.util.TestLogger;
 
 import org.junit.After;
 import org.junit.AfterClass;
-import org.junit.Assert;
 import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
-import java.util.Collections;
-import java.util.HashSet;
-import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ForkJoinPool;
 import java.util.concurrent.TimeoutException;
 
@@ -126,6 +121,7 @@ public class ResourceManagerServiceImplTest extends TestLogger {
                 ResourceManagerServiceImpl.create(
                         rmFactory,
                         new Configuration(),
+                        ResourceID.generate(),
                         rpcService,
                         haService,
                         heartbeatServices,
@@ -496,59 +492,6 @@ public class ResourceManagerServiceImplTest extends TestLogger {
         deregisterApplicationFuture.get(TIMEOUT.getSize(), TIMEOUT.getUnit());
     }
 
-    @Test
-    public void grantAndRevokeLeadership_verifyMetrics() throws Exception {
-        final Set<String> registeredMetrics = Collections.newSetFromMap(new ConcurrentHashMap<>());
-        TestingMetricRegistry metricRegistry =
-                TestingMetricRegistry.builder()
-                        .setRegisterConsumer((a, b, c) -> registeredMetrics.add(b))
-                        .setUnregisterConsumer((a, b, c) -> registeredMetrics.remove(b))
-                        .build();
-
-        final TestingResourceManagerFactory rmFactory = rmFactoryBuilder.build();
-        resourceManagerService =
-                ResourceManagerServiceImpl.create(
-                        rmFactory,
-                        new Configuration(),
-                        rpcService,
-                        haService,
-                        heartbeatServices,
-                        fatalErrorHandler,
-                        clusterInformation,
-                        null,
-                        metricRegistry,
-                        "localhost",
-                        ForkJoinPool.commonPool());
-        resourceManagerService.start();
-
-        Assert.assertEquals(0, registeredMetrics.size());
-        // grant leadership
-        leaderElectionService.isLeader(UUID.randomUUID());
-
-        assertRmStarted();
-        Set<String> expectedMetrics = new HashSet<>();
-        expectedMetrics.add(MetricNames.NUM_REGISTERED_TASK_MANAGERS);
-        expectedMetrics.add(MetricNames.TASK_SLOTS_TOTAL);
-        expectedMetrics.add(MetricNames.TASK_SLOTS_AVAILABLE);
-        Assert.assertTrue(
-                "Expected RM to register leader metrics",
-                registeredMetrics.containsAll(expectedMetrics));
-
-        // revoke leadership, block until old rm is terminated
-        revokeLeadership();
-
-        Set<String> intersection = new HashSet<>(registeredMetrics);
-        intersection.retainAll(expectedMetrics);
-        Assert.assertTrue("Expected RM to unregister leader metrics", intersection.isEmpty());
-
-        leaderElectionService.isLeader(UUID.randomUUID());
-
-        assertRmStarted();
-        Assert.assertTrue(
-                "Expected RM to re-register leader metrics",
-                registeredMetrics.containsAll(expectedMetrics));
-    }
-
     private static void blockOnFuture(CompletableFuture<?> future) {
         try {
             future.get();
@@ -570,11 +513,4 @@ public class ResourceManagerServiceImplTest extends TestLogger {
     private void assertRmStarted() throws Exception {
         leaderElectionService.getConfirmationFuture().get(TIMEOUT.getSize(), TIMEOUT.getUnit());
     }
-
-    private void revokeLeadership() {
-        ResourceManager<?> leaderResourceManager =
-                resourceManagerService.getLeaderResourceManager();
-        leaderElectionService.notLeader();
-        blockOnFuture(leaderResourceManager.getTerminationFuture());
-    }
 }
