@@ -47,8 +47,8 @@ import org.apache.flink.runtime.scheduler.exceptionhistory.ExceptionHistoryEntry
 import org.apache.flink.runtime.scheduler.exceptionhistory.RootExceptionHistoryEntry;
 import org.apache.flink.runtime.taskmanager.LocalTaskManagerLocation;
 import org.apache.flink.runtime.taskmanager.TaskManagerLocation;
-import org.apache.flink.runtime.testutils.TestingUtils;
 import org.apache.flink.runtime.util.EvictingBoundedList;
+import org.apache.flink.testutils.TestingUtils;
 import org.apache.flink.util.ExceptionUtils;
 import org.apache.flink.util.SerializedThrowable;
 import org.apache.flink.util.TestLogger;
@@ -96,7 +96,7 @@ public class JobExceptionsHandlerTest extends TestLogger {
         final ExecutionGraphInfo executionGraphInfo =
                 new ExecutionGraphInfo(new ArchivedExecutionGraphBuilder().build());
 
-        final HandlerRequest<EmptyRequestBody, JobExceptionsMessageParameters> request =
+        final HandlerRequest<EmptyRequestBody> request =
                 createRequest(executionGraphInfo.getJobId(), 10);
         final JobExceptionsInfoWithHistory response =
                 testInstance.handleRequest(request, executionGraphInfo);
@@ -115,7 +115,7 @@ public class JobExceptionsHandlerTest extends TestLogger {
 
         final ExecutionGraphInfo executionGraphInfo =
                 createExecutionGraphInfo(fromGlobalFailure(rootCause, rootCauseTimestamp));
-        final HandlerRequest<EmptyRequestBody, JobExceptionsMessageParameters> request =
+        final HandlerRequest<EmptyRequestBody> request =
                 createRequest(executionGraphInfo.getJobId(), 10);
         final JobExceptionsInfoWithHistory response =
                 testInstance.handleRequest(request, executionGraphInfo);
@@ -130,6 +130,26 @@ public class JobExceptionsHandlerTest extends TestLogger {
                 contains(historyContainsGlobalFailure(rootCause, rootCauseTimestamp)));
     }
 
+    @Test
+    public void testOnlyExceptionHistory() throws HandlerRequestException {
+        final RuntimeException rootThrowable = new RuntimeException("exception #0");
+        final long rootTimestamp = System.currentTimeMillis();
+        final RootExceptionHistoryEntry rootEntry = fromGlobalFailure(rootThrowable, rootTimestamp);
+        final ExecutionGraphInfo executionGraphInfo =
+                createExecutionGraphInfoWithoutFailureCause(rootEntry);
+        final HandlerRequest<EmptyRequestBody> request =
+                createRequest(executionGraphInfo.getJobId(), 10);
+        final JobExceptionsInfoWithHistory response =
+                testInstance.handleRequest(request, executionGraphInfo);
+
+        assertThat(response.getRootException(), is(nullValue()));
+        assertThat(response.getRootTimestamp(), is(nullValue()));
+
+        assertThat(
+                response.getExceptionHistory().getEntries(),
+                contains(historyContainsGlobalFailure(rootThrowable, rootTimestamp)));
+    }
+
     @Test
     public void testWithExceptionHistory() throws HandlerRequestException {
         final RootExceptionHistoryEntry rootCause =
@@ -144,7 +164,7 @@ public class JobExceptionsHandlerTest extends TestLogger {
 
         final ExecutionGraphInfo executionGraphInfo =
                 createExecutionGraphInfo(rootCause, otherFailure);
-        final HandlerRequest<EmptyRequestBody, JobExceptionsMessageParameters> request =
+        final HandlerRequest<EmptyRequestBody> request =
                 createRequest(executionGraphInfo.getJobId(), 10);
         final JobExceptionsInfoWithHistory response =
                 testInstance.handleRequest(request, executionGraphInfo);
@@ -175,7 +195,7 @@ public class JobExceptionsHandlerTest extends TestLogger {
                         Collections.emptySet());
 
         final ExecutionGraphInfo executionGraphInfo = createExecutionGraphInfo(failure);
-        final HandlerRequest<EmptyRequestBody, JobExceptionsMessageParameters> request =
+        final HandlerRequest<EmptyRequestBody> request =
                 createRequest(executionGraphInfo.getJobId(), 10);
         final JobExceptionsInfoWithHistory response =
                 testInstance.handleRequest(request, executionGraphInfo);
@@ -205,7 +225,7 @@ public class JobExceptionsHandlerTest extends TestLogger {
 
         final ExecutionGraphInfo executionGraphInfo =
                 createExecutionGraphInfo(rootCause, otherFailure);
-        final HandlerRequest<EmptyRequestBody, JobExceptionsMessageParameters> request =
+        final HandlerRequest<EmptyRequestBody> request =
                 createRequest(executionGraphInfo.getJobId(), 1);
         final JobExceptionsInfoWithHistory response =
                 testInstance.handleRequest(request, executionGraphInfo);
@@ -271,7 +291,7 @@ public class JobExceptionsHandlerTest extends TestLogger {
             int maxNumExceptions,
             int numExpectedException)
             throws HandlerRequestException {
-        final HandlerRequest<EmptyRequestBody, JobExceptionsMessageParameters> handlerRequest =
+        final HandlerRequest<EmptyRequestBody> handlerRequest =
                 createRequest(graph.getJobId(), numExpectedException);
         final JobExceptionsInfo jobExceptionsInfo =
                 jobExceptionsHandler.handleRequest(handlerRequest, graph);
@@ -348,12 +368,22 @@ public class JobExceptionsHandlerTest extends TestLogger {
 
     private static ExecutionGraphInfo createExecutionGraphInfo(
             RootExceptionHistoryEntry... historyEntries) {
+        return createExecutionGraphInfo(true, historyEntries);
+    }
+
+    private static ExecutionGraphInfo createExecutionGraphInfoWithoutFailureCause(
+            RootExceptionHistoryEntry... historyEntries) {
+        return createExecutionGraphInfo(false, historyEntries);
+    }
+
+    private static ExecutionGraphInfo createExecutionGraphInfo(
+            boolean setFailureCause, RootExceptionHistoryEntry... historyEntries) {
         final ArchivedExecutionGraphBuilder executionGraphBuilder =
                 new ArchivedExecutionGraphBuilder();
         final List<RootExceptionHistoryEntry> historyEntryCollection = new ArrayList<>();
 
         for (int i = 0; i < historyEntries.length; i++) {
-            if (i == 0) {
+            if (i == 0 && setFailureCause) {
                 // first entry is root cause
                 executionGraphBuilder.setFailureCause(
                         new ErrorInfo(
@@ -370,18 +400,19 @@ public class JobExceptionsHandlerTest extends TestLogger {
         return new ExecutionGraphInfo(executionGraphBuilder.build(), historyEntryCollection);
     }
 
-    private static HandlerRequest<EmptyRequestBody, JobExceptionsMessageParameters> createRequest(
-            JobID jobId, int size) throws HandlerRequestException {
+    private static HandlerRequest<EmptyRequestBody> createRequest(JobID jobId, int size)
+            throws HandlerRequestException {
         final Map<String, String> pathParameters = new HashMap<>();
         pathParameters.put(JobIDPathParameter.KEY, jobId.toString());
         final Map<String, List<String>> queryParameters = new HashMap<>();
         queryParameters.put(UpperLimitExceptionParameter.KEY, Collections.singletonList("" + size));
 
-        return new HandlerRequest<>(
+        return HandlerRequest.resolveParametersAndCreate(
                 EmptyRequestBody.getInstance(),
                 new JobExceptionsMessageParameters(),
                 pathParameters,
-                queryParameters);
+                queryParameters,
+                Collections.emptyList());
     }
 
     private static RootExceptionHistoryEntry fromGlobalFailure(Throwable cause, long timestamp) {
