@@ -31,10 +31,10 @@ import org.apache.flink.runtime.rest.messages.EmptyRequestBody;
 import org.apache.flink.runtime.rest.messages.job.metrics.AbstractAggregatedMetricsParameters;
 import org.apache.flink.runtime.rest.messages.job.metrics.AggregatedMetric;
 import org.apache.flink.runtime.rest.messages.job.metrics.AggregatedMetricsResponseBody;
-import org.apache.flink.runtime.testutils.TestingUtils;
 import org.apache.flink.runtime.webmonitor.RestfulGateway;
 import org.apache.flink.runtime.webmonitor.retriever.GatewayRetriever;
 import org.apache.flink.runtime.webmonitor.retriever.MetricQueryServiceRetriever;
+import org.apache.flink.testutils.TestingUtils;
 import org.apache.flink.util.TestLogger;
 import org.apache.flink.util.concurrent.Executors;
 
@@ -125,12 +125,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
     @Test
     public void getStores() throws Exception {
         { // test without filter
-            HandlerRequest<EmptyRequestBody, P> request =
-                    new HandlerRequest<>(
+            HandlerRequest<EmptyRequestBody> request =
+                    HandlerRequest.resolveParametersAndCreate(
                             EmptyRequestBody.getInstance(),
                             handler.getMessageHeaders().getUnresolvedMessageParameters(),
                             pathParameters,
-                            Collections.emptyMap());
+                            Collections.emptyMap(),
+                            Collections.emptyList());
             Collection<? extends MetricStore.ComponentMetricStore> subStores =
                     handler.getStores(store, request);
 
@@ -164,12 +165,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
             Tuple2<String, List<String>> filter = getFilter();
             Map<String, List<String>> queryParameters = new HashMap<>(4);
             queryParameters.put(filter.f0, filter.f1);
-            HandlerRequest<EmptyRequestBody, P> request =
-                    new HandlerRequest<>(
+            HandlerRequest<EmptyRequestBody> request =
+                    HandlerRequest.resolveParametersAndCreate(
                             EmptyRequestBody.getInstance(),
                             handler.getMessageHeaders().getUnresolvedMessageParameters(),
                             pathParameters,
-                            queryParameters);
+                            queryParameters,
+                            Collections.emptyList());
             Collection<? extends MetricStore.ComponentMetricStore> subStores =
                     handler.getStores(store, request);
 
@@ -201,12 +203,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
 
     @Test
     public void testListMetrics() throws Exception {
-        HandlerRequest<EmptyRequestBody, P> request =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> request =
+                HandlerRequest.resolveParametersAndCreate(
                         EmptyRequestBody.getInstance(),
                         handler.getMessageHeaders().getUnresolvedMessageParameters(),
                         pathParameters,
-                        Collections.emptyMap());
+                        Collections.emptyMap(),
+                        Collections.emptyList());
 
         AggregatedMetricsResponseBody response =
                 handler.handleRequest(request, MOCK_DISPATCHER_GATEWAY).get();
@@ -228,12 +231,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
         queryParams.put("get", Collections.singletonList("abc.metric1"));
         queryParams.put("agg", Collections.singletonList("min"));
 
-        HandlerRequest<EmptyRequestBody, P> request =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> request =
+                HandlerRequest.resolveParametersAndCreate(
                         EmptyRequestBody.getInstance(),
                         handler.getMessageHeaders().getUnresolvedMessageParameters(),
                         pathParameters,
-                        queryParams);
+                        queryParams,
+                        Collections.emptyList());
 
         AggregatedMetricsResponseBody response =
                 handler.handleRequest(request, MOCK_DISPATCHER_GATEWAY).get();
@@ -256,12 +260,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
         queryParams.put("get", Collections.singletonList("abc.metric1"));
         queryParams.put("agg", Collections.singletonList("max"));
 
-        HandlerRequest<EmptyRequestBody, P> request =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> request =
+                HandlerRequest.resolveParametersAndCreate(
                         EmptyRequestBody.getInstance(),
                         handler.getMessageHeaders().getUnresolvedMessageParameters(),
                         pathParameters,
-                        queryParams);
+                        queryParams,
+                        Collections.emptyList());
 
         AggregatedMetricsResponseBody response =
                 handler.handleRequest(request, MOCK_DISPATCHER_GATEWAY).get();
@@ -284,12 +289,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
         queryParams.put("get", Collections.singletonList("abc.metric1"));
         queryParams.put("agg", Collections.singletonList("sum"));
 
-        HandlerRequest<EmptyRequestBody, P> request =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> request =
+                HandlerRequest.resolveParametersAndCreate(
                         EmptyRequestBody.getInstance(),
                         handler.getMessageHeaders().getUnresolvedMessageParameters(),
                         pathParameters,
-                        queryParams);
+                        queryParams,
+                        Collections.emptyList());
 
         AggregatedMetricsResponseBody response =
                 handler.handleRequest(request, MOCK_DISPATCHER_GATEWAY).get();
@@ -312,12 +318,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
         queryParams.put("get", Collections.singletonList("abc.metric1"));
         queryParams.put("agg", Collections.singletonList("avg"));
 
-        HandlerRequest<EmptyRequestBody, P> request =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> request =
+                HandlerRequest.resolveParametersAndCreate(
                         EmptyRequestBody.getInstance(),
                         handler.getMessageHeaders().getUnresolvedMessageParameters(),
                         pathParameters,
-                        queryParams);
+                        queryParams,
+                        Collections.emptyList());
 
         AggregatedMetricsResponseBody response =
                 handler.handleRequest(request, MOCK_DISPATCHER_GATEWAY).get();
@@ -340,12 +347,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
         queryParams.put("get", Collections.singletonList("abc.metric1"));
         queryParams.put("agg", Arrays.asList("min", "max", "avg"));
 
-        HandlerRequest<EmptyRequestBody, P> request =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> request =
+                HandlerRequest.resolveParametersAndCreate(
                         EmptyRequestBody.getInstance(),
                         handler.getMessageHeaders().getUnresolvedMessageParameters(),
                         pathParameters,
-                        queryParams);
+                        queryParams,
+                        Collections.emptyList());
 
         AggregatedMetricsResponseBody response =
                 handler.handleRequest(request, MOCK_DISPATCHER_GATEWAY).get();
@@ -367,12 +375,13 @@ public abstract class AggregatingMetricsHandlerTestBase<
         Map<String, List<String>> queryParams = new HashMap<>(4);
         queryParams.put("get", Collections.singletonList("abc.metric1"));
 
-        HandlerRequest<EmptyRequestBody, P> request =
-                new HandlerRequest<>(
+        HandlerRequest<EmptyRequestBody> request =
+                HandlerRequest.resolveParametersAndCreate(
                         EmptyRequestBody.getInstance(),
                         handler.getMessageHeaders().getUnresolvedMessageParameters(),
                         pathParameters,
-                        queryParams);
+                        queryParams,
+                        Collections.emptyList());
 
         AggregatedMetricsResponseBody response =
                 handler.handleRequest(request, MOCK_DISPATCHER_GATEWAY).get();
