@@ -157,6 +157,7 @@ public class ArchivedExecutionGraphBuilder {
                 isStoppable,
                 null,
                 null,
-                "stateBackendName");
+                "stateBackendName",
+                "checkpointStorageName");
     }
 }
