@@ -20,6 +20,7 @@ package org.apache.flink.runtime.scheduler;
 
 import org.apache.flink.api.common.JobID;
 import org.apache.flink.api.common.JobStatus;
+import org.apache.flink.core.execution.SavepointFormatType;
 import org.apache.flink.queryablestate.KvStateID;
 import org.apache.flink.runtime.accumulators.AccumulatorSnapshot;
 import org.apache.flink.runtime.checkpoint.CheckpointMetrics;
@@ -41,13 +42,13 @@ import org.apache.flink.runtime.operators.coordination.OperatorEvent;
 import org.apache.flink.runtime.query.KvStateLocation;
 import org.apache.flink.runtime.state.KeyGroupRange;
 import org.apache.flink.util.concurrent.FutureUtils;
+import org.apache.flink.util.function.TriFunction;
 
 import javax.annotation.Nullable;
 
 import java.io.IOException;
 import java.net.InetSocketAddress;
 import java.util.concurrent.CompletableFuture;
-import java.util.function.BiFunction;
 import java.util.function.Consumer;
 import java.util.function.Supplier;
 
@@ -56,19 +57,24 @@ public class TestingSchedulerNG implements SchedulerNG {
     private final CompletableFuture<JobStatus> jobTerminationFuture;
     private final Runnable startSchedulingRunnable;
     private final Supplier<CompletableFuture<Void>> closeAsyncSupplier;
-    private final BiFunction<String, Boolean, CompletableFuture<String>> triggerSavepointFunction;
+    private final TriFunction<String, Boolean, SavepointFormatType, CompletableFuture<String>>
+            triggerSavepointFunction;
+    private final Supplier<CompletableFuture<String>> triggerCheckpointFunction;
     private final Consumer<Throwable> handleGlobalFailureConsumer;
 
     private TestingSchedulerNG(
             CompletableFuture<JobStatus> jobTerminationFuture,
             Runnable startSchedulingRunnable,
             Supplier<CompletableFuture<Void>> closeAsyncSupplier,
-            BiFunction<String, Boolean, CompletableFuture<String>> triggerSavepointFunction,
+            TriFunction<String, Boolean, SavepointFormatType, CompletableFuture<String>>
+                    triggerSavepointFunction,
+            Supplier<CompletableFuture<String>> triggerCheckpointFunction,
             Consumer<Throwable> handleGlobalFailureConsumer) {
         this.jobTerminationFuture = jobTerminationFuture;
         this.startSchedulingRunnable = startSchedulingRunnable;
         this.closeAsyncSupplier = closeAsyncSupplier;
         this.triggerSavepointFunction = triggerSavepointFunction;
+        this.triggerCheckpointFunction = triggerCheckpointFunction;
         this.handleGlobalFailureConsumer = handleGlobalFailureConsumer;
     }
 
@@ -175,8 +181,13 @@ public class TestingSchedulerNG implements SchedulerNG {
 
     @Override
     public CompletableFuture<String> triggerSavepoint(
-            @Nullable String targetDirectory, boolean cancelJob) {
-        return triggerSavepointFunction.apply(targetDirectory, cancelJob);
+            @Nullable String targetDirectory, boolean cancelJob, SavepointFormatType formatType) {
+        return triggerSavepointFunction.apply(targetDirectory, cancelJob, formatType);
+    }
+
+    @Override
+    public CompletableFuture<String> triggerCheckpoint() {
+        return triggerCheckpointFunction.get();
     }
 
     @Override
@@ -195,7 +206,8 @@ public class TestingSchedulerNG implements SchedulerNG {
     }
 
     @Override
-    public CompletableFuture<String> stopWithSavepoint(String targetDirectory, boolean terminate) {
+    public CompletableFuture<String> stopWithSavepoint(
+            String targetDirectory, boolean terminate, SavepointFormatType formatType) {
         failOperation();
         return null;
     }
@@ -230,8 +242,11 @@ public class TestingSchedulerNG implements SchedulerNG {
         private Runnable startSchedulingRunnable = () -> {};
         private Supplier<CompletableFuture<Void>> closeAsyncSupplier =
                 FutureUtils::completedVoidFuture;
-        private BiFunction<String, Boolean, CompletableFuture<String>> triggerSavepointFunction =
-                (ignoredA, ignoredB) -> new CompletableFuture<>();
+        private TriFunction<String, Boolean, SavepointFormatType, CompletableFuture<String>>
+                triggerSavepointFunction =
+                        (ignoredA, ignoredB, formatType) -> new CompletableFuture<>();
+        private Supplier<CompletableFuture<String>> triggerCheckpointFunction =
+                CompletableFuture::new;
         private Consumer<Throwable> handleGlobalFailureConsumer = (ignored) -> {};
 
         public Builder setJobTerminationFuture(CompletableFuture<JobStatus> jobTerminationFuture) {
@@ -250,11 +265,18 @@ public class TestingSchedulerNG implements SchedulerNG {
         }
 
         public Builder setTriggerSavepointFunction(
-                BiFunction<String, Boolean, CompletableFuture<String>> triggerSavepointFunction) {
+                TriFunction<String, Boolean, SavepointFormatType, CompletableFuture<String>>
+                        triggerSavepointFunction) {
             this.triggerSavepointFunction = triggerSavepointFunction;
             return this;
         }
 
+        public Builder setTriggerCheckpointFunction(
+                Supplier<CompletableFuture<String>> triggerCheckpointFunction) {
+            this.triggerCheckpointFunction = triggerCheckpointFunction;
+            return this;
+        }
+
         public Builder setHandleGlobalFailureConsumer(
                 Consumer<Throwable> handleGlobalFailureConsumer) {
             this.handleGlobalFailureConsumer = handleGlobalFailureConsumer;
@@ -267,6 +289,7 @@ public class TestingSchedulerNG implements SchedulerNG {
                     startSchedulingRunnable,
                     closeAsyncSupplier,
                     triggerSavepointFunction,
+                    triggerCheckpointFunction,
                     handleGlobalFailureConsumer);
         }
     }
