@@ -21,6 +21,7 @@ package org.apache.flink.runtime.scheduler.adaptive;
 import org.apache.flink.api.common.JobID;
 import org.apache.flink.api.common.JobStatus;
 import org.apache.flink.api.common.time.Time;
+import org.apache.flink.core.execution.SavepointFormatType;
 import org.apache.flink.core.testutils.CompletedScheduledFuture;
 import org.apache.flink.runtime.JobException;
 import org.apache.flink.runtime.blob.BlobWriter;
@@ -35,6 +36,7 @@ import org.apache.flink.runtime.execution.ExecutionState;
 import org.apache.flink.runtime.executiongraph.ArchivedExecutionGraph;
 import org.apache.flink.runtime.executiongraph.DefaultSubtaskAttemptNumberStore;
 import org.apache.flink.runtime.executiongraph.EdgeManager;
+import org.apache.flink.runtime.executiongraph.ErrorInfo;
 import org.apache.flink.runtime.executiongraph.Execution;
 import org.apache.flink.runtime.executiongraph.ExecutionAttemptID;
 import org.apache.flink.runtime.executiongraph.ExecutionDeploymentListener;
@@ -51,9 +53,14 @@ import org.apache.flink.runtime.executiongraph.failover.flip1.partitionrelease.P
 import org.apache.flink.runtime.io.network.partition.JobMasterPartitionTracker;
 import org.apache.flink.runtime.jobgraph.IntermediateResultPartitionID;
 import org.apache.flink.runtime.jobgraph.JobVertex;
+import org.apache.flink.runtime.jobgraph.JobVertexID;
+import org.apache.flink.runtime.operators.coordination.CoordinatorStoreImpl;
 import org.apache.flink.runtime.scheduler.DefaultVertexParallelismInfo;
 import org.apache.flink.runtime.scheduler.ExecutionGraphHandler;
 import org.apache.flink.runtime.scheduler.OperatorCoordinatorHandler;
+import org.apache.flink.runtime.scheduler.exceptionhistory.ExceptionHistoryEntry;
+import org.apache.flink.runtime.scheduler.exceptionhistory.RootExceptionHistoryEntry;
+import org.apache.flink.runtime.scheduler.exceptionhistory.TestingAccessExecution;
 import org.apache.flink.runtime.scheduler.strategy.ExecutionVertexID;
 import org.apache.flink.runtime.shuffle.ShuffleMaster;
 import org.apache.flink.runtime.taskmanager.TaskExecutionState;
@@ -68,6 +75,7 @@ import javax.annotation.Nonnull;
 import javax.annotation.Nullable;
 
 import java.time.Duration;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 import java.util.Optional;
@@ -125,7 +133,8 @@ public class ExecutingTest extends TestLogger {
                     new TestingOperatorCoordinatorHandler(),
                     log,
                     ctx,
-                    ClassLoader.getSystemClassLoader());
+                    ClassLoader.getSystemClassLoader(),
+                    new ArrayList<>());
             assertThat(mockExecutionVertex.isDeployCalled(), is(false));
         }
     }
@@ -142,7 +151,8 @@ public class ExecutingTest extends TestLogger {
                     new TestingOperatorCoordinatorHandler(),
                     log,
                     ctx,
-                    ClassLoader.getSystemClassLoader());
+                    ClassLoader.getSystemClassLoader(),
+                    new ArrayList<>());
         }
     }
 
@@ -172,7 +182,7 @@ public class ExecutingTest extends TestLogger {
                         assertThat(failingArguments.getExecutionGraph(), notNullValue());
                         assertThat(failingArguments.getFailureCause().getMessage(), is(failureMsg));
                     }));
-            ctx.setHowToHandleFailure(Executing.FailureResult::canNotRestart);
+            ctx.setHowToHandleFailure(FailureResult::canNotRestart);
             exec.handleGlobalFailure(new RuntimeException(failureMsg));
         }
     }
@@ -185,7 +195,7 @@ public class ExecutingTest extends TestLogger {
             ctx.setExpectRestarting(
                     (restartingArguments ->
                             assertThat(restartingArguments.getBackoffTime(), is(duration))));
-            ctx.setHowToHandleFailure((t) -> Executing.FailureResult.canRestart(t, duration));
+            ctx.setHowToHandleFailure((f) -> FailureResult.canRestart(f, duration));
             exec.handleGlobalFailure(new RuntimeException("Recoverable error"));
         }
     }
@@ -256,34 +266,52 @@ public class ExecutingTest extends TestLogger {
     public void testFailureReportedViaUpdateTaskExecutionStateCausesFailingOnNoRestart()
             throws Exception {
         try (MockExecutingContext ctx = new MockExecutingContext()) {
-            ExecutionGraph returnsFailedStateExecutionGraph =
-                    new MockExecutionGraph(true, Collections::emptyList);
+            StateTrackingMockExecutionGraph returnsFailedStateExecutionGraph =
+                    new StateTrackingMockExecutionGraph();
             Executing exec =
                     new ExecutingStateBuilder()
                             .setExecutionGraph(returnsFailedStateExecutionGraph)
                             .build(ctx);
 
-            ctx.setHowToHandleFailure(Executing.FailureResult::canNotRestart);
+            ctx.setHowToHandleFailure(FailureResult::canNotRestart);
             ctx.setExpectFailing(assertNonNull());
 
-            exec.updateTaskExecutionState(createFailingStateTransition());
+            Exception exception = new RuntimeException();
+            TestingAccessExecution execution =
+                    TestingAccessExecution.newBuilder()
+                            .withExecutionState(ExecutionState.FAILED)
+                            .withErrorInfo(new ErrorInfo(exception, System.currentTimeMillis()))
+                            .build();
+            returnsFailedStateExecutionGraph.registerExecution(execution);
+            TaskExecutionStateTransition taskExecutionStateTransition =
+                    createFailingStateTransition(execution.getAttemptId(), exception);
+            exec.updateTaskExecutionState(taskExecutionStateTransition);
         }
     }
 
     @Test
     public void testFailureReportedViaUpdateTaskExecutionStateCausesRestart() throws Exception {
         try (MockExecutingContext ctx = new MockExecutingContext()) {
-            ExecutionGraph returnsFailedStateExecutionGraph =
-                    new MockExecutionGraph(true, Collections::emptyList);
+            StateTrackingMockExecutionGraph returnsFailedStateExecutionGraph =
+                    new StateTrackingMockExecutionGraph();
             Executing exec =
                     new ExecutingStateBuilder()
                             .setExecutionGraph(returnsFailedStateExecutionGraph)
                             .build(ctx);
             ctx.setHowToHandleFailure(
-                    (throwable) -> Executing.FailureResult.canRestart(throwable, Duration.ZERO));
+                    (failure) -> FailureResult.canRestart(failure, Duration.ZERO));
             ctx.setExpectRestarting(assertNonNull());
 
-            exec.updateTaskExecutionState(createFailingStateTransition());
+            Exception exception = new RuntimeException();
+            TestingAccessExecution execution =
+                    TestingAccessExecution.newBuilder()
+                            .withExecutionState(ExecutionState.FAILED)
+                            .withErrorInfo(new ErrorInfo(exception, System.currentTimeMillis()))
+                            .build();
+            returnsFailedStateExecutionGraph.registerExecution(execution);
+            TaskExecutionStateTransition taskExecutionStateTransition =
+                    createFailingStateTransition(execution.getAttemptId(), exception);
+            exec.updateTaskExecutionState(taskExecutionStateTransition);
         }
     }
 
@@ -297,7 +325,16 @@ public class ExecutingTest extends TestLogger {
                             .setExecutionGraph(returnsFailedStateExecutionGraph)
                             .build(ctx);
 
-            exec.updateTaskExecutionState(createFailingStateTransition());
+            Exception exception = new RuntimeException();
+            TestingAccessExecution execution =
+                    TestingAccessExecution.newBuilder()
+                            .withExecutionState(ExecutionState.FAILED)
+                            .withErrorInfo(new ErrorInfo(exception, System.currentTimeMillis()))
+                            .build();
+            returnsFailedStateExecutionGraph.registerExecution(execution);
+            TaskExecutionStateTransition taskExecutionStateTransition =
+                    createFailingStateTransition(execution.getAttemptId(), exception);
+            exec.updateTaskExecutionState(taskExecutionStateTransition);
 
             ctx.assertNoStateTransition();
         }
@@ -339,7 +376,7 @@ public class ExecutingTest extends TestLogger {
                             .build(ctx);
 
             ctx.setExpectStopWithSavepoint(assertNonNull());
-            exec.stopWithSavepoint("file:///tmp/target", true);
+            exec.stopWithSavepoint("file:///tmp/target", true, SavepointFormatType.CANONICAL);
         }
     }
 
@@ -367,18 +404,12 @@ public class ExecutingTest extends TestLogger {
             assertThat(coordinator.isPeriodicCheckpointingStarted(), is(true));
 
             ctx.setExpectStopWithSavepoint(assertNonNull());
-            exec.stopWithSavepoint("file:///tmp/target", true);
+            exec.stopWithSavepoint("file:///tmp/target", true, SavepointFormatType.CANONICAL);
 
             assertThat(coordinator.isPeriodicCheckpointingStarted(), is(false));
         }
     }
 
-    static TaskExecutionStateTransition createFailingStateTransition() {
-        return new TaskExecutionStateTransition(
-                new TaskExecutionState(
-                        new ExecutionAttemptID(), ExecutionState.FAILED, new RuntimeException()));
-    }
-
     @Test
     public void testJobInformationMethods() throws Exception {
         try (MockExecutingContext ctx = new MockExecutingContext()) {
@@ -429,6 +460,18 @@ public class ExecutingTest extends TestLogger {
         }
     }
 
+    public static TaskExecutionStateTransition createFailingStateTransition(
+            ExecutionAttemptID attemptId) throws JobException {
+        return new TaskExecutionStateTransition(
+                new TaskExecutionState(attemptId, ExecutionState.FAILED, new RuntimeException()));
+    }
+
+    public static TaskExecutionStateTransition createFailingStateTransition(
+            ExecutionAttemptID attemptId, Exception exception) throws JobException {
+        return new TaskExecutionStateTransition(
+                new TaskExecutionState(attemptId, ExecutionState.FAILED, exception));
+    }
+
     private final class ExecutingStateBuilder {
         private ExecutionGraph executionGraph =
                 TestingDefaultExecutionGraphBuilder.newBuilder().build();
@@ -458,7 +501,8 @@ public class ExecutingTest extends TestLogger {
                     operatorCoordinatorHandler,
                     log,
                     ctx,
-                    ClassLoader.getSystemClassLoader());
+                    ClassLoader.getSystemClassLoader(),
+                    new ArrayList<>());
         }
     }
 
@@ -478,7 +522,7 @@ public class ExecutingTest extends TestLogger {
         private final StateValidator<CancellingArguments> cancellingStateValidator =
                 new StateValidator<>("cancelling");
 
-        private Function<Throwable, Executing.FailureResult> howToHandleFailure;
+        private Function<Throwable, FailureResult> howToHandleFailure;
         private Supplier<Boolean> canScaleUp = () -> false;
         private StateValidator<StopWithSavepointArguments> stopWithSavepointValidator =
                 new StateValidator<>("stopWithSavepoint");
@@ -501,7 +545,7 @@ public class ExecutingTest extends TestLogger {
             stopWithSavepointValidator.expectInput(asserter);
         }
 
-        public void setHowToHandleFailure(Function<Throwable, Executing.FailureResult> function) {
+        public void setHowToHandleFailure(Function<Throwable, FailureResult> function) {
             this.howToHandleFailure = function;
         }
 
@@ -515,7 +559,8 @@ public class ExecutingTest extends TestLogger {
         public void goToCanceling(
                 ExecutionGraph executionGraph,
                 ExecutionGraphHandler executionGraphHandler,
-                OperatorCoordinatorHandler operatorCoordinatorHandler) {
+                OperatorCoordinatorHandler operatorCoordinatorHandler,
+                List<ExceptionHistoryEntry> failureCollection) {
             cancellingStateValidator.validateInput(
                     new CancellingArguments(
                             executionGraph, executionGraphHandler, operatorCoordinatorHandler));
@@ -523,7 +568,7 @@ public class ExecutingTest extends TestLogger {
         }
 
         @Override
-        public Executing.FailureResult howToHandleFailure(Throwable failure) {
+        public FailureResult howToHandleFailure(Throwable failure) {
             return howToHandleFailure.apply(failure);
         }
 
@@ -537,7 +582,8 @@ public class ExecutingTest extends TestLogger {
                 ExecutionGraph executionGraph,
                 ExecutionGraphHandler executionGraphHandler,
                 OperatorCoordinatorHandler operatorCoordinatorHandler,
-                Duration backoffTime) {
+                Duration backoffTime,
+                List<ExceptionHistoryEntry> failureCollection) {
             restartingStateValidator.validateInput(
                     new RestartingArguments(
                             executionGraph,
@@ -552,7 +598,8 @@ public class ExecutingTest extends TestLogger {
                 ExecutionGraph executionGraph,
                 ExecutionGraphHandler executionGraphHandler,
                 OperatorCoordinatorHandler operatorCoordinatorHandler,
-                Throwable failureCause) {
+                Throwable failureCause,
+                List<ExceptionHistoryEntry> failureCollection) {
             failingStateValidator.validateInput(
                     new FailingArguments(
                             executionGraph,
@@ -568,7 +615,8 @@ public class ExecutingTest extends TestLogger {
                 ExecutionGraphHandler executionGraphHandler,
                 OperatorCoordinatorHandler operatorCoordinatorHandler,
                 CheckpointScheduling checkpointScheduling,
-                CompletableFuture<String> savepointFuture) {
+                CompletableFuture<String> savepointFuture,
+                List<ExceptionHistoryEntry> failureCollection) {
             stopWithSavepointValidator.validateInput(
                     new StopWithSavepointArguments(
                             executionGraph,
@@ -597,6 +645,9 @@ public class ExecutingTest extends TestLogger {
             cancellingStateValidator.close();
             stopWithSavepointValidator.close();
         }
+
+        @Override
+        public void archiveFailure(RootExceptionHistoryEntry failure) {}
     }
 
     static class CancellingArguments {
@@ -755,11 +806,14 @@ public class ExecutingTest extends TestLogger {
             super(
                     new MockInternalExecutionGraphAccessor(),
                     new JobVertex("test"),
+                    new DefaultVertexParallelismInfo(1, 1, max -> Optional.empty()));
+
+            initialize(
                     1,
                     Time.milliseconds(1L),
                     1L,
-                    new DefaultVertexParallelismInfo(1, 1, max -> Optional.empty()),
-                    new DefaultSubtaskAttemptNumberStore(Collections.emptyList()));
+                    new DefaultSubtaskAttemptNumberStore(Collections.emptyList()),
+                    new CoordinatorStoreImpl());
             mockExecutionVertex = executionVertexSupplier.apply(this);
         }
 
@@ -899,10 +953,10 @@ public class ExecutingTest extends TestLogger {
                 boolean releasePartitions) {}
 
         @Override
-        public void vertexFinished() {}
+        public void jobVertexFinished() {}
 
         @Override
-        public void vertexUnFinished() {}
+        public void jobVertexUnFinished() {}
 
         @Override
         public ExecutionDeploymentListener getExecutionDeploymentListener() {
@@ -910,7 +964,10 @@ public class ExecutingTest extends TestLogger {
         }
 
         @Override
-        public void notifyExecutionChange(Execution execution, ExecutionState newExecutionState) {}
+        public void notifyExecutionChange(
+                Execution execution,
+                ExecutionState previousState,
+                ExecutionState newExecutionState) {}
 
         @Override
         public EdgeManager getEdgeManager() {
@@ -936,5 +993,17 @@ public class ExecutingTest extends TestLogger {
             throw new UnsupportedOperationException(
                     "This method is not supported by the MockInternalExecutionGraphAccessor.");
         }
+
+        @Override
+        public ExecutionJobVertex getJobVertex(JobVertexID id) {
+            throw new UnsupportedOperationException(
+                    "This method is not supported by the MockInternalExecutionGraphAccessor.");
+        }
+
+        @Override
+        public boolean isDynamic() {
+            throw new UnsupportedOperationException(
+                    "This method is not supported by the MockInternalExecutionGraphAccessor.");
+        }
     }
 }
