@@ -18,6 +18,7 @@
 
 package org.apache.flink.runtime.source.coordinator;
 
+import org.apache.flink.api.common.eventtime.WatermarkAlignmentParams;
 import org.apache.flink.api.connector.source.Boundedness;
 import org.apache.flink.api.connector.source.Source;
 import org.apache.flink.api.connector.source.SourceEvent;
@@ -34,6 +35,7 @@ import org.apache.flink.core.io.SimpleVersionedSerializer;
 import org.apache.flink.core.memory.DataOutputSerializer;
 import org.apache.flink.runtime.jobgraph.OperatorID;
 import org.apache.flink.runtime.operators.coordination.ComponentClosingUtils;
+import org.apache.flink.runtime.operators.coordination.CoordinatorStoreImpl;
 import org.apache.flink.runtime.operators.coordination.MockOperatorCoordinatorContext;
 import org.apache.flink.runtime.operators.coordination.OperatorCoordinator;
 import org.apache.flink.runtime.source.event.SourceEventWrapper;
@@ -247,7 +249,9 @@ public class SourceCoordinatorTest extends SourceCoordinatorTestBase {
                         new SourceCoordinator<>(
                                 OPERATOR_NAME,
                                 new EnumeratorCreatingSource<>(() -> splitEnumerator),
-                                context)) {
+                                context,
+                                new CoordinatorStoreImpl(),
+                                WatermarkAlignmentParams.WATERMARK_ALIGNMENT_DISABLED)) {
 
             coordinator.start();
             waitUtil(
@@ -269,7 +273,9 @@ public class SourceCoordinatorTest extends SourceCoordinatorTestBase {
                                 () -> {
                                     throw failureReason;
                                 }),
-                        context);
+                        context,
+                        new CoordinatorStoreImpl(),
+                        WatermarkAlignmentParams.WATERMARK_ALIGNMENT_DISABLED);
 
         coordinator.start();
 
@@ -293,7 +299,9 @@ public class SourceCoordinatorTest extends SourceCoordinatorTestBase {
                         new SourceCoordinator<>(
                                 OPERATOR_NAME,
                                 new EnumeratorCreatingSource<>(() -> splitEnumerator),
-                                context)) {
+                                context,
+                                new CoordinatorStoreImpl(),
+                                WatermarkAlignmentParams.WATERMARK_ALIGNMENT_DISABLED)) {
 
             coordinator.start();
             coordinator.handleEventFromOperator(1, new SourceEventWrapper(new SourceEvent() {}));
@@ -334,7 +342,8 @@ public class SourceCoordinatorTest extends SourceCoordinatorTestBase {
                         new SourceCoordinator<>(
                                 OPERATOR_NAME,
                                 new EnumeratorCreatingSource<>(() -> splitEnumerator),
-                                context)) {
+                                context,
+                                new CoordinatorStoreImpl())) {
 
             coordinator.start();
             coordinator.handleEventFromOperator(1, new SourceEventWrapper(new SourceEvent() {}));
@@ -370,7 +379,12 @@ public class SourceCoordinatorTest extends SourceCoordinatorTestBase {
         final EnumeratorCreatingSource<?, ClassLoaderTestEnumerator> source =
                 new EnumeratorCreatingSource<>(ClassLoaderTestEnumerator::new);
         final SourceCoordinatorProvider<?> provider =
-                new SourceCoordinatorProvider<>("testOperator", context.getOperatorId(), source, 1);
+                new SourceCoordinatorProvider<>(
+                        "testOperator",
+                        context.getOperatorId(),
+                        source,
+                        1,
+                        WatermarkAlignmentParams.WATERMARK_ALIGNMENT_DISABLED);
 
         final OperatorCoordinator coordinator = provider.getCoordinator(context);
         coordinator.start();
@@ -392,7 +406,12 @@ public class SourceCoordinatorTest extends SourceCoordinatorTestBase {
         final EnumeratorCreatingSource<?, ClassLoaderTestEnumerator> source =
                 new EnumeratorCreatingSource<>(ClassLoaderTestEnumerator::new);
         final SourceCoordinatorProvider<?> provider =
-                new SourceCoordinatorProvider<>("testOperator", context.getOperatorId(), source, 1);
+                new SourceCoordinatorProvider<>(
+                        "testOperator",
+                        context.getOperatorId(),
+                        source,
+                        1,
+                        WatermarkAlignmentParams.WATERMARK_ALIGNMENT_DISABLED);
 
         final OperatorCoordinator coordinator = provider.getCoordinator(context);
         coordinator.resetToCheckpoint(1L, createEmptyCheckpoint());
