@@ -35,6 +35,7 @@ import org.apache.flink.runtime.metrics.MetricRegistryImpl;
 import org.apache.flink.runtime.metrics.MetricRegistryTestUtils;
 import org.apache.flink.runtime.operators.coordination.OperatorEvent;
 import org.apache.flink.runtime.operators.coordination.TestOperatorEvent;
+import org.apache.flink.runtime.operators.coordination.TestingCoordinationRequestHandler;
 import org.apache.flink.runtime.rpc.TestingRpcService;
 import org.apache.flink.runtime.taskmanager.Task;
 import org.apache.flink.runtime.testutils.CancelableInvokable;
@@ -117,6 +118,21 @@ public class TaskExecutorOperatorEventHandlingTest extends TestLogger {
         }
     }
 
+    @Test
+    public void requestToCoordinatorDeliveryFailureFailsTask() throws Exception {
+        final JobID jobId = new JobID();
+        final ExecutionAttemptID eid = new ExecutionAttemptID();
+
+        try (TaskSubmissionTestEnvironment env =
+                createExecutorWithRunningTask(
+                        jobId, eid, CoordinationRequestSendingInvokable.class)) {
+            final Task task = env.getTaskSlotTable().getTask(eid);
+
+            task.getExecutingThread().join(10_000);
+            assertEquals(ExecutionState.FAILED, task.getExecutionState());
+        }
+    }
+
     // ------------------------------------------------------------------------
     //  test setup helpers
     // ------------------------------------------------------------------------
@@ -148,6 +164,10 @@ public class TaskExecutorOperatorEventHandlingTest extends TestLogger {
                                                 (eio, oid, value) -> {
                                                     throw new RuntimeException();
                                                 })
+                                        .setDeliverCoordinationRequestFunction(
+                                                (oid, value) -> {
+                                                    throw new RuntimeException();
+                                                })
                                         .build())
                         .build();
 
@@ -225,4 +245,24 @@ public class TaskExecutorOperatorEventHandlingTest extends TestLogger {
             waitUntilCancelled();
         }
     }
+
+    /** Test invokable that fails when receiving a coordination request. */
+    public static final class CoordinationRequestSendingInvokable extends CancelableInvokable {
+
+        public CoordinationRequestSendingInvokable(Environment environment) {
+            super(environment);
+        }
+
+        @Override
+        protected void doInvoke() throws Exception {
+            getEnvironment()
+                    .getOperatorCoordinatorEventGateway()
+                    .sendRequestToCoordinator(
+                            new OperatorID(),
+                            new SerializedValue<>(
+                                    new TestingCoordinationRequestHandler.Request<>(0L)));
+
+            waitUntilCancelled();
+        }
+    }
 }
