@@ -71,7 +71,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.Future;
 import java.util.concurrent.LinkedBlockingDeque;
 import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.TimeoutException;
@@ -159,6 +158,26 @@ public class TaskTest extends TestLogger {
         assertTrue(wasCleanedUp);
     }
 
+    @Test
+    public void testCleanupWhenSwitchToInitializationFails() throws Exception {
+        createTaskBuilder()
+                .setInvokable(TestInvokableCorrect.class)
+                .setTaskManagerActions(
+                        new NoOpTaskManagerActions() {
+                            @Override
+                            public void updateTaskExecutionState(
+                                    TaskExecutionState taskExecutionState) {
+                                if (taskExecutionState.getExecutionState()
+                                        == ExecutionState.INITIALIZING) {
+                                    throw new ExpectedTestException();
+                                }
+                            }
+                        })
+                .build()
+                .run();
+        assertTrue(wasCleanedUp);
+    }
+
     @Test
     public void testRegularExecution() throws Exception {
         final QueuedNoOpTaskManagerActions taskManagerActions = new QueuedNoOpTaskManagerActions();
@@ -1256,9 +1275,8 @@ public class TaskTest extends TestLogger {
         public void invoke() {}
 
         @Override
-        public Future<Void> cancel() {
+        public void cancel() {
             fail("This should not be called");
-            return null;
         }
 
         @Override
@@ -1322,9 +1340,7 @@ public class TaskTest extends TestLogger {
         }
 
         @Override
-        public Future<Void> cancel() {
-            return CompletableFuture.completedFuture(null);
-        }
+        public void cancel() {}
     }
 
     private static class InvokableBlockingWithTrigger extends AbstractInvokable {
@@ -1490,12 +1506,11 @@ public class TaskTest extends TestLogger {
         }
 
         @Override
-        public Future<Void> cancel() throws Exception {
+        public void cancel() throws Exception {
             synchronized (this) {
                 triggerLatch.trigger();
                 wait();
             }
-            return CompletableFuture.completedFuture(null);
         }
     }
 
@@ -1517,12 +1532,11 @@ public class TaskTest extends TestLogger {
         }
 
         @Override
-        public Future<Void> cancel() {
+        public void cancel() {
             synchronized (lock) {
                 // do nothing but a placeholder
                 triggerLatch.trigger();
             }
-            return CompletableFuture.completedFuture(null);
         }
     }
 
@@ -1546,9 +1560,7 @@ public class TaskTest extends TestLogger {
         }
 
         @Override
-        public Future<Void> cancel() {
-            return CompletableFuture.completedFuture(null);
-        }
+        public void cancel() {}
     }
 
     // ------------------------------------------------------------------------
