@@ -25,6 +25,8 @@ import org.apache.flink.core.fs.CloseableRegistry;
 import org.apache.flink.core.memory.DataOutputView;
 import org.apache.flink.core.memory.DataOutputViewStreamWrapper;
 import org.apache.flink.runtime.checkpoint.CheckpointOptions;
+import org.apache.flink.runtime.checkpoint.CheckpointType;
+import org.apache.flink.runtime.checkpoint.SnapshotType;
 import org.apache.flink.runtime.state.CheckpointStreamFactory;
 import org.apache.flink.runtime.state.CheckpointStreamWithResultProvider;
 import org.apache.flink.runtime.state.CheckpointedStateScope;
@@ -64,13 +66,17 @@ import java.io.File;
 import java.io.IOException;
 import java.nio.file.Path;
 import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
+import java.util.Optional;
 import java.util.SortedMap;
+import java.util.TreeMap;
 import java.util.UUID;
+import java.util.stream.Collectors;
 
 import static org.apache.flink.contrib.streaming.state.snapshot.RocksSnapshotUtil.SST_FILE_SUFFIX;
 
@@ -96,9 +102,11 @@ public class RocksIncrementalSnapshotStrategy<K>
     @Nonnull private final UUID backendUID;
 
     /**
-     * Stores the materialized sstable files from all snapshots that build the incremental history.
+     * Stores the {@link StateHandleID IDs} of uploaded SST files that build the incremental
+     * history. Once the checkpoint is confirmed by JM, only the ID paired with {@link
+     * PlaceholderStreamStateHandle} can be sent.
      */
-    @Nonnull private final SortedMap<Long, Set<StateHandleID>> materializedSstFiles;
+    @Nonnull private final SortedMap<Long, Map<StateHandleID, Long>> uploadedStateIDs;
 
     /** The identifier of the last completed checkpoint. */
     private long lastCompletedCheckpointId;
@@ -120,7 +128,7 @@ public class RocksIncrementalSnapshotStrategy<K>
             @Nonnull CloseableRegistry cancelStreamRegistry,
             @Nonnull File instanceBasePath,
             @Nonnull UUID backendUID,
-            @Nonnull SortedMap<Long, Set<StateHandleID>> materializedSstFiles,
+            @Nonnull SortedMap<Long, Map<StateHandleID, StreamStateHandle>> uploadedStateHandles,
             @Nonnull RocksDBStateUploader rocksDBStateUploader,
             long lastCompletedCheckpointId) {
 
@@ -136,7 +144,16 @@ public class RocksIncrementalSnapshotStrategy<K>
 
         this.instanceBasePath = instanceBasePath;
         this.backendUID = backendUID;
-        this.materializedSstFiles = materializedSstFiles;
+        this.uploadedStateIDs = new TreeMap<>();
+        for (Map.Entry<Long, Map<StateHandleID, StreamStateHandle>> entry :
+                uploadedStateHandles.entrySet()) {
+            Map<StateHandleID, Long> map = new HashMap<>();
+            for (Map.Entry<StateHandleID, StreamStateHandle> stateHandleEntry :
+                    entry.getValue().entrySet()) {
+                map.put(stateHandleEntry.getKey(), stateHandleEntry.getValue().getStateSize());
+            }
+            this.uploadedStateIDs.put(entry.getKey(), map);
+        }
         this.stateUploader = rocksDBStateUploader;
         this.lastCompletedCheckpointId = lastCompletedCheckpointId;
         this.localDirectoryName = backendUID.toString().replaceAll("[\\-]", "");
@@ -151,13 +168,13 @@ public class RocksIncrementalSnapshotStrategy<K>
 
         final List<StateMetaInfoSnapshot> stateMetaInfoSnapshots =
                 new ArrayList<>(kvStateInformation.size());
-        final Set<StateHandleID> baseSstFiles =
+        final PreviousSnapshot previousSnapshot =
                 snapshotMetaData(checkpointId, stateMetaInfoSnapshots);
 
         takeDBNativeCheckpoint(snapshotDirectory);
 
         return new IncrementalRocksDBSnapshotResources(
-                snapshotDirectory, baseSstFiles, stateMetaInfoSnapshots);
+                snapshotDirectory, previousSnapshot, stateMetaInfoSnapshots);
     }
 
     @Override
@@ -177,23 +194,40 @@ public class RocksIncrementalSnapshotStrategy<K>
             return registry -> SnapshotResult.empty();
         }
 
+        final PreviousSnapshot previousSnapshot;
+        final CheckpointType.SharingFilesStrategy sharingFilesStrategy =
+                checkpointOptions.getCheckpointType().getSharingFilesStrategy();
+        switch (sharingFilesStrategy) {
+            case FORWARD_BACKWARD:
+                previousSnapshot = snapshotResources.previousSnapshot;
+                break;
+            case FORWARD:
+            case NO_SHARING:
+                previousSnapshot = EMPTY_PREVIOUS_SNAPSHOT;
+                break;
+            default:
+                throw new IllegalArgumentException(
+                        "Unsupported sharing files strategy: " + sharingFilesStrategy);
+        }
+
         return new RocksDBIncrementalSnapshotOperation(
                 checkpointId,
                 checkpointStreamFactory,
                 snapshotResources.snapshotDirectory,
-                snapshotResources.baseSstFiles,
+                previousSnapshot,
+                sharingFilesStrategy,
                 snapshotResources.stateMetaInfoSnapshots);
     }
 
     @Override
     public void notifyCheckpointComplete(long completedCheckpointId) {
-        synchronized (materializedSstFiles) {
+        synchronized (uploadedStateIDs) {
             // FLINK-23949: materializedSstFiles.keySet().contains(completedCheckpointId) make sure
             // the notified checkpointId is not a savepoint, otherwise next checkpoint will
             // degenerate into a full checkpoint
             if (completedCheckpointId > lastCompletedCheckpointId
-                    && materializedSstFiles.keySet().contains(completedCheckpointId)) {
-                materializedSstFiles
+                    && uploadedStateIDs.containsKey(completedCheckpointId)) {
+                uploadedStateIDs
                         .keySet()
                         .removeIf(checkpointId -> checkpointId < completedCheckpointId);
                 lastCompletedCheckpointId = completedCheckpointId;
@@ -203,8 +237,8 @@ public class RocksIncrementalSnapshotStrategy<K>
 
     @Override
     public void notifyCheckpointAborted(long abortedCheckpointId) {
-        synchronized (materializedSstFiles) {
-            materializedSstFiles.keySet().remove(abortedCheckpointId);
+        synchronized (uploadedStateIDs) {
+            uploadedStateIDs.keySet().remove(abortedCheckpointId);
         }
     }
 
@@ -219,7 +253,9 @@ public class RocksIncrementalSnapshotStrategy<K>
         if (localRecoveryConfig.isLocalRecoveryEnabled()) {
             // create a "permanent" snapshot directory for local recovery.
             LocalRecoveryDirectoryProvider directoryProvider =
-                    localRecoveryConfig.getLocalStateDirectoryProvider();
+                    localRecoveryConfig
+                            .getLocalStateDirectoryProvider()
+                            .orElseThrow(LocalRecoveryConfig.localRecoveryNotEnabled());
             File directory = directoryProvider.subtaskSpecificCheckpointDirectory(checkpointId);
 
             if (!directory.exists() && !directory.mkdirs()) {
@@ -258,30 +294,35 @@ public class RocksIncrementalSnapshotStrategy<K>
         }
     }
 
-    private Set<StateHandleID> snapshotMetaData(
+    private PreviousSnapshot snapshotMetaData(
             long checkpointId, @Nonnull List<StateMetaInfoSnapshot> stateMetaInfoSnapshots) {
 
         final long lastCompletedCheckpoint;
-        final Set<StateHandleID> baseSstFiles;
+        final Map<StateHandleID, Long> confirmedSstFiles;
+        final Map<StateHandleID, StreamStateHandle> uploadedSstFiles;
 
         // use the last completed checkpoint as the comparison base.
-        synchronized (materializedSstFiles) {
+        synchronized (uploadedStateIDs) {
             lastCompletedCheckpoint = lastCompletedCheckpointId;
-            baseSstFiles = materializedSstFiles.get(lastCompletedCheckpoint);
+            confirmedSstFiles = uploadedStateIDs.get(lastCompletedCheckpoint);
+            LOG.trace(
+                    "Use confirmed SST files for checkpoint {}: {}",
+                    checkpointId,
+                    confirmedSstFiles);
         }
         LOG.trace(
                 "Taking incremental snapshot for checkpoint {}. Snapshot is based on last completed checkpoint {} "
-                        + "assuming the following (shared) files as base: {}.",
+                        + "assuming the following (shared) confirmed files as base: {}.",
                 checkpointId,
                 lastCompletedCheckpoint,
-                baseSstFiles);
+                confirmedSstFiles);
 
         // snapshot meta data to save
         for (Map.Entry<String, RocksDbKvStateInfo> stateMetaInfoEntry :
                 kvStateInformation.entrySet()) {
             stateMetaInfoSnapshots.add(stateMetaInfoEntry.getValue().metaInfo.snapshot());
         }
-        return baseSstFiles;
+        return new PreviousSnapshot(confirmedSstFiles);
     }
 
     private void takeDBNativeCheckpoint(@Nonnull SnapshotDirectory outputDirectory)
@@ -319,20 +360,24 @@ public class RocksIncrementalSnapshotStrategy<K>
         @Nonnull private final SnapshotDirectory localBackupDirectory;
 
         /** All sst files that were part of the last previously completed checkpoint. */
-        @Nullable private final Set<StateHandleID> baseSstFiles;
+        @Nonnull private final PreviousSnapshot previousSnapshot;
+
+        @Nonnull private final SnapshotType.SharingFilesStrategy sharingFilesStrategy;
 
         private RocksDBIncrementalSnapshotOperation(
                 long checkpointId,
                 @Nonnull CheckpointStreamFactory checkpointStreamFactory,
                 @Nonnull SnapshotDirectory localBackupDirectory,
-                @Nullable Set<StateHandleID> baseSstFiles,
+                @Nonnull PreviousSnapshot previousSnapshot,
+                @Nonnull SnapshotType.SharingFilesStrategy sharingFilesStrategy,
                 @Nonnull List<StateMetaInfoSnapshot> stateMetaInfoSnapshots) {
 
             this.checkpointStreamFactory = checkpointStreamFactory;
-            this.baseSstFiles = baseSstFiles;
+            this.previousSnapshot = previousSnapshot;
             this.checkpointId = checkpointId;
             this.localBackupDirectory = localBackupDirectory;
             this.stateMetaInfoSnapshots = stateMetaInfoSnapshots;
+            this.sharingFilesStrategy = sharingFilesStrategy;
         }
 
         @Override
@@ -359,10 +404,9 @@ public class RocksIncrementalSnapshotStrategy<K>
                         "Metadata for job manager was not properly created.");
 
                 uploadSstFiles(sstFiles, miscFiles, snapshotCloseableRegistry);
-
-                synchronized (materializedSstFiles) {
-                    materializedSstFiles.put(checkpointId, sstFiles.keySet());
-                }
+                long checkpointedSize = metaStateHandle.getStateSize();
+                checkpointedSize += getUploadedStateSize(sstFiles.values());
+                checkpointedSize += getUploadedStateSize(miscFiles.values());
 
                 final IncrementalRemoteKeyedStateHandle jmIncrementalKeyedStateHandle =
                         new IncrementalRemoteKeyedStateHandle(
@@ -371,7 +415,8 @@ public class RocksIncrementalSnapshotStrategy<K>
                                 checkpointId,
                                 sstFiles,
                                 miscFiles,
-                                metaStateHandle.getJobManagerOwnedSnapshot());
+                                metaStateHandle.getJobManagerOwnedSnapshot(),
+                                checkpointedSize);
 
                 final DirectoryStateHandle directoryStateHandle =
                         localBackupDirectory.completeSnapshotAndGetHandle();
@@ -386,7 +431,7 @@ public class RocksIncrementalSnapshotStrategy<K>
                                     directoryStateHandle,
                                     keyGroupRange,
                                     metaStateHandle.getTaskLocalSnapshot(),
-                                    sstFiles.keySet());
+                                    sstFiles);
 
                     snapshotResult =
                             SnapshotResult.withLocalState(
@@ -410,6 +455,13 @@ public class RocksIncrementalSnapshotStrategy<K>
             }
         }
 
+        private long getUploadedStateSize(Collection<StreamStateHandle> streamStateHandles) {
+            return streamStateHandles.stream()
+                    .filter(s -> !(s instanceof PlaceholderStreamStateHandle))
+                    .mapToLong(StateObject::getStateSize)
+                    .sum();
+        }
+
         private void cleanupIncompleteSnapshot(@Nonnull List<StateObject> statesToDiscard) {
 
             try {
@@ -447,12 +499,44 @@ public class RocksIncrementalSnapshotStrategy<K>
             if (files != null) {
                 createUploadFilePaths(files, sstFiles, sstFilePaths, miscFilePaths);
 
+                final CheckpointedStateScope stateScope =
+                        sharingFilesStrategy == SnapshotType.SharingFilesStrategy.NO_SHARING
+                                ? CheckpointedStateScope.EXCLUSIVE
+                                : CheckpointedStateScope.SHARED;
                 sstFiles.putAll(
                         stateUploader.uploadFilesToCheckpointFs(
-                                sstFilePaths, checkpointStreamFactory, snapshotCloseableRegistry));
+                                sstFilePaths,
+                                checkpointStreamFactory,
+                                stateScope,
+                                snapshotCloseableRegistry));
                 miscFiles.putAll(
                         stateUploader.uploadFilesToCheckpointFs(
-                                miscFilePaths, checkpointStreamFactory, snapshotCloseableRegistry));
+                                miscFilePaths,
+                                checkpointStreamFactory,
+                                stateScope,
+                                snapshotCloseableRegistry));
+
+                synchronized (uploadedStateIDs) {
+                    switch (sharingFilesStrategy) {
+                        case FORWARD_BACKWARD:
+                        case FORWARD:
+                            uploadedStateIDs.put(
+                                    checkpointId,
+                                    sstFiles.entrySet().stream()
+                                            .collect(
+                                                    Collectors.toMap(
+                                                            Map.Entry::getKey,
+                                                            t -> t.getValue().getStateSize())));
+                            break;
+                        case NO_SHARING:
+                            break;
+                        default:
+                            // This is just a safety precaution. It is checked before creating the
+                            // RocksDBIncrementalSnapshotOperation
+                            throw new IllegalArgumentException(
+                                    "Unsupported sharing files strategy: " + sharingFilesStrategy);
+                    }
+                }
             }
         }
 
@@ -466,16 +550,12 @@ public class RocksIncrementalSnapshotStrategy<K>
                 final StateHandleID stateHandleID = new StateHandleID(fileName);
 
                 if (fileName.endsWith(SST_FILE_SUFFIX)) {
-                    final boolean existsAlready =
-                            baseSstFiles != null && baseSstFiles.contains(stateHandleID);
-
-                    if (existsAlready) {
-                        // we introduce a placeholder state handle, that is replaced with the
-                        // original from the shared state registry (created from a previous
-                        // checkpoint)
-                        sstFiles.put(stateHandleID, new PlaceholderStreamStateHandle());
+                    Optional<StreamStateHandle> uploaded =
+                            previousSnapshot.getUploaded(stateHandleID);
+                    if (uploaded.isPresent()) {
+                        sstFiles.put(stateHandleID, uploaded.get());
                     } else {
-                        sstFilePaths.put(stateHandleID, filePath);
+                        sstFilePaths.put(stateHandleID, filePath); // re-upload
                     }
                 } else {
                     miscFilePaths.put(stateHandleID, filePath);
@@ -493,7 +573,10 @@ public class RocksIncrementalSnapshotStrategy<K>
                                     checkpointId,
                                     CheckpointedStateScope.EXCLUSIVE,
                                     checkpointStreamFactory,
-                                    localRecoveryConfig.getLocalStateDirectoryProvider())
+                                    localRecoveryConfig
+                                            .getLocalStateDirectoryProvider()
+                                            .orElseThrow(
+                                                    LocalRecoveryConfig.localRecoveryNotEnabled()))
                             : CheckpointStreamWithResultProvider.createSimpleStream(
                                     CheckpointedStateScope.EXCLUSIVE, checkpointStreamFactory);
 
@@ -531,15 +614,17 @@ public class RocksIncrementalSnapshotStrategy<K>
 
     static class IncrementalRocksDBSnapshotResources implements SnapshotResources {
         @Nonnull private final SnapshotDirectory snapshotDirectory;
-        @Nonnull private final Set<StateHandleID> baseSstFiles;
+
+        @Nonnull private final PreviousSnapshot previousSnapshot;
+
         @Nonnull private final List<StateMetaInfoSnapshot> stateMetaInfoSnapshots;
 
         public IncrementalRocksDBSnapshotResources(
                 SnapshotDirectory snapshotDirectory,
-                Set<StateHandleID> baseSstFiles,
+                PreviousSnapshot previousSnapshot,
                 List<StateMetaInfoSnapshot> stateMetaInfoSnapshots) {
             this.snapshotDirectory = snapshotDirectory;
-            this.baseSstFiles = baseSstFiles;
+            this.previousSnapshot = previousSnapshot;
             this.stateMetaInfoSnapshots = stateMetaInfoSnapshots;
         }
 
@@ -561,4 +646,29 @@ public class RocksIncrementalSnapshotStrategy<K>
             }
         }
     }
+
+    private static final PreviousSnapshot EMPTY_PREVIOUS_SNAPSHOT =
+            new PreviousSnapshot(Collections.emptyMap());
+
+    private static class PreviousSnapshot {
+
+        @Nullable private final Map<StateHandleID, Long> confirmedSstFiles;
+
+        private PreviousSnapshot(@Nullable Map<StateHandleID, Long> confirmedSstFiles) {
+            this.confirmedSstFiles = confirmedSstFiles;
+        }
+
+        private Optional<StreamStateHandle> getUploaded(StateHandleID stateHandleID) {
+            if (confirmedSstFiles != null && confirmedSstFiles.containsKey(stateHandleID)) {
+                // we introduce a placeholder state handle, that is replaced with the
+                // original from the shared state registry (created from a previous checkpoint)
+                return Optional.of(
+                        new PlaceholderStreamStateHandle(confirmedSstFiles.get(stateHandleID)));
+            } else {
+                // Don't use any uploaded but not confirmed handles because they might be deleted
+                // (by TM) if the previous checkpoint failed. See FLINK-25395
+                return Optional.empty();
+            }
+        }
+    }
 }
