@@ -87,6 +87,17 @@ public interface SubtaskCheckpointCoordinator extends Closeable {
             long checkpointId, OperatorChain<?, ?> operatorChain, Supplier<Boolean> isRunning)
             throws Exception;
 
+    /**
+     * Notified on the task side once a distributed checkpoint has been subsumed.
+     *
+     * @param checkpointId The checkpoint id to notify as been subsumed.
+     * @param operatorChain The chain of operators executed by the task.
+     * @param isRunning Whether the task is running.
+     */
+    void notifyCheckpointSubsumed(
+            long checkpointId, OperatorChain<?, ?> operatorChain, Supplier<Boolean> isRunning)
+            throws Exception;
+
     /** Waits for all the pending checkpoints to finish their asynchronous step. */
     void waitForPendingCheckpoints() throws Exception;
 }
