@@ -24,33 +24,25 @@ import org.apache.flink.configuration.ReadableConfig;
 import org.apache.flink.runtime.state.CheckpointStorage;
 import org.apache.flink.runtime.state.storage.FileSystemCheckpointStorage;
 import org.apache.flink.streaming.api.CheckpointingMode;
-import org.apache.flink.util.Preconditions;
 
-import org.hamcrest.CoreMatchers;
-import org.hamcrest.Description;
-import org.hamcrest.Matcher;
-import org.hamcrest.TypeSafeMatcher;
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
+import org.assertj.core.api.InstanceOfAssertFactories;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.MethodSource;
 
-import java.util.Arrays;
-import java.util.Collection;
 import java.util.function.BiConsumer;
 import java.util.function.Function;
+import java.util.stream.Stream;
 
-import static org.hamcrest.MatcherAssert.assertThat;
+import static org.assertj.core.api.Assertions.assertThat;
 
 /**
  * Tests for configuring {@link CheckpointConfig} via {@link
  * CheckpointConfig#configure(ReadableConfig)}.
  */
-@RunWith(Parameterized.class)
 public class CheckpointConfigFromConfigurationTest {
 
-    @Parameterized.Parameters(name = "{0}")
-    public static Collection<TestSpec<?>> specs() {
-        return Arrays.asList(
+    private static Stream<TestSpec<?>> specs() {
+        return Stream.of(
                 TestSpec.testValue(CheckpointingMode.AT_LEAST_ONCE)
                         .whenSetFromFile("execution.checkpointing.mode", "AT_LEAST_ONCE")
                         .viaSetter(CheckpointConfig::setCheckpointingMode)
@@ -109,73 +101,25 @@ public class CheckpointConfigFromConfigurationTest {
                         .getterVia(CheckpointConfig::getCheckpointStorage)
                         .nonDefaultValue(
                                 new FileSystemCheckpointStorage("file:///path/to/checkpoint/dir"))
-                        .customMatcher(FileSystemCheckpointStorageMatcher::new));
+                        .customMatcher(
+                                (actualValue, expectedValue) ->
+                                        assertThat(actualValue)
+                                                .hasSameClassAs(expectedValue)
+                                                .asInstanceOf(
+                                                        InstanceOfAssertFactories.type(
+                                                                FileSystemCheckpointStorage.class))
+                                                .extracting(
+                                                        FileSystemCheckpointStorage
+                                                                ::getCheckpointPath)
+                                                .isEqualTo(
+                                                        ((FileSystemCheckpointStorage)
+                                                                        expectedValue)
+                                                                .getCheckpointPath())));
     }
 
-    /**
-     * {@code FileSystemCheckpointStorageMatcher} verifies that the set {@link CheckpointStorage} is
-     * of type {@link FileSystemCheckpointStorage} pointing to the same filesystem path.
-     */
-    private static class FileSystemCheckpointStorageMatcher
-            extends TypeSafeMatcher<CheckpointStorage> {
-        private static final Class<FileSystemCheckpointStorage> EXPECTED_CHECKPOINT_STORAGE_CLASS =
-                FileSystemCheckpointStorage.class;
-        private final FileSystemCheckpointStorage fileSystemCheckpointStorageFromSetter;
-
-        public FileSystemCheckpointStorageMatcher(CheckpointStorage checkpointStorageFromSetter) {
-            Preconditions.checkArgument(
-                    checkpointStorageFromSetter
-                            .getClass()
-                            .equals(EXPECTED_CHECKPOINT_STORAGE_CLASS));
-            this.fileSystemCheckpointStorageFromSetter =
-                    (FileSystemCheckpointStorage) checkpointStorageFromSetter;
-        }
-
-        @Override
-        public void describeTo(Description description) {
-            describeObject(fileSystemCheckpointStorageFromSetter, description);
-        }
-
-        @Override
-        protected void describeMismatchSafely(
-                CheckpointStorage checkpointStorageFromFile, Description description) {
-            if (!checkpointStorageFromFile.getClass().equals(EXPECTED_CHECKPOINT_STORAGE_CLASS)) {
-                description.appendText(
-                        "Passed object is not of type "
-                                + EXPECTED_CHECKPOINT_STORAGE_CLASS.getCanonicalName());
-            } else {
-                describeObject(
-                        (FileSystemCheckpointStorage) checkpointStorageFromFile, description);
-            }
-            super.describeMismatchSafely(checkpointStorageFromFile, description);
-        }
-
-        private static void describeObject(
-                FileSystemCheckpointStorage fileSystemCheckpointStorage, Description description) {
-            description
-                    .appendText(EXPECTED_CHECKPOINT_STORAGE_CLASS.getCanonicalName())
-                    .appendText("(")
-                    .appendText(fileSystemCheckpointStorage.getCheckpointPath().toString())
-                    .appendText(")");
-        }
-
-        @Override
-        protected boolean matchesSafely(CheckpointStorage checkpointStorageFromFile) {
-            if (!checkpointStorageFromFile.getClass().equals(EXPECTED_CHECKPOINT_STORAGE_CLASS)) {
-                return false;
-            }
-            final FileSystemCheckpointStorage fileSystemCheckpointStorageFromFile =
-                    (FileSystemCheckpointStorage) checkpointStorageFromFile;
-            return fileSystemCheckpointStorageFromFile
-                    .getCheckpointPath()
-                    .equals(fileSystemCheckpointStorageFromSetter.getCheckpointPath());
-        }
-    }
-
-    @Parameterized.Parameter public TestSpec<?> spec;
-
-    @Test
-    public void testLoadingFromConfiguration() {
+    @ParameterizedTest
+    @MethodSource("specs")
+    public void testLoadingFromConfiguration(TestSpec<?> spec) {
         CheckpointConfig configFromSetters = new CheckpointConfig();
         CheckpointConfig configFromFile = new CheckpointConfig();
 
@@ -187,8 +131,9 @@ public class CheckpointConfigFromConfigurationTest {
         spec.assertEqual(configFromFile, configFromSetters);
     }
 
-    @Test
-    public void testNotOverridingIfNotSet() {
+    @ParameterizedTest
+    @MethodSource("specs")
+    public void testNotOverridingIfNotSet(TestSpec<?> spec) {
         CheckpointConfig config = new CheckpointConfig();
 
         spec.setNonDefaultValue(config);
@@ -206,7 +151,8 @@ public class CheckpointConfigFromConfigurationTest {
         private BiConsumer<CheckpointConfig, T> setter;
         private Function<CheckpointConfig, T> getter;
 
-        private Function<T, Matcher<T>> createMatcher = CoreMatchers::equalTo;
+        private BiConsumer<T, T> customAssertion =
+                (actualValue, expectedValue) -> assertThat(actualValue).isEqualTo(expectedValue);
 
         private TestSpec(T value) {
             this.objectValue = value;
@@ -237,8 +183,8 @@ public class CheckpointConfigFromConfigurationTest {
             return this;
         }
 
-        public TestSpec<T> customMatcher(Function<T, Matcher<T>> customMatcher) {
-            this.createMatcher = customMatcher;
+        public TestSpec<T> customMatcher(BiConsumer<T, T> customAssertion) {
+            this.customAssertion = customAssertion;
             return this;
         }
 
@@ -252,13 +198,11 @@ public class CheckpointConfigFromConfigurationTest {
 
         public void assertEqual(
                 CheckpointConfig configFromFile, CheckpointConfig configFromSetters) {
-            assertThat(
-                    getter.apply(configFromFile),
-                    createMatcher.apply(getter.apply(configFromSetters)));
+            customAssertion.accept(getter.apply(configFromFile), getter.apply(configFromSetters));
         }
 
         public void assertEqualNonDefault(CheckpointConfig configFromFile) {
-            assertThat(getter.apply(configFromFile), createMatcher.apply(nonDefaultValue));
+            customAssertion.accept(getter.apply(configFromFile), nonDefaultValue);
         }
 
         @Override
