@@ -28,6 +28,7 @@ import org.apache.flink.runtime.checkpoint.TaskStateSnapshot;
 import org.apache.flink.runtime.execution.Environment;
 import org.apache.flink.runtime.executiongraph.ExecutionAttemptID;
 import org.apache.flink.runtime.io.network.api.writer.ResultPartitionWriter;
+import org.apache.flink.runtime.io.network.partition.consumer.SingleInputGateBuilder;
 import org.apache.flink.runtime.io.network.partition.consumer.StreamTestSingleInputGate;
 import org.apache.flink.runtime.jobgraph.OperatorID;
 import org.apache.flink.runtime.metrics.groups.TaskMetricGroup;
@@ -40,7 +41,7 @@ import org.apache.flink.runtime.state.TestTaskStateManagerBuilder;
 import org.apache.flink.runtime.taskmanager.CheckpointResponder;
 import org.apache.flink.runtime.taskmanager.TaskManagerRuntimeInfo;
 import org.apache.flink.runtime.taskmanager.TestCheckpointResponder;
-import org.apache.flink.runtime.throughput.ThroughputCalculator;
+import org.apache.flink.runtime.throughput.BufferDebloatConfiguration;
 import org.apache.flink.runtime.util.TestingTaskManagerRuntimeInfo;
 import org.apache.flink.streaming.api.TimeCharacteristic;
 import org.apache.flink.streaming.api.graph.StreamConfig;
@@ -56,7 +57,6 @@ import org.apache.flink.streaming.api.operators.StreamOperatorFactory;
 import org.apache.flink.streaming.runtime.partitioner.BroadcastPartitioner;
 import org.apache.flink.streaming.runtime.partitioner.ForwardPartitioner;
 import org.apache.flink.streaming.runtime.streamrecord.StreamElementSerializer;
-import org.apache.flink.util.clock.SystemClock;
 import org.apache.flink.util.function.FunctionWithException;
 
 import javax.annotation.Nullable;
@@ -70,6 +70,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Queue;
 import java.util.function.Consumer;
+import java.util.function.Function;
 
 import static org.apache.flink.util.Preconditions.checkNotNull;
 import static org.apache.flink.util.Preconditions.checkState;
@@ -100,10 +101,9 @@ public class StreamTaskMailboxTestHarnessBuilder<OUT> {
 
     private boolean setupCalled = false;
 
-    private ThroughputCalculator throughputCalculator =
-            new ThroughputCalculator(SystemClock.getInstance(), 10);
-
     private TaskManagerRuntimeInfo taskManagerRuntimeInfo = new TestingTaskManagerRuntimeInfo();
+    private Function<SingleInputGateBuilder, SingleInputGateBuilder> modifyGateBuilder =
+            Function.identity();
 
     public StreamTaskMailboxTestHarnessBuilder(
             FunctionWithException<Environment, ? extends StreamTask<OUT, ?>, Exception> taskFactory,
@@ -131,12 +131,6 @@ public class StreamTaskMailboxTestHarnessBuilder<OUT> {
         return this;
     }
 
-    public <T> StreamTaskMailboxTestHarnessBuilder<OUT> setThroughputCalculator(
-            ThroughputCalculator throughputCalculator) {
-        this.throughputCalculator = throughputCalculator;
-        return this;
-    }
-
     public <T> StreamTaskMailboxTestHarnessBuilder<OUT> setTaskManagerRuntimeInfo(
             TaskManagerRuntimeInfo taskManagerRuntimeInfo) {
         this.taskManagerRuntimeInfo = taskManagerRuntimeInfo;
@@ -148,6 +142,12 @@ public class StreamTaskMailboxTestHarnessBuilder<OUT> {
         return this;
     }
 
+    public <T> StreamTaskMailboxTestHarnessBuilder<OUT> modifyGateBuilder(
+            Function<SingleInputGateBuilder, SingleInputGateBuilder> singleInputGateBuilder) {
+        this.modifyGateBuilder = singleInputGateBuilder;
+        return this;
+    }
+
     public StreamTaskMailboxTestHarnessBuilder<OUT> addInput(TypeInformation<?> inputType) {
         return addInput(inputType, 1);
     }
@@ -229,12 +229,11 @@ public class StreamTaskMailboxTestHarnessBuilder<OUT> {
                         new MockInputSplitProvider(),
                         bufferSize,
                         taskStateManager,
-                        throughputCalculator,
                         collectNetworkEvents);
 
         streamMockEnvironment.setCheckpointResponder(taskStateManager.getCheckpointResponder());
-        initializeInputs(streamMockEnvironment);
         streamMockEnvironment.setTaskManagerInfo(taskManagerRuntimeInfo);
+        initializeInputs(streamMockEnvironment);
 
         checkState(inputGates != null, "InputGates hasn't been initialised");
 
@@ -307,7 +306,14 @@ public class StreamTaskMailboxTestHarnessBuilder<OUT> {
                         inputChannelsPerGate.get(gateIndex),
                         gateIndex,
                         inputSerializer,
-                        bufferSize);
+                        bufferSize,
+                        modifyGateBuilder.apply(
+                                new SingleInputGateBuilder()
+                                        .setBufferDebloatConfiguration(
+                                                BufferDebloatConfiguration.fromConfiguration(
+                                                        streamMockEnvironment
+                                                                .getTaskManagerInfo()
+                                                                .getConfiguration()))));
 
         StreamNode sourceVertexDummy =
                 new StreamNode(
