@@ -18,9 +18,10 @@
 
 package org.apache.flink.streaming.runtime.tasks;
 
+import org.apache.flink.core.execution.SavepointFormatType;
 import org.apache.flink.runtime.checkpoint.CheckpointMetaData;
 import org.apache.flink.runtime.checkpoint.CheckpointOptions;
-import org.apache.flink.runtime.checkpoint.CheckpointType;
+import org.apache.flink.runtime.checkpoint.SavepointType;
 import org.apache.flink.runtime.execution.CancelTaskException;
 import org.apache.flink.runtime.execution.Environment;
 import org.apache.flink.runtime.operators.testutils.DummyEnvironment;
@@ -39,7 +40,6 @@ import java.util.concurrent.LinkedBlockingQueue;
 import static org.apache.flink.util.Preconditions.checkNotNull;
 import static org.hamcrest.Matchers.instanceOf;
 import static org.hamcrest.Matchers.is;
-import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
@@ -76,24 +76,6 @@ public class SynchronousCheckpointTest {
         assertThat(eventQueue.take(), is(Event.TASK_INITIALIZED));
     }
 
-    @Test(timeout = 20_000)
-    public void synchronousCheckpointBlocksUntilNotificationForCorrectCheckpointComes()
-            throws Exception {
-        launchSynchronousSavepointAndWaitForSyncSavepointIdToBeSet();
-        assertTrue(streamTaskUnderTest.getSynchronousSavepointId().isPresent());
-
-        streamTaskUnderTest.notifyCheckpointCompleteAsync(41).get();
-        assertTrue(streamTaskUnderTest.getSynchronousSavepointId().isPresent());
-
-        streamTaskUnderTest.notifyCheckpointCompleteAsync(42).get();
-        assertFalse(streamTaskUnderTest.getSynchronousSavepointId().isPresent());
-
-        streamTaskUnderTest.stopTask();
-        waitUntilMainExecutionThreadIsFinished();
-
-        assertFalse(streamTaskUnderTest.isCanceled());
-    }
-
     @Test(timeout = 10_000)
     public void cancelShouldAlsoCancelPendingSynchronousCheckpoint() throws Throwable {
         launchSynchronousSavepointAndWaitForSyncSavepointIdToBeSet();
@@ -111,7 +93,7 @@ public class SynchronousCheckpointTest {
         streamTaskUnderTest.triggerCheckpointAsync(
                 new CheckpointMetaData(42, System.currentTimeMillis()),
                 new CheckpointOptions(
-                        CheckpointType.SAVEPOINT_SUSPEND,
+                        SavepointType.suspend(SavepointFormatType.CANONICAL),
                         CheckpointStorageLocationReference.getDefault()));
         waitForSyncSavepointIdToBeSet(streamTaskUnderTest);
     }
