@@ -25,7 +25,6 @@ import org.apache.flink.table.expressions.CallExpression;
 import org.apache.flink.table.expressions.ResolvedExpression;
 import org.apache.flink.table.expressions.utils.ResolvedExpressionMock;
 import org.apache.flink.table.functions.BuiltInFunctionDefinitions;
-import org.apache.flink.table.functions.FunctionIdentifier;
 import org.apache.flink.table.types.DataType;
 import org.apache.flink.table.types.logical.LocalZonedTimestampType;
 import org.apache.flink.table.types.logical.LogicalType;
@@ -194,10 +193,7 @@ public class SchemaResolutionTest {
                         Collections.singletonList(
                                 WatermarkSpec.of(
                                         "ts_ltz",
-                                        new CallExpression(
-                                                FunctionIdentifier.of(
-                                                        BuiltInFunctionDefinitions.SOURCE_WATERMARK
-                                                                .getName()),
+                                        CallExpression.permanent(
                                                 BuiltInFunctionDefinitions.SOURCE_WATERMARK,
                                                 Collections.emptyList(),
                                                 DataTypes.TIMESTAMP_LTZ(1)))),
@@ -225,6 +221,26 @@ public class SchemaResolutionTest {
                 Schema.newBuilder().columnByExpression("invalid", callSql("INVALID")).build(),
                 "Invalid expression for computed column 'invalid'.");
 
+        // metadata columns
+
+        testError(
+                Schema.newBuilder()
+                        .columnByMetadata("metadata", DataTypes.INT())
+                        .columnByMetadata("from_metadata", DataTypes.BIGINT(), "metadata", false)
+                        .build(),
+                "The column `metadata` and `from_metadata` in the table are both from the same metadata key 'metadata'. "
+                        + "Please specify one of the columns as the metadata column and use the "
+                        + "computed column syntax to specify the others.");
+
+        testError(
+                Schema.newBuilder()
+                        .columnByMetadata("from_metadata", DataTypes.BIGINT(), "metadata", false)
+                        .columnByMetadata("from_metadata2", DataTypes.STRING(), "metadata", true)
+                        .build(),
+                "The column `from_metadata` and `from_metadata2` in the table are both from the same metadata key 'metadata'. "
+                        + "Please specify one of the columns as the metadata column and use the "
+                        + "computed column syntax to specify the others.");
+
         // time attributes and watermarks
 
         testError(
