@@ -26,7 +26,10 @@ import org.apache.flink.table.annotation.InputGroup;
 import org.apache.flink.table.api.DataTypes;
 import org.apache.flink.table.api.TableConfig;
 import org.apache.flink.table.api.TableSchema;
+import org.apache.flink.table.catalog.CatalogTable;
+import org.apache.flink.table.catalog.ContextResolvedTable;
 import org.apache.flink.table.catalog.ObjectIdentifier;
+import org.apache.flink.table.catalog.ResolvedCatalogTable;
 import org.apache.flink.table.catalog.ResolvedSchema;
 import org.apache.flink.table.expressions.CallExpression;
 import org.apache.flink.table.expressions.Expression;
@@ -38,8 +41,8 @@ import org.apache.flink.table.functions.FunctionDefinition;
 import org.apache.flink.table.functions.FunctionIdentifier;
 import org.apache.flink.table.functions.ScalarFunction;
 import org.apache.flink.table.functions.ScalarFunctionDefinition;
-import org.apache.flink.table.operations.CatalogQueryOperation;
 import org.apache.flink.table.operations.QueryOperation;
+import org.apache.flink.table.operations.SourceQueryOperation;
 import org.apache.flink.table.types.utils.DataTypeFactoryMock;
 import org.apache.flink.table.utils.FunctionLookupMock;
 
@@ -57,6 +60,7 @@ import java.util.Optional;
 
 import static org.apache.flink.table.api.Expressions.$;
 import static org.apache.flink.table.api.Expressions.call;
+import static org.apache.flink.table.api.Expressions.col;
 import static org.apache.flink.table.api.Expressions.range;
 import static org.apache.flink.table.api.Expressions.withColumns;
 import static org.apache.flink.table.expressions.ApiExpressionUtils.valueLiteral;
@@ -97,8 +101,7 @@ public class ExpressionResolverTest {
                                         .build())
                         .select($("f0").flatten())
                         .equalTo(
-                                new CallExpression(
-                                        FunctionIdentifier.of("get"),
+                                CallExpression.permanent(
                                         BuiltInFunctionDefinitions.GET,
                                         Arrays.asList(
                                                 new FieldReferenceExpression(
@@ -112,8 +115,7 @@ public class ExpressionResolverTest {
                                                         0),
                                                 new ValueLiteralExpression("n0")),
                                         DataTypes.BIGINT()),
-                                new CallExpression(
-                                        FunctionIdentifier.of("get"),
+                                CallExpression.permanent(
                                         BuiltInFunctionDefinitions.GET,
                                         Arrays.asList(
                                                 new FieldReferenceExpression(
@@ -135,8 +137,7 @@ public class ExpressionResolverTest {
                                         .build())
                         .select($("f0").isEqual($("f1")))
                         .equalTo(
-                                new CallExpression(
-                                        FunctionIdentifier.of("equals"),
+                                CallExpression.permanent(
                                         BuiltInFunctionDefinitions.EQUALS,
                                         Arrays.asList(
                                                 new FieldReferenceExpression(
@@ -151,7 +152,7 @@ public class ExpressionResolverTest {
                                 new ScalarFunctionDefinition("func", new LegacyScalarFunc()))
                         .select(call("func", 1, $("f0")))
                         .equalTo(
-                                new CallExpression(
+                                CallExpression.permanent(
                                         FunctionIdentifier.of("func"),
                                         new ScalarFunctionDefinition(
                                                 "func", new LegacyScalarFunc()),
@@ -165,7 +166,7 @@ public class ExpressionResolverTest {
                         .lookupFunction("func", new ScalarFunc())
                         .select(call("func", 1, $("f0")))
                         .equalTo(
-                                new CallExpression(
+                                CallExpression.permanent(
                                         FunctionIdentifier.of("func"),
                                         new ScalarFunc(),
                                         Arrays.asList(
@@ -177,7 +178,7 @@ public class ExpressionResolverTest {
                         .inputSchemas(TableSchema.builder().field("f0", DataTypes.INT()).build())
                         .select(call(ScalarFunc.class, 1, $("f0")))
                         .equalTo(
-                                new CallExpression(
+                                CallExpression.anonymous(
                                         new ScalarFunc(),
                                         Arrays.asList(
                                                 valueLiteral(1),
@@ -189,7 +190,7 @@ public class ExpressionResolverTest {
                         .lookupFunction(ObjectIdentifier.of("cat", "db", "func"), new ScalarFunc())
                         .select(call("cat.db.func", 1, $("f0")))
                         .equalTo(
-                                new CallExpression(
+                                CallExpression.permanent(
                                         FunctionIdentifier.of(
                                                 ObjectIdentifier.of("cat", "db", "func")),
                                         new ScalarFunc(),
@@ -203,14 +204,14 @@ public class ExpressionResolverTest {
                         .lookupFunction("func", new ScalarFunc())
                         .select(call("func", call(new ScalarFunc(), call("func", 1, $("f0")))))
                         .equalTo(
-                                new CallExpression(
+                                CallExpression.permanent(
                                         FunctionIdentifier.of("func"),
                                         new ScalarFunc(),
                                         Collections.singletonList(
-                                                new CallExpression(
+                                                CallExpression.anonymous(
                                                         new ScalarFunc(),
                                                         Collections.singletonList(
-                                                                new CallExpression(
+                                                                CallExpression.permanent(
                                                                         FunctionIdentifier.of(
                                                                                 "func"),
                                                                         new ScalarFunc(),
@@ -240,7 +241,7 @@ public class ExpressionResolverTest {
                         .lookupFunction("func", new ScalarFunc())
                         .select(call("func", $("*")))
                         .equalTo(
-                                new CallExpression(
+                                CallExpression.permanent(
                                         FunctionIdentifier.of("func"),
                                         new ScalarFunc(),
                                         Arrays.asList(
@@ -248,7 +249,11 @@ public class ExpressionResolverTest {
                                                         "f0", DataTypes.INT(), 0, 0),
                                                 new FieldReferenceExpression(
                                                         "f1", DataTypes.STRING(), 0, 1)),
-                                        DataTypes.INT().notNull().bridgedTo(int.class))));
+                                        DataTypes.INT().notNull().bridgedTo(int.class))),
+                TestSpec.test("Test field reference with col()")
+                        .inputSchemas(TableSchema.builder().field("i", DataTypes.INT()).build())
+                        .select(col("i"))
+                        .equalTo(new FieldReferenceExpression("i", DataTypes.INT(), 0, 0)));
     }
 
     @Parameterized.Parameter public TestSpec testSpec;
@@ -357,12 +362,23 @@ public class ExpressionResolverTest {
                                     .map(
                                             schema ->
                                                     (QueryOperation)
-                                                            new CatalogQueryOperation(
-                                                                    ObjectIdentifier.of("", "", ""),
-                                                                    ResolvedSchema.physical(
-                                                                            schema.getFieldNames(),
-                                                                            schema
-                                                                                    .getFieldDataTypes())))
+                                                            new SourceQueryOperation(
+                                                                    ContextResolvedTable.anonymous(
+                                                                            new ResolvedCatalogTable(
+                                                                                    CatalogTable.of(
+                                                                                            schema
+                                                                                                    .toSchema(),
+                                                                                            null,
+                                                                                            Collections
+                                                                                                    .emptyList(),
+                                                                                            Collections
+                                                                                                    .emptyMap()),
+                                                                                    ResolvedSchema
+                                                                                            .physical(
+                                                                                                    schema
+                                                                                                            .getFieldNames(),
+                                                                                                    schema
+                                                                                                            .getFieldDataTypes())))))
                                     .toArray(QueryOperation[]::new))
                     .build();
         }
