@@ -61,9 +61,7 @@ import java.util.List;
 import java.util.stream.Collectors;
 
 import static org.apache.flink.table.types.logical.utils.LogicalTypeCasts.supportsAvoidingCast;
-import static org.hamcrest.CoreMatchers.equalTo;
-import static org.junit.Assert.assertThat;
-import static org.junit.Assert.assertTrue;
+import static org.assertj.core.api.Assertions.assertThat;
 
 /** Tests for {@link LogicalTypeCasts#supportsAvoidingCast(LogicalType, LogicalType)}. */
 @RunWith(Parameterized.class)
@@ -304,9 +302,9 @@ public class LogicalTypeCastAvoidanceTest {
 
     @Test
     public void testSupportsAvoidingCast() {
-        assertThat(supportsAvoidingCast(sourceType, targetType), equalTo(equals));
-        assertTrue(supportsAvoidingCast(sourceType, sourceType.copy()));
-        assertTrue(supportsAvoidingCast(targetType, targetType.copy()));
+        assertThat(supportsAvoidingCast(sourceType, targetType)).isEqualTo(equals);
+        assertThat(supportsAvoidingCast(sourceType, sourceType.copy())).isTrue();
+        assertThat(supportsAvoidingCast(targetType, targetType.copy())).isTrue();
     }
 
     private static DistinctType createDistinctType(String name, LogicalType sourceType) {
