@@ -28,9 +28,9 @@ import org.apache.calcite.sql.SqlFunction;
 import org.apache.calcite.sql.SqlFunctionCategory;
 import org.apache.calcite.sql.SqlGroupedWindowFunction;
 import org.apache.calcite.sql.SqlIdentifier;
+import org.apache.calcite.sql.SqlJsonConstructorNullClause;
 import org.apache.calcite.sql.SqlKind;
 import org.apache.calcite.sql.SqlOperator;
-import org.apache.calcite.sql.SqlOperatorBinding;
 import org.apache.calcite.sql.SqlPostfixOperator;
 import org.apache.calcite.sql.SqlSyntax;
 import org.apache.calcite.sql.fun.SqlStdOperatorTable;
@@ -52,7 +52,7 @@ import java.util.List;
 
 import static org.apache.flink.table.planner.plan.type.FlinkReturnTypes.ARG0_VARCHAR_FORCE_NULLABLE;
 import static org.apache.flink.table.planner.plan.type.FlinkReturnTypes.STR_MAP_NULLABLE;
-import static org.apache.flink.table.planner.plan.type.FlinkReturnTypes.VARCHAR_2000_NULLABLE;
+import static org.apache.flink.table.planner.plan.type.FlinkReturnTypes.VARCHAR_FORCE_NULLABLE;
 
 /** Operator table that contains only Flink-specific functions and operators. */
 public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
@@ -72,23 +72,9 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
         return instance;
     }
 
-    private static final SqlReturnTypeInference ROWTIME_TYPE_INFERENCE =
-            createTimeIndicatorReturnType(true, false);
     private static final SqlReturnTypeInference PROCTIME_TYPE_INFERENCE =
-            createTimeIndicatorReturnType(false, true);
-
-    private static SqlReturnTypeInference createTimeIndicatorReturnType(
-            boolean isRowTime, boolean isTimestampLtz) {
-        return ReturnTypes.explicit(
-                factory -> {
-                    if (isRowTime) {
-                        return ((FlinkTypeFactory) factory)
-                                .createRowtimeIndicatorType(false, isTimestampLtz);
-                    } else {
-                        return ((FlinkTypeFactory) factory).createProctimeIndicatorType(false);
-                    }
-                });
-    }
+            ReturnTypes.explicit(
+                    factory -> ((FlinkTypeFactory) factory).createProctimeIndicatorType(false));
 
     @Override
     public void lookupOperatorOverloads(
@@ -110,78 +96,83 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
 
     /** Function used to access a processing time attribute. */
     public static final SqlFunction PROCTIME =
-            new CalciteSqlFunction(
-                    "PROCTIME",
-                    SqlKind.OTHER_FUNCTION,
-                    PROCTIME_TYPE_INFERENCE,
-                    null,
-                    OperandTypes.NILADIC,
-                    SqlFunctionCategory.TIMEDATE,
-                    false);
+            BuiltInSqlFunction.newBuilder()
+                    .name("PROCTIME")
+                    .returnType(PROCTIME_TYPE_INFERENCE)
+                    .operandTypeChecker(OperandTypes.NILADIC)
+                    .notDeterministic()
+                    .build();
 
     /**
-     * Function used to access a event time attribute with TIMESTAMP or TIMESTAMP_LTZ type from
-     * MATCH_RECOGNIZE, for TIMESTAMP_LTZ type, we rewrite the return type in
-     * [org.apache.flink.table.planner.calcite.RelTimeIndicatorConverter].
+     * Function used to access an event time attribute with TIMESTAMP or TIMESTAMP_LTZ type from
+     * MATCH_RECOGNIZE.
      */
-    public static final SqlFunction MATCH_ROWTIME =
-            new CalciteSqlFunction(
-                    "MATCH_ROWTIME",
-                    SqlKind.OTHER_FUNCTION,
-                    ROWTIME_TYPE_INFERENCE,
-                    null,
-                    OperandTypes.NILADIC,
-                    SqlFunctionCategory.MATCH_RECOGNIZE,
-                    true);
+    public static final SqlFunction MATCH_ROWTIME = new MatchRowTimeFunction();
 
     /** Function used to access a processing time attribute from MATCH_RECOGNIZE. */
     public static final SqlFunction MATCH_PROCTIME =
-            new CalciteSqlFunction(
-                    "MATCH_PROCTIME",
-                    SqlKind.OTHER_FUNCTION,
-                    PROCTIME_TYPE_INFERENCE,
-                    null,
-                    OperandTypes.NILADIC,
-                    SqlFunctionCategory.MATCH_RECOGNIZE,
-                    false);
+            BuiltInSqlFunction.newBuilder()
+                    .name("MATCH_PROCTIME")
+                    .category(SqlFunctionCategory.MATCH_RECOGNIZE)
+                    .returnType(PROCTIME_TYPE_INFERENCE)
+                    .operandTypeChecker(OperandTypes.NILADIC)
+                    .notDeterministic()
+                    .build();
 
-    /** Function used to materialize a processing time attribute. */
-    public static final SqlFunction PROCTIME_MATERIALIZE = new ProctimeMaterializeSqlFunction();
+    /**
+     * Function that materializes a processing time attribute. After materialization the result can
+     * be used in regular arithmetical calculations.
+     */
+    public static final SqlFunction PROCTIME_MATERIALIZE =
+            BuiltInSqlFunction.newBuilder()
+                    .name("PROCTIME_MATERIALIZE")
+                    .returnType(
+                            ReturnTypes.cascade(
+                                    ReturnTypes.explicit(
+                                            SqlTypeName.TIMESTAMP_WITH_LOCAL_TIME_ZONE, 3),
+                                    SqlTypeTransforms.TO_NULLABLE))
+                    .operandTypeInference(InferTypes.RETURN_TYPE)
+                    .operandTypeChecker(OperandTypes.family(SqlTypeFamily.TIMESTAMP))
+                    .monotonicity(SqlMonotonicity.INCREASING)
+                    .notDeterministic()
+                    .build();
 
     /** Function to access the timestamp of a StreamRecord. */
-    public static final SqlFunction STREAMRECORD_TIMESTAMP = new StreamRecordTimestampSqlFunction();
+    public static final SqlFunction STREAMRECORD_TIMESTAMP =
+            BuiltInSqlFunction.newBuilder()
+                    .name("STREAMRECORD_TIMESTAMP")
+                    .returnType(ReturnTypes.explicit(SqlTypeName.BIGINT))
+                    .operandTypeInference(InferTypes.RETURN_TYPE)
+                    .operandTypeChecker(OperandTypes.family(SqlTypeFamily.NUMERIC))
+                    .build();
 
     /** Function to access the constant value of E. */
     public static final SqlFunction E =
-            new SqlFunction(
-                    "E",
-                    SqlKind.OTHER_FUNCTION,
-                    ReturnTypes.DOUBLE,
-                    null,
-                    OperandTypes.NILADIC,
-                    SqlFunctionCategory.NUMERIC);
+            BuiltInSqlFunction.newBuilder()
+                    .name("E")
+                    .returnType(ReturnTypes.DOUBLE)
+                    .operandTypeChecker(OperandTypes.NILADIC)
+                    .build();
 
     /** Function to access the constant value of PI. */
     public static final SqlFunction PI_FUNCTION =
-            new SqlFunction(
-                    "PI",
-                    SqlKind.OTHER_FUNCTION,
-                    ReturnTypes.DOUBLE,
-                    null,
-                    OperandTypes.NILADIC,
-                    SqlFunctionCategory.NUMERIC);
+            BuiltInSqlFunction.newBuilder()
+                    .name("PI")
+                    .returnType(ReturnTypes.DOUBLE)
+                    .operandTypeChecker(OperandTypes.NILADIC)
+                    .build();
 
     /** Function for concat strings, it is same with {@link #CONCAT}, but this is a function. */
     public static final SqlFunction CONCAT_FUNCTION =
-            new SqlFunction(
-                    "CONCAT",
-                    SqlKind.OTHER_FUNCTION,
-                    ReturnTypes.cascade(
-                            ReturnTypes.explicit(SqlTypeName.VARCHAR),
-                            SqlTypeTransforms.TO_NULLABLE),
-                    null,
-                    OperandTypes.repeat(SqlOperandCountRanges.from(1), OperandTypes.STRING),
-                    SqlFunctionCategory.STRING);
+            BuiltInSqlFunction.newBuilder()
+                    .name("CONCAT")
+                    .returnType(
+                            ReturnTypes.cascade(
+                                    ReturnTypes.explicit(SqlTypeName.VARCHAR),
+                                    SqlTypeTransforms.TO_NULLABLE))
+                    .operandTypeChecker(
+                            OperandTypes.repeat(SqlOperandCountRanges.from(1), OperandTypes.STRING))
+                    .build();
 
     /** Function for concat strings with a separator. */
     public static final SqlFunction CONCAT_WS =
@@ -329,7 +320,7 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             new SqlFunction(
                     "CHR",
                     SqlKind.OTHER_FUNCTION,
-                    VARCHAR_2000_NULLABLE,
+                    VARCHAR_FORCE_NULLABLE,
                     null,
                     OperandTypes.family(SqlTypeFamily.INTEGER),
                     SqlFunctionCategory.STRING);
@@ -373,7 +364,7 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             new SqlFunction(
                     "REVERSE",
                     SqlKind.OTHER_FUNCTION,
-                    VARCHAR_2000_NULLABLE,
+                    VARCHAR_FORCE_NULLABLE,
                     null,
                     OperandTypes.family(SqlTypeFamily.STRING),
                     SqlFunctionCategory.STRING);
@@ -394,7 +385,7 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             new SqlFunction(
                     "SPLIT_INDEX",
                     SqlKind.OTHER_FUNCTION,
-                    VARCHAR_2000_NULLABLE,
+                    VARCHAR_FORCE_NULLABLE,
                     null,
                     OperandTypes.or(
                             OperandTypes.family(
@@ -423,9 +414,7 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             new SqlFunction(
                     "REGEXP_EXTRACT",
                     SqlKind.OTHER_FUNCTION,
-                    ReturnTypes.cascade(
-                            ReturnTypes.explicit(SqlTypeName.VARCHAR),
-                            SqlTypeTransforms.TO_NULLABLE),
+                    VARCHAR_FORCE_NULLABLE,
                     null,
                     OperandTypes.or(OperandTypes.STRING_STRING_INTEGER, OperandTypes.STRING_STRING),
                     SqlFunctionCategory.STRING);
@@ -529,9 +518,7 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             new SqlFunction(
                     "DATE_FORMAT",
                     SqlKind.OTHER_FUNCTION,
-                    ReturnTypes.cascade(
-                            ReturnTypes.explicit(SqlTypeName.VARCHAR),
-                            SqlTypeTransforms.FORCE_NULLABLE),
+                    VARCHAR_FORCE_NULLABLE,
                     InferTypes.RETURN_TYPE,
                     OperandTypes.or(
                             OperandTypes.family(SqlTypeFamily.TIMESTAMP, SqlTypeFamily.STRING),
@@ -551,7 +538,7 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             new SqlFunction(
                     "PARSE_URL",
                     SqlKind.OTHER_FUNCTION,
-                    VARCHAR_2000_NULLABLE,
+                    VARCHAR_FORCE_NULLABLE,
                     null,
                     OperandTypes.or(
                             OperandTypes.family(SqlTypeFamily.STRING, SqlTypeFamily.STRING),
@@ -596,37 +583,31 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             };
 
     public static final SqlFunction UNIX_TIMESTAMP =
-            new SqlFunction(
-                    "UNIX_TIMESTAMP",
-                    SqlKind.OTHER_FUNCTION,
-                    ReturnTypes.BIGINT_NULLABLE,
-                    null,
-                    OperandTypes.or(
-                            OperandTypes.NILADIC,
-                            OperandTypes.family(SqlTypeFamily.STRING),
-                            OperandTypes.family(SqlTypeFamily.STRING, SqlTypeFamily.STRING)),
-                    SqlFunctionCategory.TIMEDATE) {
-
-                @Override
-                public boolean isDeterministic() {
-                    return false;
-                }
-
-                @Override
-                public SqlMonotonicity getMonotonicity(SqlOperatorBinding call) {
-                    if (call.getOperandCount() == 0) {
-                        return SqlMonotonicity.INCREASING;
-                    } else {
-                        return SqlMonotonicity.NOT_MONOTONIC;
-                    }
-                }
-            };
+            BuiltInSqlFunction.newBuilder()
+                    .name("UNIX_TIMESTAMP")
+                    .returnType(ReturnTypes.BIGINT_NULLABLE)
+                    .operandTypeChecker(
+                            OperandTypes.or(
+                                    OperandTypes.NILADIC,
+                                    OperandTypes.family(SqlTypeFamily.STRING),
+                                    OperandTypes.family(
+                                            SqlTypeFamily.STRING, SqlTypeFamily.STRING)))
+                    .notDeterministic()
+                    .monotonicity(
+                            call -> {
+                                if (call.getOperandCount() == 0) {
+                                    return SqlMonotonicity.INCREASING;
+                                } else {
+                                    return SqlMonotonicity.NOT_MONOTONIC;
+                                }
+                            })
+                    .build();
 
     public static final SqlFunction FROM_UNIXTIME =
             new SqlFunction(
                     "FROM_UNIXTIME",
                     SqlKind.OTHER_FUNCTION,
-                    VARCHAR_2000_NULLABLE,
+                    VARCHAR_FORCE_NULLABLE,
                     null,
                     OperandTypes.or(
                             OperandTypes.family(SqlTypeFamily.INTEGER),
@@ -699,19 +680,12 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
                     SqlFunctionCategory.STRING);
 
     public static final SqlFunction UUID =
-            new SqlFunction(
-                    "UUID",
-                    SqlKind.OTHER_FUNCTION,
-                    ReturnTypes.explicit(SqlTypeName.CHAR, 36),
-                    null,
-                    OperandTypes.NILADIC,
-                    SqlFunctionCategory.STRING) {
-
-                @Override
-                public boolean isDeterministic() {
-                    return false;
-                }
-            };
+            BuiltInSqlFunction.newBuilder()
+                    .name("UUID")
+                    .returnType(ReturnTypes.explicit(SqlTypeName.CHAR, 36))
+                    .operandTypeChecker(OperandTypes.NILADIC)
+                    .notDeterministic()
+                    .build();
 
     public static final SqlFunction SUBSTRING =
             new SqlFunction(
@@ -803,9 +777,7 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             new SqlFunction(
                     "CONVERT_TZ",
                     SqlKind.OTHER_FUNCTION,
-                    ReturnTypes.cascade(
-                            ReturnTypes.explicit(SqlTypeName.VARCHAR),
-                            SqlTypeTransforms.FORCE_NULLABLE),
+                    VARCHAR_FORCE_NULLABLE,
                     null,
                     OperandTypes.family(
                             SqlTypeFamily.STRING, SqlTypeFamily.STRING, SqlTypeFamily.STRING),
@@ -849,7 +821,7 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
             new SqlFunction(
                     "DECODE",
                     SqlKind.OTHER_FUNCTION,
-                    VARCHAR_2000_NULLABLE,
+                    VARCHAR_FORCE_NULLABLE,
                     null,
                     OperandTypes.family(SqlTypeFamily.BINARY, SqlTypeFamily.STRING),
                     SqlFunctionCategory.STRING);
@@ -901,6 +873,8 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
                             OperandTypes.family(SqlTypeFamily.STRING, SqlTypeFamily.STRING)),
                     SqlFunctionCategory.STRING);
 
+    public static final SqlFunction TRY_CAST = new SqlTryCastFunction();
+
     /** <code>AUXILIARY_GROUP</code> aggregate function. Only be used in internally. */
     public static final SqlAggFunction AUXILIARY_GROUP = new SqlAuxiliaryGroupAggFunction();
 
@@ -1111,7 +1085,6 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
     public static final SqlFunction ABS = SqlStdOperatorTable.ABS;
     public static final SqlFunction EXP = SqlStdOperatorTable.EXP;
     public static final SqlFunction NULLIF = SqlStdOperatorTable.NULLIF;
-    public static final SqlFunction COALESCE = SqlStdOperatorTable.COALESCE;
     public static final SqlFunction FLOOR = SqlStdOperatorTable.FLOOR;
     public static final SqlFunction CEIL = SqlStdOperatorTable.CEIL;
     public static final SqlFunction LOCALTIME = SqlStdOperatorTable.LOCALTIME;
@@ -1167,7 +1140,18 @@ public class FlinkSqlOperatorTable extends ReflectiveSqlOperatorTable {
 
     // JSON FUNCTIONS
     public static final SqlFunction JSON_EXISTS = SqlStdOperatorTable.JSON_EXISTS;
-    public static final SqlFunction JSON_VALUE = SqlStdOperatorTable.JSON_VALUE;
+    public static final SqlFunction JSON_VALUE = new SqlJsonValueFunctionWrapper("JSON_VALUE");
+    public static final SqlFunction JSON_QUERY = new SqlJsonQueryFunctionWrapper();
+    public static final SqlFunction JSON_OBJECT = new SqlJsonObjectFunctionWrapper();
+    public static final SqlAggFunction JSON_OBJECTAGG_NULL_ON_NULL =
+            SqlStdOperatorTable.JSON_OBJECTAGG;
+    public static final SqlAggFunction JSON_OBJECTAGG_ABSENT_ON_NULL =
+            SqlStdOperatorTable.JSON_OBJECTAGG.with(SqlJsonConstructorNullClause.ABSENT_ON_NULL);
+    public static final SqlFunction JSON_ARRAY = new SqlJsonArrayFunctionWrapper();
+    public static final SqlAggFunction JSON_ARRAYAGG_NULL_ON_NULL =
+            SqlStdOperatorTable.JSON_ARRAYAGG.with(SqlJsonConstructorNullClause.NULL_ON_NULL);
+    public static final SqlAggFunction JSON_ARRAYAGG_ABSENT_ON_NULL =
+            SqlStdOperatorTable.JSON_ARRAYAGG;
     public static final SqlPostfixOperator IS_JSON_VALUE = SqlStdOperatorTable.IS_JSON_VALUE;
     public static final SqlPostfixOperator IS_JSON_OBJECT = SqlStdOperatorTable.IS_JSON_OBJECT;
     public static final SqlPostfixOperator IS_JSON_ARRAY = SqlStdOperatorTable.IS_JSON_ARRAY;
