@@ -21,7 +21,6 @@ package org.apache.flink.table.planner.codegen
 import java.lang.reflect.Method
 import java.lang.{Boolean => JBoolean, Byte => JByte, Double => JDouble, Float => JFloat, Integer => JInt, Long => JLong, Object => JObject, Short => JShort}
 import java.util.concurrent.atomic.AtomicLong
-
 import org.apache.flink.api.common.ExecutionConfig
 import org.apache.flink.api.common.functions.RuntimeContext
 import org.apache.flink.core.memory.MemorySegment
@@ -33,32 +32,30 @@ import org.apache.flink.table.data.util.DataFormatConverters.IdentityConverter
 import org.apache.flink.table.data.utils.JoinedRowData
 import org.apache.flink.table.functions.UserDefinedFunction
 import org.apache.flink.table.planner.codegen.GenerateUtils.{generateInputFieldUnboxing, generateNonNullField}
+import org.apache.flink.table.planner.codegen.calls.BuiltInMethods.BINARY_STRING_DATA_FROM_STRING
 import org.apache.flink.table.runtime.dataview.StateDataViewStore
-import org.apache.flink.table.runtime.functions.NumericUtils
 import org.apache.flink.table.runtime.generated.{AggsHandleFunction, HashFunction, NamespaceAggsHandleFunction, TableAggsHandleFunction}
 import org.apache.flink.table.runtime.types.LogicalTypeDataTypeConverter.fromDataTypeToLogicalType
-import org.apache.flink.table.runtime.types.PlannerTypeUtils.isInteroperable
 import org.apache.flink.table.runtime.typeutils.TypeCheckUtils
 import org.apache.flink.table.runtime.util.{MurmurHashUtil, TimeWindowUtil}
 import org.apache.flink.table.types.DataType
 import org.apache.flink.table.types.logical.LogicalTypeRoot._
 import org.apache.flink.table.types.logical._
 import org.apache.flink.table.types.logical.utils.LogicalTypeChecks
-import org.apache.flink.table.types.logical.utils.LogicalTypeChecks.{getFieldCount, getPrecision, getScale, hasRoot}
+import org.apache.flink.table.types.logical.utils.LogicalTypeChecks.{getFieldCount, getPrecision, getScale}
 import org.apache.flink.table.types.logical.utils.LogicalTypeUtils.toInternalConversionClass
 import org.apache.flink.table.types.utils.DataTypeUtils.isInternal
+import org.apache.flink.table.utils.EncodingUtils
 import org.apache.flink.types.{Row, RowKind}
 
-import java.lang.reflect.Method
-import java.lang.{Boolean => JBoolean, Byte => JByte, Double => JDouble, Float => JFloat, Integer => JInt, Long => JLong, Object => JObject, Short => JShort}
-import java.util.concurrent.atomic.AtomicLong
-
 import scala.annotation.tailrec
 
 object CodeGenUtils {
 
   // ------------------------------- DEFAULT TERMS ------------------------------------------
 
+  val DEFAULT_LEGACY_CAST_BEHAVIOUR = "legacyCastBehaviour"
+
   val DEFAULT_TIMEZONE_TERM = "timeZone"
 
   val DEFAULT_INPUT1_TERM = "in1"
@@ -101,8 +98,6 @@ object CodeGenUtils {
 
   val ROW_KIND: String = className[RowKind]
 
-  val NUMERIC_UTIL: String = className[NumericUtils]
-
   val DECIMAL_UTIL: String = className[DecimalDataUtils]
 
   val SEGMENT: String = className[MemorySegment]
@@ -150,6 +145,16 @@ object CodeGenUtils {
     name
   }
 
+  def className(c: Class[_]): String = {
+    val name = c.getCanonicalName
+    if (name == null) {
+      throw new CodeGenException(
+        s"Class '${c.getName}' does not have a canonical name. " +
+          s"Make sure it is statically accessible.")
+    }
+    name
+  }
+
   /**
    * Returns a term for representing the given class in Java code.
    */
@@ -192,6 +197,42 @@ object CodeGenUtils {
     case _ => boxedTypeTermForType(t)
   }
 
+  /**
+   * Converts values to stringified representation to include in the codegen.
+   *
+   * This method doesn't support complex types.
+   */
+  def primitiveLiteralForType(value: Any): String = value match {
+    // ordered by type root definition
+    case _: JBoolean => value.toString
+    case _: JByte => s"((byte)$value)"
+    case _: JShort => s"((short)$value)"
+    case _: JInt => value.toString
+    case _: JLong => value.toString + "L"
+    case _: JFloat => value match {
+      case JFloat.NEGATIVE_INFINITY => "java.lang.Float.NEGATIVE_INFINITY"
+      case JFloat.POSITIVE_INFINITY => "java.lang.Float.POSITIVE_INFINITY"
+      case _ => value.toString + "f"
+    }
+    case _: JDouble => value match {
+      case JDouble.NEGATIVE_INFINITY => "java.lang.Double.NEGATIVE_INFINITY"
+      case JDouble.POSITIVE_INFINITY => "java.lang.Double.POSITIVE_INFINITY"
+      case _ => value.toString + "d"
+    }
+    case sd: StringData =>
+      qualifyMethod(BINARY_STRING_DATA_FROM_STRING) + "(\"" +
+        EncodingUtils.escapeJava(sd.toString) + "\")"
+    case td: TimestampData =>
+      s"$TIMESTAMP_DATA.fromEpochMillis(${td.getMillisecond}L, ${td.getNanoOfMillisecond})"
+    case decimalData: DecimalData =>
+      s"""$DECIMAL_UTIL.castFrom(
+         |"${decimalData.toString}",
+         |${decimalData.precision()},
+         |${decimalData.scale()})"""
+        .stripMargin
+    case _ => throw new IllegalArgumentException("Illegal literal type: " + value.getClass)
+  }
+
   @tailrec
   def boxedTypeTermForType(t: LogicalType): String = t.getTypeRoot match {
     // ordered by type root definition
@@ -211,7 +252,6 @@ object CodeGenUtils {
     case ARRAY => className[ArrayData]
     case MULTISET | MAP => className[MapData]
     case ROW | STRUCTURED_TYPE => className[RowData]
-    case TIMESTAMP_WITHOUT_TIME_ZONE | TIMESTAMP_WITH_LOCAL_TIME_ZONE => className[TimestampData]
     case DISTINCT_TYPE => boxedTypeTermForType(t.asInstanceOf[DistinctType].getSourceType)
     case NULL => className[JObject] // special case for untyped null literals
     case RAW => className[BinaryRawValueData[_]]
@@ -219,6 +259,22 @@ object CodeGenUtils {
       throw new IllegalArgumentException("Illegal type: " + t)
   }
 
+  /**
+   * Returns true if [[primitiveDefaultValue()]] returns a nullable Java type, that is,
+   * a non primitive type.
+   */
+  @tailrec
+  def isPrimitiveNullable(t: LogicalType): Boolean = t.getTypeRoot match {
+    // ordered by type root definition
+    case BOOLEAN | TINYINT | SMALLINT | INTEGER |
+         DATE | TIME_WITHOUT_TIME_ZONE | INTERVAL_YEAR_MONTH |
+         BIGINT | INTERVAL_DAY_TIME | FLOAT | DOUBLE => false
+
+    case DISTINCT_TYPE => isPrimitiveNullable(t.asInstanceOf[DistinctType].getSourceType)
+
+    case _ => true
+  }
+
   /**
     * Gets the default value for a primitive type, and null for generic types
     */
@@ -294,69 +350,6 @@ object CodeGenUtils {
       throw new IllegalArgumentException("Illegal type: " + t)
   }
 
-  // ----------------------------------------------------------------------------------------------
-
-  // Cast numeric type to another numeric type with larger range.
-  // This function must be in sync with [[NumericOrDefaultReturnTypeInference]].
-  def getNumericCastedResultTerm(expr: GeneratedExpression, targetType: LogicalType): String = {
-    (expr.resultType.getTypeRoot, targetType.getTypeRoot) match {
-      case _ if isInteroperable(expr.resultType, targetType) => expr.resultTerm
-
-      // byte -> other numeric types
-      case (TINYINT, SMALLINT) => s"(short) ${expr.resultTerm}"
-      case (TINYINT, INTEGER) => s"(int) ${expr.resultTerm}"
-      case (TINYINT, BIGINT) => s"(long) ${expr.resultTerm}"
-      case (TINYINT, DECIMAL) =>
-        val dt = targetType.asInstanceOf[DecimalType]
-        s"$DECIMAL_UTIL.castFrom(" +
-          s"${expr.resultTerm}, ${dt.getPrecision}, ${dt.getScale})"
-      case (TINYINT, FLOAT) => s"(float) ${expr.resultTerm}"
-      case (TINYINT, DOUBLE) => s"(double) ${expr.resultTerm}"
-
-      // short -> other numeric types
-      case (SMALLINT, INTEGER) => s"(int) ${expr.resultTerm}"
-      case (SMALLINT, BIGINT) => s"(long) ${expr.resultTerm}"
-      case (SMALLINT, DECIMAL) =>
-        val dt = targetType.asInstanceOf[DecimalType]
-        s"$DECIMAL_UTIL.castFrom(" +
-          s"${expr.resultTerm}, ${dt.getPrecision}, ${dt.getScale})"
-      case (SMALLINT, FLOAT) => s"(float) ${expr.resultTerm}"
-      case (SMALLINT, DOUBLE) => s"(double) ${expr.resultTerm}"
-
-      // int -> other numeric types
-      case (INTEGER, BIGINT) => s"(long) ${expr.resultTerm}"
-      case (INTEGER, DECIMAL) =>
-        val dt = targetType.asInstanceOf[DecimalType]
-        s"$DECIMAL_UTIL.castFrom(" +
-          s"${expr.resultTerm}, ${dt.getPrecision}, ${dt.getScale})"
-      case (INTEGER, FLOAT) => s"(float) ${expr.resultTerm}"
-      case (INTEGER, DOUBLE) => s"(double) ${expr.resultTerm}"
-
-      // long -> other numeric types
-      case (BIGINT, DECIMAL) =>
-        val dt = targetType.asInstanceOf[DecimalType]
-        s"$DECIMAL_UTIL.castFrom(" +
-          s"${expr.resultTerm}, ${dt.getPrecision}, ${dt.getScale})"
-      case (BIGINT, FLOAT) => s"(float) ${expr.resultTerm}"
-      case (BIGINT, DOUBLE) => s"(double) ${expr.resultTerm}"
-
-      // decimal -> other numeric types
-      case (DECIMAL, DECIMAL) =>
-        val dt = targetType.asInstanceOf[DecimalType]
-        s"$DECIMAL_UTIL.castToDecimal(" +
-          s"${expr.resultTerm}, ${dt.getPrecision}, ${dt.getScale})"
-      case (DECIMAL, FLOAT) =>
-        s"$DECIMAL_UTIL.castToFloat(${expr.resultTerm})"
-      case (DECIMAL, DOUBLE) =>
-        s"$DECIMAL_UTIL.castToDouble(${expr.resultTerm})"
-
-      // float -> other numeric types
-      case (FLOAT, DOUBLE) => s"(double) ${expr.resultTerm}"
-
-      case _ => null
-    }
-  }
-
   // -------------------------- Method & Enum ---------------------------------------
 
   def qualifyMethod(method: Method): String =
@@ -368,16 +361,12 @@ object CodeGenUtils {
   def compareEnum(term: String, enum: Enum[_]): Boolean = term == qualifyEnum(enum)
 
   def getEnum(genExpr: GeneratedExpression): Enum[_] = {
-    val split = genExpr.resultTerm.split('.')
-    val value = split.last
-    val clazz = genExpr.resultType.asInstanceOf[TypeInformationRawType[_]]
-        .getTypeInformation.getTypeClass
-    enumValueOf(clazz, value)
+   genExpr
+     .literalValue
+     .map(_.asInstanceOf[Enum[_]])
+     .getOrElse(throw new CodeGenException("Enum literal expected."))
   }
 
-  def enumValueOf[T <: Enum[T]](cls: Class[_], stringValue: String): Enum[_] =
-    Enum.valueOf(cls.asInstanceOf[Class[T]], stringValue).asInstanceOf[Enum[_]]
-
   // --------------------------- Require Check ---------------------------------------
 
   def requireNumeric(genExpr: GeneratedExpression): Unit =
@@ -426,7 +415,9 @@ object CodeGenUtils {
       throw new CodeGenException("Integer expression type expected.")
     }
 
-  def udfFieldName(udf: UserDefinedFunction): String = s"function_${udf.functionIdentifier}"
+  def udfFieldName(udf: UserDefinedFunction): String = {
+    s"function_${udf.functionIdentifier.replace('.', '$')}"
+  }
 
   def genLogInfo(logTerm: String, format: String, argTerm: String): String =
     s"""$logTerm.info("$format", $argTerm);"""
@@ -437,19 +428,11 @@ object CodeGenUtils {
 
   // -------------------------- RowData Read Access -------------------------------
 
-  def rowFieldReadAccess(
-      ctx: CodeGeneratorContext,
-      index: Int,
-      rowTerm: String,
-      fieldType: LogicalType) : String =
-    rowFieldReadAccess(ctx, index.toString, rowTerm, fieldType)
+  def rowFieldReadAccess(index: Int, rowTerm: String, fieldType: LogicalType): String =
+    rowFieldReadAccess(index.toString, rowTerm, fieldType)
 
   @tailrec
-  def rowFieldReadAccess(
-      ctx: CodeGeneratorContext,
-      indexTerm: String,
-      rowTerm: String,
-      t: LogicalType)
+  def rowFieldReadAccess(indexTerm: String, rowTerm: String, t: LogicalType)
     : String = t.getTypeRoot match {
       // ordered by type root definition
       case CHAR | VARCHAR =>
@@ -483,7 +466,7 @@ object CodeGenUtils {
       case ROW | STRUCTURED_TYPE =>
         s"$rowTerm.getRow($indexTerm, ${getFieldCount(t)})"
       case DISTINCT_TYPE =>
-        rowFieldReadAccess(ctx, indexTerm, rowTerm, t.asInstanceOf[DistinctType].getSourceType)
+        rowFieldReadAccess(indexTerm, rowTerm, t.asInstanceOf[DistinctType].getSourceType)
       case RAW =>
         s"(($BINARY_RAW_VALUE) $rowTerm.getRawValue($indexTerm))"
       case NULL | SYMBOL | UNRESOLVED =>
@@ -732,16 +715,26 @@ object CodeGenUtils {
       fieldValTerm: String,
       writerTerm: String,
       fieldType: LogicalType): String =
-    binaryWriterWriteField(ctx, index.toString, fieldValTerm, writerTerm, fieldType)
+    binaryWriterWriteField(
+      t => ctx.addReusableTypeSerializer(t), index.toString, fieldValTerm, writerTerm, fieldType)
 
-  @tailrec
   def binaryWriterWriteField(
       ctx: CodeGeneratorContext,
       indexTerm: String,
       fieldValTerm: String,
       writerTerm: String,
+      t: LogicalType): String =
+    binaryWriterWriteField(
+      t => ctx.addReusableTypeSerializer(t), indexTerm, fieldValTerm, writerTerm, t)
+
+  @tailrec
+  def binaryWriterWriteField(
+      addSerializer: LogicalType => String,
+      indexTerm: String,
+      fieldValTerm: String,
+      writerTerm: String,
       t: LogicalType)
-    : String = t.getTypeRoot match {
+  : String = t.getTypeRoot match {
     // ordered by type root definition
     case CHAR | VARCHAR =>
       s"$writerTerm.writeString($indexTerm, $fieldValTerm)"
@@ -768,23 +761,23 @@ object CodeGenUtils {
     case TIMESTAMP_WITH_TIME_ZONE =>
       throw new UnsupportedOperationException("Unsupported type: " + t)
     case ARRAY =>
-      val ser = ctx.addReusableTypeSerializer(t)
+      val ser = addSerializer(t)
       s"$writerTerm.writeArray($indexTerm, $fieldValTerm, $ser)"
     case MULTISET | MAP =>
-      val ser = ctx.addReusableTypeSerializer(t)
+      val ser = addSerializer(t)
       s"$writerTerm.writeMap($indexTerm, $fieldValTerm, $ser)"
     case ROW | STRUCTURED_TYPE =>
-      val ser = ctx.addReusableTypeSerializer(t)
+      val ser = addSerializer(t)
       s"$writerTerm.writeRow($indexTerm, $fieldValTerm, $ser)"
     case DISTINCT_TYPE =>
       binaryWriterWriteField(
-        ctx,
+        addSerializer,
         indexTerm,
         fieldValTerm,
         writerTerm,
         t.asInstanceOf[DistinctType].getSourceType)
     case RAW =>
-      val ser = ctx.addReusableTypeSerializer(t)
+      val ser = addSerializer(t)
       s"$writerTerm.writeRawValue($indexTerm, $fieldValTerm, $ser)"
     case NULL | SYMBOL | UNRESOLVED =>
       throw new IllegalArgumentException("Illegal type: " + t);
@@ -919,7 +912,7 @@ object CodeGenUtils {
     val targetTypeTerm = boxedTypeTermForType(targetType)
 
     // untyped null literal
-    if (hasRoot(internalExpr.resultType, NULL)) {
+    if (internalExpr.resultType.is(NULL)) {
       return s"($targetTypeTerm) null"
     }
 
@@ -1038,7 +1031,7 @@ object CodeGenUtils {
     val targetTypeTerm = boxedTypeTermForType(targetType)
 
     // untyped null literal
-    if (hasRoot(internalExpr.resultType, NULL)) {
+    if (internalExpr.resultType.is(NULL)) {
       return s"($targetTypeTerm) null"
     }
 
