@@ -27,7 +27,7 @@ import org.apache.flink.table.functions.{FunctionContext, UserDefinedFunction}
 import org.apache.flink.table.planner.codegen.CodeGenUtils._
 import org.apache.flink.table.planner.codegen.GenerateUtils.generateRecordStatement
 import org.apache.flink.table.planner.utils.InternalConfigOptions
-import org.apache.flink.table.runtime.functions.SqlDateTimeUtils
+import org.apache.flink.table.utils.DateTimeUtils
 import org.apache.flink.table.runtime.operators.TableStreamOperator
 import org.apache.flink.table.runtime.typeutils.{ExternalSerializer, InternalSerializers}
 import org.apache.flink.table.runtime.util.collections._
@@ -51,6 +51,10 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
   // holding a list of objects that could be used passed into generated class
   val references: mutable.ArrayBuffer[AnyRef] = new mutable.ArrayBuffer[AnyRef]()
 
+  // set of strings (lines) that will be concatenated into a single class header comment
+  private val reusableHeaderComments: mutable.LinkedHashSet[String] =
+    mutable.LinkedHashSet[String]()
+
   // set of member statements that will be added only once
   // we use a LinkedHashSet to keep the insertion order
   private val reusableMemberStatements: mutable.LinkedHashSet[String] =
@@ -143,6 +147,16 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
 
   def nullCheck: Boolean = tableConfig.getNullCheck
 
+
+  /**
+    * Add a line comment to [[reusableHeaderComments]] list which will be concatenated
+    * into a single class header comment.
+    * @param comment The comment to add for class header
+    */
+  def addReusableHeaderComment(comment: String): Unit = {
+    reusableHeaderComments.add(comment)
+  }
+
   // ---------------------------------------------------------------------------------
   // Local Variables for Code Split
   // ---------------------------------------------------------------------------------
@@ -196,6 +210,17 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
   // generate reuse code methods
   // ---------------------------------------------------------------------------------
 
+  /**
+    * @return Comment to be added as a header comment on the generated class
+    */
+  def getClassHeaderComment: String = {
+    s"""
+    |/*
+    | * ${reusableHeaderComments.mkString("\n * ")}
+    | */
+    """.stripMargin
+  }
+
   /**
     * @return code block of statements that need to be placed in the member area of the class
     *         (e.g. inner class definition)
@@ -394,11 +419,18 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
       case _ => className[ObjectHashSet[_]]
     }
 
-    addReusableMember(
-      s"final $setTypeTerm $fieldTerm = new $setTypeTerm(${elements.size});")
-
-    elements.foreach { element =>
-      val content =
+    val addElementsCode = elements.map { element =>
+      if (element.literalValue.isDefined) {
+        // Don't generate the null check in case the element is a literal expression
+        if (element.literalValue.get != null) {
+          s"""
+             |${element.code}
+             |$fieldTerm.add(${element.resultTerm});
+             |""".stripMargin
+        } else if (element.literalValue.get == null) {
+          s"$fieldTerm.addNull();"
+        }
+      } else {
         s"""
            |${element.code}
            |if (${element.nullTerm}) {
@@ -407,9 +439,23 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
            |  $fieldTerm.add(${element.resultTerm});
            |}
            |""".stripMargin
-      reusableInitStatements.add(content)
-    }
-    reusableInitStatements.add(s"$fieldTerm.optimize();")
+      }
+    }.mkString("\n")
+    val setBuildingFunctionName = newName("buildSet")
+    val setBuildingFunctionCode =
+      s"""
+         |private void $setBuildingFunctionName() {
+         |  $addElementsCode
+         |  $fieldTerm.optimize();
+         |}
+         |""".stripMargin
+
+    addReusableMember(
+      s"""
+         |final $setTypeTerm $fieldTerm = new $setTypeTerm(${elements.size});
+         |$setBuildingFunctionCode
+         |""".stripMargin)
+    reusableInitStatements.add(s"$setBuildingFunctionName();")
 
     fieldTerm
   }
@@ -526,12 +572,12 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
 
     // declaration
     reusableMemberStatements.add(s"private int $fieldTerm;")
-    val utilsName = classOf[SqlDateTimeUtils].getCanonicalName
+    val utilsName = classOf[DateTimeUtils].getCanonicalName
 
     // assignment
     val field =
     s"""
-       |$fieldTerm = $utilsName.getTimeInMills($localtimestamp.getMillisecond());
+       |$fieldTerm = $utilsName.timestampMillisToTime($localtimestamp.getMillisecond());
        |""".stripMargin
     reusablePerRecordStatements.add(field)
     fieldTerm
@@ -545,12 +591,12 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
     val fieldTerm = s"queryStartLocaltime"
 
     val queryStartLocalTimestamp = addReusableQueryLevelLocalDateTime()
-    val utilsName = classOf[SqlDateTimeUtils].getCanonicalName
+    val utilsName = classOf[DateTimeUtils].getCanonicalName
     // declaration
     reusableMemberStatements.add(
       s"""
           |private static final int $fieldTerm =
-          | $utilsName.getTimeInMills($queryStartLocalTimestamp.getMillisecond());
+          | $utilsName.timestampMillisToTime($queryStartLocalTimestamp.getMillisecond());
           | """.stripMargin)
     fieldTerm
   }
@@ -562,13 +608,13 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
     val fieldTerm = s"date"
 
     val timestamp = addReusableRecordLevelLocalDateTime()
-    val utilsName = classOf[SqlDateTimeUtils].getCanonicalName
+    val utilsName = classOf[DateTimeUtils].getCanonicalName
 
     // declaration
     reusableMemberStatements.add(s"private int $fieldTerm;")
 
     // assignment
-    val field = s"$fieldTerm = $utilsName.getDateInDays($timestamp.getMillisecond());"
+    val field = s"$fieldTerm = $utilsName.timestampMillisToDate($timestamp.getMillisecond());"
 
     reusablePerRecordStatements.add(field)
     fieldTerm
@@ -579,13 +625,13 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
    */
   def addReusableQueryLevelCurrentDate(): String = {
     val fieldTerm = s"queryStartDate"
-    val utilsName = classOf[SqlDateTimeUtils].getCanonicalName
+    val utilsName = classOf[DateTimeUtils].getCanonicalName
 
     val timestamp = addReusableQueryLevelLocalDateTime()
     reusableMemberStatements.add(
     s"""
        |private static final int $fieldTerm =
-       | $fieldTerm = $utilsName.getDateInDays($timestamp.getMillisecond());
+       | $fieldTerm = $utilsName.timestampMillisToDate($timestamp.getMillisecond());
        |""".stripMargin)
 
     fieldTerm
@@ -823,9 +869,7 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
     * @param constant constant expression
     * @return generated expression with the fieldTerm and nullTerm
     */
-  def addReusableConstant(
-      constant: GeneratedExpression,
-      nullCheck: Boolean): GeneratedExpression = {
+  def addReusableConstant(constant: GeneratedExpression): GeneratedExpression = {
     require(constant.literal, "Literal expected")
 
     val fieldTerm = newName("constant")
@@ -854,8 +898,11 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
 
   /**
     * Adds a reusable string constant to the member area of the generated class.
+    *
+    * The string must be already escaped with
+    * [[org.apache.flink.table.utils.EncodingUtils.escapeJava()]].
     */
-  def addReusableStringConstants(value: String): String = {
+  def addReusableEscapedStringConstant(value: String): String = {
     reusableStringConstants.get(value) match {
       case Some(field) => field
       case None =>
@@ -943,7 +990,7 @@ class CodeGeneratorContext(val tableConfig: TableConfig) {
 }
 
 object CodeGeneratorContext {
-  def apply(config: TableConfig): CodeGeneratorContext = {
-    new CodeGeneratorContext(config)
+  def apply(tableConfig: TableConfig): CodeGeneratorContext = {
+    new CodeGeneratorContext(tableConfig)
   }
 }
