@@ -20,21 +20,23 @@ package org.apache.flink.table.planner.codegen
 
 import org.apache.flink.streaming.api.functions.ProcessFunction
 import org.apache.flink.table.api.TableException
+import org.apache.flink.table.api.config.ExecutionConfigOptions
 import org.apache.flink.table.data.RowData
 import org.apache.flink.table.data.binary.BinaryRowData
-import org.apache.flink.table.data.util.DataFormatConverters.{DataFormatConverter, getConverterForDataType}
+import org.apache.flink.table.data.util.DataFormatConverters.{getConverterForDataType, DataFormatConverter}
 import org.apache.flink.table.functions.BuiltInFunctionDefinitions
-import org.apache.flink.table.planner.calcite.{FlinkRexBuilder, FlinkTypeFactory, RexDistinctKeyVariable, RexFieldVariable}
-import org.apache.flink.table.planner.codegen.CodeGenUtils.{requireTemporal, requireTimeInterval, _}
+import org.apache.flink.table.planner.calcite.{FlinkTypeFactory, RexDistinctKeyVariable, RexFieldVariable}
+import org.apache.flink.table.planner.codegen.CodeGenUtils._
 import org.apache.flink.table.planner.codegen.GenerateUtils._
 import org.apache.flink.table.planner.codegen.GeneratedExpression.{NEVER_NULL, NO_CODE}
 import org.apache.flink.table.planner.codegen.calls.ScalarOperatorGens._
+import org.apache.flink.table.planner.codegen.calls.SearchOperatorGen.generateSearch
 import org.apache.flink.table.planner.codegen.calls._
 import org.apache.flink.table.planner.functions.bridging.BridgingSqlFunction
 import org.apache.flink.table.planner.functions.sql.FlinkSqlOperatorTable._
 import org.apache.flink.table.planner.functions.sql.SqlThrowExceptionFunction
 import org.apache.flink.table.planner.functions.utils.{ScalarSqlFunction, TableSqlFunction}
-import org.apache.flink.table.planner.plan.utils.FlinkRexUtil
+import org.apache.flink.table.planner.plan.utils.RexLiteralUtil
 import org.apache.flink.table.runtime.types.LogicalTypeDataTypeConverter.fromLogicalTypeToDataType
 import org.apache.flink.table.runtime.types.PlannerTypeUtils.isInteroperable
 import org.apache.flink.table.runtime.typeutils.TypeCheckUtils
@@ -44,17 +46,15 @@ import org.apache.flink.table.types.logical.utils.LogicalTypeChecks.{getFieldCou
 import org.apache.flink.table.typeutils.TimeIndicatorTypeInfo
 
 import org.apache.calcite.rex._
-import org.apache.calcite.sql.{SqlKind, SqlOperator}
 import org.apache.calcite.sql.`type`.{ReturnTypes, SqlTypeName}
-import org.apache.calcite.util.{Sarg, TimestampString}
-import org.apache.flink.table.functions.{BuiltInFunctionDefinitions, FunctionDefinition}
+import org.apache.calcite.sql.{SqlKind, SqlOperator}
 
 import scala.collection.JavaConversions._
 
 /**
-  * This code generator is mainly responsible for generating codes for a given calcite [[RexNode]].
-  * It can also generate type conversion codes for the result converter.
-  */
+ * This code generator is mainly responsible for generating codes for a given calcite [[RexNode]].
+ * It can also generate type conversion codes for the result converter.
+ */
 class ExprCodeGenerator(ctx: CodeGeneratorContext, nullableInput: Boolean)
   extends RexVisitor[GeneratedExpression] {
 
@@ -412,15 +412,8 @@ class ExprCodeGenerator(ctx: CodeGeneratorContext, nullableInput: Boolean)
   }
 
   override def visitLiteral(literal: RexLiteral): GeneratedExpression = {
-    val resultType = FlinkTypeFactory.toLogicalType(literal.getType)
-    val value = resultType.getTypeRoot match {
-      case LogicalTypeRoot.TIMESTAMP_WITHOUT_TIME_ZONE |
-           LogicalTypeRoot.TIMESTAMP_WITH_LOCAL_TIME_ZONE =>
-        literal.getValueAs(classOf[TimestampString])
-      case _ =>
-        literal.getValue3
-    }
-    generateLiteral(ctx, resultType, value)
+    val res = RexLiteralUtil.toFlinkInternalValue(literal)
+    generateLiteral(ctx, res.f0, res.f1)
   }
 
   override def visitCorrelVariable(correlVariable: RexCorrelVariable): GeneratedExpression = {
@@ -472,19 +465,10 @@ class ExprCodeGenerator(ctx: CodeGeneratorContext, nullableInput: Boolean)
   override def visitCall(call: RexCall): GeneratedExpression = {
     val resultType = FlinkTypeFactory.toLogicalType(call.getType)
     if (call.getKind == SqlKind.SEARCH) {
-      val sarg = call.getOperands.get(1).asInstanceOf[RexLiteral]
-          .getValueAs(classOf[Sarg[_]])
-      val rexBuilder = new FlinkRexBuilder(FlinkTypeFactory.INSTANCE)
-      if (sarg.isPoints) {
-        val operands = FlinkRexUtil.expandSearchOperands(rexBuilder, call)
-            .map(operand => operand.accept(this))
-        return generateCallExpression(ctx, call, operands, resultType)
-      } else {
-        return RexUtil.expandSearch(
-          rexBuilder,
-          null,
-          call).accept(this)
-      }
+      return generateSearch(
+        ctx,
+        generateExpression(call.getOperands.get(0)),
+        call.getOperands.get(1).asInstanceOf[RexLiteral])
     }
 
     // convert operands and help giving untyped NULL literals a type
@@ -692,20 +676,13 @@ class ExprCodeGenerator(ctx: CodeGeneratorContext, nullableInput: Boolean)
         requireBoolean(operand)
         generateIsNotFalse(operand)
 
-      case SEARCH | IN =>
-        val left = operands.head
-        val right = operands.tail
-        generateIn(ctx, left, right)
-
-      case NOT_IN =>
-        val left = operands.head
-        val right = operands.tail
-        generateNot(ctx, generateIn(ctx, left, right))
-
       // casting
       case CAST =>
-        val operand = operands.head
-        generateCast(ctx, operand, resultType)
+        generateCast(ctx, operands.head, resultType, nullOnFailure = ctx.tableConfig
+          .getConfiguration.get(ExecutionConfigOptions.TABLE_EXEC_LEGACY_CAST_BEHAVIOUR).isEnabled)
+
+      case TRY_CAST =>
+        generateCast(ctx, operands.head, resultType, nullOnFailure = true)
 
       // Reinterpret
       case REINTERPRET =>
@@ -782,6 +759,10 @@ class ExprCodeGenerator(ctx: CodeGeneratorContext, nullableInput: Boolean)
 
       case JSON_VALUE => new JsonValueCallGen().generate(ctx, operands, resultType)
 
+      case JSON_OBJECT => new JsonObjectCallGen(call).generate(ctx, operands, resultType)
+
+      case JSON_ARRAY => new JsonArrayCallGen(call).generate(ctx, operands, resultType)
+
       case _: SqlThrowExceptionFunction =>
         val nullValue = generateNullLiteral(resultType, nullCheck = true)
         val code =
@@ -806,6 +787,7 @@ class ExprCodeGenerator(ctx: CodeGeneratorContext, nullableInput: Boolean)
 
       case bsf: BridgingSqlFunction =>
         bsf.getDefinition match {
+
           case BuiltInFunctionDefinitions.CURRENT_WATERMARK =>
             generateWatermark(ctx, contextTerm, resultType)
 
@@ -813,19 +795,27 @@ class ExprCodeGenerator(ctx: CodeGeneratorContext, nullableInput: Boolean)
             operands.foreach { operand =>
               requireComparable(operand)
             }
-            generateGreatestLeast(resultType, operands)
+            generateGreatestLeast(ctx, resultType, operands)
 
           case BuiltInFunctionDefinitions.LEAST =>
             operands.foreach { operand =>
               requireComparable(operand)
             }
-            generateGreatestLeast(resultType, operands, greatest = false)
+            generateGreatestLeast(ctx, resultType, operands, greatest = false)
+
+          case BuiltInFunctionDefinitions.JSON_STRING =>
+            new JsonStringCallGen(call).generate(ctx, operands, resultType)
 
           case BuiltInFunctionDefinitions.AGG_DECIMAL_PLUS =>
             val left = operands.head
             val right = operands(1)
             generateBinaryArithmeticOperator(ctx, "+", resultType, left, right)
 
+          case BuiltInFunctionDefinitions.AGG_DECIMAL_MINUS =>
+            val left = operands.head
+            val right = operands(1)
+            generateBinaryArithmeticOperator(ctx, "-", resultType, left, right)
+
           case _ =>
             new BridgingSqlFunctionCallGen(call).generate(ctx, operands, resultType)
         }
@@ -861,8 +851,8 @@ class ExprCodeGenerator(ctx: CodeGeneratorContext, nullableInput: Boolean)
         case None => null
         case Some(literal) =>
           getConverterForDataType(fromLogicalTypeToDataType(expr.resultType))
-              .asInstanceOf[DataFormatConverter[AnyRef, AnyRef]
-              ].toExternal(literal.asInstanceOf[AnyRef])
+            .asInstanceOf[DataFormatConverter[AnyRef, AnyRef]]
+            .toExternal(literal.asInstanceOf[AnyRef])
       }
     }.toArray
   }
