@@ -20,9 +20,8 @@ package org.apache.flink.table.planner.codegen
 
 import org.apache.flink.api.common.ExecutionConfig
 import org.apache.flink.api.common.typeinfo.{AtomicType => AtomicTypeInfo}
-import org.apache.flink.api.java.typeutils.GenericTypeInfo
 import org.apache.flink.table.data._
-import org.apache.flink.table.data.binary.BinaryRowData
+import org.apache.flink.table.data.binary.{BinaryRowData, BinaryStringData}
 import org.apache.flink.table.data.utils.JoinedRowData
 import org.apache.flink.table.data.writer.BinaryRowWriter
 import org.apache.flink.table.planner.codegen.CodeGenUtils._
@@ -30,18 +29,12 @@ import org.apache.flink.table.planner.codegen.GeneratedExpression.{ALWAYS_NULL,
 import org.apache.flink.table.planner.codegen.calls.CurrentTimePointCallGen
 import org.apache.flink.table.planner.plan.nodes.exec.spec.SortSpec
 import org.apache.flink.table.planner.plan.utils.SortUtil
+import org.apache.flink.table.planner.typeutils.SymbolUtil.calciteToCommon
 import org.apache.flink.table.runtime.typeutils.TypeCheckUtils.{isCharacterString, isReference, isTemporal}
-import org.apache.flink.table.types.logical.LogicalTypeRoot._
 import org.apache.flink.table.types.logical._
+import org.apache.flink.table.types.logical.LogicalTypeRoot._
 import org.apache.flink.table.types.logical.utils.LogicalTypeChecks.{getFieldCount, getFieldTypes}
-import org.apache.flink.table.util.TimestampStringUtils.toLocalDateTime
-
-import org.apache.calcite.avatica.util.ByteString
-import org.apache.calcite.util.TimestampString
-import org.apache.commons.lang3.StringEscapeUtils
-
-import java.math.{BigDecimal => JBigDecimal}
-import java.time.ZoneOffset
+import org.apache.flink.table.utils.EncodingUtils
 
 import scala.annotation.tailrec
 import scala.collection.mutable
@@ -62,9 +55,10 @@ object GenerateUtils {
       ctx: CodeGeneratorContext,
       returnType: LogicalType,
       operands: Seq[GeneratedExpression],
-      resultNullable: Boolean = false)
+      resultNullable: Boolean = false,
+      wrapTryCatch: Boolean = false)
       (call: Seq[String] => String): GeneratedExpression = {
-    generateCallWithStmtIfArgsNotNull(ctx, returnType, operands, resultNullable) {
+    generateCallWithStmtIfArgsNotNull(ctx, returnType, operands, resultNullable, wrapTryCatch) {
       args => ("", call(args))
     }
   }
@@ -76,7 +70,8 @@ object GenerateUtils {
       ctx: CodeGeneratorContext,
       returnType: LogicalType,
       operands: Seq[GeneratedExpression],
-      resultNullable: Boolean = false)
+      resultNullable: Boolean = false,
+      wrapTryCatch: Boolean = false)
       (call: Seq[String] => (String, String)): GeneratedExpression = {
     val resultTypeTerm = if (resultNullable) {
       boxedTypeTermForType(returnType)
@@ -95,14 +90,30 @@ object GenerateUtils {
 
     val (stmt, result) = call(operands.map(_.resultTerm))
 
+    val wrappedResultAssignment = if (wrapTryCatch) {
+      s"""
+         |try {
+         |  $stmt
+         |  $resultTerm = $result;
+         |} catch (Throwable ${newName("ignored")}) {
+         |  $nullTerm = true;
+         |  $resultTerm = $defaultValue;
+         |}
+         |""".stripMargin
+    } else {
+      s"""
+         |$stmt
+         |$resultTerm = $result;
+         |""".stripMargin
+    }
+
     val resultCode = if (ctx.nullCheck && operands.nonEmpty) {
       s"""
          |${operands.map(_.code).mkString("\n")}
          |$nullTerm = ${operands.map(_.nullTerm).mkString(" || ")};
          |$resultTerm = $defaultValue;
          |if (!$nullTerm) {
-         |  $stmt
-         |  $resultTerm = $result;
+         |  $wrappedResultAssignment
          |  $nullTermCode
          |}
          |""".stripMargin
@@ -110,16 +121,14 @@ object GenerateUtils {
       s"""
          |${operands.map(_.code).mkString("\n")}
          |$nullTerm = false;
-         |$stmt
-         |$resultTerm = $result;
+         |$wrappedResultAssignment
          |$nullTermCode
          |""".stripMargin
     } else {
       s"""
          |$nullTerm = false;
          |${operands.map(_.code).mkString("\n")}
-         |$stmt
-         |$resultTerm = $result;
+         |$wrappedResultAssignment
          |""".stripMargin
     }
 
@@ -141,22 +150,6 @@ object GenerateUtils {
   }
 
 
-  /**
-    * Generates a string result call with auxiliary statements and result expression.
-    * This will convert the String result to BinaryStringData.
-    */
-  def generateStringResultCallWithStmtIfArgsNotNull(
-      ctx: CodeGeneratorContext,
-      operands: Seq[GeneratedExpression],
-      returnType: LogicalType)
-      (call: Seq[String] => (String, String)): GeneratedExpression = {
-    generateCallWithStmtIfArgsNotNull(ctx, returnType, operands) {
-      args =>
-        val (stmt, result) = call(args)
-        (stmt, s"$BINARY_STRING.fromString($result)")
-    }
-  }
-
   /**
     * Generates a call with the nullable args.
     */
@@ -164,7 +157,8 @@ object GenerateUtils {
       ctx: CodeGeneratorContext,
       returnType: LogicalType,
       operands: Seq[GeneratedExpression],
-      resultNullable: Boolean = false)
+      resultNullable: Boolean = false,
+      wrapTryCatch: Boolean = false)
       (call: Seq[String] => String): GeneratedExpression = {
     val resultTypeTerm = if (resultNullable) {
       boxedTypeTermForType(returnType)
@@ -189,10 +183,26 @@ object GenerateUtils {
         x.resultTerm
       })
 
+
+    val wrappedResultAssignment = if (wrapTryCatch) {
+      s"""
+         |try {
+         |  $resultTerm = ${call(parameters)};
+         |} catch (Throwable ${newName("ignored")}) {
+         |  $nullTerm = true;
+         |  $resultTerm = $defaultValue;
+         |}
+         |""".stripMargin
+    } else {
+      s"""
+         |$resultTerm = ${call(parameters)};
+         |""".stripMargin
+    }
+
     val resultCode = if (ctx.nullCheck) {
       s"""
          |${operands.map(_.code).mkString("\n")}
-         |$resultTerm = ${call(parameters)};
+         |$wrappedResultAssignment
          |$nullTermCode
          |if ($nullTerm) {
          |  $resultTerm = $defaultValue;
@@ -201,7 +211,7 @@ object GenerateUtils {
     } else {
       s"""
          |${operands.map(_.code).mkString("\n")}
-         |$resultTerm = ${call(parameters)};
+         |$wrappedResultAssignment
          |$nullTermCode
        """.stripMargin
     }
@@ -296,182 +306,66 @@ object GenerateUtils {
       literalValue = Some(literalValue))
   }
 
+  /**
+   * This function accepts the Flink's internal data structures.
+   *
+   * Check [[org.apache.flink.table.planner.plan.utils.RexLiteralUtil.toFlinkInternalValue]] to
+   * convert RexLiteral value to Flink's internal data structures.
+   */
   @tailrec
   def generateLiteral(
       ctx: CodeGeneratorContext,
-      literalType: LogicalType,
-      literalValue: Any): GeneratedExpression = {
+      literalValue: Any,
+      literalType: LogicalType): GeneratedExpression = {
     if (literalValue == null) {
       return generateNullLiteral(literalType, ctx.nullCheck)
     }
-    // non-null values
     literalType.getTypeRoot match {
-      // ordered by type root definition
+      // For strings, binary and decimal, we add the literal as reusable field,
+      // as they're not cheap to construct. For the other types, the return term is directly
+      // the literal value
       case CHAR | VARCHAR =>
-        val escapedValue = StringEscapeUtils.ESCAPE_JAVA.translate(literalValue.toString)
-        val field = ctx.addReusableStringConstants(escapedValue)
-        generateNonNullLiteral(literalType, field, StringData.fromString(escapedValue))
-
-      case BOOLEAN =>
-        generateNonNullLiteral(literalType, literalValue.toString, literalValue)
+        val str = literalValue.asInstanceOf[BinaryStringData]
+        val field = ctx.addReusableEscapedStringConstant(EncodingUtils.escapeJava(str.toString))
+        generateNonNullLiteral(literalType, field, str)
 
       case BINARY | VARBINARY =>
-        val bytesVal = literalValue.asInstanceOf[ByteString].getBytes
+        val bytesVal = literalValue.asInstanceOf[Array[Byte]]
         val fieldTerm = ctx.addReusableObject(
           bytesVal, "binary", bytesVal.getClass.getCanonicalName)
         generateNonNullLiteral(literalType, fieldTerm, bytesVal)
 
       case DECIMAL =>
-        val dt = literalType.asInstanceOf[DecimalType]
-        val precision = dt.getPrecision
-        val scale = dt.getScale
         val fieldTerm = newName("decimal")
-        val decimalClass = className[DecimalData]
-        val fieldDecimal =
-          s"""
-             |$decimalClass $fieldTerm =
-             |    $DECIMAL_UTIL.castFrom("${literalValue.toString}", $precision, $scale);
-             |""".stripMargin
-        ctx.addReusableMember(fieldDecimal)
-        val value = DecimalData.fromBigDecimal(
-          literalValue.asInstanceOf[JBigDecimal], precision, scale)
-        if (value == null) {
-          generateNullLiteral(literalType, ctx.nullCheck)
-        } else {
-          generateNonNullLiteral(literalType, fieldTerm, value)
-        }
-
-      case TINYINT =>
-        val decimal = BigDecimal(literalValue.asInstanceOf[JBigDecimal])
-        generateNonNullLiteral(literalType, decimal.byteValue().toString, decimal.byteValue())
-
-      case SMALLINT =>
-        val decimal = BigDecimal(literalValue.asInstanceOf[JBigDecimal])
-        generateNonNullLiteral(literalType, decimal.shortValue().toString, decimal.shortValue())
-
-      case INTEGER =>
-        val decimal = BigDecimal(literalValue.asInstanceOf[JBigDecimal])
-        generateNonNullLiteral(literalType, decimal.intValue().toString, decimal.intValue())
-
-      case BIGINT =>
-        val decimal = BigDecimal(literalValue.asInstanceOf[JBigDecimal])
-        generateNonNullLiteral(
-          literalType, decimal.longValue().toString + "L", decimal.longValue())
-
-      case FLOAT =>
-        val floatValue = literalValue.asInstanceOf[JBigDecimal].floatValue()
-        floatValue match {
-          case Float.NegativeInfinity =>
-            generateNonNullLiteral(
-              literalType,
-              "java.lang.Float.NEGATIVE_INFINITY",
-              Float.NegativeInfinity)
-          case Float.PositiveInfinity => generateNonNullLiteral(
-            literalType,
-            "java.lang.Float.POSITIVE_INFINITY",
-            Float.PositiveInfinity)
-          case _ => generateNonNullLiteral(
-            literalType, floatValue.toString + "f", floatValue)
-        }
-
-      case DOUBLE =>
-        val doubleValue = literalValue.asInstanceOf[JBigDecimal].doubleValue()
-        doubleValue match {
-          case Double.NegativeInfinity =>
-            generateNonNullLiteral(
-              literalType,
-              "java.lang.Double.NEGATIVE_INFINITY",
-              Double.NegativeInfinity)
-          case Double.PositiveInfinity =>
-            generateNonNullLiteral(
-              literalType,
-              "java.lang.Double.POSITIVE_INFINITY",
-              Double.PositiveInfinity)
-          case _ => generateNonNullLiteral(
-            literalType, doubleValue.toString + "d", doubleValue)
-        }
-
-      case DATE =>
-        generateNonNullLiteral(literalType, literalValue.toString, literalValue)
-
-      case TIME_WITHOUT_TIME_ZONE =>
-        generateNonNullLiteral(literalType, literalValue.toString, literalValue)
-
-      case TIMESTAMP_WITHOUT_TIME_ZONE =>
-        val fieldTerm = newName("timestamp")
-        val ldt = toLocalDateTime(literalValue.asInstanceOf[TimestampString])
-        val ts = TimestampData.fromLocalDateTime(ldt)
-        val fieldTimestamp =
-          s"""
-             |$TIMESTAMP_DATA $fieldTerm =
-             |  $TIMESTAMP_DATA.fromEpochMillis(${ts.getMillisecond}L, ${ts.getNanoOfMillisecond});
-           """.stripMargin
-        ctx.addReusableMember(fieldTimestamp)
-        generateNonNullLiteral(literalType, fieldTerm, ts)
-
-      case TIMESTAMP_WITH_TIME_ZONE =>
-        throw new UnsupportedOperationException("Unsupported type: " + literalType)
-
-      case TIMESTAMP_WITH_LOCAL_TIME_ZONE =>
-        val fieldTerm = newName("timestampWithLocalZone")
-        val ins =
-          toLocalDateTime(literalValue.asInstanceOf[TimestampString])
-            .atOffset(ZoneOffset.UTC)
-            .toInstant
-        val ts = TimestampData.fromInstant(ins)
-        val fieldTimestampWithLocalZone =
-          s"""
-             |$TIMESTAMP_DATA $fieldTerm =
-             |  $TIMESTAMP_DATA.fromEpochMillis(${ts.getMillisecond}L, ${ts.getNanoOfMillisecond});
-           """.stripMargin
-        ctx.addReusableMember(fieldTimestampWithLocalZone)
+        ctx.addReusableMember(s"""
+           |${className[DecimalData]} $fieldTerm = ${primitiveLiteralForType(literalValue)};
+           |""".stripMargin)
         generateNonNullLiteral(literalType, fieldTerm, literalValue)
 
-      case INTERVAL_YEAR_MONTH =>
-        val decimal = BigDecimal(literalValue.asInstanceOf[JBigDecimal])
-        if (decimal.isValidInt) {
-          generateNonNullLiteral(literalType, decimal.intValue().toString, decimal.intValue())
-        } else {
-          throw new CodeGenException(
-            s"Decimal '$decimal' can not be converted to interval of months.")
-        }
-
-      case INTERVAL_DAY_TIME =>
-        val decimal = BigDecimal(literalValue.asInstanceOf[JBigDecimal])
-        if (decimal.isValidLong) {
-          generateNonNullLiteral(
-            literalType,
-            decimal.longValue().toString + "L",
-            decimal.longValue())
-        } else {
-          throw new CodeGenException(
-            s"Decimal '$decimal' can not be converted to interval of milliseconds.")
-        }
-
       case DISTINCT_TYPE =>
-        generateLiteral(ctx, literalType.asInstanceOf[DistinctType].getSourceType, literalValue)
-
-      // Symbol type for special flags e.g. TRIM's BOTH, LEADING, TRAILING
-      case RAW if literalType.asInstanceOf[TypeInformationRawType[_]]
-          .getTypeInformation.getTypeClass.isAssignableFrom(classOf[Enum[_]]) =>
-        generateSymbol(literalValue.asInstanceOf[Enum[_]])
+        generateLiteral(ctx, literalValue, literalType.asInstanceOf[DistinctType].getSourceType)
 
       case SYMBOL =>
-        throw new UnsupportedOperationException() // TODO support symbol?
+        generateSymbol(literalValue.asInstanceOf[Enum[_]])
+
+      case BOOLEAN | TINYINT | SMALLINT | INTEGER | BIGINT | FLOAT | DOUBLE | DATE |
+           TIME_WITHOUT_TIME_ZONE | TIMESTAMP_WITHOUT_TIME_ZONE | TIMESTAMP_WITH_LOCAL_TIME_ZONE |
+           INTERVAL_YEAR_MONTH | INTERVAL_DAY_TIME =>
+        generateNonNullLiteral(literalType, primitiveLiteralForType(literalValue), literalValue)
 
       case ARRAY | MULTISET | MAP | ROW | STRUCTURED_TYPE | NULL | UNRESOLVED =>
         throw new CodeGenException(s"Type not supported: $literalType")
     }
   }
 
-  def generateSymbol(enum: Enum[_]): GeneratedExpression = {
+  def generateSymbol(value: Enum[_]): GeneratedExpression = {
+    val convertedValue = calciteToCommon(value, true)
     GeneratedExpression(
-      qualifyEnum(enum),
+      qualifyEnum(convertedValue),
       NEVER_NULL,
       NO_CODE,
-      new TypeInformationRawType[AnyRef](new GenericTypeInfo[AnyRef](
-        enum.getDeclaringClass.asInstanceOf[Class[AnyRef]])),
-      literalValue = Some(enum))
+      new SymbolType(false),
+      literalValue = Some(convertedValue))
   }
 
   /**
@@ -714,7 +608,7 @@ object GenerateUtils {
         val fieldType = getFieldTypes(inputType).get(index)
         val resultTypeTerm = primitiveTypeTermForType(fieldType)
         val defaultValue = primitiveDefaultValue(fieldType)
-        val readCode = rowFieldReadAccess(ctx, index.toString, inputTerm, fieldType)
+        val readCode = rowFieldReadAccess(index.toString, inputTerm, fieldType)
         val Seq(fieldTerm, nullTerm) = ctx.addReusableLocalVariables(
           (resultTypeTerm, "field"),
           ("boolean", "isNull"))
@@ -876,8 +770,8 @@ object GenerateUtils {
           } else if ($isNullB) {
             return ${-nullIsLastRet};
           } else {
-            $typeTerm $fieldA = ${rowFieldReadAccess(ctx, i, leftTerm, elementType)};
-            $typeTerm $fieldB = ${rowFieldReadAccess(ctx, i, rightTerm, elementType)};
+            $typeTerm $fieldA = ${rowFieldReadAccess(i, leftTerm, elementType)};
+            $typeTerm $fieldB = ${rowFieldReadAccess(i, rightTerm, elementType)};
             int $comp = ${generateCompare(ctx, elementType, nullsIsLast, fieldA, fieldB)};
             if ($comp != 0) {
               return $comp;
@@ -929,8 +823,8 @@ object GenerateUtils {
            |} else if ($isNullB) {
            |  return ${-nullIsLastRet};
            |} else {
-           |  $typeTerm $fieldA = ${rowFieldReadAccess(ctx, index, leftTerm, t)};
-           |  $typeTerm $fieldB = ${rowFieldReadAccess(ctx, index, rightTerm, t)};
+           |  $typeTerm $fieldA = ${rowFieldReadAccess(index, leftTerm, t)};
+           |  $typeTerm $fieldB = ${rowFieldReadAccess(index, rightTerm, t)};
            |  int $comp = ${generateCompare(ctx, t, fieldSpec.getNullIsLast, fieldA, fieldB)};
            |  if ($comp != 0) {
            |    return $symbol$comp;
