@@ -29,7 +29,6 @@ import org.apache.flink.table.runtime.functions.SqlFunctionUtils
 import org.apache.flink.table.runtime.typeutils.TypeCheckUtils.{isCharacterString, isTimestamp, isTimestampWithLocalZone}
 import org.apache.flink.table.types.logical._
 
-import org.apache.calcite.runtime.SqlFunctions
 import org.apache.calcite.sql.SqlOperator
 import org.apache.calcite.sql.fun.SqlTrimFunction.Flag.{BOTH, LEADING, TRAILING}
 
@@ -55,6 +54,10 @@ object StringCallGen {
       new MethodCallGen(method).generate(ctx, operands, returnType)
     }
 
+    def fallibleMethodGen(method: Method): GeneratedExpression = {
+      new MethodCallGen(method, wrapTryCatch = true).generate(ctx, operands, returnType)
+    }
+
     val generator = operator match {
 
       case LIKE =>
@@ -187,12 +190,12 @@ object StringCallGen {
         methodGen(BuiltInMethods.STRING_TO_DATE_WITH_FORMAT)
 
       case TO_TIMESTAMP if operands.size == 1 && isCharacterString(operands.head.resultType) =>
-        methodGen(BuiltInMethods.STRING_TO_TIMESTAMP)
+        fallibleMethodGen(BuiltInMethods.STRING_TO_TIMESTAMP)
 
       case TO_TIMESTAMP if operands.size == 2 &&
           isCharacterString(operands.head.resultType) &&
           isCharacterString(operands(1).resultType) =>
-        methodGen(BuiltInMethods.STRING_TO_TIMESTAMP_WITH_FORMAT)
+        fallibleMethodGen(BuiltInMethods.STRING_TO_TIMESTAMP_WITH_FORMAT)
 
       case UNIX_TIMESTAMP if operands.size == 1 && isCharacterString(operands.head.resultType) =>
         methodGen(BuiltInMethods.UNIX_TIMESTAMP_STR)
@@ -205,17 +208,17 @@ object StringCallGen {
       case DATE_FORMAT if operands.size == 2 &&
           isTimestamp(operands.head.resultType) &&
           isCharacterString(operands(1).resultType) =>
-        methodGen(BuiltInMethods.DATE_FORMAT_TIMESTAMP_STRING)
+        methodGen(BuiltInMethods.FORMAT_TIMESTAMP_DATA)
 
       case DATE_FORMAT if operands.size == 2 &&
           isTimestampWithLocalZone(operands.head.resultType) &&
           isCharacterString(operands(1).resultType) =>
-        methodGen(BuiltInMethods.DATE_FORMAT_TIMESTAMP_STRING_TIME_ZONE)
+        methodGen(BuiltInMethods.FORMAT_TIMESTAMP_DATA_WITH_TIME_ZONE)
 
       case DATE_FORMAT if operands.size == 2 &&
           isCharacterString(operands.head.resultType) &&
           isCharacterString(operands(1).resultType) =>
-        methodGen(BuiltInMethods.DATE_FORMAT_STIRNG_STRING)
+        methodGen(BuiltInMethods.FORMAT_TIMESTAMP_STRING_FORMAT_STRING_STRING)
 
       case CONVERT_TZ if operands.size == 3 &&
           isCharacterString(operands.head.resultType) &&
@@ -341,9 +344,8 @@ object StringCallGen {
   def generateSimilarTo(
     ctx: CodeGeneratorContext,
     operands: Seq[GeneratedExpression]): GeneratedExpression = {
-    val className = classOf[SqlFunctions].getCanonicalName
     generateCallIfArgsNotNull(ctx, new BooleanType(), operands) {
-      terms => s"$className.similar(${toStringTerms(terms, operands)})"
+      terms => s"${qualifyMethod(BuiltInMethods.STRING_SIMILAR)}(${toStringTerms(terms, operands)})"
     }
   }
 
@@ -424,9 +426,8 @@ object StringCallGen {
     ctx: CodeGeneratorContext,
     operands: Seq[GeneratedExpression],
     returnType: LogicalType): GeneratedExpression = {
-    val className = classOf[SqlFunctions].getCanonicalName
     generateStringResultCallIfArgsNotNull(ctx, operands, returnType) {
-      terms => s"$className.initcap(${terms.head}.toString())"
+      terms => s"${qualifyMethod(BuiltInMethods.STRING_INITCAP)}(${terms.head}.toString())"
     }
   }
 
@@ -765,7 +766,7 @@ object StringCallGen {
       operands: Seq[GeneratedExpression]): GeneratedExpression = {
     val className = classOf[SqlFunctionUtils].getCanonicalName
     val t = new MapType(
-      new VarCharType(VarCharType.MAX_LENGTH), new VarCharType(VarCharType.MAX_LENGTH))
+      VarCharType.STRING_TYPE, VarCharType.STRING_TYPE)
     val converter = DataFormatConverters.getConverterForDataType(
       DataTypes.MAP(DataTypes.STRING(), DataTypes.STRING()))
     val converterTerm = ctx.addReusableObject(converter, "mapConverter")
@@ -797,5 +798,4 @@ object StringCallGen {
         s"new String(${terms.head}, ${terms(1)}.toString())"
     }
   }
-
 }
