@@ -22,34 +22,42 @@ import org.apache.flink.api.common.RuntimeExecutionMode
 import org.apache.flink.api.dag.Transformation
 import org.apache.flink.configuration.ExecutionOptions
 import org.apache.flink.streaming.api.graph.StreamGraph
-import org.apache.flink.table.api.{ExplainDetail, TableConfig, TableException}
-import org.apache.flink.table.catalog.{CatalogManager, FunctionCatalog, ObjectIdentifier}
-import org.apache.flink.table.delegation.Executor
-import org.apache.flink.table.operations.{CatalogSinkModifyOperation, ModifyOperation, Operation, QueryOperation}
-import org.apache.flink.table.planner.operations.PlannerQueryOperation
+import org.apache.flink.table.api.PlanReference.{ContentPlanReference, FilePlanReference, ResourcePlanReference}
+import org.apache.flink.table.api.internal.TableEnvironmentInternal
+import org.apache.flink.table.api.{ExplainDetail, PlanReference, TableConfig, TableException}
+import org.apache.flink.table.catalog.{CatalogManager, FunctionCatalog}
+import org.apache.flink.table.delegation.{Executor, InternalPlan}
+import org.apache.flink.table.module.ModuleManager
+import org.apache.flink.table.operations.{ModifyOperation, Operation}
+import org.apache.flink.table.planner.plan.ExecNodeGraphInternalPlan
 import org.apache.flink.table.planner.plan.`trait`._
 import org.apache.flink.table.planner.plan.nodes.exec.ExecNodeGraph
 import org.apache.flink.table.planner.plan.nodes.exec.processor.ExecNodeGraphProcessor
+import org.apache.flink.table.planner.plan.nodes.exec.serde.{JsonSerdeUtil, SerdeContext}
 import org.apache.flink.table.planner.plan.nodes.exec.stream.StreamExecNode
 import org.apache.flink.table.planner.plan.nodes.exec.utils.ExecNodePlanDumper
 import org.apache.flink.table.planner.plan.optimize.{Optimizer, StreamCommonSubGraphBasedOptimizer}
 import org.apache.flink.table.planner.plan.utils.FlinkRelOptUtil
 import org.apache.flink.table.planner.utils.DummyStreamExecutionEnvironment
 
+import org.apache.flink.shaded.jackson2.com.fasterxml.jackson.databind.ObjectReader
+
 import org.apache.calcite.plan.{ConventionTraitDef, RelTrait, RelTraitDef}
-import org.apache.calcite.rel.logical.LogicalTableModify
 import org.apache.calcite.sql.SqlExplainLevel
 
+import java.io.{File, IOException}
 import java.util
 
 import _root_.scala.collection.JavaConversions._
 
 class StreamPlanner(
     executor: Executor,
-    config: TableConfig,
+    tableConfig: TableConfig,
+    moduleManager: ModuleManager,
     functionCatalog: FunctionCatalog,
     catalogManager: CatalogManager)
-  extends PlannerBase(executor, config, functionCatalog, catalogManager, isStreamingMode = true) {
+  extends PlannerBase(executor, tableConfig, moduleManager, functionCatalog, catalogManager,
+    isStreamingMode = true) {
 
   override protected def getTraitDefs: Array[RelTraitDef[_ <: RelTrait]] = {
     Array(
@@ -65,7 +73,7 @@ class StreamPlanner(
   override protected def getExecNodeGraphProcessors: Seq[ExecNodeGraphProcessor] = Seq()
 
   override protected def translateToPlan(execGraph: ExecNodeGraph): util.List[Transformation[_]] = {
-    validateAndOverrideConfiguration()
+    beforeTranslation()
     val planner = createDummyPlanner()
     val transformations = execGraph.getRootNodes.map {
       case node: StreamExecNode[_] => node.translateToPlan(planner)
@@ -73,48 +81,20 @@ class StreamPlanner(
         throw new TableException("Cannot generate DataStream due to an invalid logical plan. " +
           "This is a bug and should not happen. Please file an issue.")
     }
-    cleanupInternalConfigurations()
+    afterTranslation()
     transformations
   }
 
   override def explain(operations: util.List[Operation], extraDetails: ExplainDetail*): String = {
-    require(operations.nonEmpty, "operations should not be empty")
-    validateAndOverrideConfiguration()
-    val sinkRelNodes = operations.map {
-      case queryOperation: QueryOperation =>
-        val relNode = getRelBuilder.queryOperation(queryOperation).build()
-        relNode match {
-          // SQL: explain plan for insert into xx
-          case modify: LogicalTableModify =>
-            // convert LogicalTableModify to CatalogSinkModifyOperation
-            val qualifiedName = modify.getTable.getQualifiedName
-            require(qualifiedName.size() == 3, "the length of qualified name should be 3.")
-            val modifyOperation = new CatalogSinkModifyOperation(
-              ObjectIdentifier.of(qualifiedName.get(0), qualifiedName.get(1), qualifiedName.get(2)),
-              new PlannerQueryOperation(modify.getInput)
-            )
-            translateToRel(modifyOperation)
-          case _ =>
-            relNode
-        }
-      case modifyOperation: ModifyOperation =>
-        translateToRel(modifyOperation)
-      case o => throw new TableException(s"Unsupported operation: ${o.getClass.getCanonicalName}")
-    }
-    val optimizedRelNodes = optimize(sinkRelNodes)
-    val execGraph = translateToExecNodeGraph(optimizedRelNodes)
-
-    val transformations = translateToPlan(execGraph)
-    cleanupInternalConfigurations()
-
-    val streamGraph = executor.createPipeline(transformations, config.getConfiguration, null)
-      .asInstanceOf[StreamGraph]
+    val (sinkRelNodes, optimizedRelNodes, execGraph, streamGraph) = getExplainGraphs(operations)
 
     val sb = new StringBuilder
     sb.append("== Abstract Syntax Tree ==")
     sb.append(System.lineSeparator)
     sinkRelNodes.foreach { sink =>
-      sb.append(FlinkRelOptUtil.toString(sink))
+      // use EXPPLAN_ATTRIBUTES to make the ast result more readable
+      // and to keep the previous behavior
+      sb.append(FlinkRelOptUtil.toString(sink, SqlExplainLevel.EXPPLAN_ATTRIBUTES))
       sb.append(System.lineSeparator)
     }
 
@@ -151,16 +131,67 @@ class StreamPlanner(
   private def createDummyPlanner(): StreamPlanner = {
     val dummyExecEnv = new DummyStreamExecutionEnvironment(getExecEnv)
     val executor = new DefaultExecutor(dummyExecEnv)
-    new StreamPlanner(executor, config, functionCatalog, catalogManager)
+    new StreamPlanner(executor, tableConfig, moduleManager, functionCatalog, catalogManager)
   }
 
-  override def explainJsonPlan(jsonPlan: String, extraDetails: ExplainDetail*): String = {
-    validateAndOverrideConfiguration()
-    val execGraph = ExecNodeGraph.createExecNodeGraph(jsonPlan, createSerdeContext)
-    val transformations = translateToPlan(execGraph)
-    cleanupInternalConfigurations()
+  override def loadPlan(planReference: PlanReference): InternalPlan = {
+    val ctx = createSerdeContext
+    val objectReader: ObjectReader = JsonSerdeUtil.createObjectReader(ctx)
+    val execNodeGraph = planReference match {
+      case filePlanReference: FilePlanReference =>
+        objectReader.readValue(filePlanReference.getFile, classOf[ExecNodeGraph])
+      case contentPlanReference: ContentPlanReference =>
+        objectReader.readValue(contentPlanReference.getContent, classOf[ExecNodeGraph])
+      case resourcePlanReference: ResourcePlanReference =>
+        val url = resourcePlanReference.getClassLoader
+          .getResource(resourcePlanReference.getResourcePath)
+        if (url == null) {
+          throw new IOException(
+            "Cannot load the plan reference from classpath: " + planReference)
+        }
+        objectReader.readValue(new File(url.toURI), classOf[ExecNodeGraph])
+      case _ => throw new IllegalStateException(
+        "Unknown PlanReference. This is a bug, please contact the developers")
+    }
 
-    val streamGraph = executor.createPipeline(transformations, config.getConfiguration, null)
+    new ExecNodeGraphInternalPlan(
+      JsonSerdeUtil.createObjectWriter(ctx)
+        .withDefaultPrettyPrinter()
+        .writeValueAsString(execNodeGraph),
+      execNodeGraph)
+  }
+
+  override def compilePlan(
+     modifyOperations: util.List[ModifyOperation]): InternalPlan = {
+    beforeTranslation()
+    val relNodes = modifyOperations.map(translateToRel)
+    val optimizedRelNodes = optimize(relNodes)
+    val execGraph = translateToExecNodeGraph(optimizedRelNodes)
+    afterTranslation()
+
+    new ExecNodeGraphInternalPlan(
+      JsonSerdeUtil.createObjectWriter(createSerdeContext)
+        .withDefaultPrettyPrinter()
+        .writeValueAsString(execGraph),
+      execGraph)
+  }
+
+  override def translatePlan(plan: InternalPlan): util.List[Transformation[_]] = {
+    beforeTranslation()
+    val execGraph = plan.asInstanceOf[ExecNodeGraphInternalPlan].getExecNodeGraph
+    val transformations = translateToPlan(execGraph)
+    afterTranslation()
+    transformations
+  }
+
+  override def explainPlan(plan: InternalPlan, extraDetails: ExplainDetail*): String = {
+    beforeTranslation()
+    val execGraph = plan.asInstanceOf[ExecNodeGraphInternalPlan].getExecNodeGraph
+    val transformations = translateToPlan(execGraph)
+    afterTranslation()
+
+    // We pass only the configuration to avoid reconfiguration with the rootConfiguration
+    val streamGraph = executor.createPipeline(transformations, tableConfig.getConfiguration, null)
       .asInstanceOf[StreamGraph]
 
     val sb = new StringBuilder
@@ -178,9 +209,9 @@ class StreamPlanner(
     sb.toString()
   }
 
-  override def validateAndOverrideConfiguration(): Unit = {
-    super.validateAndOverrideConfiguration()
-    val runtimeMode = getConfiguration.get(ExecutionOptions.RUNTIME_MODE)
+  override def beforeTranslation(): Unit = {
+    super.beforeTranslation()
+    val runtimeMode = getTableConfig.get(ExecutionOptions.RUNTIME_MODE)
     if (runtimeMode != RuntimeExecutionMode.STREAMING) {
       throw new IllegalArgumentException(
         "Mismatch between configured runtime mode and actual runtime mode. " +
