@@ -37,6 +37,7 @@ import org.apache.flink.types.RowKind
 
 import org.apache.calcite.rel.RelNode
 import org.apache.calcite.util.ImmutableBitSet
+import org.apache.flink.table.catalog.{ManagedTableListener, ResolvedCatalogBaseTable}
 
 import scala.collection.JavaConversions._
 
@@ -45,15 +46,12 @@ import scala.collection.JavaConversions._
  */
 class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOptimizeContext] {
 
-  private val SATISFY_MODIFY_KIND_SET_TRAIT_VISITOR = new SatisfyModifyKindSetTraitVisitor
-  private val SATISFY_UPDATE_KIND_TRAIT_VISITOR = new SatisfyUpdateKindTraitVisitor
-
   override def optimize(
       root: RelNode,
       context: StreamOptimizeContext): RelNode = {
     // step1: satisfy ModifyKindSet trait
     val physicalRoot = root.asInstanceOf[StreamPhysicalRel]
-    val rootWithModifyKindSet = SATISFY_MODIFY_KIND_SET_TRAIT_VISITOR.visit(
+    val rootWithModifyKindSet = new SatisfyModifyKindSetTraitVisitor().visit(
       physicalRoot,
       // we do not propagate the ModifyKindSet requirement and requester among blocks
       // set default ModifyKindSet requirement and requester for root
@@ -76,8 +74,9 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
       Seq(UpdateKindTrait.NONE)
     }
 
+    val updateKindTraitVisitor = new SatisfyUpdateKindTraitVisitor(context)
     val finalRoot = requiredUpdateKindTraits.flatMap { requiredUpdateKindTrait =>
-      SATISFY_UPDATE_KIND_TRAIT_VISITOR.visit(rootWithModifyKindSet, requiredUpdateKindTrait)
+      updateKindTraitVisitor.visit(rootWithModifyKindSet, requiredUpdateKindTrait)
     }
 
     // step3: sanity check and return non-empty root
@@ -124,7 +123,7 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
         requiredTrait: ModifyKindSetTrait,
         requester: String): StreamPhysicalRel = rel match {
       case sink: StreamPhysicalSink =>
-        val name = s"Table sink '${sink.tableIdentifier.asSummaryString()}'"
+        val name = s"Table sink '${sink.contextResolvedTable.getIdentifier.asSummaryString()}'"
         val queryModifyKindSet = deriveQueryDefaultChangelogMode(sink.getInput, name)
         val sinkRequiredTrait = ModifyKindSetTrait.fromChangelogMode(
           sink.tableSink.getChangelogMode(queryModifyKindSet))
@@ -215,8 +214,9 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
         val providedTrait = new ModifyKindSetTrait(builder.build())
         createNewNode(window, children, providedTrait, requiredTrait, requester)
 
-      case _: StreamPhysicalWindowAggregate | _: StreamPhysicalWindowRank =>
-        // WindowAggregate and WindowRank support insert-only in input
+      case _: StreamPhysicalWindowAggregate | _: StreamPhysicalWindowRank |
+           _: StreamPhysicalWindowDeduplicate =>
+        // WindowAggregate, WindowRank, WindowDeduplicate support insert-only in input
         val children = visitChildren(rel, ModifyKindSetTrait.INSERT_ONLY)
         val providedTrait = ModifyKindSetTrait.INSERT_ONLY
         createNewNode(rel, children, providedTrait, requiredTrait, requester)
@@ -406,8 +406,7 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
    * <p>After traversed by this visitor, every node should have a correct [[UpdateKindTrait]]
    * or returns None if the planner doesn't support to satisfy the required [[UpdateKindTrait]].
    */
-  private class SatisfyUpdateKindTraitVisitor {
-
+  private class SatisfyUpdateKindTraitVisitor(private val context: StreamOptimizeContext) {
     /**
      * Try to satisfy the required [[UpdateKindTrait]] from root.
      *
@@ -475,11 +474,13 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
         createNewNode(rel, children, requiredTrait)
 
       case _: StreamPhysicalWindowAggregate | _: StreamPhysicalWindowRank |
-           _: StreamPhysicalDeduplicate | _: StreamPhysicalTemporalSort | _: StreamPhysicalMatch |
+           _: StreamPhysicalWindowDeduplicate | _: StreamPhysicalDeduplicate |
+           _: StreamPhysicalTemporalSort | _: StreamPhysicalMatch |
            _: StreamPhysicalOverAggregate | _: StreamPhysicalIntervalJoin |
            _: StreamPhysicalPythonOverAggregate | _: StreamPhysicalWindowJoin =>
-        // WindowAggregate, WindowTableAggregate, Deduplicate, TemporalSort, CEP,
-        // OverAggregate, and IntervalJoin, WindowJoin require nothing about UpdateKind.
+        // WindowAggregate, WindowTableAggregate, WindowRank, WindowDeduplicate, Deduplicate,
+        // TemporalSort, CEP, OverAggregate, and IntervalJoin, WindowJoin require nothing about
+        // UpdateKind.
         val children = visitChildren(rel, UpdateKindTrait.NONE)
         createNewNode(rel, children, requiredTrait)
 
@@ -627,6 +628,27 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
         }
 
       case normalize: StreamPhysicalChangelogNormalize =>
+        val contextResolvedTable = normalize.contextResolvedTable
+        val tableIdentifier = contextResolvedTable.getIdentifier
+        if (!contextResolvedTable.isAnonymous
+          && requiredTrait == UpdateKindTrait.ONLY_UPDATE_AFTER) {
+          val catalogName = tableIdentifier.getCatalogName
+          val catalog = context.getCatalogManager.getCatalog(catalogName).orElse(null)
+          val catalogTable = contextResolvedTable.getResolvedTable[ResolvedCatalogBaseTable[_]]
+          if (ManagedTableListener.isManagedTable(catalog, catalogTable)) {
+            // if requiredTrait is ONLY_UPDATE_AFTER and table is ManagedTable,
+            // we can eliminate current normalize stage,
+            // cuz ManagedTable has preserved complete delete messages.
+            val input = normalize.getInput match {
+              case exchange: StreamPhysicalExchange =>
+                exchange.getInput
+              case _ =>
+                normalize.getInput
+            }
+            val inputPhysicalRel = input.asInstanceOf[StreamPhysicalRel]
+            return this.visit(inputPhysicalRel, UpdateKindTrait.ONLY_UPDATE_AFTER)
+          }
+        }
         // changelog normalize currently only supports input only sending UPDATE_AFTER
         val children = visitChildren(normalize, UpdateKindTrait.ONLY_UPDATE_AFTER)
         // use requiredTrait as providedTrait,
@@ -771,12 +793,10 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
         // if sink's pk(s) are not exactly match input changeLogUpsertKeys then it will fallback
         // to beforeAndAfter mode for the correctness
         var requireBeforeAndAfter: Boolean = false
-        val sinkDefinedPks = toScala(sink.catalogTable.getResolvedSchema
-            .getPrimaryKey).map(_.getColumns).map(toScala[String]).getOrElse(Seq())
+        val sinkDefinedPks = sink.contextResolvedTable.getResolvedSchema.getPrimaryKeyIndexes
 
         if (sinkDefinedPks.nonEmpty) {
-          val sinkColumns = sink.catalogTable.getResolvedSchema.getColumnNames
-          val sinkPks = ImmutableBitSet.of(sinkDefinedPks.map(sinkColumns.indexOf): _*)
+          val sinkPks = ImmutableBitSet.of(sinkDefinedPks: _*)
           val fmq = FlinkRelMetadataQuery.reuseOrCreate(sink.getCluster.getMetadataQuery)
           val changeLogUpsertKeys = fmq.getUpsertKeys(sink.getInput)
           // if input is UA only, primary key != upsert key (upsert key can be null) we should
@@ -811,10 +831,8 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
           .getTableConfig
       val inputChangelogMode = ChangelogPlanUtils.getChangelogMode(
         sink.getInput.asInstanceOf[StreamPhysicalRel]).get
-      val catalogTable = sink.catalogTable
-      val primaryKeys = toScala(catalogTable.getResolvedSchema
-          .getPrimaryKey).map(_.getColumns).map(toScala[String]).getOrElse(Seq())
-      val upsertMaterialize = tableConfig.getConfiguration.get(
+      val primaryKeys = sink.contextResolvedTable.getResolvedSchema.getPrimaryKeyIndexes
+      val upsertMaterialize = tableConfig.get(
         ExecutionConfigOptions.TABLE_EXEC_SINK_UPSERT_MATERIALIZE) match {
         case UpsertMaterialize.FORCE => primaryKeys.nonEmpty
         case UpsertMaterialize.NONE => false
@@ -824,8 +842,7 @@ class FlinkChangelogModeInferenceProgram extends FlinkOptimizeProgram[StreamOpti
           val inputInsertOnly = inputChangelogMode.containsOnly(RowKind.INSERT)
 
           if (!sinkAcceptInsertOnly && !inputInsertOnly && primaryKeys.nonEmpty) {
-            val columnNames = catalogTable.getResolvedSchema.getColumnNames
-            val pks = ImmutableBitSet.of(primaryKeys.map(columnNames.indexOf): _*)
+            val pks = ImmutableBitSet.of(primaryKeys: _*)
             val fmq = FlinkRelMetadataQuery.reuseOrCreate(sink.getCluster.getMetadataQuery)
             val changeLogUpsertKeys = fmq.getUpsertKeys(sink.getInput)
             // if input has update and primary key != upsert key (upsert key can be null) we should
