@@ -22,7 +22,7 @@ import org.apache.flink.table.planner.plan.nodes.FlinkConventions
 import org.apache.flink.table.planner.plan.nodes.logical.FlinkLogicalTableFunctionScan
 import org.apache.flink.table.planner.plan.nodes.physical.stream.StreamPhysicalWindowTableFunction
 import org.apache.flink.table.planner.plan.utils.WindowUtil
-import org.apache.flink.table.planner.plan.utils.WindowUtil.convertToWindowingStrategy
+import org.apache.flink.table.planner.plan.utils.WindowUtil.{convertToWindowingStrategy, validateTimeFieldWithTimeAttribute}
 
 import org.apache.calcite.plan.{RelOptRule, RelOptRuleCall, RelTraitSet}
 import org.apache.calcite.rel.RelNode
@@ -49,13 +49,16 @@ class StreamPhysicalWindowTableFunctionRule  extends ConverterRule(
     val traitSet: RelTraitSet = rel.getTraitSet.replace(FlinkConventions.STREAM_PHYSICAL)
     val newInput = RelOptRule.convert(scan.getInput(0), FlinkConventions.STREAM_PHYSICAL)
 
+    val windowTableFunction = scan.getCall.asInstanceOf[RexCall]
+    val inputRowType = newInput.getRowType
+    // Time field of window table function in streaming mode should be with time attribute
+    validateTimeFieldWithTimeAttribute(windowTableFunction, inputRowType)
     new StreamPhysicalWindowTableFunction(
       scan.getCluster,
       traitSet,
       newInput,
       scan.getRowType,
-      convertToWindowingStrategy(scan.getCall.asInstanceOf[RexCall], newInput.getRowType),
-      false
+      convertToWindowingStrategy(windowTableFunction, inputRowType)
     )
   }
 }
