@@ -70,12 +70,11 @@ object IntervalJoinUtil {
       leftLogicalFieldCnt: Int,
       joinRowType: RelDataType,
       rexBuilder: RexBuilder,
-      config: TableConfig): (Option[WindowBounds], Option[RexNode]) = {
+      tableConfig: TableConfig): (Option[WindowBounds], Option[RexNode]) = {
 
     // Converts the condition to conjunctive normal form (CNF)
     val cnfCondition = FlinkRexUtil.toCnf(rexBuilder,
-      config.getConfiguration.getInteger(FlinkRexUtil.TABLE_OPTIMIZER_CNF_NODES_LIMIT),
-      predicate)
+      tableConfig.get(FlinkRexUtil.TABLE_OPTIMIZER_CNF_NODES_LIMIT), predicate)
 
     // split the condition into time predicates and other predicates
     // We need two range predicates or an equality predicate for a properly bounded window join.
@@ -119,7 +118,8 @@ object IntervalJoinUtil {
     }
 
     // assemble window bounds from predicates
-    val streamTimeOffsets = timePreds.map(computeWindowBoundFromPredicate(_, rexBuilder, config))
+    val streamTimeOffsets = timePreds.map(
+      computeWindowBoundFromPredicate(_, rexBuilder, tableConfig))
     val (leftLowerBound, leftUpperBound) =
       streamTimeOffsets match {
         case Seq(Some(x: WindowBound), Some(y: WindowBound)) if x.isLeftLower && !y.isLeftLower =>
@@ -323,7 +323,7 @@ object IntervalJoinUtil {
   private def computeWindowBoundFromPredicate(
       timePred: TimePredicate,
       rexBuilder: RexBuilder,
-      config: TableConfig): Option[WindowBound] = {
+      tableConfig: TableConfig): Option[WindowBound] = {
 
     val isLeftLowerBound: Boolean =
       timePred.pred.getKind match {
@@ -338,7 +338,7 @@ object IntervalJoinUtil {
       }
 
     // reduce predicate to constants to compute bounds
-    val (leftLiteral, rightLiteral) = reduceTimeExpression(timePred, rexBuilder, config)
+    val (leftLiteral, rightLiteral) = reduceTimeExpression(timePred, rexBuilder, tableConfig)
 
     if (leftLiteral.isEmpty || rightLiteral.isEmpty) {
       return None
@@ -370,15 +370,15 @@ object IntervalJoinUtil {
     * Replaces the time attributes on both sides of a time predicate by a zero literal and
     * reduces the expressions on both sides to a long literal.
     *
-    * @param timePred   The time predicate which both sides are reduced.
-    * @param rexBuilder A RexBuilder
-    * @param config     A TableConfig.
+    * @param timePred    The time predicate which both sides are reduced.
+    * @param rexBuilder  A RexBuilder
+    * @param tableConfig A TableConfig.
     * @return The values of the reduced literals on both sides of the time comparison predicate.
     */
   private def reduceTimeExpression(
       timePred: TimePredicate,
       rexBuilder: RexBuilder,
-      config: TableConfig): (Option[Long], Option[Long]) = {
+      tableConfig: TableConfig): (Option[Long], Option[Long]) = {
 
     /**
      * Checks if the given call is a materialization call for either proctime or rowtime.
@@ -426,7 +426,7 @@ object IntervalJoinUtil {
     val rightSideWithLiteral = replaceTimeFieldWithLiteral(rightSide)
 
     // reduce expression to literal
-     val exprReducer = new ExpressionReducer(config, allowChangeNullability = true)
+     val exprReducer = new ExpressionReducer(tableConfig, allowChangeNullability = true)
     val originList = new util.ArrayList[RexNode]()
     originList.add(leftSideWithLiteral)
     originList.add(rightSideWithLiteral)
