@@ -20,20 +20,20 @@ package org.apache.flink.table.planner.plan.utils
 import org.apache.flink.table.api.TableException
 import org.apache.flink.table.functions.{AggregateFunction, UserDefinedFunction}
 import org.apache.flink.table.planner.CalcitePair
-import org.apache.flink.table.planner.expressions.PlannerNamedWindowProperty
 import org.apache.flink.table.planner.functions.aggfunctions.DeclarativeAggregateFunction
-import org.apache.flink.table.planner.plan.nodes.ExpressionFormat
-import org.apache.flink.table.planner.plan.nodes.ExpressionFormat.ExpressionFormat
+import org.apache.flink.table.planner.plan.utils.ExpressionDetail.ExpressionDetail
+import org.apache.flink.table.planner.plan.utils.ExpressionFormat.ExpressionFormat
+import org.apache.flink.table.runtime.groupwindow.NamedWindowProperty
 
-import com.google.common.collect.{ImmutableList, ImmutableMap}
+import com.google.common.collect.ImmutableMap
 import org.apache.calcite.rel.`type`.RelDataType
 import org.apache.calcite.rel.core.Window.Group
 import org.apache.calcite.rel.core.{AggregateCall, Window}
 import org.apache.calcite.rel.hint.RelHint
 import org.apache.calcite.rel.{RelCollation, RelWriter}
 import org.apache.calcite.rex._
-import org.apache.calcite.sql.SqlKind
 import org.apache.calcite.sql.SqlMatchRecognize.AfterOption
+import org.apache.calcite.sql.{SqlExplainLevel, SqlKind}
 
 import java.util
 import java.util.{SortedSet => JSortedSet}
@@ -59,6 +59,11 @@ object RelExplainUtil {
     case _ => ExpressionFormat.Prefix
   }
 
+  def preferExpressionDetail(pw: RelWriter): ExpressionDetail = pw.getDetailLevel match {
+    case SqlExplainLevel.EXPPLAN_ATTRIBUTES => ExpressionDetail.Explain
+    case _ => ExpressionDetail.Digest
+  }
+
   /**
     * Converts field names corresponding to given indices to String.
     */
@@ -267,7 +272,7 @@ object RelExplainUtil {
       outputRowType: RelDataType,
       aggInfoList: AggregateInfoList,
       grouping: Array[Int],
-      windowProperties: Seq[PlannerNamedWindowProperty],
+      windowProperties: Seq[NamedWindowProperty],
       isLocal: Boolean = false,
       isGlobal: Boolean = false): String = {
     stringifyStreamAggregationToString(
@@ -306,7 +311,7 @@ object RelExplainUtil {
       aggInfoList: AggregateInfoList,
       grouping: Array[Int],
       shuffleKey: Option[Array[Int]],
-      windowProperties: Seq[PlannerNamedWindowProperty],
+      windowProperties: Seq[NamedWindowProperty],
       isLocal: Boolean,
       isGlobal: Boolean): String = {
 
@@ -505,7 +510,7 @@ object RelExplainUtil {
     val outputFieldNames = outputRowType.getFieldNames
 
     val aggStrings = namedAggregates.map(_.getKey).map(
-      a => s"${a.getAggregation}(${
+      a => s"""${a.getAggregation}(${
         val prefix = if (a.isDistinct) {
           "DISTINCT "
         } else {
@@ -525,7 +530,8 @@ object RelExplainUtil {
         } else {
           "*"
         })
-      })")
+      })""".stripMargin
+    )
 
     val output = if (outputInputName) inputFieldNames ++ aggStrings else aggStrings
     output.zip(outputFieldNames.drop(rowTypeOffset)).map {
@@ -557,7 +563,9 @@ object RelExplainUtil {
             outputFieldName
           }
         case (l: RexLiteral, i) => s"${l.getValue3} AS ${outFieldNames.get(i)}"
-        case (_, i) => outFieldNames.get(i)
+        case (n, i) =>
+          val exprStr = FlinkRexUtil.getExpressionString(n, inFieldNames)
+          s"$exprStr AS ${outFieldNames.get(i)}"
       }.mkString("{", ", ", "}")
     }.mkString(", ")
   }
@@ -604,16 +612,39 @@ object RelExplainUtil {
     buf.toString
   }
 
+  /**
+   * Converts conditions list to String.
+   */
+  def conditionsToString(
+      conditions : Option[util.List[RexNode]],
+      inputRowType: RelDataType,
+      f: (RexNode, List[String], Option[List[RexNode]],
+        ExpressionFormat, ExpressionDetail) => String,
+      expressionFormat: ExpressionFormat = ExpressionFormat.Prefix,
+      expressionDetail: ExpressionDetail = ExpressionDetail.Digest): String = {
+    if (conditions.isEmpty) {
+      return "[]"
+    }
+    val inFieldNames = inputRowType.getFieldNames.toList
+    conditions.get.map {cond => if (cond == null) {
+      "NULL"
+    } else {
+      f(cond, inFieldNames, None, expressionFormat, expressionDetail)
+    }}.mkString(", ")
+  }
+
   def conditionToString(
       calcProgram: RexProgram,
-      f: (RexNode, List[String], Option[List[RexNode]], ExpressionFormat) => String,
-      expressionFormat: ExpressionFormat = ExpressionFormat.Prefix): String = {
+      f: (RexNode, List[String], Option[List[RexNode]],
+        ExpressionFormat, ExpressionDetail) => String,
+      expressionFormat: ExpressionFormat = ExpressionFormat.Prefix,
+      expressionDetail: ExpressionDetail = ExpressionDetail.Digest): String = {
     val cond = calcProgram.getCondition
     val inputFieldNames = calcProgram.getInputRowType.getFieldNames.toList
     val localExprs = calcProgram.getExprList.toList
 
     if (cond != null) {
-      f(cond, inputFieldNames, Some(localExprs), expressionFormat)
+      f(cond, inputFieldNames, Some(localExprs), expressionFormat, expressionDetail)
     } else {
       ""
     }
@@ -621,15 +652,17 @@ object RelExplainUtil {
 
   def selectionToString(
       calcProgram: RexProgram,
-      expression: (RexNode, List[String], Option[List[RexNode]], ExpressionFormat) => String,
-      expressionFormat: ExpressionFormat = ExpressionFormat.Prefix): String = {
+      expression: (RexNode, List[String], Option[List[RexNode]],
+        ExpressionFormat, ExpressionDetail) => String,
+      expressionFormat: ExpressionFormat = ExpressionFormat.Prefix,
+      expressionDetail: ExpressionDetail = ExpressionDetail.Digest): String = {
 
     val proj = calcProgram.getProjectList.toList
     val inFields = calcProgram.getInputRowType.getFieldNames.toList
     val localExprs = calcProgram.getExprList.toList
     val outFields = calcProgram.getOutputRowType.getFieldNames.toList
 
-    proj.map(expression(_, inFields, Some(localExprs), expressionFormat))
+    proj.map(expression(_, inFields, Some(localExprs), expressionFormat, expressionDetail))
         .zip(outFields).map { case (e, o) =>
       if (e != o) {
         e + " AS " + o
@@ -642,10 +675,12 @@ object RelExplainUtil {
   def correlateToString(
       inputType: RelDataType,
       rexCall: RexCall,
-      expression: (RexNode, List[String], Option[List[RexNode]]) => String): String = {
+      expression: (RexNode, List[String], Option[List[RexNode]], ExpressionDetail) => String,
+      expressionDetail: ExpressionDetail): String = {
     val name = rexCall.getOperator.toString
     val inFields = inputType.getFieldNames.toList
-    val operands = rexCall.getOperands.map(expression(_, inFields, None)).mkString(",")
+    val operands = rexCall.getOperands.map(
+        expression(_, inFields, None, expressionDetail)).mkString(",")
     s"table($name($operands))"
   }
 
@@ -758,7 +793,7 @@ object RelExplainUtil {
       grouping: Array[Int],
       rowType: RelDataType,
       aggs: Seq[AggregateCall],
-      namedProperties: Seq[PlannerNamedWindowProperty],
+      namedProperties: Seq[NamedWindowProperty],
       withOutputFieldNames: Boolean = true): String = {
     val inFields = inputType.getFieldNames
     val isTableAggregate = AggregateUtil.isTableAggregate(aggs)
@@ -818,9 +853,10 @@ object RelExplainUtil {
   def measuresDefineToString(
     measures: ImmutableMap[String, RexNode],
     fieldNames: List[String],
-    expression: (RexNode, List[String], Option[List[RexNode]]) => String): String =
+    expression: (RexNode, List[String], Option[List[RexNode]], ExpressionDetail) => String,
+    expressionDetail: ExpressionDetail): String =
     measures.map {
-      case (k, v) => s"${expression(v, fieldNames, None)} AS $k"
+      case (k, v) => s"${expression(v, fieldNames, None, expressionDetail)} AS $k"
     }.mkString(", ")
 
   /**
