@@ -18,10 +18,9 @@
 
 package org.apache.flink.table.planner.plan.utils
 
-import org.apache.flink.table.planner.plan.metadata.FlinkRelMetadataQuery
-import org.apache.flink.table.planner.plan.nodes.ExpressionFormat
-import org.apache.flink.table.planner.plan.nodes.logical.FlinkLogicalJoin
 import org.apache.flink.table.planner.plan.`trait`.RelWindowProperties
+import org.apache.flink.table.planner.plan.metadata.FlinkRelMetadataQuery
+import org.apache.flink.table.planner.plan.nodes.logical.FlinkLogicalJoin
 import org.apache.flink.table.planner.utils.Logging
 
 import org.apache.calcite.rel.RelNode
@@ -242,7 +241,8 @@ object WindowJoinUtil extends Logging {
         join.getCondition,
         inputFieldNames,
         None,
-        ExpressionFormat.Infix)
+        ExpressionFormat.Infix,
+        ExpressionDetail.Digest)
       LOG.warn(
         "Currently, window join requires JOIN ON condition must contain both window starts " +
           "equality of input tables and window ends equality of input tables.\n" +
