@@ -21,7 +21,6 @@ package org.apache.flink.table.planner.plan.utils
 import org.apache.flink.table.api.{DataTypes, TableConfig, TableException, ValidationException}
 import org.apache.flink.table.planner.JBigDecimal
 import org.apache.flink.table.planner.calcite.FlinkTypeFactory
-import org.apache.flink.table.planner.expressions._
 import org.apache.flink.table.planner.functions.sql.{FlinkSqlOperatorTable, SqlWindowTableFunction}
 import org.apache.flink.table.planner.plan.`trait`.RelWindowProperties
 import org.apache.flink.table.planner.plan.logical._
@@ -29,6 +28,7 @@ import org.apache.flink.table.planner.plan.metadata.FlinkRelMetadataQuery
 import org.apache.flink.table.planner.plan.utils.AggregateUtil.inferAggAccumulatorNames
 import org.apache.flink.table.planner.plan.utils.WindowEmitStrategy.{TABLE_EXEC_EMIT_EARLY_FIRE_ENABLED, TABLE_EXEC_EMIT_LATE_FIRE_ENABLED}
 import org.apache.flink.table.planner.typeutils.RowTypeUtils
+import org.apache.flink.table.runtime.groupwindow._
 import org.apache.flink.table.runtime.types.LogicalTypeDataTypeConverter.fromDataTypeToLogicalType
 import org.apache.flink.table.types.logical.TimestampType
 import org.apache.flink.table.types.logical.utils.LogicalTypeChecks.canBeTimeAttributeType
@@ -69,6 +69,20 @@ object WindowUtil {
     }
   }
 
+  /**
+   * Excludes window_start, window_end and window_time properties from grouping keys.
+   */
+  def groupingExcludeWindowStartEndTimeColumns(
+      grouping: ImmutableBitSet,
+      windowProperties: RelWindowProperties): (
+    ImmutableBitSet, ImmutableBitSet, ImmutableBitSet, ImmutableBitSet) = {
+    val startColumns = windowProperties.getWindowStartColumns.intersect(grouping)
+    val endColumns = windowProperties.getWindowEndColumns.intersect(grouping)
+    val timeColumns = windowProperties.getWindowTimeColumns.intersect(grouping)
+    val newGrouping = grouping.except(startColumns).except(endColumns).except(timeColumns)
+    (startColumns, endColumns, timeColumns, newGrouping)
+  }
+
   /**
    * Returns true if the [[RexNode]] is a window table-valued function call.
    */
@@ -169,6 +183,17 @@ object WindowUtil {
     (program, calcFieldShifting.toArray, newTimeAttributeIndex, !containsTimeAttribute)
   }
 
+  def validateTimeFieldWithTimeAttribute(
+      windowCall: RexCall,
+      inputRowType: RelDataType): Unit = {
+    val timeIndex = getTimeAttributeIndex(windowCall.operands(1))
+    val fieldType = inputRowType.getFieldList.get(timeIndex).getType
+    if (!FlinkTypeFactory.isTimeIndicatorType(fieldType)) {
+      throw new ValidationException(
+        s"The window function requires the timecol is a time attribute type, but is $fieldType.")
+    }
+  }
+
   /**
    * Converts a [[RexCall]] into [[TimeAttributeWindowingStrategy]], the [[RexCall]] must be a
    * window table-valued function call.
@@ -183,10 +208,6 @@ object WindowUtil {
 
     val timeIndex = getTimeAttributeIndex(windowCall.operands(1))
     val fieldType = inputRowType.getFieldList.get(timeIndex).getType
-    if (!FlinkTypeFactory.isTimeIndicatorType(fieldType)) {
-      throw new ValidationException("Window can only be defined on a time attribute column, " +
-        "but is type of " + fieldType)
-    }
     val timeAttributeType = FlinkTypeFactory.toLogicalType(fieldType)
     if (!canBeTimeAttributeType(timeAttributeType)) {
       throw new ValidationException("The supported time indicator type are TIMESTAMP" +
@@ -233,9 +254,8 @@ object WindowUtil {
    * throws exception when the configurations are set.
    */
   def checkEmitConfiguration(tableConfig: TableConfig): Unit = {
-    val conf = tableConfig.getConfiguration
-    if (conf.getBoolean(TABLE_EXEC_EMIT_EARLY_FIRE_ENABLED) ||
-      conf.getBoolean(TABLE_EXEC_EMIT_LATE_FIRE_ENABLED)) {
+    if (tableConfig.get(TABLE_EXEC_EMIT_EARLY_FIRE_ENABLED) ||
+      tableConfig.get(TABLE_EXEC_EMIT_LATE_FIRE_ENABLED)) {
       throw new TableException("Currently, window table function based aggregate doesn't " +
         s"support early-fire and late-fire configuration " +
         s"'${TABLE_EXEC_EMIT_EARLY_FIRE_ENABLED.key()}' and " +
@@ -251,7 +271,7 @@ object WindowUtil {
       grouping: Array[Int],
       aggCalls: Seq[AggregateCall],
       windowing: WindowingStrategy,
-      namedWindowProperties: Seq[PlannerNamedWindowProperty],
+      namedWindowProperties: Seq[NamedWindowProperty],
       inputRowType: RelDataType,
       typeFactory: FlinkTypeFactory): RelDataType = {
     val groupSet = ImmutableBitSet.of(grouping: _*)
@@ -268,9 +288,9 @@ object WindowUtil {
       // use types from windowing strategy which keeps the precision and timestamp type
       // cast the type to not null type, because window properties should never be null
       val timeType = namedProp.getProperty match {
-        case _: PlannerWindowStart | _: PlannerWindowEnd =>
+        case _: WindowStart | _: WindowEnd =>
           new TimestampType(false, 3)
-        case _: PlannerRowtimeAttribute | _: PlannerProctimeAttribute =>
+        case _: RowtimeAttribute | _: ProctimeAttribute =>
           windowing.getTimeAttributeType.copy(false)
       }
       builder.add(namedProp.getName, typeFactory.createFieldTypeFromLogicalType(timeType))
