@@ -29,7 +29,6 @@ import org.apache.flink.table.functions.python.PythonFunctionKind;
 import org.apache.flink.types.Row;
 
 import java.sql.Timestamp;
-import java.time.LocalDateTime;
 import java.util.Random;
 import java.util.TimeZone;
 
@@ -102,7 +101,8 @@ public class JavaUserDefinedScalarFunctions {
     /** Testing open method is called. */
     public static class UdfWithOpen extends ScalarFunction {
 
-        private boolean isOpened = false;
+        // transient make this class serializable by class name
+        private transient boolean isOpened = false;
 
         @Override
         public void open(FunctionContext context) throws Exception {
@@ -293,24 +293,4 @@ public class JavaUserDefinedScalarFunctions {
             return PythonFunctionKind.PANDAS;
         }
     }
-
-    /** A Python UDF that returns current timestamp with any input. */
-    public static class PythonTimestampScalarFunction extends ScalarFunction
-            implements PythonFunction {
-
-        @DataTypeHint("TIMESTAMP(3)")
-        public LocalDateTime eval(@DataTypeHint(inputGroup = InputGroup.ANY) Object... o) {
-            return LocalDateTime.now();
-        }
-
-        @Override
-        public byte[] getSerializedPythonFunction() {
-            return new byte[0];
-        }
-
-        @Override
-        public PythonEnv getPythonEnv() {
-            return null;
-        }
-    }
 }
