@@ -18,10 +18,8 @@
 
 package org.apache.flink.table.api
 
-import org.apache.flink.api.common.RuntimeExecutionMode
 import org.apache.flink.api.common.typeinfo.Types.STRING
 import org.apache.flink.api.scala._
-import org.apache.flink.configuration.ExecutionOptions
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment
 import org.apache.flink.streaming.api.scala.{StreamExecutionEnvironment => ScalaStreamExecutionEnvironment}
 import org.apache.flink.table.api.bridge.java.StreamTableEnvironment
@@ -29,39 +27,32 @@ import org.apache.flink.table.api.bridge.scala.{StreamTableEnvironment => ScalaS
 import org.apache.flink.table.api.config.TableConfigOptions
 import org.apache.flink.table.api.internal.{TableEnvironmentImpl, TableEnvironmentInternal}
 import org.apache.flink.table.catalog._
-import org.apache.flink.table.functions.TestGenericUDF
 import org.apache.flink.table.planner.factories.utils.TestCollectionTableFactory
 import org.apache.flink.table.planner.runtime.utils.TestingAppendSink
 import org.apache.flink.table.planner.utils.TableTestUtil.{readFromResource, replaceStageId}
 import org.apache.flink.table.planner.utils.{TableTestUtil, TestTableSourceSinks, TestTableSourceWithTime}
+import org.apache.flink.test.util.AbstractTestBase
 import org.apache.flink.types.{Row, RowKind}
-import org.apache.flink.util.{CollectionUtil, FileUtils, TestLogger}
+import org.apache.flink.util.{CollectionUtil, FileUtils}
 
-import org.junit.Assert.{assertEquals, assertFalse, assertTrue, fail}
-import org.junit.rules.{ExpectedException, TemporaryFolder}
+import org.junit.Assert.{assertEquals, assertFalse, assertTrue}
+import org.junit.rules.TemporaryFolder
 import org.junit.runner.RunWith
 import org.junit.runners.Parameterized
 import org.junit.{Assert, Before, Rule, Test}
 
-import _root_.java.io.{File, FileFilter}
-import _root_.java.lang.{Long => JLong}
-import _root_.java.util
+import java.io.{File, FileFilter}
+import java.lang.{Long => JLong}
+import java.util
 
-import _root_.scala.collection.mutable
+import scala.annotation.meta.getter
+import scala.collection.mutable
 
 @RunWith(classOf[Parameterized])
-class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends TestLogger {
+class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends AbstractTestBase {
 
-  // used for accurate exception information checking.
-  val expectedException: ExpectedException = ExpectedException.none()
-
-  @Rule
-  def thrown: ExpectedException = expectedException
-
-  private val _tempFolder = new TemporaryFolder()
-
-  @Rule
-  def tempFolder: TemporaryFolder = _tempFolder
+  @(Rule @getter)
+  val tempFolder: TemporaryFolder = new TemporaryFolder()
 
   var tEnv: TableEnvironment = _
 
@@ -84,45 +75,6 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     TestTableSourceSinks.createPersonCsvTemporaryTable(tEnv, "MyTable")
   }
 
-  @Test
-  def testSetPlannerType: Unit = {
-    tEnv.getConfig.getConfiguration.set(TableConfigOptions.TABLE_PLANNER, PlannerType.OLD)
-
-    TestTableSourceSinks.createCsvTemporarySinkTable(
-      tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
-
-
-    thrown.expect(classOf[IllegalArgumentException])
-    thrown.expectMessage(
-      "Mismatch between configured planner and actual planner. " +
-        "Currently, the 'table.planner' can only be set " +
-        "when instantiating the table environment. Subsequent changes are not supported. " +
-        "Please instantiate a new TableEnvironment if necessary."
-    )
-
-    tEnv.executeSql("insert into MySink1 select first from MyTable")
-  }
-
-  @Test
-  def testSetExecutionMode(): Unit = {
-    if (isStreaming) {
-      tEnv.getConfig.getConfiguration.set(ExecutionOptions.RUNTIME_MODE, RuntimeExecutionMode.BATCH)
-    } else {
-      tEnv.getConfig.getConfiguration.set(ExecutionOptions.RUNTIME_MODE,
-        RuntimeExecutionMode.STREAMING)
-    }
-
-    thrown.expect(classOf[IllegalArgumentException])
-    thrown.expectMessage(
-      "Mismatch between configured runtime mode and actual runtime mode. " +
-        "Currently, the 'execution.runtime-mode' can only be set when instantiating the " +
-        "table environment. Subsequent changes are not supported. " +
-        "Please instantiate a new TableEnvironment if necessary."
-    )
-
-    tEnv.explainSql("select first from MyTable")
-  }
-
   @Test
   def testExecuteTwiceUsingSameTableEnv(): Unit = {
     val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
@@ -136,8 +88,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     checkEmptyFile(sink2Path)
 
     val table1 = tEnv.sqlQuery("select first from MyTable")
-    tEnv.insertInto(table1, "MySink1")
-    tEnv.execute("test1")
+    table1.executeInsert("MySink1").await()
     assertFirstValues(sink1Path)
     checkEmptyFile(sink2Path)
 
@@ -146,8 +97,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     assertFalse(new File(sink1Path).exists())
 
     val table2 = tEnv.sqlQuery("select last from MyTable")
-    tEnv.insertInto(table2, "MySink2")
-    tEnv.execute("test2")
+    table2.executeInsert("MySink2").await()
     assertFalse(new File(sink1Path).exists())
     assertLastValues(sink2Path)
   }
@@ -158,160 +108,10 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
       tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
 
     val table1 = tEnv.sqlQuery("select first from MyTable")
-    tEnv.insertInto(table1, "MySink1")
-
-    tEnv.explain(false)
-    tEnv.execute("test1")
+    table1.executeInsert("MySink1").await()
     assertFirstValues(sinkPath)
   }
 
-  @Test
-  def testExplainAndExecuteMultipleSink(): Unit = {
-    val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
-      tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
-
-    val sink2Path = TestTableSourceSinks.createCsvTemporarySinkTable(
-      tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink2")
-
-    val table1 = tEnv.sqlQuery("select first from MyTable")
-    tEnv.insertInto(table1, "MySink1")
-    val table2 = tEnv.sqlQuery("select last from MyTable")
-    tEnv.insertInto(table2, "MySink2")
-
-    tEnv.explain(false)
-    tEnv.execute("test1")
-    assertFirstValues(sink1Path)
-    assertLastValues(sink2Path)
-  }
-
-  @Test
-  def testExplainTwice(): Unit = {
-    val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
-      tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
-
-    val sink2Path = TestTableSourceSinks.createCsvTemporarySinkTable(
-      tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink2")
-
-    val table1 = tEnv.sqlQuery("select first from MyTable")
-    tEnv.insertInto(table1, "MySink1")
-    val table2 = tEnv.sqlQuery("select last from MyTable")
-    tEnv.insertInto(table2, "MySink2")
-
-    val result1 = tEnv.explain(false)
-    val result2 = tEnv.explain(false)
-    assertEquals(replaceStageId(result1), replaceStageId(result2))
-  }
-
-  @Test
-  def testSqlUpdateAndToDataStream(): Unit = {
-    if (!tableEnvName.equals("StreamTableEnvironment")) {
-      return
-    }
-    val streamEnv = StreamExecutionEnvironment.getExecutionEnvironment
-    val streamTableEnv = StreamTableEnvironment.create(streamEnv, settings)
-    TestTableSourceSinks.createPersonCsvTemporaryTable(streamTableEnv, "MyTable")
-    val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
-      streamTableEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
-    checkEmptyFile(sink1Path)
-
-    streamTableEnv.sqlUpdate("insert into MySink1 select first from MyTable")
-
-    val table = streamTableEnv.sqlQuery("select last from MyTable where id > 0")
-    val resultSet = streamTableEnv.toAppendStream(table, classOf[Row])
-    val sink = new TestingAppendSink
-    resultSet.addSink(sink)
-
-    val explain = streamTableEnv.explain(false)
-    assertEquals(
-      replaceStageId(readFromResource("/explain/testSqlUpdateAndToDataStream.out")),
-      replaceStageId(explain))
-
-    streamTableEnv.execute("test1")
-    assertFirstValues(sink1Path)
-
-    // the DataStream program is not executed
-    assertFalse(sink.isInitialized)
-
-    deleteFile(sink1Path)
-
-    streamEnv.execute("test2")
-    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
-    // the table program is not executed again
-    assertFileNotExist(sink1Path)
-  }
-
-  @Test
-  def testToDataStreamAndSqlUpdate(): Unit = {
-    if (!tableEnvName.equals("StreamTableEnvironment")) {
-      return
-    }
-    val streamEnv = StreamExecutionEnvironment.getExecutionEnvironment
-    val streamTableEnv = StreamTableEnvironment.create(streamEnv, settings)
-    TestTableSourceSinks.createPersonCsvTemporaryTable(streamTableEnv, "MyTable")
-    val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
-      streamTableEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
-    checkEmptyFile(sink1Path)
-
-    val table = streamTableEnv.sqlQuery("select last from MyTable where id > 0")
-    val resultSet = streamTableEnv.toAppendStream(table, classOf[Row])
-    val sink = new TestingAppendSink
-    resultSet.addSink(sink)
-
-    streamTableEnv.sqlUpdate("insert into MySink1 select first from MyTable")
-
-    val explain = streamTableEnv.explain(false)
-    assertEquals(
-      replaceStageId(readFromResource("/explain/testSqlUpdateAndToDataStream.out")),
-      replaceStageId(explain))
-
-    streamEnv.execute("test2")
-    // the table program is not executed
-    checkEmptyFile(sink1Path)
-    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
-
-    streamTableEnv.execute("test1")
-    assertFirstValues(sink1Path)
-    // the DataStream program is not executed again because the result in sink is not changed
-    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
-  }
-
-  @Test
-  def testFromToDataStreamAndSqlUpdate(): Unit = {
-    if (!tableEnvName.equals("StreamTableEnvironment")) {
-      return
-    }
-    val streamEnv = ScalaStreamExecutionEnvironment.getExecutionEnvironment
-    val streamTableEnv = ScalaStreamTableEnvironment.create(streamEnv, settings)
-    val t = streamEnv.fromCollection(getPersonData)
-      .toTable(streamTableEnv, 'first, 'id, 'score, 'last)
-    streamTableEnv.registerTable("MyTable", t)
-    val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
-      streamTableEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
-    checkEmptyFile(sink1Path)
-
-    val table = streamTableEnv.sqlQuery("select last from MyTable where id > 0")
-    val resultSet = streamTableEnv.toAppendStream[Row](table)
-    val sink = new TestingAppendSink
-    resultSet.addSink(sink)
-
-    streamTableEnv.sqlUpdate("insert into MySink1 select first from MyTable")
-
-    val explain = streamTableEnv.explain(false)
-    assertEquals(
-      replaceStageId(readFromResource("/explain/testFromToDataStreamAndSqlUpdate.out")),
-      replaceStageId(explain))
-
-    streamEnv.execute("test2")
-    // the table program is not executed
-    checkEmptyFile(sink1Path)
-    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
-
-    streamTableEnv.execute("test1")
-    assertFirstValues(sink1Path)
-    // the DataStream program is not executed again because the result in sink is not changed
-    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
-  }
-
   @Test
   def testExecuteSqlWithInsertInto(): Unit = {
     val sinkPath = TestTableSourceSinks.createCsvTemporarySinkTable(
@@ -329,7 +129,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
       return
     }
 
-    val sinkPath = _tempFolder.newFolder().toString
+    val sinkPath = tempFolder.newFolder().toString
     tEnv.executeSql(
       s"""
          |create table MySink (
@@ -352,7 +152,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
   }
 
   @Test
-  def testExecuteSqlAndSqlUpdate(): Unit = {
+  def testExecuteSqlAndExecuteInsert(): Unit = {
     val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
       tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
     val sink2Path = TestTableSourceSinks.createCsvTemporarySinkTable(
@@ -370,9 +170,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     new File(sink1Path).delete()
     assertFalse(new File(sink1Path).exists())
 
-    val table2 = tEnv.sqlQuery("select last from MyTable")
-    tEnv.insertInto(table2, "MySink2")
-    tEnv.execute("test2")
+    tEnv.sqlQuery("select last from MyTable").executeInsert("MySink2").await()
     assertFalse(new File(sink1Path).exists())
     assertLastValues(sink2Path)
   }
@@ -409,6 +207,78 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     assertFileNotExist(sink1Path)
   }
 
+  @Test
+  def testToDataStreamAndExecuteSql(): Unit = {
+    if (!tableEnvName.equals("StreamTableEnvironment")) {
+      return
+    }
+    val streamEnv = StreamExecutionEnvironment.getExecutionEnvironment
+    val streamTableEnv = StreamTableEnvironment.create(streamEnv, settings)
+    TestTableSourceSinks.createPersonCsvTemporaryTable(streamTableEnv, "MyTable")
+    val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
+      streamTableEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
+    checkEmptyFile(sink1Path)
+
+    val table = streamTableEnv.sqlQuery("select last from MyTable where id > 0")
+    val resultSet = streamTableEnv.toAppendStream(table, classOf[Row])
+    val sink = new TestingAppendSink
+    resultSet.addSink(sink)
+
+    val insertStmt = "insert into MySink1 select first from MyTable"
+
+    val explain = streamTableEnv.explainSql(insertStmt)
+    assertEquals(
+      replaceStageId(readFromResource("/explain/testSqlUpdateAndToDataStream.out")),
+      replaceStageId(explain))
+
+    streamEnv.execute("test2")
+    // the table program is not executed
+    checkEmptyFile(sink1Path)
+    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
+
+    streamTableEnv.executeSql(insertStmt).await()
+    assertFirstValues(sink1Path)
+    // the DataStream program is not executed again because the result in sink is not changed
+    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
+  }
+
+  @Test
+  def testFromToDataStreamAndExecuteSql(): Unit = {
+    if (!tableEnvName.equals("StreamTableEnvironment")) {
+      return
+    }
+    val streamEnv = ScalaStreamExecutionEnvironment.getExecutionEnvironment
+    val streamTableEnv = ScalaStreamTableEnvironment.create(streamEnv, settings)
+    val t = streamEnv.fromCollection(getPersonData)
+      .toTable(streamTableEnv, 'first, 'id, 'score, 'last)
+    streamTableEnv.registerTable("MyTable", t)
+    val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
+      streamTableEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
+    checkEmptyFile(sink1Path)
+
+    val table = streamTableEnv.sqlQuery("select last from MyTable where id > 0")
+    val resultSet = streamTableEnv.toAppendStream[Row](table)
+    val sink = new TestingAppendSink
+    resultSet.addSink(sink)
+
+    val insertStmt = "insert into MySink1 select first from MyTable"
+
+    val explain = streamTableEnv.explainSql(insertStmt)
+    assertEquals(
+      replaceStageId(readFromResource("/explain/testFromToDataStreamAndSqlUpdate.out")),
+      replaceStageId(explain))
+
+    streamEnv.execute("test2")
+    // the table program is not executed
+    checkEmptyFile(sink1Path)
+    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
+
+    streamTableEnv.executeSql(insertStmt).await()
+    assertFirstValues(sink1Path)
+    // the DataStream program is not executed again because the result in sink is not changed
+    assertEquals(getExpectedLastValues.sorted, sink.getAppendResults.sorted)
+  }
+
   @Test
   def testExecuteInsert(): Unit = {
     val sinkPath = TestTableSourceSinks.createCsvTemporarySinkTable(
@@ -420,13 +290,23 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     assertFirstValues(sinkPath)
   }
 
+  @Test
+  def testExecuteInsert2(): Unit = {
+    val sinkPath = TestTableSourceSinks.createCsvTemporarySinkTable(
+      tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink")
+    checkEmptyFile(sinkPath)
+    val tableResult = tEnv.executeSql("execute insert into MySink select first from MyTable")
+    checkInsertTableResult(tableResult, "default_catalog.default_database.MySink")
+    assertFirstValues(sinkPath)
+  }
+
   @Test
   def testExecuteInsertOverwrite(): Unit = {
     if(isStreaming) {
       // Streaming mode not support overwrite for FileSystemTableSink.
       return
     }
-    val sinkPath = _tempFolder.newFolder().toString
+    val sinkPath = tempFolder.newFolder().toString
     tEnv.executeSql(
       s"""
          |create table MySink (
@@ -449,8 +329,8 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
 
   @Test
   def testTableDMLSync(): Unit = {
-    tEnv.getConfig.getConfiguration.set(TableConfigOptions.TABLE_DML_SYNC, Boolean.box(true));
-    val sink1Path = _tempFolder.newFolder().toString
+    tEnv.getConfig.set(TableConfigOptions.TABLE_DML_SYNC, Boolean.box(true))
+    val sink1Path = tempFolder.newFolder().toString
     tEnv.executeSql(
       s"""
          |create table MySink1 (
@@ -464,7 +344,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
        """.stripMargin
     )
 
-    val sink2Path = _tempFolder.newFolder().toString
+    val sink2Path = tempFolder.newFolder().toString
     tEnv.executeSql(
       s"""
          |create table MySink2 (
@@ -477,7 +357,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
        """.stripMargin
     )
 
-    val sink3Path = _tempFolder.newFolder().toString
+    val sink3Path = tempFolder.newFolder().toString
     tEnv.executeSql(
       s"""
          |create table MySink3 (
@@ -540,13 +420,35 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     assertLastValues(sink2Path)
   }
 
+  @Test
+  def testExecuteStatementSet(): Unit = {
+    val sink1Path = TestTableSourceSinks.createCsvTemporarySinkTable(
+      tEnv, new TableSchema(Array("first"), Array(STRING)), "MySink1")
+
+    val sink2Path = TestTableSourceSinks.createCsvTemporarySinkTable(
+      tEnv, new TableSchema(Array("last"), Array(STRING)), "MySink2")
+
+    val tableResult = tEnv.executeSql(
+    """execute statement set begin
+        |insert into MySink1 select first from MyTable;
+        |insert into MySink2 select last from MyTable;
+        |end""".stripMargin)
+    checkInsertTableResult(
+      tableResult,
+      "default_catalog.default_database.MySink1",
+      "default_catalog.default_database.MySink2")
+
+    assertFirstValues(sink1Path)
+    assertLastValues(sink2Path)
+  }
+
   @Test
   def testStatementSetWithOverwrite(): Unit = {
     if(isStreaming) {
       // Streaming mode not support overwrite for FileSystemTableSink.
       return
     }
-    val sink1Path = _tempFolder.newFolder().toString
+    val sink1Path = tempFolder.newFolder().toString
     tEnv.executeSql(
       s"""
          |create table MySink1 (
@@ -559,7 +461,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
        """.stripMargin
     )
 
-    val sink2Path = _tempFolder.newFolder().toString
+    val sink2Path = tempFolder.newFolder().toString
     tEnv.executeSql(
       s"""
          |create table MySink2 (
@@ -605,7 +507,7 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
       // Streaming mode not support overwrite for FileSystemTableSink.
       return
     }
-    val sinkPath = _tempFolder.newFolder().toString
+    val sinkPath = tempFolder.newFolder().toString
     tEnv.executeSql(
       s"""
          |create table MySink (
@@ -632,11 +534,23 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
 
   @Test
   def testExecuteSelect(): Unit = {
-    val query =
+    val query = {
       """
         |select id, concat(concat(`first`, ' '), `last`) as `full name`
         |from MyTable where mod(id, 2) = 0
       """.stripMargin
+    }
+    testExecuteSelectInternal(query)
+    val query2 = {
+      """
+        |execute select id, concat(concat(`first`, ' '), `last`) as `full name`
+        |from MyTable where mod(id, 2) = 0
+      """.stripMargin
+    }
+    testExecuteSelectInternal(query2)
+  }
+
+  def testExecuteSelectInternal(query: String): Unit = {
     val tableResult = tEnv.executeSql(query)
     assertTrue(tableResult.getJobClient.isPresent)
     assertEquals(ResultKind.SUCCESS_WITH_CONTENT, tableResult.getResultKind)
@@ -737,77 +651,6 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     tableEnv.executeSql("insert into dest2 select x from src").await()
   }
 
-  @Test
-  def testTemporaryOperationListener(): Unit = {
-    val listener = new ListenerCatalog("listener_cat")
-    val currentCat = tEnv.getCurrentCatalog
-    tEnv.registerCatalog(listener.getName, listener)
-    // test temporary table
-    tEnv.executeSql("create temporary table tbl1 (x int)")
-    assertEquals(0, listener.numTempTable)
-    tEnv.executeSql(s"create temporary table ${listener.getName}.`default`.tbl1 (x int)")
-    assertEquals(1, listener.numTempTable)
-    val tableResult = tEnv.asInstanceOf[TableEnvironmentInternal].getCatalogManager
-      .getTable(ObjectIdentifier.of(listener.getName, "default", "tbl1"))
-    assertTrue(tableResult.isPresent)
-    assertEquals(listener.tableComment, tableResult.get().getTable.getComment)
-    tEnv.executeSql("drop temporary table tbl1")
-    assertEquals(1, listener.numTempTable)
-    tEnv.executeSql(s"drop temporary table ${listener.getName}.`default`.tbl1")
-    assertEquals(0, listener.numTempTable)
-    tEnv.useCatalog(listener.getName)
-    tEnv.executeSql("create temporary table tbl1 (x int)")
-    assertEquals(1, listener.numTempTable)
-    tEnv.executeSql("drop temporary table tbl1")
-    assertEquals(0, listener.numTempTable)
-    tEnv.useCatalog(currentCat)
-
-    // test temporary view
-    tEnv.executeSql("create temporary view v1 as select 1")
-    assertEquals(0, listener.numTempTable)
-    tEnv.executeSql(s"create temporary view ${listener.getName}.`default`.v1 as select 1")
-    assertEquals(1, listener.numTempTable)
-    val viewResult = tEnv.asInstanceOf[TableEnvironmentInternal].getCatalogManager
-      .getTable(ObjectIdentifier.of(listener.getName, "default", "v1"))
-    assertTrue(viewResult.isPresent)
-    assertEquals(listener.tableComment, viewResult.get().getTable.getComment)
-    tEnv.executeSql("drop temporary view v1")
-    assertEquals(1, listener.numTempTable)
-    tEnv.executeSql(s"drop temporary view ${listener.getName}.`default`.v1")
-    assertEquals(0, listener.numTempTable)
-    tEnv.useCatalog(listener.getName)
-    tEnv.executeSql("create temporary view v1 as select 1")
-    assertEquals(1, listener.numTempTable)
-    tEnv.executeSql("drop temporary view  v1")
-    assertEquals(0, listener.numTempTable)
-    tEnv.useCatalog(currentCat)
-
-    // test temporary function
-    val clzName = "foo.class.name"
-    try {
-      tEnv.executeSql(s"create temporary function func1 as '${clzName}'")
-      fail("Creating a temporary function with invalid class should fail")
-    } catch {
-      case _: Exception => //expected
-    }
-    assertEquals(0, listener.numTempFunc)
-    tEnv.executeSql(
-      s"create temporary function ${listener.getName}.`default`.func1 as '${clzName}'")
-    assertEquals(1, listener.numTempFunc)
-    tEnv.executeSql("drop temporary function if exists func1")
-    assertEquals(1, listener.numTempFunc)
-    tEnv.executeSql(s"drop temporary function ${listener.getName}.`default`.func1")
-    assertEquals(0, listener.numTempFunc)
-    tEnv.useCatalog(listener.getName)
-    tEnv.executeSql(s"create temporary function func1 as '${clzName}'")
-    assertEquals(1, listener.numTempFunc)
-    tEnv.executeSql("drop temporary function func1")
-    assertEquals(0, listener.numTempFunc)
-    tEnv.useCatalog(currentCat)
-
-    listener.close()
-  }
-
   def getPersonData: List[(String, Int, Double, String)] = {
     val data = new mutable.MutableList[(String, Int, Double, String)]
     data.+=(("Mike", 1, 12.3, "Smith"))
@@ -874,38 +717,6 @@ class TableEnvironmentITCase(tableEnvName: String, isStreaming: Boolean) extends
     }
   }
 
-  class ListenerCatalog(name: String)
-    extends GenericInMemoryCatalog(name) with TemporaryOperationListener {
-
-    val tableComment: String = "listener_comment"
-    val funcClzName: String = classOf[TestGenericUDF].getName
-
-    var numTempTable = 0
-    var numTempFunc = 0
-
-    override def onCreateTemporaryTable(tablePath: ObjectPath, table: CatalogBaseTable)
-    : CatalogBaseTable = {
-      numTempTable += 1
-      if (table.isInstanceOf[CatalogTable]) {
-        new CatalogTableImpl(table.getSchema, table.getOptions, tableComment)
-      } else {
-        val view = table.asInstanceOf[CatalogView]
-        new CatalogViewImpl(view.getOriginalQuery, view.getExpandedQuery,
-          view.getSchema, view.getOptions, tableComment)
-      }
-    }
-
-    override def onDropTemporaryTable(tablePath: ObjectPath): Unit = numTempTable -= 1
-
-    override def onCreateTemporaryFunction(functionPath: ObjectPath, function: CatalogFunction)
-    : CatalogFunction = {
-      numTempFunc += 1
-      new CatalogFunctionImpl(funcClzName, function.getFunctionLanguage)
-    }
-
-    override def onDropTemporaryFunction(functionPath: ObjectPath): Unit = numTempFunc -= 1
-  }
-
 }
 
 object TableEnvironmentITCase {
