@@ -18,49 +18,53 @@
 
 package org.apache.flink.table.planner.expressions.utils
 
-import java.util.Collections
-import org.apache.calcite.plan.hep.{HepPlanner, HepProgramBuilder}
-import org.apache.calcite.rel.RelNode
-import org.apache.calcite.rel.logical.LogicalCalc
-import org.apache.calcite.rel.rules._
-import org.apache.calcite.rex.RexNode
-import org.apache.calcite.sql.`type`.SqlTypeName.VARCHAR
-import org.apache.flink.api.common.{JobID, TaskInfo}
+import org.apache.flink.api.common.TaskInfo
 import org.apache.flink.api.common.functions.util.RuntimeUDFContext
 import org.apache.flink.api.common.functions.{MapFunction, RichFunction, RichMapFunction}
 import org.apache.flink.api.java.typeutils.RowTypeInfo
-import org.apache.flink.configuration.Configuration
+import org.apache.flink.configuration.{ConfigOption, Configuration}
 import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment
 import org.apache.flink.table.api
 import org.apache.flink.table.api.bridge.java.internal.StreamTableEnvironmentImpl
-import org.apache.flink.table.api.{EnvironmentSettings, TableConfig, ValidationException}
+import org.apache.flink.table.api.config.{ExecutionConfigOptions, TableConfigOptions}
+import org.apache.flink.table.api.{EnvironmentSettings, TableConfig, TableException, ValidationException}
 import org.apache.flink.table.data.RowData
 import org.apache.flink.table.data.binary.BinaryRowData
 import org.apache.flink.table.data.conversion.{DataStructureConverter, DataStructureConverters}
 import org.apache.flink.table.data.util.DataFormatConverters
 import org.apache.flink.table.data.util.DataFormatConverters.DataFormatConverter
-import org.apache.flink.table.expressions.{Expression, ExpressionParser}
+import org.apache.flink.table.delegation.ExpressionParser
+import org.apache.flink.table.expressions.Expression
 import org.apache.flink.table.functions.ScalarFunction
 import org.apache.flink.table.planner.codegen.{CodeGeneratorContext, ExprCodeGenerator, FunctionCodeGenerator}
 import org.apache.flink.table.planner.delegation.PlannerBase
+import org.apache.flink.table.planner.utils.TestingTableEnvironment
+import org.apache.flink.table.runtime.generated.GeneratedFunction
 import org.apache.flink.table.runtime.types.TypeInfoLogicalTypeConverter.fromTypeInfoToLogicalType
 import org.apache.flink.table.types.AbstractDataType
 import org.apache.flink.table.types.logical.{RowType, VarCharType}
 import org.apache.flink.table.types.utils.TypeConversions
 import org.apache.flink.types.Row
 
+import org.apache.calcite.plan.hep.{HepPlanner, HepProgramBuilder}
+import org.apache.calcite.rel.RelNode
+import org.apache.calcite.rel.logical.LogicalCalc
+import org.apache.calcite.rel.rules._
+import org.apache.calcite.rex.RexNode
+import org.apache.calcite.sql.`type`.SqlTypeName.VARCHAR
+
 import org.junit.Assert.{assertEquals, assertTrue, fail}
 import org.junit.rules.ExpectedException
 import org.junit.{After, Before, Rule}
 
+import java.time.ZoneId
+import java.util.Collections
+
 import scala.collection.JavaConverters._
 import scala.collection.mutable
-import scala.collection.mutable.ListBuffer
 
 abstract class ExpressionTestBase {
 
-  val config = new TableConfig()
-
   // (originalExpr, optimizedExpr, expectedResult)
   private val validExprs = mutable.ArrayBuffer[(String, RexNode, String)]()
   // (originalSqlExpr, keywords, exceptionClass)
@@ -70,11 +74,14 @@ abstract class ExpressionTestBase {
     .ArrayBuffer[(Expression, String, Class[_ <: Throwable])]()
 
   private val env = StreamExecutionEnvironment.createLocalEnvironment(4)
-  private val setting = EnvironmentSettings.newInstance().inStreamingMode().build()
+  private val settings = EnvironmentSettings.newInstance().inStreamingMode().build()
   // use impl class instead of interface class to avoid
   // "Static methods in interface require -target:jvm-1.8"
-  private val tEnv = StreamTableEnvironmentImpl.create(env, setting, config)
+  private val tEnv = StreamTableEnvironmentImpl.create(env, settings)
     .asInstanceOf[StreamTableEnvironmentImpl]
+
+  val config = tEnv.getConfig
+
   private val resolvedDataType = if (containsLegacyTypes) {
     TypeConversions.fromLegacyInfoToDataType(typeInfo)
   } else {
@@ -87,8 +94,7 @@ abstract class ExpressionTestBase {
 
   // setup test utils
   private val tableName = "testTable"
-  protected val nullable = "null"
-  protected val notNullable = "not null"
+  protected val nullable = "NULL"
 
   // used for accurate exception information checking.
   val expectedException: ExpectedException = ExpectedException.none()
@@ -98,6 +104,10 @@ abstract class ExpressionTestBase {
 
   @Before
   def prepare(): Unit = {
+    config.set(
+      ExecutionConfigOptions.TABLE_EXEC_LEGACY_CAST_BEHAVIOUR,
+      ExecutionConfigOptions.LegacyCastBehaviour.DISABLED
+    )
     if (containsLegacyTypes) {
       val ds = env.fromCollection(Collections.emptyList[Row](), typeInfo)
       tEnv.createTemporaryView(tableName, ds, typeInfo.getFieldNames.map(api.$): _*)
@@ -211,15 +221,17 @@ abstract class ExpressionTestBase {
   }
 
   // return the codegen function instances
-  def getCodeGenFunctions(sqlExprs: List[String]) : MapFunction[RowData, BinaryRowData] = {
+  def getCodeGenFunctions(
+        sqlExprs: List[String]): GeneratedFunction[MapFunction[RowData, BinaryRowData]] = {
     val testSqlExprs = mutable.ArrayBuffer[(String, RexNode, String)]()
     sqlExprs.foreach(exp => addSqlTestExpr(exp, null, testSqlExprs, null))
     getCodeGenFunction(testSqlExprs.map(r => r._2).toList)
   }
 
   // return the codegen function instances
-  def evaluateFunctionResult(mapper: MapFunction[RowData, BinaryRowData])
-  : List[String] = {
+  def evaluateFunctionResult(
+        generatedFunction: GeneratedFunction[MapFunction[RowData, BinaryRowData]]): List[String] = {
+    val mapper = generatedFunction.newInstance(getClass.getClassLoader)
     val isRichFunction = mapper.isInstanceOf[RichFunction]
 
     // call setRuntimeContext method and open method for RichFunction
@@ -247,28 +259,35 @@ abstract class ExpressionTestBase {
         .asInstanceOf[DataStructureConverter[RowData, Row]]
       converter.toInternalOrNull(testData)
     }
-    val result = mapper.map(testRow)
+    try {
+      val result = mapper.map(testRow)
 
-    // call close method for RichFunction
-    if (isRichFunction) {
-      mapper.asInstanceOf[RichMapFunction[_, _]].close()
-    }
-
-    val resultList = new ListBuffer[String]()
-    for (index <- 0 until result.getArity) {
-      // adapt string result
-      val item = if (!result.asInstanceOf[BinaryRowData].isNullAt(index)) {
-        result.asInstanceOf[BinaryRowData].getString(index).toString
-      } else {
-        null
+      // call close method for RichFunction
+      if (isRichFunction) {
+        mapper.asInstanceOf[RichMapFunction[_, _]].close()
       }
-      resultList += item
+
+      Seq.range(0, result.getArity).map(index =>
+        if (!result.isNullAt(index)) {
+          result.getString(index).toString
+        } else {
+          null
+        }
+      ).toList
+    } catch {
+      case te: TableException =>
+        // TableException are exception that might be expected,
+        // so we don't wrap them with more details
+        throw te
+      case e: Throwable =>
+        throw new AssertionError(
+          "Error when executing the expression. Expression code:\n" + generatedFunction.getCode, e)
     }
-    resultList.toList
   }
 
   private def testTableApiTestExpr(tableApiString: String, expected: String): Unit = {
-    addTableApiTestExpr(ExpressionParser.parseExpression(tableApiString), expected, validExprs)
+    addTableApiTestExpr(
+      ExpressionParser.INSTANCE.parseExpression(tableApiString), expected, validExprs)
   }
 
   private def addSqlTestExpr(
@@ -340,11 +359,12 @@ abstract class ExpressionTestBase {
           assertEquals(
             s"Wrong result $original optimized to: [$optimizedExpr]",
             expected,
-            if (actual == null) "null" else actual)
+            if (actual == null) "NULL" else actual)
       }
   }
 
-  private def getCodeGenFunction(rexNodes: List[RexNode]): MapFunction[RowData, BinaryRowData] = {
+  private def getCodeGenFunction(rexNodes: List[RexNode]):
+    GeneratedFunction[MapFunction[RowData, BinaryRowData]] = {
     val ctx = CodeGeneratorContext(config)
     val inputType = if (containsLegacyTypes) {
       fromTypeInfoToLogicalType(typeInfo)
@@ -358,7 +378,7 @@ abstract class ExpressionTestBase {
 
     // generate code
     val resultType = RowType.of(Seq.fill(rexNodes.size)(
-      new VarCharType(VarCharType.MAX_LENGTH)): _*)
+      VarCharType.STRING_TYPE): _*)
 
     val exprs = stringTestExprs.map(exprGenerator.generateExpression)
     val genExpr = exprGenerator.generateResultExpression(exprs, resultType, classOf[BinaryRowData])
@@ -369,14 +389,13 @@ abstract class ExpressionTestBase {
          |return ${genExpr.resultTerm};
         """.stripMargin
 
-    val genFunc = FunctionCodeGenerator.generateFunction[MapFunction[RowData, BinaryRowData]](
+    FunctionCodeGenerator.generateFunction[MapFunction[RowData, BinaryRowData]](
       ctx,
       "TestFunction",
       classOf[MapFunction[RowData, BinaryRowData]],
       bodyCode,
       resultType,
       inputType)
-    genFunc.newInstance(getClass.getClassLoader)
   }
 
   def testData: Row
