@@ -647,10 +647,10 @@ class FlinkRelMdColumnUniquenessTest extends FlinkRelMdHandlerTestBase {
       tablePartiallyProjectedKeyStreamScan
     )
       .foreach { scan =>
-        assertNull(mq.areColumnsUnique(scan, ImmutableBitSet.of(0)))
-        assertNull(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1)))
-        assertNull(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1, 2)))
-        assertNull(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1, 2, 3)))
+        assertFalse(mq.areColumnsUnique(scan, ImmutableBitSet.of(0)))
+        assertFalse(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1)))
+        assertFalse(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1, 2)))
+        assertFalse(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1, 2, 3)))
       }
   }
 
@@ -663,10 +663,10 @@ class FlinkRelMdColumnUniquenessTest extends FlinkRelMdHandlerTestBase {
       tableSourceTableNonKeyStreamScan
     )
       .foreach { scan =>
-        assertNull(mq.areColumnsUnique(scan, ImmutableBitSet.of(0)))
-        assertNull(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1)))
-        assertNull(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1, 2)))
-        assertNull(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1, 2, 3)))
+        assertFalse(mq.areColumnsUnique(scan, ImmutableBitSet.of(0)))
+        assertFalse(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1)))
+        assertFalse(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1, 2)))
+        assertFalse(mq.areColumnsUnique(scan, ImmutableBitSet.of(0, 1, 2, 3)))
       }
   }
 }
