@@ -19,16 +19,18 @@
 package org.apache.flink.table.planner.plan.metadata
 
 import org.apache.flink.api.common.typeinfo.{BasicTypeInfo, SqlTimeTypeInfo}
-import org.apache.flink.table.api.{DataTypes, TableException, TableSchema}
-import org.apache.flink.table.catalog.{CatalogTable, Column, ObjectIdentifier, ResolvedCatalogTable, ResolvedSchema, UniqueConstraint}
+import org.apache.flink.table.api.{DataTypes, TableConfig, TableException, TableSchema}
+import org.apache.flink.table.catalog.{CatalogTable, Column, ContextResolvedTable, ObjectIdentifier, ResolvedCatalogTable, ResolvedSchema, UniqueConstraint}
 import org.apache.flink.table.connector.ChangelogMode
 import org.apache.flink.table.connector.source.{DynamicTableSource, ScanTableSource}
+import org.apache.flink.table.module.ModuleManager
 import org.apache.flink.table.plan.stats.{ColumnStats, TableStats}
-import org.apache.flink.table.planner.calcite.{FlinkTypeFactory, FlinkTypeSystem}
+import org.apache.flink.table.planner.calcite.{FlinkContext, FlinkContextImpl, FlinkTypeFactory, FlinkTypeSystem}
 import org.apache.flink.table.planner.plan.schema.{FlinkPreparingTableBase, TableSourceTable}
 import org.apache.flink.table.planner.plan.stats.FlinkStatistic
 import org.apache.flink.table.runtime.types.TypeInfoLogicalTypeConverter.fromTypeInfoToLogicalType
 import org.apache.flink.table.types.logical.{BigIntType, DoubleType, IntType, LocalZonedTimestampType, LogicalType, TimestampKind, TimestampType, VarCharType}
+import org.apache.flink.table.utils.CatalogManagerMocks
 
 import org.apache.calcite.config.CalciteConnectionConfig
 import org.apache.calcite.jdbc.CalciteSchema
@@ -196,7 +198,7 @@ object MetadataTestUtil {
     val fieldNames = Array("a", "b", "c", "proctime", "rowtime")
     val fieldTypes = Array[LogicalType](
       new BigIntType(),
-      new VarCharType(VarCharType.MAX_LENGTH),
+      VarCharType.STRING_TYPE,
       new IntType(),
       new LocalZonedTimestampType(true, TimestampKind.PROCTIME, 3),
       new TimestampType(true, TimestampKind.ROWTIME, 3))
@@ -215,7 +217,7 @@ object MetadataTestUtil {
     val fieldNames = Array("a", "b", "c", "proctime", "rowtime")
     val fieldTypes = Array[LogicalType](
       new BigIntType(),
-      new VarCharType(VarCharType.MAX_LENGTH),
+      VarCharType.STRING_TYPE,
       new IntType(),
       new LocalZonedTimestampType(true, TimestampKind.PROCTIME, 3),
       new TimestampType(true, TimestampKind.ROWTIME, 3))
@@ -236,7 +238,7 @@ object MetadataTestUtil {
     val fieldTypes = Array[LogicalType](
       new IntType(),
       new BigIntType(),
-      new VarCharType(VarCharType.MAX_LENGTH),
+      VarCharType.STRING_TYPE,
       new LocalZonedTimestampType(true, TimestampKind.PROCTIME, 3),
       new TimestampType(true, TimestampKind.ROWTIME, 3))
 
@@ -250,6 +252,14 @@ object MetadataTestUtil {
     getMetadataTable(fieldNames, fieldTypes, new FlinkStatistic(tableStats))
   }
 
+  private val flinkContext = new FlinkContextImpl(
+    false,
+    TableConfig.getDefault,
+    new ModuleManager,
+    null,
+    CatalogManagerMocks.createEmptyCatalogManager,
+    null)
+
   private def createProjectedTableSourceTable(): Table = {
     val resolvedSchema = new ResolvedSchema(
       util.Arrays.asList(
@@ -268,12 +278,14 @@ object MetadataTestUtil {
       Seq(new BigIntType(false), new DoubleType(), new VarCharType(false, 100)))
 
     new MockTableSourceTable(
-      ObjectIdentifier.of("default_catalog", "default_database", "projected_table_source_table"),
       rowType,
       new TestTableSource(),
       true,
-      new ResolvedCatalogTable(catalogTable, resolvedSchema),
-      Array("project=[a, c, d]"))
+      ContextResolvedTable.temporary(
+        ObjectIdentifier.of(
+          "default_catalog", "default_database", "projected_table_source_table"),
+        new ResolvedCatalogTable(catalogTable, resolvedSchema)),
+      flinkContext)
   }
 
   private def createTableSourceTable1(): Table = {
@@ -308,12 +320,14 @@ object MetadataTestUtil {
       Seq(new BigIntType(false), new IntType(), new VarCharType(false, 100), new BigIntType(false)))
 
     new MockTableSourceTable(
-      ObjectIdentifier.of("default_catalog", "default_database", "TableSourceTable1"),
       rowType,
       new TestTableSource(),
       true,
-      new ResolvedCatalogTable(catalogTable, resolvedSchema),
-      Array("project=[a, b, c, d]"))
+      ContextResolvedTable.temporary(
+        ObjectIdentifier.of("default_catalog", "default_database", "TableSourceTable1"),
+        new ResolvedCatalogTable(catalogTable, resolvedSchema)
+      ),
+      flinkContext)
   }
 
   private def createTableSourceTable2(): Table = {
@@ -334,12 +348,14 @@ object MetadataTestUtil {
       Seq(new BigIntType(false), new IntType(), new VarCharType(false, 100), new BigIntType(false)))
 
     new MockTableSourceTable(
-      ObjectIdentifier.of("default_catalog", "default_database", "TableSourceTable2"),
       rowType,
       new TestTableSource(),
       true,
-      new ResolvedCatalogTable(catalogTable, resolvedSchema),
-      Array("project=[a, b, c, d]"))
+      ContextResolvedTable.temporary(
+        ObjectIdentifier.of("default_catalog", "default_database", "TableSourceTable2"),
+        new ResolvedCatalogTable(catalogTable, resolvedSchema)
+      ),
+      flinkContext)
   }
 
   private def createTableSourceTable3(): Table = {
@@ -360,12 +376,14 @@ object MetadataTestUtil {
       Seq(new BigIntType(false), new IntType(), new VarCharType(false, 100), new BigIntType(false)))
 
     new MockTableSourceTable(
-      ObjectIdentifier.of("default_catalog", "default_database", "TableSourceTable3"),
       rowType,
       new TestTableSource(),
       true,
-      new ResolvedCatalogTable(catalogTable, resolvedSchema),
-      Array("project=[a, b, c, d]"))
+      ContextResolvedTable.temporary(
+        ObjectIdentifier.of("default_catalog", "default_database", "TableSourceTable3"),
+        new ResolvedCatalogTable(catalogTable, resolvedSchema)
+      ),
+      flinkContext)
   }
 
   private def createProjectedTableSourceTableWithPartialCompositePrimaryKey(): Table = {
@@ -384,15 +402,17 @@ object MetadataTestUtil {
       Seq(new BigIntType(false)))
 
     new MockTableSourceTable(
-      ObjectIdentifier.of(
-        "default_catalog",
-        "default_database",
-        "projected_table_source_table_with_partial_pk"),
       rowType,
       new TestTableSource(),
       true,
-      new ResolvedCatalogTable(catalogTable, resolvedSchema),
-      Array("project=[a]"))
+      ContextResolvedTable.temporary(
+        ObjectIdentifier.of(
+          "default_catalog",
+          "default_database",
+          "projected_table_source_table_with_partial_pk"),
+        new ResolvedCatalogTable(catalogTable, resolvedSchema)
+      ),
+      flinkContext)
   }
 
   private def getCatalogTable(resolvedSchema: ResolvedSchema) = {
@@ -457,21 +477,19 @@ class TestTableSource extends ScanTableSource {
 }
 
 class MockTableSourceTable(
-    tableIdentifier: ObjectIdentifier,
     rowType: RelDataType,
     tableSource: DynamicTableSource,
     isStreamingMode: Boolean,
-    catalogTable: ResolvedCatalogTable,
-    extraDigests: Array[String] = Array.empty)
+    contextResolvedTable: ContextResolvedTable,
+    flinkContext: FlinkContext)
   extends TableSourceTable(
     null,
-    tableIdentifier,
     rowType,
     FlinkStatistic.UNKNOWN,
     tableSource,
     isStreamingMode,
-    catalogTable,
-    extraDigests)
+    contextResolvedTable,
+    flinkContext)
   with Table {
   override def getRowType(typeFactory: RelDataTypeFactory): RelDataType = rowType
 
