@@ -79,6 +79,7 @@ class SelectivityEstimatorTest {
   private def mockScan(
       statistic: FlinkStatistic = FlinkStatistic.UNKNOWN,
       tableConfig: TableConfig = TableConfig.getDefault): TableScan = {
+    val moduleManager = new ModuleManager
     val catalogManager = CatalogManagerMocks.createEmptyCatalogManager()
     val rootSchema = CalciteSchema.createRootSchema(true, false).plus()
     val table = new MockMetaTable(relDataType, statistic)
@@ -87,7 +88,8 @@ class SelectivityEstimatorTest {
       new PlannerContext(
         false,
         tableConfig,
-        new FunctionCatalog(tableConfig, catalogManager, new ModuleManager),
+        moduleManager,
+        new FunctionCatalog(tableConfig, catalogManager, moduleManager),
         catalogManager,
         CalciteSchema.from(rootSchema),
         util.Arrays.asList(
