@@ -840,8 +840,9 @@ class RankTest extends TableTestBase {
 
   @Test
   def testUpdatableRankAfterIntermediateScan(): Unit = {
-    util.tableEnv.getConfig.getConfiguration.setBoolean(
-      RelNodeBlockPlanBuilder.TABLE_OPTIMIZER_REUSE_OPTIMIZE_BLOCK_WITH_DIGEST_ENABLED, true)
+    util.tableEnv.getConfig.set(
+      RelNodeBlockPlanBuilder.TABLE_OPTIMIZER_REUSE_OPTIMIZE_BLOCK_WITH_DIGEST_ENABLED,
+      Boolean.box(true))
     util.tableEnv.executeSql(
       """
         |CREATE VIEW v1 AS
