@@ -19,11 +19,15 @@
 package org.apache.flink.table.planner.plan.stream.sql
 
 import org.apache.flink.api.scala._
+import org.apache.flink.core.testutils.FlinkAssertions
+import org.apache.flink.core.testutils.FlinkAssertions.anyCauseMatches
 import org.apache.flink.table.api._
 import org.apache.flink.table.api.config.ExecutionConfigOptions
 import org.apache.flink.table.planner.expressions.utils.Func0
 import org.apache.flink.table.planner.factories.TestValuesTableFactory.MockedLookupTableSource
 import org.apache.flink.table.planner.utils.TableTestBase
+
+import org.assertj.core.api.Assertions.assertThatThrownBy
 import org.junit.Test
 
 class TableScanTest extends TableTestBase {
@@ -99,6 +103,25 @@ class TableScanTest extends TableTestBase {
     util.verifyExecPlan("SELECT * FROM MetadataTable")
   }
 
+  @Test
+  def testDDLWithMetadataThatConflictsWithPhysicalColumn(): Unit = {
+    util.addTable(
+      s"""
+         |CREATE TABLE MetadataTable (
+         |  `timestamp` TIMESTAMP(9),
+         |  `metadata_timestamp` TIMESTAMP(0) METADATA FROM 'timestamp',
+         |  `other` STRING METADATA,
+         |  `computed_other` AS UPPER(`other`),
+         |  `computed_timestamp` AS CAST(`metadata_timestamp` AS STRING)
+         |) WITH (
+         |  'connector' = 'values',
+         |  'bounded' = 'false',
+         |  'readable-metadata' = 'timestamp:TIMESTAMP(0), other:STRING'
+         |)
+       """.stripMargin)
+    util.verifyExecPlan("SELECT * FROM MetadataTable")
+  }
+
   @Test
   def testDDLWithMetadataColumnProjectionPushDown(): Unit = {
     // tests reordering, skipping, and casting of metadata
@@ -158,6 +181,54 @@ class TableScanTest extends TableTestBase {
     util.verifyExecPlan("SELECT * FROM src WHERE a > 1")
   }
 
+  @Test
+  def testDDLWithMultipleColumnsFromSameMetadataKey(): Unit = {
+    assertThatThrownBy(() =>
+      util.tableEnv.executeSql(
+        """
+          |CREATE TABLE source (
+          |  a INT METADATA,
+          |  b INT METADATA FROM 'a'
+          |) WITH (
+          |  'connector' = 'COLLECTION'
+          |)
+          |""".stripMargin)).satisfies(
+      FlinkAssertions.anyCauseMatches(
+        classOf[ValidationException],
+        "The column `a` and `b` in the table are both from the same metadata key 'a'. " +
+          "Please specify one of the columns as the metadata column and use the computed column" +
+          " syntax to specify the others."))
+  }
+
+  @Test
+  def testDDLWithMultipleColumnsFromSameMetadataKey2(): Unit = {
+    util.tableEnv.executeSql(
+      """
+        |CREATE TABLE source (
+        |  a INT METADATA
+        |) WITH (
+        |  'connector' = 'COLLECTION'
+        |)
+        |""".stripMargin)
+    assertThatThrownBy(() =>
+      util.tableEnv.executeSql(
+        """
+          |CREATE TABLE like_source (
+          |  b INT METADATA FROM 'a'
+          |)
+          |WITH (
+          |  'connector' = 'COLLECTION'
+          |) LIKE source (
+          |  INCLUDING METADATA
+          |)
+          |""".stripMargin
+      )).satisfies(anyCauseMatches(
+      "The column `a` and `b` in the table are both from the same metadata key 'a'. " +
+        "Please specify one of the columns as the metadata column and use the computed column" +
+        " syntax to specify the others."
+    ))
+  }
+
   @Test
   def testKeywordsWithWatermarkComputedColumn(): Unit = {
     // Create table with field as atom expression.
@@ -288,8 +359,8 @@ class TableScanTest extends TableTestBase {
 
   @Test
   def testJoinOnChangelogSourceWithEventsDuplicate(): Unit = {
-    util.tableEnv.getConfig.getConfiguration.setBoolean(
-      ExecutionConfigOptions.TABLE_EXEC_SOURCE_CDC_EVENTS_DUPLICATE, true)
+    util.tableEnv.getConfig.set(
+      ExecutionConfigOptions.TABLE_EXEC_SOURCE_CDC_EVENTS_DUPLICATE, Boolean.box(true))
     verifyJoinOnSource("I,UB,UA")
   }
 
@@ -356,8 +427,8 @@ class TableScanTest extends TableTestBase {
 
   @Test
   def testChangelogSourceWithEventsDuplicate(): Unit = {
-    util.tableEnv.getConfig.getConfiguration.setBoolean(
-      ExecutionConfigOptions.TABLE_EXEC_SOURCE_CDC_EVENTS_DUPLICATE, true)
+    util.tableEnv.getConfig.set(
+      ExecutionConfigOptions.TABLE_EXEC_SOURCE_CDC_EVENTS_DUPLICATE, Boolean.box(true))
     util.addTable(
       """
         |CREATE TABLE src (
@@ -668,8 +739,8 @@ class TableScanTest extends TableTestBase {
         |  'changelog-mode' = 'I,UB,UA,D'
         |)
       """.stripMargin)
-    util.tableEnv.getConfig.getConfiguration.setBoolean(
-      ExecutionConfigOptions.TABLE_EXEC_SOURCE_CDC_EVENTS_DUPLICATE, true)
+    util.tableEnv.getConfig.set(
+      ExecutionConfigOptions.TABLE_EXEC_SOURCE_CDC_EVENTS_DUPLICATE, Boolean.box(true))
 
     thrown.expect(classOf[TableException])
     thrown.expectMessage("Configuration 'table.exec.source.cdc-events-duplicate' is enabled " +
