@@ -36,8 +36,6 @@ class CorrelateTest extends TableTestBase {
     val table = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
 
     val function = new TableFunc1
-    util.addFunction("func1", function)
-
     val result1 = table.joinLateral(function('c) as 's).select('c, 's)
     util.verifyExecPlan(result1)
   }
@@ -48,7 +46,6 @@ class CorrelateTest extends TableTestBase {
     val table = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
 
     val function = new TableFunc1
-    util.addFunction("func1", function)
     // test overloading
     val result2 = table.joinLateral(function('c, "$") as 's).select('c, 's)
     util.verifyExecPlan(result2)
@@ -59,8 +56,6 @@ class CorrelateTest extends TableTestBase {
     val util = streamTestUtil()
     val table = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
     val function = new TableFunc1
-    util.addFunction("func1", function)
-
     val result = table.leftOuterJoinLateral(function('c) as 's, true).select('c, 's)
     util.verifyExecPlan(result)
   }
@@ -70,7 +65,6 @@ class CorrelateTest extends TableTestBase {
     val util = streamTestUtil()
     val table = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
     val function = new TableFunc2
-    util.addFunction("func2", function)
     val scalarFunc = new Func13("pre")
 
     val result = table.joinLateral(
@@ -84,8 +78,6 @@ class CorrelateTest extends TableTestBase {
     val util = streamTestUtil()
     val table = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
     val function = new HierarchyTableFunction
-    util.addFunction("hierarchy", function)
-
     val result = table.joinLateral(function('c) as ('name, 'adult, 'len))
     util.verifyExecPlan(result)
   }
@@ -95,8 +87,6 @@ class CorrelateTest extends TableTestBase {
     val util = streamTestUtil()
     val table = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
     val function = new PojoTableFunc
-    util.addFunction("pojo", function)
-
     val result = table.joinLateral(function('c))
     util.verifyExecPlan(result)
   }
@@ -106,8 +96,6 @@ class CorrelateTest extends TableTestBase {
     val util = streamTestUtil()
     val table = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
     val function = new TableFunc2
-    util.addFunction("func2", function)
-
     val result = table
       .joinLateral(function('c) as ('name, 'len))
       .select('c, 'name, 'len)
@@ -120,8 +108,6 @@ class CorrelateTest extends TableTestBase {
     val util = streamTestUtil()
     val table = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
     val function = new TableFunc1
-    util.addFunction("func1", function)
-
     val result = table.joinLateral(function('c.substring(2)) as 's)
     util.verifyExecPlan(result)
   }
@@ -131,8 +117,6 @@ class CorrelateTest extends TableTestBase {
     val util = streamTestUtil()
     val sourceTable = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
     val function = new TableFunc0
-    util.addFunction("func1", function)
-
     val result = sourceTable.select('a, 'b, 'c)
       .joinLateral(function('c) as('d, 'e))
       .select('c, 'd, 'e)
@@ -158,7 +142,6 @@ class CorrelateTest extends TableTestBase {
 
     val sourceTable = util.addTableSource[(Int, Long, String)]("MyTable", 'a, 'b, 'c)
     val function = new TableFunc0
-    util.addFunction("func1", function)
     val result = sourceTable.select('a, 'b, 'c)
       .joinLateral(function('c) as('d, 'e))
       .select('c, 'd, 'e)
