@@ -22,7 +22,7 @@ import org.apache.flink.api.common.typeinfo.Types
 import org.apache.flink.table.api.{DataTypes, TableConfig}
 import org.apache.flink.table.catalog.{CatalogManager, FunctionCatalog}
 import org.apache.flink.table.expressions.ApiExpressionUtils.{unresolvedCall, unresolvedRef, valueLiteral}
-import org.apache.flink.table.expressions.{Expression, ExpressionParser}
+import org.apache.flink.table.expressions.Expression
 import org.apache.flink.table.functions.BuiltInFunctionDefinitions.{EQUALS, GREATER_THAN, LESS_THAN, LESS_THAN_OR_EQUAL}
 import org.apache.flink.table.functions.{AggregateFunctionDefinition, FunctionIdentifier}
 import org.apache.flink.table.module.ModuleManager
@@ -32,13 +32,13 @@ import org.apache.flink.table.planner.functions.sql.FlinkSqlOperatorTable
 import org.apache.flink.table.planner.functions.utils.ScalarSqlFunction
 import org.apache.flink.table.planner.utils.{DateTimeTestUtil, IntSumAggFunction}
 import org.apache.flink.table.utils.CatalogManagerMocks
-
 import org.apache.calcite.rel.`type`.RelDataType
 import org.apache.calcite.rex.{RexBuilder, RexNode}
 import org.apache.calcite.sql.SqlPostfixOperator
 import org.apache.calcite.sql.`type`.SqlTypeName
 import org.apache.calcite.sql.fun.{SqlStdOperatorTable, SqlTrimFunction}
 import org.apache.calcite.util.{DateString, TimeString, TimestampString}
+import org.apache.flink.table.delegation.ExpressionParser
 import org.hamcrest.CoreMatchers.is
 import org.junit.Assert.{assertArrayEquals, assertEquals, assertThat, assertTrue}
 import org.junit.Test
@@ -46,7 +46,6 @@ import org.junit.Test
 import java.math.BigDecimal
 import java.time.ZoneId
 import java.util.{Arrays, TimeZone, List => JList}
-
 import scala.collection.JavaConverters._
 
 /**
@@ -112,8 +111,8 @@ class RexNodeExtractorTest extends RexNodeTestBase {
     val builder: RexBuilder = new FlinkRexBuilder(typeFactory)
     val expr = buildConditionExpr()
 
-    val firstExp = ExpressionParser.parseExpression("id > 6")
-    val secondExp = ExpressionParser.parseExpression("amount * price < 100")
+    val firstExp = ExpressionParser.INSTANCE.parseExpression("id > 6")
+    val secondExp = ExpressionParser.INSTANCE.parseExpression("amount * price < 100")
     val expected: Array[Expression] = Array(firstExp, secondExp)
 
     val (convertedExpressions, unconvertedRexNodes) =
@@ -147,7 +146,8 @@ class RexNodeExtractorTest extends RexNodeTestBase {
         relBuilder,
         functionCatalog)
 
-    val expected: Array[Expression] = Array(ExpressionParser.parseExpression("amount >= id"))
+    val expected: Array[Expression] = Array(
+      ExpressionParser.INSTANCE.parseExpression("amount >= id"))
     assertExpressionArrayEquals(expected, convertedExpressions)
     assertEquals(0, unconvertedRexNodes.length)
   }
@@ -197,9 +197,9 @@ class RexNodeExtractorTest extends RexNodeTestBase {
         functionCatalog)
 
     val expected: Array[Expression] = Array(
-      ExpressionParser.parseExpression("amount < 100 || price == 100 || price === 200"),
-      ExpressionParser.parseExpression("id > 100 || price == 100 || price === 200"),
-      ExpressionParser.parseExpression("!(amount <= id)"))
+      ExpressionParser.INSTANCE.parseExpression("amount < 100 || price == 100 || price === 200"),
+      ExpressionParser.INSTANCE.parseExpression("id > 100 || price == 100 || price === 200"),
+      ExpressionParser.INSTANCE.parseExpression("!(amount <= id)"))
     assertExpressionArrayEquals(expected, convertedExpressions)
     assertEquals(0, unconvertedRexNodes.length)
   }
@@ -237,10 +237,10 @@ class RexNodeExtractorTest extends RexNodeTestBase {
         functionCatalog)
 
     val expected: Array[Expression] = Array(
-      ExpressionParser.parseExpression("amount < 100"),
-      ExpressionParser.parseExpression("amount <= id"),
-      ExpressionParser.parseExpression("id > 100"),
-      ExpressionParser.parseExpression("price === 100")
+      ExpressionParser.INSTANCE.parseExpression("amount < 100"),
+      ExpressionParser.INSTANCE.parseExpression("amount <= id"),
+      ExpressionParser.INSTANCE.parseExpression("id > 100"),
+      ExpressionParser.INSTANCE.parseExpression("price === 100")
     )
 
     assertExpressionArrayEquals(expected, convertedExpressions)
@@ -393,16 +393,16 @@ class RexNodeExtractorTest extends RexNodeTestBase {
         functionCatalog)
 
     val expected: Array[Expression] = Array(
-      ExpressionParser.parseExpression("amount < id"),
-      ExpressionParser.parseExpression("amount <= id"),
-      ExpressionParser.parseExpression("amount <> id"),
-      ExpressionParser.parseExpression("amount == id"),
-      ExpressionParser.parseExpression("amount >= id"),
-      ExpressionParser.parseExpression("amount > id"),
-      ExpressionParser.parseExpression("amount + id == 100"),
-      ExpressionParser.parseExpression("amount - id == 100"),
-      ExpressionParser.parseExpression("amount * id == 100"),
-      ExpressionParser.parseExpression("amount / id == 100")
+      ExpressionParser.INSTANCE.parseExpression("amount < id"),
+      ExpressionParser.INSTANCE.parseExpression("amount <= id"),
+      ExpressionParser.INSTANCE.parseExpression("amount <> id"),
+      ExpressionParser.INSTANCE.parseExpression("amount == id"),
+      ExpressionParser.INSTANCE.parseExpression("amount >= id"),
+      ExpressionParser.INSTANCE.parseExpression("amount > id"),
+      ExpressionParser.INSTANCE.parseExpression("amount + id == 100"),
+      ExpressionParser.INSTANCE.parseExpression("amount - id == 100"),
+      ExpressionParser.INSTANCE.parseExpression("amount * id == 100"),
+      ExpressionParser.INSTANCE.parseExpression("amount / id == 100")
     )
     assertExpressionArrayEquals(expected, convertedExpressions)
     assertEquals(0, unconvertedRexNodes.length)
@@ -508,7 +508,7 @@ class RexNodeExtractorTest extends RexNodeTestBase {
     assertEquals(0, unconvertedRexNodes.length)
 
     assertExpressionArrayEquals(
-      Array(ExpressionParser.parseExpression("amount <= id")),
+      Array(ExpressionParser.INSTANCE.parseExpression("amount <= id")),
       Array(convertedExpressions(1)))
   }
 
