@@ -51,29 +51,6 @@ class CalcITCase extends StreamingTestBase {
   @Rule
   def usesLegacyRows: LegacyRowResource = LegacyRowResource.INSTANCE
 
-  @Test
-  def testCastNumericToBooleanInProjection(): Unit ={
-    val sqlQuery =
-      "SELECT CAST(1 AS BOOLEAN), CAST(0 AS BOOLEAN), CAST(1.1 AS BOOLEAN), CAST(0.00 AS BOOLEAN)"
-
-    val outputType = InternalTypeInfo.ofFields(
-      new BooleanType(),
-      new BooleanType(),
-      new BooleanType(),
-      new BooleanType()
-    )
-
-    val result = tEnv.sqlQuery(sqlQuery).toAppendStream[RowData]
-    val sink = new TestingAppendRowDataSink(outputType)
-    result.addSink(sink)
-    env.execute()
-
-    val expected = List(
-      "+I(true,false,true,false)"
-    )
-    assertEquals(expected.sorted, sink.getAppendResults.sorted)
-  }
-
   @Test
   def testCastNumericToBooleanInCondition(): Unit ={
     val sqlQuery =
@@ -81,10 +58,6 @@ class CalcITCase extends StreamingTestBase {
          | SELECT * FROM MyTableRow WHERE b = CAST(1 AS BOOLEAN)
          | UNION ALL
          | SELECT * FROM MyTableRow WHERE b = CAST(0 AS BOOLEAN)
-         | UNION ALL
-         | SELECT * FROM MyTableRow WHERE b = CAST(1.1 AS BOOLEAN)
-         | UNION ALL
-         | SELECT * FROM MyTableRow WHERE b = CAST(0.0 AS BOOLEAN)
          |""".stripMargin
 
     val rowData1: GenericRowData = new GenericRowData(2)
@@ -117,8 +90,6 @@ class CalcITCase extends StreamingTestBase {
     env.execute()
 
     val expected = List(
-      "+I(1,true)",
-      "+I(2,false)",
       "+I(1,true)",
       "+I(2,false)"
     )
@@ -181,8 +152,8 @@ class CalcITCase extends StreamingTestBase {
     tEnv.registerTable("MyTableRow", t)
 
     val outputType = InternalTypeInfo.ofFields(
-      new VarCharType(VarCharType.MAX_LENGTH),
-      new VarCharType(VarCharType.MAX_LENGTH),
+      VarCharType.STRING_TYPE,
+      VarCharType.STRING_TYPE,
       new IntType())
 
     val result = tEnv.sqlQuery(sqlQuery).toAppendStream[RowData]
@@ -522,7 +493,7 @@ class CalcITCase extends StreamingTestBase {
     val tableId = TestValuesTableFactory.registerData(rows)
 
     // We need a fixed timezone to make sure this test can run on machines across the world
-    tEnv.getConfig.getConfiguration.setString("table.local-time-zone", "Europe/Berlin")
+    tEnv.getConfig.set("table.local-time-zone", "Europe/Berlin")
 
     tEnv.executeSql(s"""
                        |CREATE TABLE T (
