@@ -29,6 +29,7 @@ import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.MemorySize;
 import org.apache.flink.configuration.StateBackendOptions;
 import org.apache.flink.configuration.TaskManagerOptions;
+import org.apache.flink.core.execution.SavepointFormatType;
 import org.apache.flink.core.fs.Path;
 import org.apache.flink.runtime.client.JobCancellationException;
 import org.apache.flink.runtime.client.JobStatusMessage;
@@ -63,7 +64,7 @@ import scala.concurrent.duration.FiniteDuration;
 
 import static org.apache.flink.changelog.fs.FsStateChangelogOptions.BASE_PATH;
 import static org.apache.flink.changelog.fs.FsStateChangelogStorageFactory.IDENTIFIER;
-import static org.apache.flink.configuration.CheckpointingOptions.STATE_CHANGE_LOG_STORAGE;
+import static org.apache.flink.configuration.StateChangelogOptions.STATE_CHANGE_LOG_STORAGE;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
@@ -414,7 +415,7 @@ public class ClassLoaderITCase extends TestLogger {
             try {
                 savepointPath =
                         clusterClient
-                                .triggerSavepoint(jobId, null)
+                                .triggerSavepoint(jobId, null, SavepointFormatType.CANONICAL)
                                 .get(deadline.timeLeft().toMillis(), TimeUnit.MILLISECONDS);
             } catch (Exception cause) {
                 LOG.info("Failed to trigger savepoint. Retrying...", cause);
