@@ -18,8 +18,9 @@
 
 package org.apache.flink.api.scala.migration
 
-import java.util
+import org.apache.flink.FlinkVersion
 
+import java.util
 import org.apache.flink.api.common.accumulators.IntCounter
 import org.apache.flink.api.common.functions.RichFlatMapFunction
 import org.apache.flink.api.common.state._
@@ -40,67 +41,112 @@ import org.apache.flink.streaming.api.functions.co.KeyedBroadcastProcessFunction
 import org.apache.flink.streaming.api.functions.sink.RichSinkFunction
 import org.apache.flink.streaming.api.functions.source.SourceFunction
 import org.apache.flink.streaming.api.watermark.Watermark
-import org.apache.flink.test.checkpointing.utils.SavepointMigrationTestBase
-import org.apache.flink.testutils.migration.MigrationVersion
+import org.apache.flink.test.checkpointing.utils.SnapshotMigrationTestBase
+import org.apache.flink.test.checkpointing.utils.SnapshotMigrationTestBase.{ExecutionMode, SnapshotSpec, SnapshotType}
 import org.apache.flink.util.Collector
 import org.junit.runner.RunWith
 import org.junit.runners.Parameterized
-import org.junit.{Assert, Ignore, Test}
+import org.junit.{Assert, Test}
 
+import java.util.stream.Collectors
 import scala.util.{Failure, Try}
 
 object StatefulJobWBroadcastStateMigrationITCase {
 
-  @Parameterized.Parameters(name = "Migrate Savepoint / Backend: {0}")
-  def parameters: util.Collection[(MigrationVersion, String)] = {
-    util.Arrays.asList(
-      (MigrationVersion.v1_5, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_5, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_6, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_6, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_7, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_7, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_8, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_8, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_9, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_9, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_10, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_10, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_11, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_11, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_12, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_12, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_13, StateBackendLoader.MEMORY_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_13, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_14, StateBackendLoader.HASHMAP_STATE_BACKEND_NAME),
-      (MigrationVersion.v1_14, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME))
+  // TODO increase this to newer version to create and test snapshot migration for newer versions
+  val currentVersion = FlinkVersion.v1_14
+
+  // TODO change this to CREATE_SNAPSHOT to (re)create binary snapshots
+  // TODO Note: You should generate the snapshot based on the release branch instead of the
+  // master.
+  val executionMode = ExecutionMode.VERIFY_SNAPSHOT
+
+  @Parameterized.Parameters(name = "Test snapshot: {0}")
+  def parameters: util.Collection[SnapshotSpec] = {
+    // Note: It is not safe to restore savepoints created in a Scala applications with Flink
+    // version 1.7 or below. The reason is that up to version 1.7 the underlying Scala serializer
+    // used names of anonymous classes that depend on the relative position/order in code, e.g.,
+    // if two anonymous classes, instantiated inside the same class and from the same base class,
+    // change order in the code their names are switched.
+    // As a consequence, changes in code may result in restore failures.
+    // This was fixed in version 1.8, see: https://issues.apache.org/jira/browse/FLINK-10493
+    var parameters: util.List[SnapshotSpec] = new util.LinkedList[SnapshotSpec]()
+    parameters.addAll(
+      SnapshotSpec.withVersions(
+        StateBackendLoader.MEMORY_STATE_BACKEND_NAME,
+        SnapshotType.SAVEPOINT_CANONICAL,
+        FlinkVersion.rangeOf(FlinkVersion.v1_8, FlinkVersion.v1_13)))
+    parameters.addAll(
+      SnapshotSpec.withVersions(
+        StateBackendLoader.HASHMAP_STATE_BACKEND_NAME,
+        SnapshotType.SAVEPOINT_CANONICAL,
+        FlinkVersion.rangeOf(FlinkVersion.v1_14, currentVersion)))
+    parameters.addAll(
+      SnapshotSpec.withVersions(
+        StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME,
+        SnapshotType.SAVEPOINT_CANONICAL,
+        FlinkVersion.rangeOf(FlinkVersion.v1_15, currentVersion)))
+    parameters.addAll(
+      SnapshotSpec.withVersions(
+        StateBackendLoader.HASHMAP_STATE_BACKEND_NAME,
+        SnapshotType.SAVEPOINT_NATIVE,
+        FlinkVersion.rangeOf(FlinkVersion.v1_15, currentVersion)))
+    parameters.addAll(
+      SnapshotSpec.withVersions(
+        StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME,
+        SnapshotType.SAVEPOINT_NATIVE,
+        FlinkVersion.rangeOf(FlinkVersion.v1_15, currentVersion)))
+    parameters.addAll(
+      SnapshotSpec.withVersions(
+        StateBackendLoader.HASHMAP_STATE_BACKEND_NAME,
+        SnapshotType.CHECKPOINT,
+        FlinkVersion.rangeOf(FlinkVersion.v1_15, currentVersion)))
+    parameters.addAll(
+      SnapshotSpec.withVersions(
+        StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME,
+        SnapshotType.CHECKPOINT,
+        FlinkVersion.rangeOf(FlinkVersion.v1_15, currentVersion)))
+    if (executionMode == ExecutionMode.CREATE_SNAPSHOT) {
+      parameters = parameters.stream()
+        .filter(x => x.getFlinkVersion().equals(currentVersion))
+        .collect(Collectors.toList())
+    }
+    parameters
   }
 
-  // TODO to generate savepoints for a specific Flink version / backend type,
-  // TODO change these values accordingly, e.g. to generate for 1.3 with RocksDB,
-  // TODO set as (MigrationVersion.v1_3, StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME)
-  // TODO Note: You should generate the savepoint based on the release branch instead of the master.
-  val GENERATE_SAVEPOINT_VER: MigrationVersion = MigrationVersion.v1_14
-  val GENERATE_SAVEPOINT_BACKEND_TYPE: String = StateBackendLoader.HASHMAP_STATE_BACKEND_NAME
+  def getSnapshotPath(snapshotSpec: SnapshotSpec): String = {
+    val path = new StringBuilder(s"stateful-scala-with-broadcast-udf-migration-itcase")
+    path ++= s"-flink${snapshotSpec.getFlinkVersion()}"
+    path ++= s"-${snapshotSpec.getStateBackendType()}"
+    snapshotSpec.getSnapshotType() match {
+      case SnapshotType.SAVEPOINT_CANONICAL =>
+        path ++= "-savepoint"
+      case SnapshotType.SAVEPOINT_NATIVE =>
+        path ++= "-savepoint-native"
+      case SnapshotType.CHECKPOINT =>
+        path ++= "-checkpoint"
+      case _ => throw new UnsupportedOperationException
+    }
+    path.toString()
+  }
 
   val NUM_ELEMENTS = 4
 }
 
 /**
-  * ITCase for migration Scala state types across different Flink versions.
-  */
+ * ITCase for migration Scala state types across different Flink versions.
+ */
 @RunWith(classOf[Parameterized])
-class StatefulJobWBroadcastStateMigrationITCase(
-                                        migrationVersionAndBackend: (MigrationVersion, String))
-  extends SavepointMigrationTestBase with Serializable {
+class StatefulJobWBroadcastStateMigrationITCase(snapshotSpec: SnapshotSpec)
+  extends SnapshotMigrationTestBase with Serializable {
 
   @Test
-  @Ignore
-  def testCreateSavepointWithBroadcastState(): Unit = {
+  def testSavepointWithBroadcast(): Unit = {
+
     val env = StreamExecutionEnvironment.getExecutionEnvironment
     env.setStreamTimeCharacteristic(TimeCharacteristic.EventTime)
 
-    StatefulJobWBroadcastStateMigrationITCase.GENERATE_SAVEPOINT_BACKEND_TYPE match {
+    snapshotSpec.getStateBackendType match {
       case StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME =>
         env.setStateBackend(new EmbeddedRocksDBStateBackend())
       case StateBackendLoader.MEMORY_STATE_BACKEND_NAME =>
@@ -109,6 +155,7 @@ class StatefulJobWBroadcastStateMigrationITCase(
         env.setStateBackend(new HashMapStateBackend())
       case _ => throw new UnsupportedOperationException
     }
+    env.enableChangelogStateBackend(false)
 
     lazy val firstBroadcastStateDesc = new MapStateDescriptor[Long, Long](
       "broadcast-state-1",
@@ -120,7 +167,6 @@ class StatefulJobWBroadcastStateMigrationITCase(
       BasicTypeInfo.STRING_TYPE_INFO,
       BasicTypeInfo.STRING_TYPE_INFO)
 
-    env.setStateBackend(new MemoryStateBackend)
     env.enableCheckpointing(500)
     env.setParallelism(4)
     env.setMaxParallelism(4)
@@ -145,95 +191,45 @@ class StatefulJobWBroadcastStateMigrationITCase(
         new CheckpointedSource(4)).setMaxParallelism(1).uid("checkpointedBroadcastSource")
       .broadcast(firstBroadcastStateDesc, secondBroadcastStateDesc)
 
-    stream
-      .connect(broadcastStream)
-      .process(new TestBroadcastProcessFunction)
-      .addSink(new AccumulatorCountingSink)
+    if (StatefulJobWBroadcastStateMigrationITCase.executionMode == ExecutionMode.CREATE_SNAPSHOT) {
+      stream
+        .connect(broadcastStream)
+        .process(new TestBroadcastProcessFunction)
+        .addSink(new AccumulatorCountingSink)
 
-    executeAndSavepoint(
-      env,
-      s"src/test/resources/stateful-scala-with-broadcast" +
-        s"-udf-migration-itcase-flink" +
-        s"${StatefulJobWBroadcastStateMigrationITCase.GENERATE_SAVEPOINT_VER}" +
-        s"-${StatefulJobWBroadcastStateMigrationITCase.GENERATE_SAVEPOINT_BACKEND_TYPE}-savepoint",
-      new Tuple2(
-        AccumulatorCountingSink.NUM_ELEMENTS_ACCUMULATOR,
-        StatefulJobWBroadcastStateMigrationITCase.NUM_ELEMENTS
+      executeAndSnapshot(
+        env,
+        s"src/test/resources/"
+          + StatefulJobWBroadcastStateMigrationITCase.getSnapshotPath(snapshotSpec),
+        snapshotSpec.getSnapshotType(),
+        new Tuple2(
+          AccumulatorCountingSink.NUM_ELEMENTS_ACCUMULATOR,
+          StatefulJobWBroadcastStateMigrationITCase.NUM_ELEMENTS
+        )
       )
-    )
-  }
+    } else if (
+      StatefulJobWBroadcastStateMigrationITCase.executionMode == ExecutionMode.VERIFY_SNAPSHOT) {
+      val expectedFirstState: Map[Long, Long] =
+        Map(0L -> 0L, 1L -> 1L, 2L -> 2L, 3L -> 3L)
+      val expectedSecondState: Map[String, String] =
+        Map("0" -> "0", "1" -> "1", "2" -> "2", "3" -> "3")
 
-  @Test
-  def testRestoreSavepointWithBroadcast(): Unit = {
+      stream
+        .connect(broadcastStream)
+        .process(new VerifyingBroadcastProcessFunction(expectedFirstState, expectedSecondState))
+        .addSink(new AccumulatorCountingSink)
 
-    val env = StreamExecutionEnvironment.getExecutionEnvironment
-    env.setStreamTimeCharacteristic(TimeCharacteristic.EventTime)
-
-    migrationVersionAndBackend._2 match {
-      case StateBackendLoader.ROCKSDB_STATE_BACKEND_NAME =>
-        env.setStateBackend(new EmbeddedRocksDBStateBackend())
-      case StateBackendLoader.MEMORY_STATE_BACKEND_NAME =>
-        env.setStateBackend(new MemoryStateBackend())
-      case StateBackendLoader.HASHMAP_STATE_BACKEND_NAME =>
-        env.setStateBackend(new HashMapStateBackend())
-      case _ => throw new UnsupportedOperationException
+      restoreAndExecute(
+        env,
+        SnapshotMigrationTestBase.getResourceFilename(
+          StatefulJobWBroadcastStateMigrationITCase.getSnapshotPath(snapshotSpec)),
+        new Tuple2(
+          AccumulatorCountingSink.NUM_ELEMENTS_ACCUMULATOR,
+          StatefulJobWBroadcastStateMigrationITCase.NUM_ELEMENTS)
+      )
+    } else {
+      throw new UnsupportedOperationException("Unsupported execution mode.")
     }
-
-    lazy val firstBroadcastStateDesc = new MapStateDescriptor[Long, Long](
-      "broadcast-state-1",
-      BasicTypeInfo.LONG_TYPE_INFO.asInstanceOf[TypeInformation[Long]],
-      BasicTypeInfo.LONG_TYPE_INFO.asInstanceOf[TypeInformation[Long]])
-
-    lazy val secondBroadcastStateDesc = new MapStateDescriptor[String, String](
-      "broadcast-state-2",
-      BasicTypeInfo.STRING_TYPE_INFO,
-      BasicTypeInfo.STRING_TYPE_INFO)
-
-    env.setStateBackend(new MemoryStateBackend)
-    env.enableCheckpointing(500)
-    env.setParallelism(4)
-    env.setMaxParallelism(4)
-
-    val stream = env
-      .addSource(
-        new CheckpointedSource(4)).setMaxParallelism(1).uid("checkpointedSource")
-      .keyBy(
-        new KeySelector[(Long, Long), Long] {
-          override def getKey(value: (Long, Long)): Long = value._1
-        }
-      )
-      .flatMap(new StatefulFlatMapper)
-      .keyBy(
-        new KeySelector[(Long, Long), Long] {
-          override def getKey(value: (Long, Long)): Long = value._1
-        }
-      )
-
-    val broadcastStream = env
-      .addSource(
-        new CheckpointedSource(4)).setMaxParallelism(1).uid("checkpointedBroadcastSource")
-      .broadcast(firstBroadcastStateDesc, secondBroadcastStateDesc)
-
-    val expectedFirstState: Map[Long, Long] =
-      Map(0L -> 0L, 1L -> 1L, 2L -> 2L, 3L -> 3L)
-    val expectedSecondState: Map[String, String] =
-      Map("0" -> "0", "1" -> "1", "2" -> "2", "3" -> "3")
-
-    stream
-      .connect(broadcastStream)
-      .process(new VerifyingBroadcastProcessFunction(expectedFirstState, expectedSecondState))
-      .addSink(new AccumulatorCountingSink)
-
-    restoreAndExecute(
-      env,
-      SavepointMigrationTestBase.getResourceFilename(
-        s"stateful-scala-with-broadcast" +
-          s"-udf-migration-itcase-flink${migrationVersionAndBackend._1}" +
-          s"-${migrationVersionAndBackend._2}-savepoint"),
-      new Tuple2(
-        AccumulatorCountingSink.NUM_ELEMENTS_ACCUMULATOR,
-        StatefulJobWBroadcastStateMigrationITCase.NUM_ELEMENTS)
-    )
   }
 }
 
