@@ -20,7 +20,6 @@ package org.apache.flink.yarn;
 
 import org.apache.flink.api.common.JobID;
 import org.apache.flink.api.common.JobStatus;
-import org.apache.flink.api.common.time.Deadline;
 import org.apache.flink.client.deployment.ClusterDeploymentException;
 import org.apache.flink.client.deployment.ClusterSpecification;
 import org.apache.flink.client.program.ClusterClient;
@@ -76,7 +75,6 @@ import javax.annotation.Nonnull;
 import java.io.File;
 import java.io.IOException;
 import java.nio.charset.StandardCharsets;
-import java.time.Duration;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
@@ -107,7 +105,6 @@ public class YARNHighAvailabilityITCase extends YarnTestBase {
     @ClassRule public static final TemporaryFolder FOLDER = new TemporaryFolder();
 
     private static final String LOG_DIR = "flink-yarn-tests-ha";
-    private static final Duration TIMEOUT = Duration.ofSeconds(200L);
 
     private static TestingServer zkServer;
     private static String storageDir;
@@ -262,8 +259,7 @@ public class YARNHighAvailabilityITCase extends YarnTestBase {
                             yarnClient.getApplicationReport(applicationId);
                     return applicationReport.getCurrentApplicationAttemptId().getAttemptId()
                             >= attemptId;
-                },
-                Deadline.fromNow(TIMEOUT));
+                });
         log.info("Attempt {} id detected.", attemptId);
     }
 
@@ -319,8 +315,7 @@ public class YARNHighAvailabilityITCase extends YarnTestBase {
                                         EnumSet.of(
                                                 YarnApplicationState.KILLED,
                                                 YarnApplicationState.FINISHED))
-                                .isEmpty(),
-                Deadline.fromNow(TIMEOUT));
+                                .isEmpty());
     }
 
     private void waitForJobTermination(
@@ -329,7 +324,7 @@ public class YARNHighAvailabilityITCase extends YarnTestBase {
         log.info("Sending stop job signal");
         stopJobSignal.signal();
         final CompletableFuture<JobResult> jobResult = restClusterClient.requestJobResult(jobId);
-        jobResult.get(TIMEOUT.toMillis(), TimeUnit.MILLISECONDS);
+        jobResult.get(200, TimeUnit.SECONDS);
     }
 
     @Nonnull
@@ -389,8 +384,7 @@ public class YARNHighAvailabilityITCase extends YarnTestBase {
                 () -> {
                     final Set<Integer> curPids = getApplicationMasterPids(processName);
                     return origPids.stream().noneMatch(curPids::contains);
-                },
-                Deadline.fromNow(TIMEOUT));
+                });
     }
 
     private Set<Integer> getApplicationMasterPids(final String processName)
@@ -417,8 +411,7 @@ public class YARNHighAvailabilityITCase extends YarnTestBase {
                             && jobDetails.getJobVertexInfos().stream()
                                     .map(toExecutionState())
                                     .allMatch(isRunning());
-                },
-                Deadline.fromNow(TIMEOUT));
+                });
     }
 
     private static Function<JobDetailsInfo.JobVertexDetailsInfo, ExecutionState>
@@ -436,8 +429,7 @@ public class YARNHighAvailabilityITCase extends YarnTestBase {
             final int expectedFullRestarts)
             throws Exception {
         CommonTestUtils.waitUntilCondition(
-                () -> getJobFullRestarts(restClusterClient, jobId) >= expectedFullRestarts,
-                Deadline.fromNow(TIMEOUT));
+                () -> getJobFullRestarts(restClusterClient, jobId) >= expectedFullRestarts);
     }
 
     private static int getJobFullRestarts(
