@@ -105,7 +105,7 @@ if [ $STAGE == $STAGE_PYTHON ]; then
 	run_with_watchdog "./flink-python/dev/lint-python.sh" $CALLBACK_ON_TIMEOUT
 	EXIT_CODE=$?
 else
-	MVN_TEST_OPTIONS="-Dflink.tests.with-openssl -Dflink.tests.check-segment-multiple-free"
+	MVN_TEST_OPTIONS="-Dflink.tests.with-openssl -Dflink.tests.check-segment-multiple-free -Darchunit.freeze.store.default.allowStoreUpdate=false"
 	if [ $STAGE = $STAGE_FINEGRAINED_RESOURCE_MANAGEMENT ]; then
 		if [[ ${PROFILE} == *"enable-adaptive-scheduler"* ]]; then
 			echo "Skipping fine grained resource management test stage in adaptive scheduler job"
